package de.tud.kom.p2psim.impl.analyzer;

import java.io.Writer;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.analyzer.Analyzer.OperationAnalyzer;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.KBRChordNode;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * This is just a little helper when working with chord. 
 * It creates a system outs at the end of a simulation with basic information
 * about the chord ring. 
 * 
 * @author Julius Rueckert
 *
 */
public class ChordStructureAnalyzer implements OperationAnalyzer {

	private final static Logger log = SimLogger.getLogger(ChordStructureAnalyzer.class);
	
	public ChordStructureAnalyzer(){
	}

	@Override
	public void start() {
		log.debug("ChordStructureAnalyzer started");
	}

	@Override
	public void stop(Writer output) {
	
		printFingertest();
		printRingTest();

	}
	
	Map<BigInteger, KBRChordNode> hosts = new HashMap<BigInteger, KBRChordNode>();
	
	
	@Override
	public void operationFinished(Operation<?> op) {
		KBRChordNode node = (KBRChordNode) op.getComponent().getHost().getOverlay(KBRChordNode.class);
		hosts.put(node.getChordID().getValue(), node);
	}

	@Override
	public void operationInitiated(Operation<?> op) {
		KBRChordNode node = (KBRChordNode) op.getComponent().getHost().getOverlay(KBRChordNode.class);
		hosts.put(node.getChordID().getValue(), node);
	}


	Map<Message, NetID> netSender = new HashMap<Message, NetID>();

	/*
	 * The following methods are just for debugging
	 */

	private void printFingertest(){
		KBRChordNode node = null;
		boolean fingersSet = true;
		
		for (Entry<BigInteger,KBRChordNode> entry : hosts.entrySet()) {
			node = entry.getValue();
			
			
			
			List<ChordContact> fingers = node.getFingerTable().allContacts();
			
			for (ChordContact finger : fingers) {
				if(finger==null)
					fingersSet = false;
			}
			if(fingers.size() != 160)
				fingersSet = false;
		}
		System.out.println("=Fingers==============");
		System.out.println("All fingers set? "+(fingersSet?"Yes":"No"));
		System.out.println("===============");
	}
	
	private void printRingTest(){
		KBRChordNode node = null;
		for (Entry<BigInteger,KBRChordNode> entry : hosts.entrySet()) {
			node = entry.getValue();
			break;
		}
		boolean consistent = areSuccPredLinksConsistent();
		
		System.out.println("=Ring==============");
		System.out.println("Is Ring consistent? "+(consistent?"Yes":"No"));
		if(consistent){
			System.out.println("Is it there only one ring? "+(countRingMembers(node)==hosts.size()?"Yes":"No"));
			System.out.println("Size of Ring: "+hosts.size());
		}
		System.out.println("===============");
	}
	
	private boolean areSuccPredLinksConsistent(){
		for (Entry<BigInteger,KBRChordNode> entry : hosts.entrySet()) {
			BigInteger localID = entry.getKey();
			
			if(entry.getValue().getPre()==null || entry.getValue().nextActiveSucc()==null){
				return false;
			} else {
				KBRChordNode pre = hosts.get(entry.getValue().getPre().getOverlayID().getValue());
				KBRChordNode succ = hosts.get(entry.getValue().nextActiveSucc().getOverlayID().getValue());
			
				if(pre.nextActiveSucc().getOverlayID().getValue().compareTo(localID) != 0 ||
				   succ.getPre()== null ||
				   succ.getPre().getOverlayID().getValue().compareTo(localID) != 0){
					System.out.println("The ring is not stabilized yet.");
					return false;
				}
			}
		}
		return true;
	}
	
	private int countRingMembers(KBRChordNode startNode){
		
		KBRChordNode currentSucc = hosts.get(startNode.nextActiveSucc().getOverlayID().getValue());
		int count = 1;
		while(currentSucc.getChordID().getValue().compareTo(
							startNode.getChordID().getValue()) != 0){
			count++;
			currentSucc = hosts.get(currentSucc.nextActiveSucc().getOverlayID().getValue());
		}
		return count;
	}

}
