package de.tud.kom.p2psim.impl.analyzer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import de.tud.kom.p2psim.api.analyzer.Analyzer.OperationAnalyzer;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.network.NetMessage;
import de.tud.kom.p2psim.impl.overlay.dht.chord.operations.LookupOperation;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class DHTAnalyzer extends NetWorkloadAnalyzer implements OperationAnalyzer {

	private boolean isRunning;

	private Map<Integer, LookupData> lookupPerformancs;

	private Map<NetID, PeerData> peerPerformances;

	private long numStaleMsg, numMsgDropped;

	class LookupData {
		public int numHops;

		public long startTime;

		public long finishTime;

		public boolean successful;
	}

	class PeerData {
		// public long msgReceived=0;
	}

	public DHTAnalyzer() {
		super();
		lookupPerformancs = new TreeMap<Integer, LookupData>();
		peerPerformances = new HashMap<NetID, PeerData>();
		this.isRunning = false;
	}

	public void start() {
		super.start();
		this.isRunning = true;
		lookupPerformancs.clear();
		peerPerformances.clear();
		this.numMsgDropped = 0;
	}

	public void operationInitiated(Operation<?> op) {
		if (isRunning) {
			if (op instanceof LookupOperation) {
				LookupOperation lookup = (LookupOperation) op;
				if (lookup.forValue()) {
					LookupData d = lookupPerformancs.get(lookup.getOperationID());
					if (d == null) {
						d = new LookupData();
						d.startTime = Simulator.getCurrentTime();
						lookupPerformancs.put(lookup.getOperationID(), d);
					} else {
						d.startTime = Simulator.getCurrentTime();
					}
				}
			}
		}
	}

	public void operationFinished(Operation<?> op) {
		if (isRunning) {
			if (op instanceof LookupOperation) {
				LookupOperation lookup = (LookupOperation) op;
				if (lookup.forValue()) {
					LookupData d = lookupPerformancs.get(lookup.getOperationID());
					if (d == null) {
						System.out.println("impossible!! in DHT analyser");
					} else {
						d.finishTime = Simulator.getCurrentTime();
						d.successful = lookup.isSuccessful();
						d.numHops = lookup.getHops();
					}
				}
			}
		}
	}

	public void netMsgDrop(NetMessage msg, NetID id) {
		super.netMsgDrop(msg, id);
		numMsgDropped++;

	}

	public void stop(Writer output) {
		super.stop(output);
		try {
			this.isRunning = false;
			ArrayList<Integer> simuTime = new ArrayList<Integer>();
			ArrayList<Integer> lookupHop = new ArrayList<Integer>();
			ArrayList<Integer> lookupResp = new ArrayList<Integer>();
			 ArrayList<Integer> sumLookups = new ArrayList<Integer>();
			 ArrayList<Float> sumLookupSuccRatio = new ArrayList<Float>();
			// ArrayList<Float> aveHops = new ArrayList<Float>();
			// ArrayList<Float> aveResp = new ArrayList<Float>();
			int sumLookup = 0, sumHops = 0, sumResponse = 0;
			int sumLookupSucc = 0;

			Iterator<LookupData> list = this.lookupPerformancs.values().iterator();
			while (list.hasNext()) {
				LookupData data = list.next();
				sumLookup++;
				if (data.successful) {
					sumLookupSucc++;
					simuTime.add((int) (data.startTime / 1000000));

					lookupHop.add(data.numHops);
					sumHops += data.numHops;

					lookupResp.add((int) ((data.finishTime - data.startTime) / 1000));
					sumResponse += lookupResp.get(lookupResp.size() - 1);
				}
				 if (sumLookup%20 ==0 ) {
				 sumLookups.add(sumLookup);
				 sumLookupSuccRatio.add(sumLookupSucc * 100f / sumLookup);
				 }

			}

			// ################ SimulationTime func FILE ################
			File f = new File("outputs/monitoring/timeFunc_seed_" + Simulator.getSeed() + ".dat");
			boolean exists = f.exists();

			// create directories if necessary
			if (!exists) {
				File parent = f.getParentFile();
				if (parent != null) { // should always be true
					parent.mkdirs();
				}
			}

			BufferedWriter bw = new BufferedWriter(new FileWriter(f, true));
			if (!exists) {
				bw.write("#SimulationTime\t Hops_per_lookup\t ResponsTime_per_lookup");
				bw.newLine();
			} else
				bw.newLine();
			for (int i = 0; i < simuTime.size(); i++) {
				bw.write(simuTime.get(i) + "\t" + lookupHop.get(i) + "\t" + lookupResp.get(i));
				bw.newLine();
			}

			bw.close();

			// ################ num_of_Ops func FILE ################
			f = new File("outputs/monitoring/numOps_func_seed_" + Simulator.getSeed() + ".dat");
			exists = f.exists();

			// create directories if necessary
			if (!exists) {
				File parent = f.getParentFile();
				if (parent != null) { // should always be true
					parent.mkdirs();
				}
			}

			bw = new BufferedWriter(new FileWriter(f, true));
			if (!exists) {
				bw.write("#sum_lookupOps\t lookupSuccRatio\t aveLookupHops\t aveLookupResponse");
				bw.newLine();
			} else
				bw.newLine();
			for (int i = 0; i < sumLookups.size(); i++) {
				bw.write(sumLookups.get(i) + "\t" + sumLookupSuccRatio.get(i));
				bw.newLine();
			}
			float aveHops = sumHops / (float) sumLookupSucc;
			float aveResp = sumResponse / (float) sumLookupSucc;
			bw.write("# aveHops: "+aveHops);
			bw.newLine();
			bw.write("# aveResp: "+ aveResp);
			bw.newLine();
			bw.close();
		} catch (IOException e) {
			System.err.println("Problems in writing results" + e);
		}
	}

	public String toString() {
		return "DHT analyzer";
	}
}
