package de.tud.kom.p2psim.impl.application.KBRApplication;

import java.util.HashMap;
import java.util.Random;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.KBR;
import de.tud.kom.p2psim.api.overlay.KBRListener;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.impl.application.AbstractApplication;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.AnnounceNewDocumentMessage;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.QueryForDocumentMessage;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.QueryResultMessage;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.RequestDocumentMessage;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.TransferDocumentMessage;
import de.tud.kom.p2psim.impl.application.KBRApplication.operations.AnnounceNewDocumentOperation;
import de.tud.kom.p2psim.impl.application.KBRApplication.operations.QueryForDocumentOperation;
import de.tud.kom.p2psim.impl.application.KBRApplication.operations.QueryResultOperation;
import de.tud.kom.p2psim.impl.application.KBRApplication.operations.RequestDocumentOperation;
import de.tud.kom.p2psim.impl.application.KBRApplication.operations.TransferDocumentOperation;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * This is a dummy application to check some basic functionality of KBR overlays.
 * 
 * @author Julius Rueckert
 *
 */
public class KBRDummyApplication extends AbstractApplication implements KBRListener {
	
	private final static Logger log = SimLogger.getLogger(KBRDummyApplication.class);
	private KBR node;
	private HashMap<OverlayKey, OverlayContact> localDB = new HashMap<OverlayKey, OverlayContact>();
	private OverlayID localOverlayID;
	private static Random rGen = new Random(Simulator.getSeed());
	
	/**
	 * @param node
	 */
	public KBRDummyApplication(KBR node){
		this.node = node;
		localOverlayID = node.getLocalOverlayContact().getOverlayID();
		
		// Set the application as KBRListener
		node.setKBRListener(this);
	}
	
	@Override
	public void deliver(OverlayKey key, Message msg) {
		if(key==null){
			// It is a direct message to this node
			
			if(msg instanceof QueryResultMessage){
				QueryResultMessage queryResMsg = (QueryResultMessage)msg;
				log.debug(localOverlayID+" - Received query result for rank "+queryResMsg.getKey()+" from "+queryResMsg.getSenderContact().getOverlayID());
				
				if(queryResMsg.getDocumentProvider() != null){
					log.debug(localOverlayID+" - Got a query result for key "+queryResMsg.getKey()+": File provider is "+queryResMsg.getDocumentProvider().getOverlayID());
					
					RequestDocumentOperation operation = new RequestDocumentOperation(this, 
																			  queryResMsg.getKey(),
																			  queryResMsg.getDocumentProvider(),
																			  new OperationCallback(){

						@Override
						public void calledOperationFailed(Operation op) {
							// TODO Auto-generated method stub
							
						}

						@Override
						public void calledOperationSucceeded(Operation op) {
							// TODO Auto-generated method stub
							
						}});
					
					operation.scheduleImmediately();
					
				} else {
					log.debug(localOverlayID+" - Got a query result for key "+queryResMsg.getKey()+": The root of the key does not hold such a document.");
				}
				
			} else if (msg instanceof RequestDocumentMessage){
				RequestDocumentMessage requestDocumentMsg = (RequestDocumentMessage)msg;
				log.debug(localOverlayID+" - Received request for document: "+requestDocumentMsg.getKeyOfDocument());
				
				TransferDocumentOperation operation = new TransferDocumentOperation(this,
																			requestDocumentMsg.getKeyOfDocument(),
																			requestDocumentMsg.getSenderContact(),
																			new OperationCallback(){

					@Override
					public void calledOperationFailed(Operation op) {
						// TODO Auto-generated method stub
						
					}

					@Override
					public void calledOperationSucceeded(Operation op) {
						// TODO Auto-generated method stub
						
					}});
				
				operation.scheduleImmediately();
			} else if (msg instanceof TransferDocumentMessage){
				
				TransferDocumentMessage transferDocumentMsg = (TransferDocumentMessage)msg;
				
				log.debug(localOverlayID+" - Received requested document: "+transferDocumentMsg.getKeyOfDocument()); 
			}
			
		} else if(node.isRootOf(key)){
			// It is a message delivered here because this is the root of the key
			
			if(msg instanceof AnnounceNewDocumentMessage){
				// As this message was delivered to this node, it must be the root of the key.
				// So we store the information about the key and the node that holds the key.
				
				AnnounceNewDocumentMessage announceMsg = (AnnounceNewDocumentMessage)msg;
				
				// Store key of file and sender of message in local database
				localDB.put(announceMsg.getDocumentKey(), announceMsg.getSenderContact());

				log.debug(localOverlayID+" - Stored information about new file (Key: "+announceMsg.getDocumentKey()+")");
			} else if(msg instanceof QueryForDocumentMessage){
				// As this message was delivered to this node, it must be the root of the 
				
				QueryForDocumentMessage queryMessage = (QueryForDocumentMessage)msg;
				log.debug(localOverlayID+" - Received query message for rank:"+queryMessage.getKey());
				
				OverlayContact documentProvider = lookupDocumentProviderInLocalDB(queryMessage.getKey());

				OverlayContact receiver = queryMessage.getSenderContact();
				QueryResultOperation operation = new QueryResultOperation(this, 
																	      queryMessage.getKey(), 
																	      documentProvider, 
																	      receiver, 
																	      new OperationCallback(){

					@Override
					public void calledOperationFailed(Operation op) {
						// TODO Auto-generated method stub
						
					}

					@Override
					public void calledOperationSucceeded(Operation op) {
						// TODO Auto-generated method stub
						
					}});
				
				operation.scheduleImmediately();
			} 
		} else {
			log.error(localOverlayID+" - I received a message that is not for me!");
		}

	}

	@Override
	public void forward(OverlayKey key, Message msg, OverlayContact nextHopAgent) {
		// TODO Auto-generated method stub
	}

	@Override
	public void update(OverlayContact contact, boolean joined) {
		if (joined)
			log.debug(contact.getOverlayID()+" - joined");
		else
			log.debug(contact.getOverlayID()+" - left");
	}
	
	/**
	 * @return the KBR node on which the application is running
	 */
	public KBR getNode(){
		return node;
	}
	
	/**
	 * Stores an new document and announces the existence in the overlay. Documents have
	 * no names but ranks that identifies them.
	 * 
	 * @param rank the rank oft the new document
	 * @param size the size in bytes of the new document
	 */
	public void storeNewFile(int rank, int size){
		log.debug(localOverlayID+" - Store new file initiated (rank:"+rank+" size:"+size+")");
		
		// Create the document
		OverlayKey key = node.getNewOverlayKey(rank);
		KBRDocument doc = new KBRDocument(key);
		doc.setSize(size);
		
		AnnounceNewDocumentOperation operation = new AnnounceNewDocumentOperation(this, doc ,new OperationCallback(){

			@Override
			public void calledOperationFailed(Operation op) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void calledOperationSucceeded(Operation op) {
				// TODO Auto-generated method stub
				
			}});
		operation.scheduleImmediately();
	}
	
	
	public void storeRandomNewFile(){
		
		int ranRank = rGen.nextInt();
		int ranSize = rGen.nextInt(200);
		
		storeNewFile(ranRank, ranSize);
	}
	
	/**
	 * Queries for document and transfers it to this host.
	 * 
	 * @param rank the rank that identifies the document
	 */
	public void getDocument(int rank){
		OverlayKey key = node.getNewOverlayKey(rank);
		log.debug(localOverlayID+" - Start procedure to get the document with rank "+rank+" (key: "+key+")");
		
		QueryForDocumentOperation operation = new QueryForDocumentOperation(this, key , new OperationCallback(){

			@Override
			public void calledOperationFailed(Operation op) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void calledOperationSucceeded(Operation op) {
				// TODO Auto-generated method stub
				
			}});
		operation.scheduleImmediately();
	}
	
	public void getRandomDocument(){
		Random rGen = new Random();
		int ranRank = rGen.nextInt();
		
		getDocument(ranRank);
	}
	
	
	private OverlayContact lookupDocumentProviderInLocalDB(OverlayKey key){
		 Set<Entry<OverlayKey, OverlayContact>> dbAsSet = localDB.entrySet();
		 
		 for (Entry<OverlayKey, OverlayContact> entry : dbAsSet) {
			if(entry.getKey().compareTo(key) == 0){
				return entry.getValue();
			}
		}
		return null;
	}
	
	
}
