package de.tud.kom.p2psim.impl.application.KBRApplication.operations;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.impl.application.KBRApplication.KBRDummyApplication;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.QueryResultMessage;
import de.tud.kom.p2psim.impl.common.AbstractOperation;

/**
 * This operation sends a QueryResultMessage direct to a contact
 * 
 * @author Julius Rueckert
 *
 */
public class QueryResultOperation extends AbstractOperation {

	private KBRDummyApplication app;
	private OverlayKey key;
	private OverlayContact senderContact;
	private OverlayContact documentProvider;
	private OverlayContact receiverContact;
	
	/**
	 * @param app the application that created the operation
	 * @param key the key of the documents that was queried 
	 * @param documentProvider the provider of the document
	 * @param receiverContact the contact of the receiver of the message
	 * @param callback
	 */
	public QueryResultOperation(KBRDummyApplication app, OverlayKey key, OverlayContact documentProvider, OverlayContact receiverContact, OperationCallback callback){
		super(app, callback);
		
		this.key = key;
		this.app = app;
		this.senderContact = app.getNode().getLocalOverlayContact();
		this.documentProvider = documentProvider;
		this.receiverContact = receiverContact;
	}
	
	@Override
	protected void execute() {
		
		Message msg = new QueryResultMessage(senderContact, key, documentProvider);
		
		// Route the message direct to the receiver (direct means the key is null and 
		// the receiver is given as hint)
		app.getNode().route(null, msg, receiverContact);
		
		operationFinished(true);
	}

	@Override
	public Object getResult() {
		// TODO Auto-generated method stub
		return null;
	}

}
