package de.tud.kom.p2psim.impl.application.KBRApplication.operations;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.impl.application.KBRApplication.KBRDummyApplication;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.RequestDocumentMessage;
import de.tud.kom.p2psim.impl.common.AbstractOperation;

/**
 * This operation requests a document to be transfered. Therefore the document provider
 * was already determined by a query for the key of the file.
 * 
 * @author Julius Rueckert
 *
 */
public class RequestDocumentOperation extends AbstractOperation {

	KBRDummyApplication app;
	OverlayKey keyOfDocument;
	OverlayContact receiverContact;
	
	/**
	 * @param app the application that created the operation
	 * @param keyOfFile the key of the document to request
	 * @param receiverContact the contact of the receiver of the message
	 * @param callback
	 */
	public RequestDocumentOperation(KBRDummyApplication app, OverlayKey keyOfFile, OverlayContact receiverContact, OperationCallback callback){
		super(app, callback);
		
		this.app = app;
		this.keyOfDocument = keyOfFile;
		this.receiverContact = receiverContact;
	}
	
	@Override
	protected void execute() {
		// Send a direct message to the file provider
		
		OverlayContact senderContact = app.getNode().getLocalOverlayContact();
		Message requestFileMessage = new RequestDocumentMessage(keyOfDocument, senderContact);
		app.getNode().route(null, requestFileMessage, receiverContact);

		operationFinished(true);
	}

	@Override
	public Object getResult() {
		// TODO Auto-generated method stub
		return null;
	}

}
