package de.tud.kom.p2psim.impl.application.KBRApplication.operations;

import java.util.Collection;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.storage.Document;
import de.tud.kom.p2psim.impl.application.KBRApplication.KBRDummyApplication;
import de.tud.kom.p2psim.impl.application.KBRApplication.messages.TransferDocumentMessage;
import de.tud.kom.p2psim.impl.common.AbstractOperation;

/**
 * This operation transfers a document direct to a contact
 * 
 * @author Julius Rueckert
 *
 */
public class TransferDocumentOperation extends AbstractOperation {

	KBRDummyApplication app;
	OverlayKey keyOfDocument;
	OverlayContact receiverContact;
	
	/**
	 * @param app the application that created the operation
	 * @param keyOfDocument the key of the document to be sent
	 * @param receiverContact the contact of the receiver of the message
	 * @param callback
	 */
	public TransferDocumentOperation(KBRDummyApplication app, OverlayKey keyOfDocument, OverlayContact receiverContact, OperationCallback callback){
		super(app, callback);
		
		this.app = app;
		this.keyOfDocument = keyOfDocument;
		this.receiverContact = receiverContact;
	}
	
	@Override
	protected void execute() {
		OverlayContact senderContact = app.getNode().getLocalOverlayContact();

		// Load document from local storage
		Document doc = getDocumnetFromStorage(keyOfDocument);
		
		// Send file direct to the receiver without routing for the key
		TransferDocumentMessage msg = new TransferDocumentMessage(keyOfDocument, senderContact, doc);
		app.getNode().route(null, msg, receiverContact);
		
		operationFinished(true);
	}

	@Override
	public Object getResult() {
		// TODO Auto-generated method stub
		return null;
	}
	
	private Document getDocumnetFromStorage(OverlayKey key){
		Collection<OverlayKey> keys = app.getHost().getStorage().listDocumentKeys();
		
		for (OverlayKey currentKey : keys) {
			if(currentKey.compareTo(key) == 0){
				return app.getHost().getStorage().loadDocument(currentKey);
			}
		}
		return null;
	}

}
