package de.tud.kom.p2psim.impl.network;

import java.util.Random;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.network.NetProtocol;
import de.tud.kom.p2psim.impl.network.AbstractNetMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.TCPMessage;
import de.tud.kom.p2psim.impl.transport.UDPMessage;

/**
 * Extends a normal IPv4 Message to work with priorities
 * 
 * @author Julius Rueckert
 */
public class IPv4PriorityMessage extends IPv4Message {
	
	/**
	 * The delay priority of the NetMessage
	 */
	private byte delayPriority;
	
	/**
	 * The loss priority of the NetMessage
	 */
	private byte lossPriority;
	
	/**
	 * The time since the last insertion of the NetMessage into a buffer
	 */
	private long insertInBufferTime = 0;
	

	public IPv4PriorityMessage(Message payload, NetID receiver, NetID sender) {
		super(payload, receiver, sender);
		
		//initialize the delayPriority of the message with a random byte [-128,127] 
		Random randomGenerator = new Random();
		this.delayPriority = (byte)randomGenerator.nextInt();
		
		//initialize the lossPriority of the message with a random byte [-128,127] 
		this.lossPriority = (byte)randomGenerator.nextInt();
		
	}
	
	/**
	 * @return the delayPriority of the message used for bandwidth management
	 */
	public byte getDelayPriority() {
		return this.delayPriority;
	}

	/**
	 * @return the lossPriority of the message used for bandwidth management
	 */
	public byte getLossPriority() {
		return lossPriority;
	}
	
	/**
	 * Set the insert in Buffer time to the current simulator time
	 */
	public void setInsertedInBuffer(){
		insertInBufferTime = Simulator.getCurrentTime();
	}
	
	/**
	 * @return the time since the insertion into buffer
	 */
	public long getTimeSinceBufferInsert(){
		return Simulator.getCurrentTime() - insertInBufferTime;
	}

}