package de.tud.kom.p2psim.impl.network.gnp;

import de.tud.kom.p2psim.impl.simengine.Simulator;

import java.awt.*;
import java.awt.geom.Point2D;
import java.util.ArrayList;

/**
 * Assigns peers to x,y coordinates accoring to the grey colors of a 8bit bitmap.
 * PeerDistributionFromBitmap must be initialized first. Then, getNextPeerLocation returns a
 * peers location accoring to the ditribution based on the bitmap.
 *
 * @author Andre Mink, Sebastian Kaune
 */
public class PeerDistributionFromBitmap {
    private static PeerDistributionFromBitmap instance;
    private int mapWidth;
    private int mapHeight;
    private boolean initialized;
    private ArrayList map;

    public static PeerDistributionFromBitmap getInstance() {
        if (instance == null) instance = new PeerDistributionFromBitmap();
        return instance;
    }

    /**
     * Sets the path to the bitmap and the number of peers of this experiment
     *
     * @param pathToBitmap  path to the bitmap
     * @param numberOfPeers number of peers of this experiment
     */
    public void initialize(String pathToBitmap, int numberOfPeers) {
        BitmapLoader bl = new BitmapLoader(pathToBitmap);

        map = bl.assignPeers(numberOfPeers, bl.getDistributionFromBitmap());

        mapWidth = bl.width;
        mapHeight = bl.height;

        initialized = true;
        instance = this;
    }

    /**
     * Returns the next x,y coordinate of a peer based on the bitmap
     *
     * @return the next x,y coordinate of a peer
     */
    public Point2D.Double getNextPeerLocation() {
        Point p = instance.getNextLocation();
        double x = -1;
        double y = -1;
        while (x < 0.0 || x > mapWidth || y < 0.0 || y > mapHeight) {
            x = p.x - Simulator.getRandom().nextDouble() + 0.5;
            y = p.y - Simulator.getRandom().nextDouble() + 0.5;
        }
        return new Point2D.Double(x, y);
    }

    /**
     * Returns the status
     *
     * @return <code>true</code> if PeerDistributionFromBitmap has been initilaized,
     *         <code>false</code> otherwise
     */
    public boolean isInitialized() {
        return initialized;
    }


    public PeerDistributionFromBitmap() {
        this.map = new ArrayList();
        this.mapHeight = 0;
        this.mapWidth = 0;
        this.initialized = false;
    }

    private Point getNextLocation() {
        int i = Simulator.getRandom().nextInt(map.size());
        return (Point) map.remove(i);
    }

}
