package de.tud.kom.p2psim.impl.network.gnp.topology;

import de.tud.kom.p2psim.api.network.NetPosition;

/**
 * Implementation of NetPosition for Position and distance measurnment on the earth.
 * 
 * @author Gerald Klunker
 * @version 0.1, 05.02.2008
 *
 */
public class GeographicPosition implements NetPosition {

	private double latitude;
	private double longitude;

	
	/**
	 * 
	 * @param longitude
	 * @param latitude
	 */
	public GeographicPosition(double longitude, double latitude) {
		this.longitude = longitude;
		this.latitude = latitude;
	}

	/**
	 * @return geographical distance in km
	 */
	public double getDistance(NetPosition point) {
		double pi = 3.14159265;
		double radConverter = pi / 180;
		double lat1 = latitude * radConverter;
		double lat2 = ((GeographicPosition) point).getLatitude() * radConverter;
		double delta_lat = lat2 - lat1;
		double delta_lon = (((GeographicPosition) point).getLongitude() - longitude) * radConverter;
		double temp = Math.pow(Math.sin(delta_lat / 2), 2) + Math.cos(lat1) * Math.cos(lat2) * Math.pow(Math.sin(delta_lon / 2), 2);
		return 2 * 6378.2 * Math.atan2(Math.sqrt(temp), Math.sqrt(1 - temp));
	}

	/**
	 * 
	 * @return latitude of position
	 */
	public double getLatitude() {
		return latitude;
	}

	/**
	 * 
	 * @return longitude of position
	 */
	public double getLongitude() {
		return longitude;
	}

}
