package de.tud.kom.p2psim.impl.network.gnp.topology;

import java.io.File;

/**
 * Adapt this class for generating an xml-Host file without gui.
 * 
 * @author Gerald Klunker
 * @version 0.1, 17.01.2008
 * 
 */
public class XmlHostGenerator {

	public static void main(String[] args) {

		HostMap map = new HostMap();

		// Import Skitter Files
		System.out.println("=== Import Skitter Files ===");
		map.importSkitterFile(new File("data/Skitter/f-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/sjc_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/uoregon_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/lhr_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/h-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/mwest_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/yto_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/i-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/e-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/champagne_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/riesling_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/cam_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/neu_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/m-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/d-root_2007_08_clean.skitter"));
		map.importSkitterFile(new File("data/Skitter/arin_2007_08_clean.skitter"));

		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Keep 6 maximum separated Monitors and delete Hosts with less
		// connectivity
		System.out.println("=== Keep 6 maximum separated Monitors and delete Hosts with less connectivity ===");
		map.makeSymmetrical();
		map.removeMonitorsKeepMaximumSparation(6);
		map.removeHosts(0);
		map.removeHosts(1);
		map.removeHosts(2);
		map.removeHosts(3);
		map.removeHosts(4);
		map.removeHosts(5);
		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Import GeoIP Files and locate Hosts
		System.out.println("=== Import GeoIP File and locate Hosts ===");
		map.setGeolocationDatabase("data/GeoIP/GeoLiteCity20070901.dat");
//		map.setIspLocationDatabase("data/GeoIP/GeoIPISP.csv");
		map.getCountryLookup().importPingErCountryRegionFile(new File("data/PingER/region_country.txt"));
		map.setLocationOfHosts();

		// Remove unlocatable Hosts
		System.out.println("=== Remove unlocatable Hosts ===");
		map.removeHosts(map.getLocations().get("# UNLOCATABLE"));
		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Import PingER Files
		System.out.println("=== Import PingER Files ===");
		map.importPingErAverageRtt(new File("data/PingER/average_rtt_Continent_To_Continent_date=2007-08_.tsv"));
		map.importPingErAverageRtt(new File("data/PingER/average_rtt_Continent_To_Country_date=2007-08_.tsv"));
		map.importPingErAverageRtt(new File("data/PingER/average_rtt_Country_To_Continent_date=2007-08_.tsv"));
		map.importPingErAverageRtt(new File("data/PingER/average_rtt_Country_To_Country_date=2007-08_.tsv"));
		map.importPingErMinimumRtt(new File("data/PingER/minimum_rtt_Continent_To_Continent_date=2007-08_.tsv"));
		map.importPingErMinimumRtt(new File("data/PingER/minimum_rtt_Continent_To_Country_date=2007-08_.tsv"));
		map.importPingErMinimumRtt(new File("data/PingER/minimum_rtt_Country_To_Continent_date=2007-08_.tsv"));
		map.importPingErMinimumRtt(new File("data/PingER/minimum_rtt_Country_To_Country_date=2007-08_.tsv"));
		map.importPingErDelayVariation(new File("data/PingER/ipdv_Continent_To_Continent_date=2007-08_.tsv"));
		map.importPingErDelayVariation(new File("data/PingER/ipdv_Continent_To_Country_date=2007-08_.tsv"));
		map.importPingErDelayVariation(new File("data/PingER/ipdv_Country_To_Continent_date=2007-08_.tsv"));
		map.importPingErDelayVariation(new File("data/PingER/ipdv_Country_To_Country_date=2007-08_.tsv"));
		map.importPingErPacketLoss(new File("data/PingER/packet_loss_Continent_To_Continent_date=2007-08_.tsv"));
		map.importPingErPacketLoss(new File("data/PingER/packet_loss_Continent_To_Country_date=2007-08_.tsv"));
		map.importPingErPacketLoss(new File("data/PingER/packet_loss_Country_To_Continent_date=2007-08_.tsv"));
		map.importPingErPacketLoss(new File("data/PingER/packet_loss_Country_To_Country_date=2007-08_.tsv"));
	
	
		// Built Country-, Region-, and World-Groups
		System.out.println("=== Built Country-, Region-, and World-Groups ===");
		map.builtWorldGroups();
		map.builtRegionGroups();
		map.builtPingerGroups();
		map.builtCountryGroups();

		// Calculate GnpSpace
		System.out.println("=== Calculate GnpSpace ===");
		map.setGnpRef(GnpSpace.getGnp(5, 1000, 12, map));

		// Remove Hosts with a maximum Relative Error >= 0.5
		System.out.println("=== Remove Hosts with a maximum Relative Error >= 0.5 ===");
		map.removeHostsWithMaximumRelativeError(0.5);
		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Save to XML-File
		System.out.println("=== Save to XML-File Error > 0.5 ===");
		map.exportToXml(new File("0.5.xml"));

		// Remove Hosts with a maximum Relative Error >= 0.2
		System.out.println("=== Remove Hosts with a maximum Relative Error >= 0.2 ===");
		map.removeHostsWithMaximumRelativeError(0.2);
		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Save to XML-File
		System.out.println("=== Save to XML-File Error > 0.2 ===");
		map.exportToXml(new File("0.2.xml"));
		
		
		// Remove Hosts with a maximum Relative Error >= 0.1
		System.out.println("=== Remove Hosts with a maximum Relative Error >= 0.1 ===");
		map.removeHostsWithMaximumRelativeError(0.1);
		System.out.println("=> Hosts: " + map.getNoOfHosts());

		// Save to XML-File
		System.out.println("=== Save to XML-File Error > 0.1 ===");
		map.exportToXml(new File("0.1.xml"));
	}
}
