package de.tud.kom.p2psim.impl.network.gnpPrio;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import de.tud.kom.p2psim.impl.network.AbstractNetLayer;

/**
 * 
 * @author Gerald Klunker
 * @version 0.01, 07/12/12
 */
public class GnpNetBandwidthAllocation {

	private double allocatedBandwidth = 0.0;
	private double bandwidthNeeds = 0.0;

	private AbstractNetLayer receiver;
	private AbstractNetLayer sender;

	private double bidSender[] = new double[2];
	private double bidReciever[] = new double[2];

	private boolean minBidSender = false;
	private boolean minBidReceiver = false;
	
	private LinkedList<Byte> delayPriorities;
	
	public GnpNetBandwidthAllocation(AbstractNetLayer sender, AbstractNetLayer receiver) {
		this.sender = sender;
		this.receiver = receiver;
		bidSender[0] = 0;
		bidSender[1] = 0;
		bidReciever[0] = 0;
		bidReciever[1] = 0;
		delayPriorities = new LinkedList<Byte>();
	}
	
	public double getAllocatedBandwidth() {
		return allocatedBandwidth;
	}

	public void setAllocatedBandwidth(double allocatedBandwidth) {
		this.allocatedBandwidth = allocatedBandwidth;
	}

	public double getBandwidthNeeds() {
		return bandwidthNeeds;
	}

	public void setBandwidthNeeds(double bandwidthNeeds) {
		this.bandwidthNeeds = bandwidthNeeds;
	}

	public AbstractNetLayer getReceiver() {
		return receiver;
	}

	public AbstractNetLayer getSender() {
		return sender;
	}
	
	public void removeDelayPriority(byte priority){
		delayPriorities.remove(priority);
	}
	
	public void addDelayPriority(byte priority){
		delayPriorities.add(priority);
	}
	
	public byte getHighestDelayPriority(){
		if(!delayPriorities.isEmpty())
			return Collections.max(delayPriorities);
		
		return Byte.MIN_VALUE;
	}
	
	/**
	 * Maps the delay priority of the allocations from [-128,127] to [1,256] and calculates the sum 
	 * over them
	 * 
	 * @return the sum of the mapped priorities
	 */
	public int getDelayPrioritySum(){
		int sum = 0;
		
		for (Byte priority : delayPriorities) {
			int mappedPrio = priority + 129;
			sum += mappedPrio;
		}
		if(sum == 0)
			return 1;
		else
			return sum;
	}
	
	public void resetPriority(){
		delayPriorities.clear();
	}

	/*
	 * Eventbased Allocation only
	 */
	
	
	public void initConnection() {
		bidSender[0] = 0;
		bidSender[1] = 0;
		bidReciever[0] = 0;
		bidReciever[1] = 0;
		minBidSender = false;
		minBidReceiver = false;
		sender.setCurrentUpBandwidth(sender.getMaxUploadBandwidth());
		receiver.setCurrentDownBandwidth(receiver.getMaxDownloadBandwidth());
	}
	
	public void setBid(double bid, boolean isMinimal, boolean sender, long step) {
		if (sender) {
			int posC = (int) (step % 2);
			bidSender[posC] = bid;
			minBidSender = isMinimal;
		} else {
			int posC = (int) (step % 2);
			bidReciever[posC] = bid;
			minBidReceiver = isMinimal;
		}
	}

	public double getCurrentBid(boolean sender, long step) {
		if (sender) {
			return bidSender[(int) (step % 2)];
		} else {
			return bidReciever[(int) (step % 2)];
		}
	}

	public double getPreviousBid(boolean sender, long step) {
		if (sender) {
			return bidSender[(int) ((step + 1) % 2)];
		} else {
			return bidReciever[(int) ((step + 1) % 2)];
		}
	}

	public boolean isBidRepeated(boolean sender) {
		if (sender) {
			return Math.abs(bidSender[0] - bidSender[1]) <= 0.0001;
		} else {
			return Math.abs(bidReciever[0] - bidReciever[1]) <= 0.0001;
		}
	}

	public boolean isMinBid(boolean sender) {
		if (sender) {
			return minBidSender;
		} else {
			return minBidReceiver;
		}
	}

}