package de.tud.kom.p2psim.impl.network.gnpPrio;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import de.tud.kom.p2psim.api.network.NetLayer;
import de.tud.kom.p2psim.impl.network.AbstractNetLayer;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class GnpNetBandwidthManagerPeriodical extends AbstractGnpNetBandwidthManager {

	private HashSet<NetLayer> changedSenders;
	private HashSet<NetLayer> changedReceivers;

	private Set<GnpNetBandwidthAllocation> changedAllocations; // within last
																// realocation

	private String bandwidthMode = "weighted";
	
	public GnpNetBandwidthManagerPeriodical() {
		super();
		changedSenders = new HashSet<NetLayer>();
		changedReceivers = new HashSet<NetLayer>();
		changedAllocations = new HashSet<GnpNetBandwidthAllocation>();
	}

	@Override
	public GnpNetBandwidthAllocation addConnection(AbstractNetLayer sender,
			AbstractNetLayer receiver, double bandwidth, byte priority) {
		changedSenders.add(sender);
		changedReceivers.add(receiver);
//		sender.setCurrentUpBandwidth(sender.getMaxUploadBandwidth());
//		receiver.setCurrentDownBandwidth(receiver.getMaxDownloadBandwidth());
		GnpNetBandwidthAllocation ba = super.addConnection(sender, receiver, bandwidth, priority);
		return ba;
	}

	@Override
	public GnpNetBandwidthAllocation removeConnection(AbstractNetLayer sender,AbstractNetLayer receiver, double bandwidth) {
		GnpNetBandwidthAllocation ba =  super.removeConnection(sender, receiver, bandwidth);
		if (connectionsSenderToReceiver.containsKey(sender))
			changedSenders.add(sender);
		else
			changedSenders.remove(sender);
		if (connectionsReceiverToSender.containsKey(receiver))
			changedReceivers.add(receiver);
		else
			changedReceivers.remove(receiver);
		return ba;
	};

	@Override
	public void allocateBandwidth() {
		HashSet<AbstractNetLayer> chSenders = (HashSet<AbstractNetLayer>) changedSenders.clone();
		HashSet<AbstractNetLayer> chReceivers = (HashSet<AbstractNetLayer>) changedReceivers.clone();
		changedSenders.clear();
		changedReceivers.clear();
		changedAllocations.clear();
		for (AbstractNetLayer host : chSenders) {
			host.setCurrentUpBandwidth(host.getMaxUploadBandwidth());
			Set<GnpNetBandwidthAllocation> temp = new HashSet<GnpNetBandwidthAllocation>();
			temp.addAll(connectionsSenderToReceiver.get(host).values());
			
			// decide which bandwidth mode to use
			if(bandwidthMode.equals("exclusive"))
				priorityShareExclusive(temp, true);
			else
				priorityShareWeighted(temp, true);
		}
		for (AbstractNetLayer host : chReceivers) {
			host.setCurrentDownBandwidth(host.getMaxDownloadBandwidth());
			Set<GnpNetBandwidthAllocation> temp = new HashSet<GnpNetBandwidthAllocation>();
			temp.addAll(connectionsReceiverToSender.get(host).values());
			
			if(bandwidthMode.equals("exclusive"))
				priorityShareExclusive(temp, false);
			else
				priorityShareWeighted(temp, false);
		}
	}

	/**
	 * Shares the bandwidth of one host between the GnpNetBandwidthAllocation of this host either 
	 * as sender or receiver. The sharing is based on the priority of the allocations. The allocation 
	 * with the highest priority gets the full bandwidth it needs. If there is some bandwidth left
	 * the allocation with the next lower priority is served and so on.
	 *  
	 * @param unassigned
	 * @param isSender
	 */
	private void priorityShareExclusive(Collection<GnpNetBandwidthAllocation> unassigned, boolean isSender) {

		if (unassigned.isEmpty())
			return;

		// initialize the variables for the search that take place next
		int maxPrioritySum = Byte.MIN_VALUE;
		GnpNetBandwidthAllocation maxBandwidthAllocation = null;
		
		// search the allocation with the highest priority sum
		for (GnpNetBandwidthAllocation ba : unassigned) {

			// has the current allocation a higher priority sum as all others before?
			if (ba.getHighestDelayPriority() >= maxPrioritySum) {
				maxPrioritySum = ba.getDelayPrioritySum();
				maxBandwidthAllocation = ba;
			} 				
		}

		double bandwidthToShare = 0;
		if (isSender)
			bandwidthToShare = maxBandwidthAllocation.getSender().getCurrentUploadBandwidth();
		else
			bandwidthToShare = maxBandwidthAllocation.getReceiver().getCurrentDownloadBandwidth();

		
		double bandwidthPossibleForAllocation =  Math.min(maxBandwidthAllocation.getReceiver().getCurrentDownloadBandwidth()
												  + maxBandwidthAllocation.getAllocatedBandwidth(), 
												  maxBandwidthAllocation.getBandwidthNeeds());
		
		// the allocation does not need the whole bandwidth?
		if (bandwidthPossibleForAllocation < bandwidthToShare) {
			
			// do we have to change the allocation or is there no change?
			if (maxBandwidthAllocation.getAllocatedBandwidth() != bandwidthPossibleForAllocation) {
				changedAllocations.add(maxBandwidthAllocation);
				maxBandwidthAllocation.setAllocatedBandwidth(bandwidthPossibleForAllocation);
				if (isSender)
					changedReceivers.add(maxBandwidthAllocation.getReceiver());
				else
					changedSenders.add(maxBandwidthAllocation.getSender());
			}

			if (isSender)
				// set the new bandwidth for the upload to (old bandwidth for upload - bandwidth for allocation)
				maxBandwidthAllocation.getSender().setCurrentUpBandwidth(
						maxBandwidthAllocation.getSender().getCurrentUploadBandwidth()
								- maxBandwidthAllocation.getAllocatedBandwidth());
			else
				maxBandwidthAllocation.getReceiver().setCurrentDownBandwidth(
						maxBandwidthAllocation.getReceiver().getCurrentDownloadBandwidth()
								- maxBandwidthAllocation.getAllocatedBandwidth());

			// remove allocation from the unassigned allocations
			unassigned.remove(maxBandwidthAllocation);
			
			// share the rest of the bandwidth between the unassigned allocations
			priorityShareExclusive(unassigned, isSender);
			
		} else { // the allocation with the highest priority needs more bandwidth than available
			
			for (GnpNetBandwidthAllocation ba : unassigned) {
				
				// do we have to change the allocation or is there no change?
				if (ba.getAllocatedBandwidth() != bandwidthToShare) {
					changedAllocations.add(ba);
					ba.setAllocatedBandwidth(bandwidthToShare);
					if (isSender)
						changedReceivers.add(ba.getReceiver());
					else
						changedSenders.add(ba.getSender());
				}
				
				// set the current bandwidth to 0, because it's fully allocated
				if (isSender)
					ba.getSender().setCurrentUpBandwidth(0);
				else
					ba.getReceiver().setCurrentDownBandwidth(0);
			}

		}
	}
	
	/**
	 * Shares the bandwidth of one host between the GnpNetBandwidthAllocation of this host either 
	 * as sender or receiver. The sharing is based on the priority of the allocations. Each 
	 * allocation gets a weighted ratio of the bandwidth based on the sum of priorities.
	 *  
	 * @param unassigned
	 * @param isSender
	 */
	private void priorityShareWeighted(Collection<GnpNetBandwidthAllocation> unassigned, boolean isSender) {

		if (unassigned.isEmpty())
			return;
		
		int connectionDelayPrioritySum = 0;
		
		for (GnpNetBandwidthAllocation ba : unassigned) {
			connectionDelayPrioritySum += ba.getDelayPrioritySum();
		}
		
		for (GnpNetBandwidthAllocation ba : unassigned) {
	
			double bandwidthToShare = 0;
			if (isSender)
				bandwidthToShare = ba.getSender().getCurrentUploadBandwidth();
			else
				bandwidthToShare = ba.getReceiver().getCurrentDownloadBandwidth();

			
			double bandwidthPossibleForAllocation =  Math.min(ba.getReceiver().getCurrentDownloadBandwidth()
													  + ba.getAllocatedBandwidth(), 
													  ba.getBandwidthNeeds());
		
			// calculate the max bandwidth for the allocation based on the priorities 
			double maxBandwidthForAllocation;
			
			if(ba.getDelayPrioritySum() != connectionDelayPrioritySum)
				maxBandwidthForAllocation = bandwidthToShare * ba.getDelayPrioritySum() / connectionDelayPrioritySum;
			else
				maxBandwidthForAllocation = bandwidthToShare;
					
			// the allocation does not need the whole bandwidth?
			if (bandwidthPossibleForAllocation < maxBandwidthForAllocation) {
				
				// do we have to change the allocation or is there no change?
				if (ba.getAllocatedBandwidth() != bandwidthPossibleForAllocation) {
					changedAllocations.add(ba);
					ba.setAllocatedBandwidth(bandwidthPossibleForAllocation);
					if (isSender)
						changedReceivers.add(ba.getReceiver());
					else
						changedSenders.add(ba.getSender());
				}
				
			} else { // the allocation needs more bandwidth than available for it
				
				// do we have to change the allocation or is there no change?
				if (ba.getAllocatedBandwidth() != maxBandwidthForAllocation) {
					changedAllocations.add(ba);
					ba.setAllocatedBandwidth(maxBandwidthForAllocation);
					if (isSender)
						changedReceivers.add(ba.getReceiver());
					else
						changedSenders.add(ba.getSender());
				}
			}
				
			// set the current bandwidth to (old bandwidth - maxBandwidthForAllocation)
			if (isSender)
				ba.getSender().setCurrentUpBandwidth(
						ba.getSender().getCurrentUploadBandwidth()
								- maxBandwidthForAllocation);
			else
				ba.getReceiver().setCurrentDownBandwidth(
						ba.getReceiver().getCurrentDownloadBandwidth()
								- maxBandwidthForAllocation);
		
			
		}
	}


	@Override
	public BandwidthAllocation getBandwidthAllocationType() {
		return AbstractGnpNetBandwidthManager.BandwidthAllocation.PERIODICAL;
	}

	@Override
	public Set<GnpNetBandwidthAllocation> getChangedAllocations() {
		return changedAllocations;
	}
	
	
	public void setBandwidthMode(String mode){
		this.bandwidthMode = mode;
	}
	
}