package de.tud.kom.p2psim.impl.network.gnpPrio;

public class HaversineHelpers {

	
	/** Radius of the earth, in meters, at the equator. */
	private static final double GLOBE_RADIUS_EQUATOR = 6378000;

	/** Radius of the earth, in meters, at the poles. */
	private static final double GLOBE_RADIUS_POLES = 6357000;
	
	
	protected static final double radians(double degrees) {
		return degrees * (2 * Math.PI) / 360;
	}

	protected static final double degrees(double radians) {
		return radians * 360 / (2 * Math.PI);
	}

	protected static final double square(double d) {
		return d * d;
	}

	/**
	 * Computes the earth's radius of curvature at a particular latitude,
	 * assuming that the earth is a squashed sphere with elliptical
	 * cross-section.
	 * 
	 * @param lat -
	 *            latitude in radians. This is the angle that a point at this
	 *            latitude makes with the horizontal.
	 */
	protected static final double globeRadiusOfCurvature(double lat) {
		double a = GLOBE_RADIUS_EQUATOR; // major axis
		double b = GLOBE_RADIUS_POLES; // minor axis
		double e = Math.sqrt(1 - square(b / a)); // eccentricity
		return a * Math.sqrt(1 - square(e)) / (1 - square(e * Math.sin(lat)));
	}
}
