package de.tud.kom.p2psim.impl.network.gnpPrio;

import java.util.Comparator;

import de.tud.kom.p2psim.api.network.NetMessage;
import de.tud.kom.p2psim.api.simengine.SimulationEvent;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class TransferProgress {

	private NetMessage message;

	private double bandwidth; // in Simulator Time Units;
	private double remainingBytes;
	private long scheduledAt;
	private byte delayPriority;
	private byte lossPriority;
	
	public boolean firstSchedule = true;
	public boolean obsolete = false;
	public SimulationEvent relatedEvent = null;

	public TransferProgress(NetMessage msg, double bandwidth,
			double remainingBytes, long scheduledAt, byte delayPriority, byte lossPriority) {
		this.message = msg;
		this.bandwidth = bandwidth / Simulator.SECOND_UNIT;
		this.remainingBytes = remainingBytes;
		this.scheduledAt = scheduledAt;
		this.delayPriority = delayPriority;
		this.lossPriority = lossPriority;
	}

	public NetMessage getMessage() {
		return message;
	}

	public double getRemainingBytes(long time) {
		long interval = time - scheduledAt;
		return remainingBytes - (interval * bandwidth);
	}

	public byte getDelayPriority(){
		return delayPriority;
	}
	
	public byte getLossPriority(){
		return lossPriority;
	}
	
	public double getBandwidth() {
		return bandwidth;
	}
	
	public static Comparator<TransferProgress> getLossPriorityComparator(){
		return new Comparator<TransferProgress>(){

			public int compare(TransferProgress o1, TransferProgress o2) {
				if(o1.getLossPriority() < o2.getLossPriority()) return -1;
				if(o1.getLossPriority() == o2.getLossPriority()) {
					return 0;
				} else {
					return 1;
				}
			}
			
			
		};
	}

}
