package de.tud.kom.p2psim.impl.overlay.dht;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayMessage;

/**
 * this message is used to transport "forward" instruction to the next node
 * @author Yue Sheng (edited by Julius Rueckert)
 */
public class ForwardMsg extends AbstractOverlayMessage<OverlayID>{

	private OverlayKey key;
	private Message msg;
	private int hops;
	
	/**
	 * @param sender
	 * @param receiver
	 * @param key
	 * @param msg
	 * @param numberOfHops 
	 */
	public ForwardMsg(OverlayID sender, OverlayID receiver, OverlayKey key, Message msg, int numberOfHops){
		super(sender, receiver);
		this.key=key;
		this.msg=msg;
		this.hops=numberOfHops;
	}
	
	/**
	 * @param sender
	 * @param receiver
	 * @param key
	 * @param msg
	 */
	public ForwardMsg(OverlayID sender, OverlayID receiver, OverlayKey key, Message msg){
		super(sender, receiver);
		this.key=key;
		this.msg=msg;
		this.hops=0;
	}
	
	@Override
	public long getSize() {
		return msg.getSize();
	}

	public OverlayKey getKey() {
		return key;
	}

	@Override
	public Message getPayload(){
		return msg;
	}
		
	/**
	 * @return the current number of hops
	 */
	public int getHops(){
		return hops;
	}
	
	/**
	 * Increases the number of hops by one
	 */
	public void incHops(){
		hops++;
	}
	
}
