package de.tud.kom.p2psim.impl.overlay.dht;

import java.util.List;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.overlay.KBR;
import de.tud.kom.p2psim.api.overlay.KBRListener;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.transport.TransMessageListener;
import de.tud.kom.p2psim.api.transport.TransMsgEvent;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * This is the message listener for KBR layers
 * @author Yue Sheng (improved by Julius Rueckert)
 * 
 * @param <T> The concrete implementation of OverlayID used
 * @param <S> The concrete implementation of OverlayContact used
 *
 */
public class KBRMsgHandler<T extends OverlayID, S extends OverlayContact<T>> implements TransMessageListener{
	
	private final static Logger log = SimLogger.getLogger(KBRMsgHandler.class);
	private KBR<T,S> masterAsKBR;
	private AbstractOverlayNode masterAsOverlayNode;
	private KBRListener listener;
	
	/**
	 * Info: If you want to create an instance of this class inside an overlay node
	 * which inherits from AbstractOverlayNode and realizes the interface KBR you have
	 * to pass for the first and second parameter "this". 
	 * 
	 * @param kbr
	 * @param overlayNode
	 * @param listener
	 */
	public KBRMsgHandler(KBR<T,S> kbr, AbstractOverlayNode overlayNode, KBRListener listener){
		this.masterAsKBR = kbr;
		this.masterAsOverlayNode = overlayNode;
		this.listener = listener;
	}
	
	public void messageArrived(TransMsgEvent receivingEvent) {
		Message msg = receivingEvent.getPayload();
		if (msg instanceof ForwardMsg) {
			ForwardMsg fm = (ForwardMsg) msg;
			OverlayKey key = fm.getKey();
			Message appMsg = fm.getPayload();
			OverlayContact nextHop = null;
			int numberOfHops = fm.getHops();
			numberOfHops++;
			
			if(key==null || masterAsKBR.isRootOf(key)){
				// Deliver the message if the key is null (it is a direct message to that node) or
				// the node is the root of the key
				listener.deliver(key, appMsg);
				
				// Inform monitors about delivery
				Simulator.getMonitor().overlayMessageDelivered(masterAsKBR.getLocalOverlayContact(), msg, numberOfHops);
				
				return;
			} else {
				List<S> posNextHops = masterAsKBR.local_lookup(key, 1);
				if(posNextHops.size() > 0) {
					nextHop = masterAsKBR.local_lookup(key, 1).get(0);
					
					if(nextHop!=null){
						listener.forward(key, appMsg, nextHop);
						fm = new ForwardMsg(masterAsOverlayNode.getOverlayID(), nextHop.getOverlayID(), key, appMsg, numberOfHops);
						masterAsOverlayNode.getTransLayer().send(fm, nextHop.getTransInfo(), masterAsOverlayNode.getPort(), TransProtocol.UDP);  
						
						// Inform monitors about forwarding
						Simulator.getMonitor().overlayMessageForwarded(masterAsKBR.getLocalOverlayContact(), nextHop, msg, numberOfHops);
					}
				} else {
					// No next hop could be determined. The message is dropped.
					Simulator.getMonitor().queryFailed(masterAsKBR.getLocalOverlayContact(), appMsg);
				}
			}
		} 
	}
	

}
