package de.tud.kom.p2psim.impl.overlay.dht.centralized;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.overlay.KBR;
import de.tud.kom.p2psim.api.overlay.KBRListener;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.dht.ForwardMsg;
import de.tud.kom.p2psim.impl.overlay.dht.KBRMsgHandler;

public class KBRClientNode extends ClientNode implements KBR<OverlayIDImpl, CentralizedContact<OverlayIDImpl>> {

	private KBRListener kbrListener;
	private KBRMsgHandler kbrMsgHandler;
	
	public KBRClientNode(TransLayer transLayer, short port){
		super(transLayer, port);
		
	}
	
	@Override
	public List<CentralizedContact<OverlayIDImpl>> local_lookup(OverlayKey key, int num) {
		List<CentralizedContact<OverlayIDImpl>> nexthops = new LinkedList<CentralizedContact<OverlayIDImpl>>();
		
		// Add the only next hop: the server
		nexthops.add(new CentralizedContact<OverlayIDImpl>(new OverlayIDImpl(true),
											getServerAddress()));
		return nexthops;
	}

	@Override
	public List<CentralizedContact<OverlayIDImpl>> neighborSet(int num) {
		List<CentralizedContact<OverlayIDImpl>> neighbors = new LinkedList<CentralizedContact<OverlayIDImpl>>();
		
		// Add the only neighbor: the server
		neighbors.add(new CentralizedContact<OverlayIDImpl>(new OverlayIDImpl(true),
											getServerAddress()));
		return neighbors;
	}

	@Override
	public OverlayID[] range(CentralizedContact contact, int rank) {
		
		OverlayID[] range = new OverlayID[2];
		
		// The client is responsible for only himself (range: [ownID, ownID])
		range[0] = getOverlayID();
		range[1] = range[0];
		
		return range;
	}

	@Override
	public List<CentralizedContact<OverlayIDImpl>> replicaSet(OverlayKey key, int maxRank) {
		List<CentralizedContact<OverlayIDImpl>> replicas = new LinkedList<CentralizedContact<OverlayIDImpl>>();
		
		// Add the only possible replica: the server
		replicas.add(new CentralizedContact<OverlayIDImpl>(new OverlayIDImpl(true),
											getServerAddress()));
		return replicas;
	}

	@Override
	public void route(OverlayKey key, Message msg, CentralizedContact hint) {
		CentralizedContact nextHop=null;
		if (hint != null) {
			nextHop = hint;
		} else if (key != null) {
			nextHop = local_lookup(key, 1).get(0);
		} else {
			System.err.println("Both key and hint are null!!");
			return;
		}
		kbrListener.forward(key, msg, nextHop);
		ForwardMsg fm = new ForwardMsg(getOverlayID(), nextHop.getOverlayID(), key, msg);
		getTransLayer().send(fm, nextHop.getTransInfo(), getPort(), TransProtocol.UDP);

	}

	@Override
	public boolean isRootOf(OverlayKey key) {
		// Clients can not be roots of documents
		return false;
	}

	@Override
	public OverlayKey getNewOverlayKey(int rank) {
		return new OverlayKeyImpl(Integer.toString(rank));
	}

	@Override
	public void setKBRListener(KBRListener listener) {
		this.kbrListener = listener;
		kbrMsgHandler = new KBRMsgHandler(this, this, kbrListener);
		// add KBR handler to the listeners list
		this.getTransLayer().addTransMsgListener(this.kbrMsgHandler, this.getPort());
	}

	@Override
	public CentralizedContact<OverlayIDImpl> getLocalOverlayContact() {
		OverlayIDImpl overlayID = (OverlayIDImpl)getHost().getOverlay(KBR.class).getOverlayID();
		short port = getHost().getOverlay(KBR.class).getPort();
		TransInfo transInfo = getHost().getTransLayer().getLocalTransInfo(port);
		
		return new CentralizedContact<OverlayIDImpl>(overlayID, transInfo); 
	}

}
