package de.tud.kom.p2psim.impl.overlay.dht.centralized;

import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.api.overlay.KBR;
import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;

/**
 * This is the factory to create instances of KBRClientNode and KBRServerNode
 *
 */
public class KBRClientServerFactory implements ComponentFactory {

	private short clientPort;

	private boolean isServer;

	BootstrapManager bootstrap = new SimpleBootstrapManager();

	private KBRClientNode createClient(Host host) {
		KBRClientNode clientNode = new KBRClientNode(host.getTransLayer(), clientPort);
		clientNode.setBootstrap(bootstrap);
		return clientNode;
	}

	private KBRServerNode createServer(Host host) {
		KBRServerNode serverNode = new KBRServerNode(host.getTransLayer());
		serverNode.setBootstrap(bootstrap);
		return serverNode;
	}

	/**
	 * The port which will be used by clients
	 * @param port
	 */
	public void setPort(long port) {
		this.clientPort = (short) port;
	}

	/**
	 * Whether the next node will be ServerNode or ClientNode
	 * @param isServer
	 */
	public void setIsServer(boolean isServer) {
		this.isServer = isServer;
	}

	public OverlayNode createComponent(Host host) {
		OverlayNode node;
		if (isServer)
			node = createServer(host);
		else
			node = createClient(host);

		return node;
	}
	
	@Override
	public String toString() {
		return "Centralized KBR Client/Server Factory";
	}

}
