package de.tud.kom.p2psim.impl.overlay.dht.centralized;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.overlay.DHTObject;
import de.tud.kom.p2psim.api.overlay.KBR;
import de.tud.kom.p2psim.api.overlay.KBRListener;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.application.filesharing.Torrent;
import de.tud.kom.p2psim.impl.overlay.dht.ForwardMsg;
import de.tud.kom.p2psim.impl.overlay.dht.KBRMsgHandler;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordID;

public class KBRServerNode extends ServerNode implements KBR<OverlayIDImpl, CentralizedContact<OverlayIDImpl>> {

	private KBRListener kbrListener;
	private KBRMsgHandler kbrMsgHandler;
	
	public KBRServerNode(TransLayer transLayer) {
		super(transLayer);
	}

	@Override
	public List<CentralizedContact<OverlayIDImpl>> local_lookup(OverlayKey key, int num) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<CentralizedContact<OverlayIDImpl>> neighborSet(int num) {
		List<CentralizedContact<OverlayIDImpl>> neighbors = new LinkedList<CentralizedContact<OverlayIDImpl>>();
		
		// Extract all clients from the servers index
		Map<OverlayKey, DHTObject> list = listIndex();
		for (Map.Entry<OverlayKey, DHTObject> entry : list.entrySet()) {
			// Return if the maximal number of neighbors is reached
			if(neighbors.size() >= num)
				return neighbors;
			
			DHTObject data = entry.getValue();
			if(data instanceof Torrent){
				Torrent torrent = (Torrent) data;
				TransInfo trans = torrent.getOwnerInfo();
				OverlayIDImpl overlayID = (OverlayIDImpl)torrent.getOwnerID();
				
				neighbors.add(new CentralizedContact<OverlayIDImpl>(overlayID, 
													 trans));
			} else {
				//TODO: Fehler - sollte nicht passieren. Schauen, ob wirklich der Fall
			}
		}
		return neighbors;
	}

	@Override
	public OverlayID[] range(CentralizedContact contact, int rank) {
		OverlayID[] bounds = null;
		
		// Is the contact the server itself?
		if(contact.getTransInfo().getNetId() == getHost().getNetLayer().getNetID() &
				((OverlayIDImpl)contact.getOverlayID()).isServer){
			bounds = new OverlayID[2];
			
			// The server is responsible for himself and all clients
			bounds[0] = new OverlayIDImpl(false);
			bounds[1] = new OverlayIDImpl(true);
		}

		return bounds;
	}

	@Override
	public List<CentralizedContact<OverlayIDImpl>> replicaSet(OverlayKey key, int maxRank) {
		
		// The server is the only participant that holds objects in a DHT. 
		// So the list is empty.
		return new LinkedList<CentralizedContact<OverlayIDImpl>>();
	}

	@Override
	public void route(OverlayKey key, Message msg, CentralizedContact hint) {
		CentralizedContact nextHop=null;
		
		if (hint != null) {
			nextHop = hint;
		} else {
			// This should only happen if the server is also a client
			nextHop = this.getLocalOverlayContact();
		}
			
		kbrListener.forward(key, msg, nextHop);
		ForwardMsg fm = new ForwardMsg(getOverlayID(), nextHop.getOverlayID(), key, msg);
		getTransLayer().send(fm, nextHop.getTransInfo(), getPort(), TransProtocol.UDP);
		
	}


	@Override
	public boolean isRootOf(OverlayKey key) {
		// the server is root of all files
		return true;
	}

	@Override
	public OverlayKey getNewOverlayKey(int rank) {
		
		return new OverlayKeyImpl(Integer.toString(rank));
	}

	@Override
	public void setKBRListener(KBRListener listener) {
		this.kbrListener = listener;
		kbrMsgHandler = new KBRMsgHandler(this, this, kbrListener);
		// add KBR handler to the listeners list
		this.getTransLayer().addTransMsgListener(this.kbrMsgHandler, this.getPort());
	}
	
	@Override
	public CentralizedContact<OverlayIDImpl> getLocalOverlayContact() {
		OverlayIDImpl overlayID = (OverlayIDImpl)getHost().getOverlay(KBR.class).getOverlayID();
		short port = getHost().getOverlay(KBR.class).getPort();
		TransInfo transInfo = getHost().getTransLayer().getLocalTransInfo(port);
		
		return new CentralizedContact<OverlayIDImpl>(overlayID, transInfo); 
	}

}
