package de.tud.kom.p2psim.impl.overlay.dht.chord;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode.PeerStatus;

/**
 * This bootstrap manager maintains 5 active nodes at the moment
 * 
 * @author Yue Sheng
 */
public class ChordBootstrapManager implements BootstrapManager {
	final int Length = 5;

	int i = 0; // index of bootstrap list

	private ChordNode[] activeNodes;

	/**
	 * constructor
	 */
	public ChordBootstrapManager() {
		this.activeNodes = new ChordNode[Length];
	}

	public List<TransInfo> getBootstrapInfo() {
		List<TransInfo> list = new LinkedList<TransInfo>();
		for (ChordNode n : this.activeNodes) {
			list.add(n.getLocalContact().getTransInfo());
		}
		return list;
	}

	public void registerNode(OverlayNode node) {
		ChordNode n = (ChordNode) node;
		i = (i + 1) % Length;
		activeNodes[i] = n;
		assert !n.getLocalContact().isAlive():"dead node inserted to bootstrap" ;
	}

	public void unregisterNode(OverlayNode node) {
		ChordNode n = (ChordNode) node;
		for(int j=0;j<Length;j++){
			if(activeNodes[j].equals(n)){
				activeNodes[j]=null;
				assert !n.getLocalContact().isAlive():"live node removed from bootstrap" ;
			}
		}
	}

	/**
	 * @return The the list of active nodes in bootstrap manager
	 */
	public ChordNode pick() {
		for(int j=0;j<Length;j++){
			ChordNode n = activeNodes[j];
			if(n !=null && n.getPeerStatus().equals(PeerStatus.PRESENT)){
				return n;
			}
		}
		return null;
	}

}
