package de.tud.kom.p2psim.impl.overlay.dht.chord;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.transport.TransInfo;

/**
 * @author Yue Sheng
 * @version 3.0, Apr 10, 2008
 *
 */
public class ChordContact implements OverlayContact<ChordID>, Cloneable{
	private ChordID id;
	private TransInfo address;
	private boolean isAlive;
	
	/**
	 * @param id the overlay ID
	 * @param trans the IP address information
	 */
	public ChordContact(ChordID id, TransInfo trans){
		this.id = id;
		this.address = trans;
	}
	
	
	/**
	 * @param alive the status of liveness of this overlay contact
	 */
	public void setAlive(boolean alive){
		this.isAlive= alive;
	}
	
	/**
	 * @return if the overlay contact is alive or not
	 */
	public boolean isAlive(){
		return this.isAlive;
	}
	
	/**
	 * @param contact
	 * @return integer indicating the comparison result
	 */
	public int compareTo (ChordContact contact ) {
		return this.id.compareTo(contact.getOverlayID());
	}
	/**
	 * @param another
	 * @return equality
	 */
	public boolean equals(Object another){
		if(another==null || !(another instanceof ChordContact)){
			return false;
		} else
			return this.id.equals(((ChordContact)another).id);
	}
	
	public int hashCode() {
		return id.hashCode();
	}
	
	/**getters and setters*/
	public ChordID getOverlayID() {
		return this.id;
	}
	
	public TransInfo getTransInfo() {
		return this.address;
	}
	
	/**
	 * @param another
	 * @return distance
	 */
	public BigInteger getDistance(ChordContact another){
		return this.id.getDistance(another.getOverlayID());
	}
	
	
	public ChordContact clone(){
		ChordContact clo = new ChordContact(this.id, this.address);
		clo.setAlive(this.isAlive);
		return clo;
	}
}
