package de.tud.kom.p2psim.impl.overlay.dht.chord;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.math.BigInteger;

import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransInfo;

/**
 * @author Yue Sheng
 * 
 */
public class ChordID implements OverlayID,Serializable {
	private BigInteger id;

	/**
	 * @param transInfo
	 *            the IP address of a node
	 */
	public ChordID(TransInfo transInfo) {
		id = new ChordIDGenerator().getID(transInfo);
	}

	/**
	 * @param i some BigInteger used to construct a ChordID
	 */
	public ChordID(BigInteger i) {
		this.id = i;
	}

	public byte[] getBytes() {
		byte[] buf = null;
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutput out = new ObjectOutputStream(bos);
			out.writeObject(this);
			out.close();
			buf = bos.toByteArray();

		} catch (IOException e) {
			System.err.println("IOException!");
		}
		return buf;
	}

	public Object getUniqueValue() {
		return id;
	}

	/**
	 * @return the concrete ID
	 */
	public BigInteger getValue() {
		return id;
	}

	public int compareTo(OverlayID id1) {
		ChordID id2 = (ChordID) id1;
		return id.compareTo(id2.getValue());
	}

	/**
	 * @param another
	 *            Another ChordOverlayID
	 * @return the distance
	 */
	public BigInteger getDistance(ChordID another) {
		BigInteger overlayID = another.getValue();
		return this.id.subtract(overlayID).abs();
	}

	/**
	 * @param another
	 *            Another overlay id
	 * @return weather both overlay id are equal
	 */
	public boolean equals(ChordID another) {
		return this.id.compareTo(another.getValue()) == 0;
	}

	/**
	 * @param a
	 * @param b
	 * @return whether this ChordID locates between a and b on the Chord ring
	 */
	public boolean between(ChordID a, ChordID b) {
		if (a.compareTo(b) < 0) {
			if (this.compareTo(a) > 0 && this.compareTo(b) < 0) {
				return true;
			}
		} else {
			if (this.compareTo(a) > 0 || this.compareTo(b) < 0) {
				return true;
			}
		}
		return false;
	}
	
	
	public String toString(){
		return id.toString();
	}
}
