package de.tud.kom.p2psim.impl.overlay.dht.chord;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * @author Yue Sheng
 */
public class FingerTable {
	private static Logger log = SimLogger.getLogger(FingerTable.class);
	
	private ChordNode node;
	
	/**
	 * The paper says that the size of the finger table should be 160.
	 * For further research a possibility was added to change the size at runtime.
	 */
	private int ftSize = 160;

	private ChordContact[] finger;

	int i = 0; // the i-th entry of finger table

	public FingerTable(ChordNode n) {
		this.node = n;
		finger = new ChordContact[ftSize];
	}

	public List<ChordContact> allContacts() {
		List<ChordContact> list = new ArrayList<ChordContact>();
		for (int i = 0; i < finger.length; i++) {
			list.add(finger[i]);
		}
		return list;
	}

	public void resetContacts() {
		finger = new ChordContact[ftSize];
	}

	public ChordContact getContact(ChordID id) {
		for (ChordContact contact : finger) {
			if (contact != null && contact.getOverlayID().equals(id)) {
				return contact;
			}
		}
		return null;
	}

	public void removeContact(ChordID id) {
		for (ChordContact contact : finger) {
			if (contact != null && contact.getOverlayID().equals(id)) {
				contact = null;
				return;
			}
		}
	}

	public void setContact(int i, ChordContact contact) {
		finger[i] = contact;
	}

	public ChordContact getFinger(int i) {
		return finger[i];
	}

	public void incPointer() {
		i = (i + 1) % ftSize;
	}

	/* getter and setters */
	public int getI() {
		return i;
	}

	/**
	 * @param id
	 * @return the closest predecessor of the given id in the finger table
	 */
	public ChordContact getClosestPre(ChordID id) {
		ChordID n = node.getChordID(), finger_i = null;
		for (int i = ftSize-1; i >= 0; i--) { // search finger table
			if (finger[i] == null || !finger[i].isAlive())
				continue;
			finger_i = finger[i].getOverlayID();
			if (finger_i.between(n, id)) {
				return finger[i];
			}
		}
		
		return node.getLocalContact();
	}

	public int getFtSize() {
		return ftSize;
	}

	private static int highestSizeSeen = 0; 
	
	public void setFtSize(int ftSize) {
		int oldSize = this.ftSize;
		this.ftSize = ftSize;
		
		if(ftSize > highestSizeSeen){
			highestSizeSeen = ftSize;
			log.info(Simulator.getCurrentTime()/Simulator.MINUTE_UNIT+" New FTsize: "+ftSize);
		}
		
		ChordContact[] newFt = new ChordContact[ftSize];
		
		for (int i=0; i<finger.length && i<newFt.length; i++) {
			newFt[i] = finger[i];
		}
		
		finger = newFt;
	}

}
