package de.tud.kom.p2psim.impl.overlay.dht.chord;

import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.operations.CheckPreOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.operations.FixFingerOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.operations.StabilizeOperation;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Yue Sheng
 * @version 3.0, Apr 17, 2008
 *
 */
public class OperationListener implements OperationCallback<Object> {
	
	private ChordNode masterNode;
	
	/*
	 * Split the intervals
	 * @author Julius Rueckert
	 */
	private long fixFingerInterval = 10*Simulator.SECOND_UNIT;
	
	private long stabilizeInterval = 10*Simulator.SECOND_UNIT;
	
	private long checkPreInterval = 10*Simulator.SECOND_UNIT;
	/**
	 * @param master
	 */
	public OperationListener(ChordNode master){
		masterNode = master;
	}
	
	public void calledOperationFailed(Operation<Object> op) {
		
	}

	public void calledOperationSucceeded(Operation<Object> op) {
		if (op instanceof StabilizeOperation) {
			StabilizeOperation stabilize = (StabilizeOperation) op;
			if(masterNode.getPeerStatus().equals(AbstractOverlayNode.PeerStatus.PRESENT)){
				stabilize = new StabilizeOperation(masterNode,this);
				stabilize.scheduleWithDelay(stabilizeInterval);
			}
		} else if(op instanceof FixFingerOperation){
			FixFingerOperation fixFingers = (FixFingerOperation) op;
			if(masterNode.getPeerStatus().equals(AbstractOverlayNode.PeerStatus.PRESENT)){
				fixFingers = new FixFingerOperation(masterNode,this);
				fixFingers.scheduleWithDelay(fixFingerInterval);
			}
		} else if(op instanceof CheckPreOperation){
			CheckPreOperation checkPre = (CheckPreOperation) op;
			if(masterNode.getPeerStatus().equals(AbstractOverlayNode.PeerStatus.PRESENT)){
				checkPre = new CheckPreOperation(masterNode, this);
				checkPre.scheduleWithDelay(checkPreInterval);
			}
		} 
	}

	/*
	 * Added getters and setters 
	 * @author Julius Rueckert
	 */
	
	public long getFixFingerInterval() {
		return fixFingerInterval;
	}

	public void setFixFingerInterval(long fixFingerInterval) {
		this.fixFingerInterval = fixFingerInterval;
	}

	public long getStabilizeInterval() {
		return stabilizeInterval;
	}

	public void setStabilizeInterval(long stabilizeInterval) {
		this.stabilizeInterval = stabilizeInterval;
	}

	public long getCheckPreInterval() {
		return checkPreInterval;
	}

	public void setCheckPreInterval(long checkPreInterval) {
		this.checkPreInterval = checkPreInterval;
	}
	
}
