package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransMessageCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.*;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Yue Sheng
 * @version 3.0, April 30, 2008
 * 
 */
public class CheckLivenessOperation extends AbstractOperation<ChordNode, Object> implements TransMessageCallback {
	private ChordContact target;

	private PingMsg pingMsg;

	private int cmdID;

	/**
	 * @param node
	 *            The master node
	 * @param target
	 *            The target node to be checked
	 * @param callback
	 */
	public CheckLivenessOperation(ChordNode node, ChordContact target, OperationCallback<Object> callback) {
		super(node, callback);
		this.target = target;
		pingMsg = new PingMsg(node.getChordID(), target.getOverlayID());
	}

	@Override
	protected void execute() {
		short senderPort = getComponent().getPort();
		TransInfo targetAddr = target.getTransInfo();
		cmdID = getComponent().getTransLayer().sendAndWait(pingMsg, targetAddr, senderPort, TransProtocol.UDP, this, 3 * Simulator.SECOND_UNIT);
	}

	public void messageTimeoutOccured(int commId) {
		if (this.cmdID == commId){
			operationFinished(false);
		}
		// change peer status and bootstrap-manager
		// TODO this 2 lines should be in connectivity changed.
		// target.setPeerStatus(PeerStatus.ABSENT);
		// getComponent().getBootstrap().unregisterNode(target);
	}

	public void receive(Message msg, TransInfo senderInfo, int commId) {
		if (this.cmdID==commId && msg instanceof PongMsg) { // got a reply of ping
			operationFinished(true);
		}
	}

	@Override
	public Object getResult() {
		return this;
	}
}
