package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordID;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordKey;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.FingerTable;

/**
 * @author Yue Sheng
 * @version 3.0, Apr 18, 2008
 *
 */
public class FixFingerOperation extends AbstractOperation<ChordNode, Object>{
	
	private final BigInteger m = new BigInteger("2").pow(160);
	
	/**
	 * @param node
	 * @param callback
	 */
	public FixFingerOperation(ChordNode node, OperationCallback<Object> callback){
		super(node, callback);
	}

	@Override
	protected void execute() {
		ChordNode master = getComponent();
		final FingerTable ft = master.getFingerTable();
		Integer ftSize = ft.getFtSize();
		ft.incPointer();
		Integer ftI = ft.getI();
		BigInteger idOfMaster = (BigInteger) master.getOverlayID().getUniqueValue();
//		BigInteger idToLookUpForFinger = (idOfMaster.add(new BigInteger("2").pow(ft.getI()))).mod(m);
		
		/**
		 * To enable the use of a finger table wit more or less than 160 entries.
		 * The i-th entry of the finger table points at the node responsible for
		 * the key with the value (ownID+2^{160*i/ftSize}) mod 2^160
		 */
		BigInteger idToLookUpForFinger = (idOfMaster.add(( new BigInteger("2")).pow(160*ftI/(ftSize)))).mod(m);
		
		ChordKey k = new ChordKey(idToLookUpForFinger);
		
		master.nodeLookup(k, new OperationCallback<Object>(){

			public void calledOperationFailed(Operation<Object> op) {
				// not possible
				operationFinished(true);
			}

			public void calledOperationSucceeded(Operation<Object> op) {
				ChordContact node = (ChordContact) op.getResult();
				ft.setContact(ft.getI(),node);
				operationFinished(true);
			}
		});
	}

	

	@Override
	public Object getResult() {
		return this;
	}

	
}
