package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransMessageCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.GetInfoReply;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.GetInfoRequest;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Yue Sheng
 * get information from other nodes
 */
public class GetInfoOperation extends AbstractOperation<ChordNode, Object> implements TransMessageCallback {
	private ChordContact target;
	private int cmdID;
	private String type;
	private	OverlayKey key=null;
	Object result=null;
	
	public GetInfoOperation(ChordNode component, ChordContact target, String type, OperationCallback<Object> callback) {
		super(component, callback);
		this.target=target;
		this.type=type;
	}
	
	public GetInfoOperation(ChordNode component, ChordContact target, OverlayKey key, OperationCallback<Object> callback) {
		this(component,target,"dhtObject",callback);
		this.key = key;
	}
	@Override
	protected void execute() {
		GetInfoRequest request;
		if(this.type.equalsIgnoreCase("dhtObject")){
			request = new GetInfoRequest(getComponent().getChordID(), target.getOverlayID(), key);
		}else {
			request = new GetInfoRequest(getComponent().getChordID(),target.getOverlayID(), type);
		}
		cmdID = getComponent().getTransLayer().sendAndWait(request, target.getTransInfo(), getComponent().getPort(), TransProtocol.UDP, this, 3 * Simulator.SECOND_UNIT);
	}

	public void receive(Message msg, TransInfo senderInfo, int commId) {
		if (this.cmdID == commId && msg instanceof GetInfoReply) {
			GetInfoReply getPreRpl = (GetInfoReply) msg;
			result =getPreRpl.getResult();
		}
		operationFinished(true);
	}
	
	public void messageTimeoutOccured(int commId) {
		if(this.cmdID==commId){
//			System.out.println("getInfo request time out!!");
			target.setAlive(false);
		}
		operationFinished(false); // means: the target node is down
	}
	
	@Override
	public Object getResult() {
		return result;
	}
}
