package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import java.net.ProtocolException;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode.PeerStatus;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordID;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.IndexTransferMsg;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.LeaveMsg;

/**
 * @author Yue Sheng
 */
public class LeaveOperation extends AbstractOperation<ChordNode, Object>{

	public LeaveOperation(ChordNode component, OperationCallback<Object> callback) {
		super(component, callback);
	}

	@Override
	protected void execute() {
		ChordNode master = getComponent();
		ChordContact succ = master.nextActiveSucc();
		// move all keys to the successor
		if(!master.getIndex().isEmpty()){
			IndexTransferMsg msg = new IndexTransferMsg(master.getChordID(),succ.getOverlayID(),master.getIndex());
			master.getTransLayer().send(msg, succ.getTransInfo(), master.getPort(), TransProtocol.UDP);
		}
		// disconnect
		master.setPeerStatus(PeerStatus.TO_JOIN);
		master.setAlive(false);
		// notify predecessor
		LeaveMsg leave;
		if(master.getPre()!=null){
			leave = new LeaveMsg(master.getChordID(), master.getPre().getOverlayID(), master.getLocalContact(), master.getPre());
			master.getTransLayer().send(leave, master.getPre().getTransInfo(), master.getPort(), TransProtocol.UDP);
		}
		// notify successor
		leave = new LeaveMsg(master.getChordID(), succ.getOverlayID(), master.getLocalContact(), master.getPre());
		master.getTransLayer().send(leave, succ.getTransInfo(), master.getPort(), TransProtocol.UDP);
		operationFinished(true);
	}

	@Override
	public Object getResult() {
		// TODO Auto-generated method stub
		return null;
	}
	

}
