package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.DHTObject;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordID;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordKey;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.LookupReply;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.LookupRequest;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * @author Yue Sheng
 */
public class LookupOperation extends AbstractOperation<ChordNode, Object> {
	private final static Logger log = SimLogger.getLogger(LookupOperation.class);
	
	private ChordNode master = null;

	private ChordKey key = null;

	private ChordContact keyRoot = null; // the root node of a key

	DHTObject value = null;

	private boolean forValue;

	int hops=0;

	/**
	 * @param node
	 *            The master node of this operation
	 * @param key
	 *            The key used during the lookup
	 * @param callback
	 *            The operation listener
	 * @param forValue
	 *            Whether this operation is used for nodeLookup or valueLookup
	 */
	public LookupOperation(ChordNode node, ChordKey key, OperationCallback callback, boolean forValue) {
		super(node, callback);
		this.master = node;
		this.key = key;
		this.forValue = forValue;
	}

	@Override
	protected void execute() {
		scheduleOperationTimeout(50 * Simulator.SECOND_UNIT);
		ChordID keyID = new ChordID(key.getKeyID());
		ChordContact succ = master.nextActiveSucc();
		
		if(succ!=null){
			ChordID nId = master.getChordID(), succId = succ.getOverlayID();

			if (keyID.between(nId, succId) || keyID.equals(succId)) {
				keyRoot = succ;
				// master.lookupFinished(getOperationID());
				if (forValue) {
					localLookup(keyRoot);
				} else {
					hops = 1;
					operationFinished(true);
				}
			} else { // forward to next hop
				ChordContact nextHop = master.closestPre(keyID);
				LookupRequest lookupReq = new LookupRequest(master.getChordID(), nextHop.getOverlayID(), key, forValue);
				lookupReq.setStarter(master.getLocalContact());
				lookupReq.setOpID(getOperationID());
				master.getTransLayer().send(lookupReq, nextHop.getTransInfo(), master.getPort(), TransProtocol.UDP);
				master.registerLookup(getOperationID(), this);
			}		
		} else {
			log.debug(Simulator.getSimulatedRealtime()+" - "+getComponent().getChordID()+" lookup not possible - there are no succs");
		}
	}

	@Override
	protected void operationTimeoutOccured() {
		master.lookupFinished(getOperationID());
		log.debug(Simulator.getSimulatedRealtime()+" - "+getComponent().getChordID()+" lookup timeout occured.");
		operationFinished(false);
	}

	@Override
	public Object getResult() {
		if (forValue) {
			return value;
		} else {
			return keyRoot;
		}
	}

	public void found(LookupReply reply) {
		hops = reply.getHopCount();
		master.lookupFinished(getOperationID());
		keyRoot = (ChordContact) reply.getResult();
		if (forValue) {
			localLookup(keyRoot);
		} else {
			hops++;
			operationFinished(true);
		}
	}


	private void localLookup(final ChordContact node) {
		GetInfoOperation op = new GetInfoOperation(master, node, key, new OperationCallback<Object>() {

			public void calledOperationFailed(Operation<Object> op) {
				// means: node is down
				node.setAlive(false);
				execute();
			}

			public void calledOperationSucceeded(Operation<Object> op) {
				value = (DHTObject) op.getResult();
				hops ++;
				if (value != null){
					// System.out.println("valueLookup succeed with hops:"+hops);//test
					operationFinished(true);
				}else{ 
					System.out.println("valueLookup failed with hops: "+hops+"///"+ ((ChordNode) op.getComponent()).getOverlayID().getUniqueValue());//test
					operationFinished(false);
				}
			}
		});
		op.scheduleImmediately();
	}

	public boolean forValue() {
		return forValue;
	}

	public int getHops() {
		return this.hops;
	}
}
