package de.tud.kom.p2psim.impl.overlay.dht.chord.operations;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.DHTObject;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransMessageCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordContact;
import de.tud.kom.p2psim.impl.overlay.dht.chord.ChordNode;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.StoreReply;
import de.tud.kom.p2psim.impl.overlay.dht.chord.messages.StoreRequest;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Yue Sheng
 *
 */
public class StoreOperation extends AbstractOperation<ChordNode, Object> implements TransMessageCallback{

	ChordNode master;
	StoreRequest storeReq;
	StoreReply reply;
	OverlayKey key;
	DHTObject obj;
	
	/**
	 * @param component
	 * @param key
	 * @param obj
	 * @param callback
	 */
	public StoreOperation(ChordNode component,OverlayKey key, DHTObject obj, OperationCallback<Object> callback) {
		super(component, callback);
		master = getComponent();
		this.key=key;
		this.obj=obj;
 	}

	@Override
	protected void execute() {
		scheduleOperationTimeout(1*Simulator.MINUTE_UNIT);
		master.nodeLookup(key, new OperationCallback<ChordContact>(){

			public void calledOperationFailed(Operation<ChordContact> op) {
				System.out.println("inStoreOp: nodeLookup failed~~");//test
				operationFinished(false);
			}

			public void calledOperationSucceeded(Operation<ChordContact> op) {
				ChordContact target = op.getResult();
				storeReq = new StoreRequest(master.getChordID(), target.getOverlayID(), key, obj);
				sendRequest(target.getTransInfo());
			}
		});
		
	}
	
	private void sendRequest(TransInfo receiverInfo){
		master.getTransLayer().sendAndWait(storeReq, receiverInfo, master.getPort(), TransProtocol.UDP, this, 3 * Simulator.SECOND_UNIT);
	}

	public void messageTimeoutOccured(int commId) {
		System.out.println("inStoreOp: after storing, no reply~~");//test
		operationFinished(false);
	}

	public void receive(Message msg, TransInfo senderInfo, int commId) {
		if (msg instanceof StoreReply) {
			this.reply = (StoreReply) msg;
			System.out.println("Store op succeeded to node: "+ reply.getSender().getValue());//test
			operationFinished(true);
		}
		
	}

	@Override
	public Object getResult() {
		return reply.getDescription();
	}
}
