package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTableConfig;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.OperationsConfig;

/**
 * Configuration settings for Kademlia components. All methods should return
 * constant values.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public interface ComponentsConfig extends RoutingTableConfig, OperationsConfig {

    /**
     * @return the interval in simulation time units between two routing table
     *         bucket refresh executions.
     */
    public long getRefreshInterval();

    /**
     * @return the interval in simulation time units between two data item
     *         republish executions.
     */
    public long getRepublishInterval();

    /**
     * @return the interval in simulation time units between two random data
     *         lookup operations executed on one host.
     */
    public long getPeriodicLookupInterval();

    /**
     * @return the time in simulation time units after which data items are
     *         evicted from a node's local database (if they have not been
     *         republished since).
     */
    public long getDataExpirationTime();

    /**
     * @return the number of data items that are available in the overlay
     *         network.
     */
    public int getNumberOfDataItems();

}
