package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components;

import java.util.Set;

import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.Node.VisibilityRestrictableNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;

/**
 * A request handler that answers both data lookups and node (k closest nodes)
 * lookups with a visibility restricted set of nodes from the local routing
 * table.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class GenerallyRestrictedRequestHandler<H extends HKademliaOverlayID>
	extends RequestHandler<H> {

    /**
     * The node that owns this request handler.
     */
    private final VisibilityRestrictableNode<H> myNode;

    /**
     * Constructs a new message handler that restricts results for both data and
     * node lookups. It has to be manually registered as ProximityListener of
     * <code>myNode</code>'s routing table and as a TransMessageListener of
     * <code>manager</code>.
     * 
     * @param manager
     *                the TransLayer used to reply to messages.
     * @param myNode
     *                the VisibilityRestrictableNode that owns this message
     *                handler.
     * @param conf
     *                an ComponentsConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public GenerallyRestrictedRequestHandler(final TransLayer manager,
	    final VisibilityRestrictableNode<H> myNode, ComponentsConfig conf) {
	super(manager, myNode, conf);
	this.myNode = myNode;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Set<KademliaOverlayContact<H>> nodeLookupForNode(
	    final KademliaOverlayKey key) {
	return myNode.getKademliaRoutingTable().visibilityRestrictedLocalLookup(key,
		config.getBucketSize());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Set<KademliaOverlayContact<H>> nodeLookupForData(
	    final KademliaOverlayKey key) {
	return myNode.getKademliaRoutingTable().visibilityRestrictedLocalLookup(key,
		config.getBucketSize());
    }
}
