package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components;

import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.Node.HierarchyRestrictableNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.HKademliaRoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTable.HierarchyRestrictableRoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.HKademlia2OperationFactory;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;

/**
 * A node in an overlay network that supports routing in virtual hierarchies
 * ("hierarchical Kademlia"), version 2. For details about nodes, see
 * {@link AbstractKademliaNode}.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class HKademlia2Node<H extends HKademliaOverlayID> extends
	AbstractKademliaNode<H> implements HierarchyRestrictableNode<H> {

    /**
     * This node's routing table.
     */
    private final HierarchyRestrictableRoutingTable<H> routingTable;

    /**
     * This node's operation factory (used to construct lookup, store etc.
     * operations)
     */
    private final HKademlia2OperationFactory<H> operationFactory;

    /**
     * Constructs a new hierarchical Kademlia node (version 2).
     * 
     * @param myContact
     *                the KademliaOverlayContact of the new node.
     * @param messageManager
     *                the TransLayer of the new node.
     * @param conf
     *                a ComponentsConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public HKademlia2Node(final KademliaOverlayContact<H> myContact,
	    final TransLayer messageManager, final ComponentsConfig conf) {
	super(myContact, messageManager, conf);
	routingTable = new HKademliaRoutingTable<H>(myContact, conf);
	operationFactory = new HKademlia2OperationFactory<H>(this, conf);

	// construct handler for incoming requests & new neighbours
	final RequestHandler<H> requestHandler = new HRequestHandler<H>(
		getMessageManager(), this, conf);
	getMessageManager().addTransMsgListener(requestHandler, getPort());
	routingTable.registerProximityListener(requestHandler);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final HKademlia2OperationFactory<H> getOperationFactory() {
	return this.operationFactory;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final HierarchyRestrictableRoutingTable<H> getKademliaRoutingTable() {
	return this.routingTable;
    }

}
