package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components;

import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.KademliaRoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.KademliaOperationFactory;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;

/**
 * A standard Kademlia node. For details about nodes, see
 * {@link AbstractKademliaNode}.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class KademliaNode<T extends KademliaOverlayID> extends
AbstractKademliaNode<T> {

	/**
	 * This node's routing table.
	 */
	private final RoutingTable<T> routingTable;

	/**
	 * This node's operation factory (used to construct lookup, store etc.
	 * operations)
	 */
	private final KademliaOperationFactory<T> operationFactory;

	/**
	 * Constructs a new standard Kademlia node.
	 * 
	 * @param myContact
	 *                the KademliaOverlayContact of the new node.
	 * @param messageManager
	 *                the TransLayer of the new node.
	 * @param conf
	 *                a ComponentsConfig reference that permits to retrieve
	 *                configuration "constants".
	 */
	public KademliaNode(final KademliaOverlayContact<T> myContact,
			final TransLayer msgMgr, final ComponentsConfig conf) {
		super(myContact, msgMgr, conf);
		routingTable = new KademliaRoutingTable<T>(myContact, conf);
		operationFactory = new KademliaOperationFactory<T>(this, conf);

		// construct handler for incoming requests & new neighbours
		final RequestHandler<T> requestHandler = new RequestHandler<T>(
				getMessageManager(), this, conf);
		getMessageManager().addTransMsgListener(requestHandler, getPort());
		routingTable.registerProximityListener(requestHandler);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public final KademliaOperationFactory<T> getOperationFactory() {
		return this.operationFactory;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public final RoutingTable<T> getKademliaRoutingTable() {
		return this.routingTable;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public final String toString() {
		return "[KademliaNode|" + getLocalContact().getOverlayID()
		+ "; status=" + getPeerStatus() + "]";
	}

}
