package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;

/**
 * This class implements the visit method for PseudoRootNode, because requests
 * for that type are always forwarded to the actual root node stored in the
 * PseudoRootNode.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
abstract class AbstractNodeVisitor<T extends KademliaOverlayID> implements
	NodeVisitor<T> {

    /**
     * {@inheritDoc}
     */
    @Override
    public final void visit(final Node<T> node) {
	throw new UnsupportedOperationException("This type of node ("
		+ node.getClass() + ") is not supported by this visitor.");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final void visit(final PseudoRootNode<T> node) {
	// The pseudo root forwards all requests to the actual root node.
	node.root.accept(this);
    }

}
