package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import java.math.BigInteger;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;


/**
 * Represents a node in the routing tree.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
interface Node<T extends KademliaOverlayID> {

    /**
     * Accepts and calls back the given visitor (according to the GoF design
     * pattern "Visitor").
     * 
     * @param visitor
     *                the visitor that is to be called back.
     */
    public void accept(NodeVisitor<T> visitor);

    /**
     * @return the KademliaOverlayID of the peer that owns this routing table.
     */
    public T getOwnID();

    /**
     * @return the level of this node. The root node has level 0, and each child
     *         node has the level of its parent plus one. (If there are pseudo
     *         nodes on top of the root node, these might have negative level
     *         values.)
     */
    public int getLevel();

    /**
     * Returns the prefix that all nodes saved in this subtree have in common.
     * 
     * @return the prefix that all nodes saved in this subtree have in common.
     */
    public BigInteger getPrefix();

}
