package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;

/**
 * Visitor for Nodes according to the GoF design pattern "Visitor". Can be used
 * to add functionality to the routing table, or implement different variations
 * of methods without having to change the routing table itself.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
interface NodeVisitor<T extends KademliaOverlayID> {

    /**
     * The callback function if a Node of an "unknown" subtype is visited (for
     * example in test cases when stubs are used).
     * 
     * @param node
     *                the currently visited Node.
     */
    public void visit(Node<T> node);

    /**
     * The callback function if a PseudoRootNode is visited.
     * 
     * @param node
     *                the currently visited PseudoRootNode.
     */
    public void visit(PseudoRootNode<T> node);

    /**
     * The callback function if a BranchNode is visited.
     * 
     * @param node
     *                the currently visited BranchNode.
     */
    public void visit(BranchNode<T> node);

    /**
     * The callback function if a LeafNode is visited.
     * 
     * @param node
     *                the currently visited LeafNode.
     */
    public void visit(LeafNode<T> node);

}
