package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.TypesConfig;

/**
 * Configuration settings for the routing table. All methods should return
 * constant values.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public interface RoutingTableConfig extends TypesConfig {

    /**
     * @return the binary length of the Kademlia overlay identifiers
     *         (KademliaOverlayID).
     */
    public int getIDLength();

    /**
     * @return the order of the routing tree. (Each node can have up to
     *         {@code 2^getRoutingTreeOrder()} children.) In the Kademlia paper,
     *         this parameter is called {@code b}.
     */
    public int getRoutingTreeOrder();

    /**
     * @return the size of a bucket. In the Kademlia paper, this parameter is
     *         called {@code k}.
     */
    public int getBucketSize();

    /**
     * @return the size of the replacement cache.
     */
    public int getReplacementCacheSize();

    /**
     * @return the number of marks that allow the routing table to remove a
     *         routing table entry of a contact that has not responded to
     *         messages. That is, if the contact has failed to respond to
     *         queries {@code getStaleCounter()} times, it may be dropped.
     */
    public int getStaleCounter();

    /**
     * @return (an approximate of) the total number of peers that will be part
     *         of the overlay network.
     */
    public int getNumberOfPeers();

}
