package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.measurement.file;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Map;

/**
 * Reads data lookup files from stdin, calculates the average distribution of
 * messages over depths, and writes to stout (column1=depth, column2=message
 * ratio).
 * <p>
 * First, the absolute count of messages sent per depth in each lookup is
 * transformed into a depth ratio, that is which part of the messages of one
 * lookup has been sent with which cluster depth. Finally, an average is
 * calculated over all lookups in this representation.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class DataMsgDepthAvgCalculator {

    /**
     * @param args
     *                0: true=consider successful lookups, false=consider failed
     *                lookups.
     */
    public static void main(String[] args) throws IOException {
	final boolean expectedResult = Boolean.valueOf(args[0]);
	final BufferedReader in = new BufferedReader(new InputStreamReader(
		System.in));
	final BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
		System.out));
	final Map<Integer, Double> depthAverageSums = new HashMap<Integer, Double>();

	double totalLookups = 0;
	double sentMsgs;
	Map<Integer, Integer> msgDepths;
	Double currAvgSum;

	String line = null;
	DataLookupQuality tmp;
	while ((line = in.readLine()) != null) {
	    tmp = DataLookupQuality.fromString(line);
	    if (tmp != null && tmp.successful == expectedResult) {
		totalLookups++;
		sentMsgs = 0;
		msgDepths = tmp.sentMessages;
		for (final Map.Entry<Integer, Integer> entry : msgDepths
			.entrySet()) {
		    sentMsgs += entry.getValue();
		}
		for (final Map.Entry<Integer, Integer> entry : msgDepths
			.entrySet()) {
		    currAvgSum = depthAverageSums.get(entry.getKey());
		    if (currAvgSum == null) {
			currAvgSum = entry.getValue() / sentMsgs;
		    } else {
			currAvgSum += entry.getValue() / sentMsgs;
		    }
		    depthAverageSums.put(entry.getKey(), currAvgSum);
		}
	    }
	}

	out.write("# DEPTH AVG_FRACTION");
	out.newLine();
	for (final Map.Entry<Integer, Double> entry : depthAverageSums
		.entrySet()) {
	    out.write(entry.getKey().toString());
	    out.append(' ');
	    out.write(Double.valueOf(entry.getValue() / totalLookups)
		    .toString());
	    out.newLine();
	}
	out.close();
    }

}
