package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.measurement.file;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Reads measurement data from files.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class MeasurementReader {

    private Map<Integer, OpDelay> opDelay;

    private Map<Integer, OpTraffic> opTraffic;

    private Map<Integer, DataLookupQuality> dataQuality;

    private Map<Integer, NodeLookupQuality> nodeQuality;

    /**
     * Reads operation delay information from the given file.
     * 
     * @param path
     * @throws IOException
     */
    public final void readOpDelay(final String path) throws IOException {
	opDelay = new HashMap<Integer, OpDelay>();
	FileReader fr = new FileReader(path);
	BufferedReader in = new BufferedReader(fr);
	String line = null;
	OpDelay tmp;
	in.readLine(); // ignore first line
	while ((line = in.readLine()) != null) {
	    tmp = OpDelay.fromString(line);
	    opDelay.put(tmp.getOperationID(), tmp);
	}
    }

    /**
     * Reads operation traffic information from the given file.
     * 
     * @param path
     * @throws IOException
     */
    public final void readOpTraffic(final String path) throws IOException {
	opTraffic = new HashMap<Integer, OpTraffic>();
	FileReader fr = new FileReader(path);
	BufferedReader in = new BufferedReader(fr);
	String line = null;
	OpTraffic tmp;
	in.readLine(); // ignore first line
	while ((line = in.readLine()) != null) {
	    tmp = OpTraffic.fromString(line);
	    opTraffic.put(tmp.getOperationID(), tmp);
	}
    }

    /**
     * Reads data lookup quality information from the given file.
     * 
     * @param path
     * @throws IOException
     */
    public final void readDataQuality(final String path) throws IOException {
	dataQuality = new HashMap<Integer, DataLookupQuality>();
	FileReader fr = new FileReader(path);
	BufferedReader in = new BufferedReader(fr);
	String line = null;
	DataLookupQuality tmp;
	in.readLine(); // ignore first line
	while ((line = in.readLine()) != null) {
	    tmp = DataLookupQuality.fromString(line);
//	    dataQuality.put(tmp.getOperationID(), tmp); // FIXME
	}
    }

    /**
     * Reads node lookup quality information from the given file.
     * 
     * @param path
     * @throws IOException
     */
    public final void readNodeQuality(final String path) throws IOException {
	nodeQuality = new HashMap<Integer, NodeLookupQuality>();
	FileReader fr = new FileReader(path);
	BufferedReader in = new BufferedReader(fr);
	String line = null;
	NodeLookupQuality tmp;
	in.readLine(); // ignore first line
	while ((line = in.readLine()) != null) {
	    tmp = NodeLookupQuality.fromString(line);
	    nodeQuality.put(tmp.getOperationID(), tmp);
	}
    }

}
