package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.measurement.file;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Information about the quality of a node lookup.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public final class NodeLookupQuality {

    private static final Pattern filePattern = Pattern
	    .compile("^(\\d+) (\\d+) (\\d+) (\\d+) (\\d+)$");

    private final int opID;

    private final int perfectContacts;

    private final int offlineContacts;

    private final int missedBetterContacts;

    private final int size;

    /**
     * Constructs a new NodeLookupQuality. All arguments are passed as Strings
     * and internally parsed.
     * 
     * @param operationID
     *                the operation identifier.
     * @param perfectContacts
     *                the number of perfect contacts.
     * @param offlineContacts
     *                the number of offline contacts.
     * @param missedBetterContacts
     *                the number of missed closer contacts.
     * @param numberOfContacts
     *                the total number of contacts.
     */
    public NodeLookupQuality(final String operationID,
	    final String perfectContacts, final String offlineContacts,
	    final String missedBetterContacts, final String numberOfContacts) {
	this.opID = Integer.valueOf(operationID);
	this.perfectContacts = Integer.valueOf(perfectContacts);
	this.offlineContacts = Integer.valueOf(offlineContacts);
	this.missedBetterContacts = Integer.valueOf(missedBetterContacts);
	this.size = Integer.valueOf(numberOfContacts);
    }

    /**
     * @return the identifier of the operation.
     */
    public final int getOperationID() {
	return opID;
    }

    /**
     * @return the number of perfect contacts.
     */
    public final int getPerfectContacts() {
	return perfectContacts;
    }

    /**
     * @return the number of offline contacts.
     */
    public final int getOfflineContacts() {
	return offlineContacts;
    }

    /**
     * @return the number of missed closer contacts.
     */
    public final int getMissedCloserContacts() {
	return missedBetterContacts;
    }

    /**
     * @return the total number of contacts found.
     */
    public final int getSize() {
	return size;
    }

    /**
     * Constructs and returns a new NodeLookupQuality from the given String that
     * represents one line of a file.
     * 
     * @param str
     *                one line.
     * @return the new NodeLookupQuality.
     */
    public static NodeLookupQuality fromString(final String str) {
	final Matcher m = filePattern.matcher(str);
	if (!m.matches()) {
	    System.err.println("Wrong line: '" + str + "'");
	}
	return new NodeLookupQuality(m.group(1), m.group(2), m.group(3), m
		.group(4), m.group(5));
    }
}
