package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;


/**
 * An abstract operation factory that saves configuration information that is
 * useful for subclasses.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public abstract class AbstractOperationFactory<T extends KademliaOverlayID>
	implements OperationFactory<T> {

    /**
     * Configuration values ("constants").
     */
    protected final OperationsConfig config;
    
    /**
     * Operations of this Node that have not yet finished.
     */
    private final Set<KademliaOperation<T>> runningOperations;

    /**
     * Constructs a new abstract operation factory and sets the given
     * configuration information.
     * 
     * @param conf
     *                an OperationsConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public AbstractOperationFactory(final OperationsConfig conf) {
	this.config = conf;
	this.runningOperations = new HashSet<KademliaOperation<T>>();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void operationConstructed(KademliaOperation<T> newOperation) {
	this.runningOperations.add(newOperation);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void operationFinished(KademliaOperation<T> finishedOperation) {
	this.runningOperations.remove(finishedOperation);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void abortAllOperations() {
	final List<KademliaOperation<T>> runningOpsCopy;
	runningOpsCopy = new ArrayList<KademliaOperation<T>>(runningOperations);
	for (final KademliaOperation<T> op : runningOpsCopy) {
	    op.abort();
	}
    }
}
