package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.TypesConfig;

/**
 * Configuration settings for Kademlia operations. All methods should return
 * constant values.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public interface OperationsConfig extends TypesConfig {

    /**
     * @return the maximum number of simulation time units that a lookup
     *         operation may take.
     */
    public long getLookupOperationTimeout();

    /**
     * @return the time in simulation time units after which a message without
     *         reply times out. That is, the receiver of the message is
     *         considered unresponsive.
     */
    public long getLookupMessageTimeout();

    /**
     * @return the size of a bucket. In the Kademlia paper, this parameter is
     *         called {@code k}.
     */
    public int getBucketSize();

    /**
     * @return the maximum number of simultaneous, concurrent lookups during a
     *         node lookup.
     */
    public int getMaxConcurrentLookups();

    /**
     * @return the order of the routing tree. (Each node can have up to
     *         {@code 2^getRoutingTreeOrder()} children.) In the Kademlia paper,
     *         this parameter is called {@code b}.
     */
    public int getRoutingTreeOrder();
    
    /**
     * @return the interval in which the routing table buckets are refreshed.
     *         Simulation time units are used.
     * @see Simulator for time units.
     */
    public long getRefreshInterval();

}
