package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.lookup;

import java.util.Set;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.Node.VisibilityRestrictableNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.OperationsConfig;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;


/**
 * A lookup coordinator that initially fills the set of the k closest known
 * nodes with a visibility restricted lookup to the local routing table.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class VisibilityRestrictedLookupCoordinator<H extends HKademliaOverlayID>
	extends StandardLookupCoordinator<H> {

    /**
     * The VisibilityRestrictableNode that has initiated this lookup.
     */
    protected final VisibilityRestrictableNode<H> myNode;

    /**
     * Constructs a new lookup coordinator that carries out visibility
     * restricted local lookups.
     * 
     * @param lookupKey
     *                the KademliaOverlayKey that is to be looked up.
     * @param node
     *                the VisibilityRestrictableNode that initiates this lookup.
     * @param conf
     *                an OperationsConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public VisibilityRestrictedLookupCoordinator(
	    final KademliaOverlayKey lookupKey,
	    final VisibilityRestrictableNode<H> node,
	    final OperationsConfig conf) {
	super(lookupKey, node, conf);
	myNode = node;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Set<KademliaOverlayContact<H>> localLookup(
	    final KademliaOverlayKey key) {
	return myNode.getKademliaRoutingTable().visibilityRestrictedLocalLookup(key,
		config.getBucketSize());
    }
}
