package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types;

/**
 * Configuration settings for general Kademlia types. All methods should return
 * constant values.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public interface TypesConfig {

    /**
     * @return the height of the hierarchy tree. 0 means "no hierarchy". Each
     *         step of the hierarchy consumes {@link #getHierarchyTreeOrder()}
     *         bits in the node identifier.
     */
    public int getHierarchyDepth();

    /**
     * @return the number of branching bits per hierarchy step. That is, each
     *         node (cluster) in the hierarchy tree has
     *         {@code 2^getHierarchyTreeOrder()} children.
     */
    public int getHierarchyTreeOrder();

    /**
     * @return the binary length of the Kademlia overlay identifiers
     *         (KademliaOverlayID).
     */
    public int getIDLength();

    /**
     * @return the assumed size of a data item (in bytes) that is transferred at
     *         the overlay (application) layer.
     */
    public int getDataSize();

}
