package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.messages;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.AbstractKademliaOperation.Reason;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.TypesConfig;

/**
 * Message format for data lookups.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public final class DataLookupMsg<T extends KademliaOverlayID> extends KademliaMsg<T> {

    /**
     * The KademliaOverlayKey of the data to be looked up.
     */
    private final KademliaOverlayKey key;

    /**
     * Constructs a new message used to look up the data item associated with
     * <code>dataKey</code>.
     * 
     * @param sender
     *                the KademliaOverlayID of the sender of this message.
     * @param destination
     *                the KademliaOverlayID of the destination of this message.
     * @param dataKey
     *                the KademliaOverlayKey associated with the data item to be
     *                looked up.
     * @param why
     *                the reason why this message will be sent.
     * @param conf
     *                a TypesConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public DataLookupMsg(final T sender, final T destination,
	    final KademliaOverlayKey dataKey, final Reason why,
	    final TypesConfig conf) {
	super(sender, destination, why, conf);
	this.key = dataKey;
    }

    /**
     * @return the KademliaOverlayKey associated with the data to be looked up.
     */
    public final KademliaOverlayKey getDataKey() {
	return this.key;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected final long getOtherFieldSize() {
	return (config.getIDLength() / 8);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final String toString() {
	return "[DataLookupMsg|from:" + getSender() + "; to:"
		+ getDestination() + "; reason:" + getReason() + "; lookup:"
		+ key + "]";
    }

}
