package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.messages;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.AbstractKademliaOperation.Reason;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.TypesConfig;

/**
 * Format for a message that requests the k closest known nodes around a given
 * key.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class KClosestNodesLookupMsg<T extends KademliaOverlayID> extends
	KademliaMsg<T> {

    /**
     * The KademliaOverlayKey of the node of which the k closest known
     * neighbours are to be looked up.
     */
    private final KademliaOverlayKey key;

    /**
     * Constructs a new message used to look up the k closest known neighbours
     * of <code>nodeKey</code>.
     * 
     * @param sender
     *                the KademliaOverlayID of the sender of this message.
     * @param destination
     *                the KademliaOverlayID of the destination of this message.
     * @param dataKey
     *                the KademliaOverlayKey of the node of which the k closest
     *                known neighbours are to be returned.
     * @param why
     *                the reason why this message will be sent.
     * @param conf
     *                a TypesConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public KClosestNodesLookupMsg(final T sender, final T destination,
	    final KademliaOverlayKey nodeKey, final Reason why,
	    final TypesConfig conf) {
	super(sender, destination, why, conf);
	this.key = nodeKey;
    }

    /**
     * @return the KademliaOverlayKey of the node of which the k closest known
     *         neighbours are to be looked up.
     */
    public final KademliaOverlayKey getNodeKey() {
	return this.key;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected long getOtherFieldSize() {
	return (config.getIDLength() / 8);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
	return "[KClosestNodesLookupMsg|from:" + getSender() + "; to:"
		+ getDestination() + "; reason:" + getReason() + "; lookup:"
		+ key + "]";
    }

}
