package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.messages;

import java.util.ArrayList;
import java.util.Collection;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.operations.AbstractKademliaOperation.Reason;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.TypesConfig;


/**
 * Format for messages that carry up to k nodes (in response to a node / data /
 * bucket lookup).
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public final class NodeListMsg<T extends KademliaOverlayID> extends KademliaMsg<T> {

    /**
     * Up to {@link TypesConfig#getBucketSize()} nodes in a list.
     */
    private final Collection<KademliaOverlayContact<T>> kNodes;

    /**
     * Constructs a new message that may carry a list of up to
     * {@link TypesConfig#getBucketSize()} nodes.
     * 
     * @param sender
     *                the KademliaOverlayID of the sender of this message.
     * @param destination
     *                the KademliaOverlayID of the destination of this message.
     * @param kNodes
     *                a Collection of up to K KademliaOverlayContacts. This
     *                collection will be copied.
     * @param why
     *                the reason why this message will be sent.
     * @param conf
     *                a TypesConfig reference that permits to retrieve
     *                configuration "constants".
     */
    public NodeListMsg(final T sender, final T destination,
	    final Collection<KademliaOverlayContact<T>> kNodes,
	    final Reason why, final TypesConfig conf) {
	super(sender, destination, why, conf);
	this.kNodes = new ArrayList<KademliaOverlayContact<T>>(kNodes);
    }

    /**
     * @return the list of nodes that is saved in this message.
     */
    public final Collection<KademliaOverlayContact<T>> getNodes() {
	return this.kNodes;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected final long getOtherFieldSize() {
	return kNodes.size() * (config.getIDLength() / 8);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final String toString() {
	return "[NodeListMsg|from:" + getSender() + "; to:" + getDestination()
		+ "; reason:" + getReason() + "; contents:" + kNodes + "]";
    }
}
