package de.tud.kom.p2psim.impl.overlay.gia.Analyzer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigInteger;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tud.kom.p2psim.api.analyzer.Analyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.OperationAnalyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.TransAnalyzer;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayMessage;
import de.tud.kom.p2psim.impl.overlay.gia.FileFactory;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAPingMessage;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryHitMessage;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryMessage;
import de.tud.kom.p2psim.impl.overlay.gia.Operations.BaseOperation;
import de.tud.kom.p2psim.impl.overlay.gia.Operations.GIAConnectOperation;
import de.tud.kom.p2psim.impl.overlay.gia.Operations.GIAPingOperation;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.AbstractTransMessage;

public class GIAAnalyzer implements Analyzer, TransAnalyzer, OperationAnalyzer{
	
	private Map<BigInteger,Double> lookupHopCount;
	private Map<BigInteger,Long> lookupStarted;
	private Map<BigInteger,Long> lookupDelay;
	private Map<BigInteger,Boolean> dataOnline;
	private Map<BigInteger,Long> lookupReplyDelay;
	private Map<BigInteger,Double> relativDelayPenalty;
	private Map<GIAOverlayID, Integer> loadBalance1Peers;
	private Map<GIAOverlayID, Integer> loadBalance10Peers;
	private Map<GIAOverlayID, Integer> loadBalance100Peers;
	private Map<GIAOverlayID, Integer> loadBalance1000Peers;
	private Map<GIAOverlayID, Integer> loadBalance10000Peers;
	private List<BigInteger> opIDs;
	private List<Double> loadBalance1;
	private List<Double> loadBalance10;
	private List<Double> loadBalance100;
	private List<Double> loadBalance1000;
	private List<Double> loadBalance10000;
	private Map<Long, Double> hopsPerSecond;
	private Map<Long, Long> delayPerSecond;
	private Map<Long, Double> rdpPerSecond;
	private Map<Long, Double> successPerSecond;
	private Map<Long, Integer> queriesPerSecond;
	private List<Long> seconds;
	private double lookups;
	private double succeeded;
	private long totalDelay;
	private double successRate;
	private Map<GIAOverlayID, GIAOverlayNode> nodes;
	private List<GIAOverlayID> peers1;
	private List<GIAOverlayID> peers10;
	private List<GIAOverlayID> peers100;
	private List<GIAOverlayID> peers1000;
	private List<GIAOverlayID> peers10000;
	private double hopCount;
	private double numberOfMessages;
	private double numberOfQueryMessages;
	private int maxHops;
	private double rdp;
	private double loadBalanceRatio1;
	private double loadBalanceRatio10;
	private double loadBalanceRatio100;
	private double loadBalanceRatio1000;
	private double loadBalanceRatio10000;

	public void start() {
		lookupHopCount = new HashMap<BigInteger, Double>();
		lookupDelay = new HashMap<BigInteger, Long>();
		lookupStarted = new HashMap<BigInteger, Long>();
		dataOnline = new HashMap<BigInteger, Boolean>();
		loadBalance1Peers = new HashMap<GIAOverlayID, Integer>();
		loadBalance10Peers = new HashMap<GIAOverlayID, Integer>();
		loadBalance100Peers = new HashMap<GIAOverlayID, Integer>();
		loadBalance1000Peers = new HashMap<GIAOverlayID, Integer>();
		loadBalance10000Peers = new HashMap<GIAOverlayID, Integer>();
		lookupReplyDelay = new HashMap<BigInteger, Long>();
		relativDelayPenalty = new HashMap<BigInteger, Double>();
		opIDs = new LinkedList<BigInteger>();
		loadBalance1 = new LinkedList<Double>();
		loadBalance10 = new LinkedList<Double>();
		loadBalance100 = new LinkedList<Double>();
		loadBalance1000 = new LinkedList<Double>();
		loadBalance10000 = new LinkedList<Double>();
		hopsPerSecond = new HashMap<Long, Double>();
		delayPerSecond = new HashMap<Long, Long>();
		rdpPerSecond = new HashMap<Long, Double>();
		successPerSecond = new HashMap<Long, Double>();
		queriesPerSecond = new HashMap<Long, Integer>();
		seconds = new LinkedList<Long>();
		lookups = 0;
		succeeded = 0;
		nodes = new HashMap<GIAOverlayID, GIAOverlayNode>();
		peers1 = new LinkedList<GIAOverlayID>();
		peers10 = new LinkedList<GIAOverlayID>();
		peers100 = new LinkedList<GIAOverlayID>();
		peers1000 = new LinkedList<GIAOverlayID>();
		peers10000 = new LinkedList<GIAOverlayID>();
		numberOfMessages = 0;
		numberOfQueryMessages = 0;
		maxHops = 0;
		loadBalanceRatio1 = 0;
		loadBalanceRatio10 = 0;
		loadBalanceRatio100 = 0;
		loadBalanceRatio1000 = 0;
		loadBalanceRatio10000 = 0;
	}

	public void stop(Writer output) {
		/*for(Gnutella06OverlayNode node : nodes){
			System.out.println(node.getOverlayID().toString()+ " is SuperPeer " +node.isSuperpeer() + " is connected to:");
			for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().allContacts()){
				System.out.println("Superpeer: " + contact.getOverlayID().toString());
			}
			for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().inactiveContacts()){
				System.out.println("inactive Superpeer: " + contact.getOverlayID().toString());
			}
			if(node.isSuperpeer()){
				for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().activeLeafContacts()){
					System.out.println("Peer: " + contact.getOverlayID().toString());
				}
				for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().inactiveLeafContacts()){
					System.out.println("inactive Peer: " + contact.getOverlayID().toString());
				}
			}
		}*/
		
		
		int counter = 1;
		for(BigInteger opID : opIDs){
			if(lookupDelay.get(opID) != null){
				System.out.println(lookupStarted.get(opID) + " Hops: " +lookupHopCount.get(opID)+ " Delay: " + lookupDelay.get(opID));
				totalDelay += lookupDelay.get(opID);
				counter++;
			}
			if(lookupHopCount.get(opID) !=null){
				hopCount += lookupHopCount.get(opID);
			}
			if(lookupReplyDelay.get(opID) != null && lookupDelay.get(opID) != null){
				double delayPenalty = (double)lookupDelay.get(opID)/(double)lookupReplyDelay.get(opID);
				relativDelayPenalty.put(opID, delayPenalty);
				rdp += delayPenalty;
			}
		}
		if(succeeded != 0){
			successRate = (succeeded/lookups)*100;
			System.out.println("Lookups: " +lookups+ " Succeeded; " +succeeded+ " Rate: " +successRate);
			System.out.println("Average Delay: " + totalDelay/lookupDelay.size());
			System.out.println("Average Hops: " + hopCount/lookupHopCount.size());
		}
		
		double peer1Messages = 0;
		for(GIAOverlayID peer : peers1){
			if(loadBalance1Peers.containsKey(peer)){
				loadBalance1.add((double)loadBalance1Peers.get(peer));
				peer1Messages += loadBalance1Peers.get(peer);
			}
		}
		Collections.sort(loadBalance1);
		
		double peer10Messages = 0;
		for(GIAOverlayID peer : peers10){
			if(loadBalance10Peers.containsKey(peer)){
				loadBalance10.add((double)loadBalance10Peers.get(peer));
				peer10Messages += loadBalance10Peers.get(peer);
			}
		}
		Collections.sort(loadBalance10);
		
		double peer100Messages = 0;
		for(GIAOverlayID peer : peers100){
			if(loadBalance100Peers.containsKey(peer)){
				loadBalance100.add((double)loadBalance100Peers.get(peer));
				peer100Messages += loadBalance100Peers.get(peer);
			}
		}
		Collections.sort(loadBalance100);
		
		double peer1000Messages = 0;
		for(GIAOverlayID peer : peers1000){
			if(loadBalance1000Peers.containsKey(peer)){
				loadBalance1000.add((double)loadBalance1000Peers.get(peer));
				peer1000Messages += loadBalance1000Peers.get(peer);
			}
		}
		Collections.sort(loadBalance1000);

		double peer10000Messages = 0;
		for(GIAOverlayID peer : peers10000){
			if(loadBalance10000Peers.containsKey(peer)){
				loadBalance10000.add((double)loadBalance10000Peers.get(peer));
				peer10000Messages += loadBalance10000Peers.get(peer);
			}
		}
		Collections.sort(loadBalance10000);
		
		for(BigInteger opID: opIDs){
			long second = lookupStarted.get(opID)/10000000;
			if(second >= 5){	
				if(!seconds.contains(second)){
					seconds.add(second);
				}
				if(queriesPerSecond.containsKey(second)){
					queriesPerSecond.put(second, queriesPerSecond.get(second) + 1);
				}
				else{
					queriesPerSecond.put(second, 1);
				}
				if(lookupHopCount.containsKey(opID)){
					if(hopsPerSecond.containsKey(second)){
						double temp = hopsPerSecond.get(second) + (double)lookupHopCount.get(opID);
						hopsPerSecond.put(second, temp);
					}
					else{
						hopsPerSecond.put(second, (double)lookupHopCount.get(opID));
					}
				}
				if(lookupDelay.containsKey(opID)){
					if(delayPerSecond.containsKey(second)){
						delayPerSecond.put(second, delayPerSecond.get(second) + lookupDelay.get(opID));
					}
					else{
						delayPerSecond.put(second, lookupDelay.get(opID));
					}
				}
				if(relativDelayPenalty.containsKey(opID)){
					if(rdpPerSecond.containsKey(second)){
						rdpPerSecond.put(second, rdpPerSecond.get(second) + relativDelayPenalty.get(opID));
					}
					else{
						rdpPerSecond.put(second, relativDelayPenalty.get(opID));
					}
				}
				if(lookupHopCount.containsKey(opID)){
					if(successPerSecond.containsKey(second)){
						successPerSecond.put(second, successPerSecond.get(second) + 1);
					}
					else{
						successPerSecond.put(second, 1.0);
					}
				}
			}	
		}
		
		successRate = 0;
		for(long second : seconds){
			if(second >= 5){
				if(successPerSecond.get(second) != null){
					hopsPerSecond.put(second, hopsPerSecond.get(second)/successPerSecond.get(second));
					delayPerSecond.put(second, delayPerSecond.get(second)/(int)(1000*successPerSecond.get(second)));
					rdpPerSecond.put(second, rdpPerSecond.get(second)/successPerSecond.get(second));
					successPerSecond.put(second, (100*successPerSecond.get(second))/queriesPerSecond.get(second));
					successRate += successPerSecond.get(second);
				}	
			}	
		}
		successRate /= seconds.size();
		
		loadBalanceRatio1 = loadBalance1.get(loadBalance1.size() - 1)/loadBalance1.get(loadBalance1.size()/2);
		loadBalanceRatio10 = loadBalance10.get(loadBalance10.size() - 1)/loadBalance10.get(loadBalance10.size()/2);
		loadBalanceRatio100 = loadBalance100.get(loadBalance100.size() - 1)/loadBalance100.get(loadBalance100.size()/2);
		loadBalanceRatio1000 = loadBalance1000.get(loadBalance1000.size() - 1)/loadBalance1000.get(loadBalance1000.size()/2);
		loadBalanceRatio10000 = loadBalance10000.get(loadBalance10000.size() - 1)/loadBalance10000.get(loadBalance10000.size()/2);
		double percentageOfQueries = numberOfQueryMessages/numberOfMessages;
		
		String f = "GIA-Analyzerausgabe.dat";
	    FileWriter fstream = null;
		try {
			fstream = new FileWriter(f);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write("	GIA-Analyzer	\n");
			out.write("	-----------------------	\n");
			out.write("	\n");
			out.write("	\n");
			out.write("number of nodes "+nodes.size()+" \n");
			out.write("	\n");
			out.write("Lookups: " +lookups+ " Succeeded; " +succeeded+ " Rate: " +successRate + " \n");
			out.write("	\n");
			out.write("Average Delay: " + totalDelay/lookupDelay.size() + " \n");
			out.write("	\n");
			out.write("Average Hops: " + hopCount/lookupHopCount.size() + " \n");
			out.write("	\n");
			out.write("Percentage of Queries: " +percentageOfQueries+"\n");
			out.write("	\n");
			out.write("QueryMessages/Query: " +numberOfQueryMessages/lookups+" \n");
			out.write("	\n");
			out.write("Average RDP: " + rdp/relativDelayPenalty.size()+" \n");
			out.write("	\n");
			out.write("Load Balance Ratio 1: " + loadBalanceRatio1 + " \n");
			out.write(" \n");
			out.write("Load Balance Ratio 10: " + loadBalanceRatio10 + " \n");
			out.write(" \n");
			out.write("Load Balance Ratio 100: " + loadBalanceRatio100 + " \n");
			out.write(" \n");
			out.write("Load Balance Ratio 1000: " + loadBalanceRatio1000 + " \n");
			out.write(" \n");
			out.write("Load Balance Ratio 10000: " + loadBalanceRatio10000 + " \n");
			out.write(" \n");
			/*for(BigInteger opID : opIDs){
				if(lookupDelay.get(opID) != null){
					out.write(lookupStarted.get(opID) + " Hops: " +lookupHopCount.get(opID)+ " Delay: " + lookupDelay.get(opID) + " RDP: " +relativDelayPenalty.get(opID) +" QueryHit Delay: "+lookupReplyDelay.get(opID)+" \n");
					
				}
			}
			out.write("	\n");
			out.write("Failed Queries: \n");
			out.write("	\n");
			for(BigInteger opID : opIDs){
				if(lookupStarted.containsKey(opID) && !lookupDelay.containsKey(opID)){
					out.write(lookupStarted.get(opID) + " data available: " +dataOnline.get(opID)+" \n");
				}
			}*/
			out.write("second   success \n");
			for(long second : seconds){
				out.write(second*10+ "  " +successPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   hops \n");
			for(long second : seconds){
				out.write(second*10+ "  " +hopsPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   delay \n");
			for(long second : seconds){
				out.write(second *10+ "  " +delayPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   rdp \n");
			for(long second : seconds){
				out.write(second*10+ "  " +rdpPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("	\n");
			out.write("Loadbalance 1 peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance1.size(); i++){
				double percent = (loadBalance1.get(loadBalance1.size()- 1 - i)/peer1Messages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average 1 peer load: "+peer1Messages/loadBalance1Peers.size()+ " \n");
			out.write("	\n");
			out.write("Loadbalance 10 Peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance10.size(); i++){
				double percent = (loadBalance10.get(loadBalance10.size()- 1 - i)/peer10Messages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average 10 Peer load: "+peer10Messages/loadBalance10Peers.size()+ " \n");
			out.write("	\n");
			out.write("Loadbalance 100 Peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance100.size(); i++){
				double percent = (loadBalance100.get(loadBalance100.size()- 1 - i)/peer100Messages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average 100 Peer load: "+peer100Messages/loadBalance100Peers.size()+ " \n");
			out.write(" \n");
			out.write("Loadbalance 1000 Peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance1000.size(); i++){
				double percent = (loadBalance1000.get(loadBalance1000.size()- 1 - i)/peer1000Messages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average 1000 Peer load: "+peer1000Messages/loadBalance1000Peers.size()+ " \n");
			out.write("	\n");
			out.write("Loadbalance 10000 Peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance10000.size(); i++){
				double percent = (loadBalance10000.get(loadBalance10000.size()- 1 - i)/peer10000Messages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average 10000 Peer load: "+peer10000Messages/loadBalance10000Peers.size()+ " \n");
			out.write("	\n");
			out.write("Diameter: " + maxHops);
			out.close();         
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void transMsgReceived(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		AbstractOverlayMessage<GIAOverlayID> abMessage = (AbstractOverlayMessage<GIAOverlayID>) message;
		if(peers1.contains(abMessage.getReceiver())){
			if(loadBalance1Peers.containsKey(abMessage.getReceiver())){
				loadBalance1Peers.put(abMessage.getReceiver(), loadBalance1Peers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalance1Peers.put(abMessage.getReceiver(), 1);
		}
		else if(peers10.contains(abMessage.getReceiver())){
			if(loadBalance10Peers.containsKey(abMessage.getReceiver())){
				loadBalance10Peers.put(abMessage.getReceiver(), loadBalance10Peers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalance10Peers.put(abMessage.getReceiver(), 1);
		}
		else if(peers100.contains(abMessage.getReceiver())){
			if(loadBalance100Peers.containsKey(abMessage.getReceiver())){
				loadBalance100Peers.put(abMessage.getReceiver(), loadBalance100Peers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalance100Peers.put(abMessage.getReceiver(), 1);
		}
		else if(peers1000.contains(abMessage.getReceiver())){
			if(loadBalance1000Peers.containsKey(abMessage.getReceiver())){
				loadBalance1000Peers.put(abMessage.getReceiver(), loadBalance1000Peers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalance1000Peers.put(abMessage.getReceiver(), 1);
		}
		else if(peers10000.contains(abMessage.getReceiver())){
			if(loadBalance10000Peers.containsKey(abMessage.getReceiver())){
				loadBalance10000Peers.put(abMessage.getReceiver(), loadBalance10000Peers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalance10000Peers.put(abMessage.getReceiver(), 1);
		}
		
		if (message instanceof GIAQueryHitMessage){
			GIAQueryHitMessage reply = (GIAQueryHitMessage)message.getPayload();
			if(opIDs.contains(reply.getDescriptor()) && lookupStarted.containsKey(reply.getDescriptor()) && !lookupDelay.containsKey(reply.getDescriptor())){
				lookupHopCount.put(reply.getDescriptor(), (double)reply.getHops());
				lookupDelay.put(reply.getDescriptor(), Simulator.getCurrentTime() - lookupStarted.get(reply.getDescriptor()));
				succeeded++;
				long temp = lookupReplyDelay.get(reply.getDescriptor());
				lookupReplyDelay.put(reply.getDescriptor(), Simulator.getCurrentTime() - temp);
			}
		}
		else if(message instanceof GIAPingMessage){
			GIAPingMessage ping = (GIAPingMessage)message.getPayload();
			
			if(ping.getHops() >= maxHops){
				maxHops = ping.getHops();
			}
		}
	}

	public void transMsgSent(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		numberOfMessages++;
		
		if (message instanceof GIAQueryMessage){
			numberOfQueryMessages++;
			GIAQueryMessage query = (GIAQueryMessage)message.getPayload();
			BigInteger opID = query.getDescriptor();
			if(!opIDs.contains(opID)){
				lookupStarted.put(opID, Simulator.getCurrentTime());
				opIDs.add(opID);
				lookups++;
				if(FileFactory.getInstance().getFileSharingKeys().contains(query.getKey())){
					dataOnline.put(opID, true);
				}
				else {
					dataOnline.put(opID, false);
				}	
			}
		}
		else if (message instanceof GIAQueryHitMessage){
			GIAQueryHitMessage reply = (GIAQueryHitMessage)message.getPayload();
			lookupReplyDelay.put(reply.getDescriptor(), Simulator.getCurrentTime());
		}
		
	}

	public void operationFinished(Operation<?> op) {
		// TODO Auto-generated method stub
		
	}

	public void operationInitiated(Operation<?> op) {
		if(op instanceof GIAPingOperation){
			BaseOperation operation = (BaseOperation) op;
			if(!nodes.containsKey((GIAOverlayID)operation.getComponent().getOverlayID())){
				nodes.put((GIAOverlayID)operation.getComponent().getOverlayID(), operation.getComponent());
				if(operation.getComponent().getCapacity() == 1){
					peers1.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 10){
					peers10.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 100){
					peers100.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 1000){
					peers1000.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 10000){
					peers10000.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
			}
		}
		else if(op instanceof GIAConnectOperation){
			GIAConnectOperation operation = (GIAConnectOperation) op;
			if(!nodes.containsKey((GIAOverlayID)operation.getComponent().getOverlayID())){
				nodes.put((GIAOverlayID)operation.getComponent().getOverlayID(), operation.getComponent());
				if(operation.getComponent().getCapacity() == 1){
					peers1.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 10){
					peers10.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 100){
					peers100.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 1000){
					peers1000.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
				else if(operation.getComponent().getCapacity() == 10000){
					peers10000.add((GIAOverlayID)operation.getComponent().getOverlayID());
				}
			}
		}
		
		
		/*if(op instanceof Gnutella06QueryOperation){
			int opID = op.getOperationID();
			if(!lookupStarted.containsKey(opID)){
				lookupStarted.put(opID, Simulator.getCurrentTime());
				opIDs.add(opID);
				lookups++;
			}
		}
		else if (op instanceof Gnutella06QueryHitOperation){
			Gnutella06QueryHitOperation queryHit = (Gnutella06QueryHitOperation) op;
			if(lookupStarted.containsKey(queryHit.getOperationID())){
				lookupHopCount.put(queryHit.getOperationID(), queryHit.getHops());
				lookupDelay.put(queryHit.getOperationID(), Simulator.getCurrentTime() - lookupStarted.get(queryHit.getOperationID()));
				lookupStarted.remove(queryHit.getOperationID());
				succeeded++;
			}
		}*/
	}

}
