package de.tud.kom.p2psim.impl.overlay.gia.Analyzer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.analyzer.Analyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.TransAnalyzer;
import de.tud.kom.p2psim.impl.overlay.gia.GIABootstrapManager;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryHitMessage;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella.messages.QueryHitMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.AbstractTransMessage;

public class GIAQueryAnalyzer implements Analyzer, TransAnalyzer {
	
	private Map<BigInteger, Long> queryHitsWithTime = new HashMap<BigInteger, Long>();
	private Map<BigInteger, Long> queriesWithTime = new HashMap<BigInteger, Long>();
	private Map<BigInteger, Long> queryHitsWithDelay = new HashMap<BigInteger, Long>();
	private Map<BigInteger, List<Long>> queryHitsWithListOfTimes = new HashMap<BigInteger, List<Long>>();
	private Map<Integer, List<BigInteger>> queriesTimeSteps = new HashMap<Integer, List<BigInteger>>();
	private Map<Integer, List<BigInteger>> queryHitsTimeSteps = new HashMap<Integer, List<BigInteger>>();
	private Map<Integer, List<BigInteger>> queriesSuccededWithTimeSteps = new HashMap<Integer, List<BigInteger>>();
	private Map<Integer, List<Integer>> aktiveNodes = new HashMap<Integer, List<Integer>>();
	private Map<BigInteger, List<Integer>> queryHitCount = new HashMap<BigInteger, List<Integer>>();
	private Map<BigInteger, Double> queryHitsWithHops = new HashMap<BigInteger, Double>();
	private Map<BigInteger, Integer> queryHopCount = new HashMap<BigInteger, Integer>();
	private Map<Integer, List<Double>> hopCountWithTimeSteps = new HashMap<Integer, List<Double>>();
	private Map<Integer, Integer> aktiveNodesWithTimeSteps = new HashMap<Integer, Integer>();
	private List<BigInteger> queryHits = new LinkedList<BigInteger>();
	private List<BigInteger> queries = new LinkedList<BigInteger>();
	private List<Integer> timeStepsQuery = new LinkedList<Integer>();
	private List<Integer> timeStepsQueryHits = new LinkedList<Integer>();
	private Map<BigInteger, Long> queriesFailedWithTime = new HashMap<BigInteger, Long>();
	private List<BigInteger> queriesFailed = new LinkedList<BigInteger>();
	private List<Integer> timeSteps = new LinkedList<Integer>();
	private int timeStepsQH;
	private int timeStepsQ;
	private int timeStepsQHF;
	private int totalCounterQueriesFailed;
	private int totalCounterQueryHits;
	private int anzahlAktiveNodes;



	public void start() {
		// TODO Auto-generated method stub
		
	}

	public void stop(Writer output) {
		NumberFormat n = NumberFormat.getInstance();
		n.setMaximumFractionDigits(1);
		long queryDelay = 0;
		for(int i = 0; i<queryHits.size(); i++){
			BigInteger descriptor = queryHits.get(i);
			long time = queryHitsWithTime.get(descriptor);
			timeStepsQH = (int)(time /10000000);
			if (!queryHitsTimeSteps.containsKey(timeStepsQH)){
				queryHitsTimeSteps.put(timeStepsQH, new LinkedList<BigInteger>());
			}
			queryHitsTimeSteps.get(timeStepsQH).add(descriptor);
			if(!timeStepsQueryHits.contains(timeStepsQH)){
				timeStepsQueryHits.add(timeStepsQH);
			}
		}
		for(int i = 0; i<queries.size(); i++){
			BigInteger descriptor = queries.get(i);
			long time = queriesWithTime.get(descriptor);
			timeStepsQ = (int)(time /10000000);
			if (!queriesTimeSteps.containsKey(timeStepsQ)){
				queriesTimeSteps.put(timeStepsQ, new LinkedList<BigInteger>());
			}
			queriesTimeSteps.get(timeStepsQ).add(descriptor);
			if(!timeStepsQuery.contains(timeStepsQ)){
				timeStepsQuery.add(timeStepsQ);
			}
		}
		
		// Hops berechnen, die beim QueryHit gebraucht werden
		int hopCountTimeSteps =0;
		for (int i=0; i<queryHits.size(); i++){
			double hops = 0;
			double hopsAveraged = 0;
			
			BigInteger descriptor = queryHits.get(i);
			long time = queriesFailedWithTime.get(descriptor);
			timeStepsQHF = (int) (time/10000000);
			if(queriesFailedWithTime.containsKey(descriptor)){
//				queriesFailedWithTime.get(descriptor);
				queriesFailedWithTime.remove(descriptor);
			}
			if(queriesFailed.contains(descriptor)){
				queriesFailed.remove(descriptor);
							}
			if (!queriesSuccededWithTimeSteps.containsKey(timeStepsQHF)){
				queriesSuccededWithTimeSteps.put(timeStepsQHF, new LinkedList<BigInteger>());
			}
			queriesSuccededWithTimeSteps.get(timeStepsQHF).add(descriptor);
//			hops = queryHitCount.get(descriptor);
/**			if (hopCountWithTimeSteps.containsKey(timeStepsQHF)){
				hopCountTimeSteps = hopCountWithTimeSteps.get(timeStepsQHF);
			}
			hopCountTimeSteps += hops;
			hopCountWithTimeSteps.put(timeStepsQHF, hopCountTimeSteps);
**/
			List<Integer> hopsList = new LinkedList<Integer>();
			double hopsPerMsgIdAveraged = 0;
			if (queryHitCount.containsKey(descriptor)){
				hopsList = queryHitCount.get(descriptor);
			}
			for (int j=0; j < hopsList.size(); j++){
				hops += (double) hopsList.get(j) ;
				
//				if (hopCountWithTimeSteps.containsKey(timeStepsQHF)){
//					hopCountTimeSteps = hopCountWithTimeSteps.get(timeStepsQHF);
//				}
//				hopCountTimeSteps += hops;
//				hopCountWithTimeSteps.put(timeStepsQHF, hopCountTimeSteps);
			}
			hopsAveraged = hops/ (double) hopsList.size();
			queryHitsWithHops.put(descriptor, hopsAveraged);
//			hopCountTimeSteps += hopsAveraged;
			if (!hopCountWithTimeSteps.containsKey(timeStepsQHF)){
				hopCountWithTimeSteps.put(timeStepsQHF, new LinkedList<Double>());
			}
			hopCountWithTimeSteps.get(timeStepsQHF).add(hopsAveraged);
			
			// delay der QueryHits berechnen
			int counter = 0;
			long delayTotal = 0;
			List<Long> timeList = queryHitsWithListOfTimes.get(descriptor);
			List<Long> delayList = new LinkedList<Long>();
			long querySentTime = queriesWithTime.get(descriptor);
			for (int k=0; k< timeList.size(); k++){
				long tempTime = timeList.get(k);
				long delay = tempTime - querySentTime;
				delayList.add(delay);
				delayTotal += delay;
				counter++;
			}
			queryHitsWithListOfTimes.put(descriptor, delayList);
			queryHitsWithDelay.put(descriptor, delayTotal/counter);
		}
		
		// Aktive Knoten berechnen
		for (int q=0; q<timeSteps.size(); q++){
			int nodes = 0;
			int nodesAveraged;
			int timeStep = timeSteps.get(q);
			List<Integer> tempo = aktiveNodes.get(timeStep);
			anzahlAktiveNodes = tempo.size();
			for (int w=0; w<tempo.size(); w++){
				nodes += tempo.get(w);
			}
			nodesAveraged = nodes / tempo.size();
			aktiveNodesWithTimeSteps.put(timeStep, nodesAveraged);
		}
		

		String f = "Analyzerausgabe3.dat";
	    FileWriter fstream = null;
		try {
			fstream = new FileWriter(f);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write("#	GIA-Analyzer	\n");
			out.write("#	-----------------------	\n");
			out.write("#	\n");
			out.write("#	\n");
			out.write("#Ausgabe der Queries und QueryHits nach zeitR�umen	\n");
			out.write("#	\n");
			out.write("#Zeit[s]		queries		Erfolgreich		Failed		Hops	Delay	Aktive Knoten			Erfolgreich in %	\n");
			for (int i =0; i< timeStepsQuery.size(); i++){
				BigInteger descriptor;
				long queryDelayTotal = 0;
				int secCounter = 0;
				List<BigInteger> descriptorList = new LinkedList<BigInteger>();
				int temp = timeStepsQuery.get(i);
				int queriesTotal = queriesTimeSteps.get(temp).size();
				int queriesSucceded = 0;
				if(queriesSuccededWithTimeSteps.containsKey(temp)){
					queriesSucceded = queriesSuccededWithTimeSteps.get(temp).size();
					descriptorList = queriesSuccededWithTimeSteps.get(temp);
					for (int d= 0; d< descriptorList.size(); d++){
						descriptor = descriptorList.get(d);
						queryDelayTotal += queryHitsWithDelay.get(descriptor);
						secCounter++;
					}
					if(secCounter != 0){
						queryDelayTotal = queryDelayTotal/secCounter;
					}
				}
				int queriesFailed = queriesTotal - queriesSucceded;
				double queryHopsAveraged = 0;
				if(hopCountWithTimeSteps.containsKey(temp)){
					for (int a = 0; a < hopCountWithTimeSteps.get(temp).size();a++){
						queryHopsAveraged += hopCountWithTimeSteps.get(temp).get(a);
					}
					if (queriesSucceded != 0){
						queryHopsAveraged= queryHopsAveraged/queriesSucceded;
					}	
				} else {
					queryHopsAveraged = 0;
				}
			
				totalCounterQueryHits += queriesSucceded;
				totalCounterQueriesFailed += queriesFailed;
				double queriesPerCent = (double)(queriesSucceded * 100)/ (double) queriesTotal ;
				if(queriesTimeSteps.containsKey(temp)){
					int zeit = temp*10 + 10;
					out.write(temp*10 + "	-	" + zeit + "			" + queriesTotal + "			" + queriesSucceded + "			" + queriesFailed + "		" + n.format(queryHopsAveraged) + "			" + queryDelayTotal + "		"+ aktiveNodesWithTimeSteps.get(temp) +  "				" + n.format(queriesPerCent) + "	\n");
				}
			}
			out.write("	\n");
			out.write("Total: 						" + totalCounterQueryHits + "			" + totalCounterQueriesFailed + "	\n");
			
			out.write("	\n");
			out.write("Zeit[s]	Anzahl queryHits	\n");
			int queryHitsTotal = 0;
			for (int i =0; i< timeStepsQueryHits.size(); i++){
				int temp = timeStepsQuery.get(i);
				if(queryHitsTimeSteps.containsKey(temp)){
					int zeit = temp*10 + 10;
					queryHitsTotal += queryHitsTimeSteps.get(temp).size();
					out.write(temp*10 + "	-	" + zeit + "				" + queryHitsTimeSteps.get(temp).size() + "	\n");
				}
			}
			out.write("	\n");
			out.write("Total:				" + queryHitsTotal + " \n");
			out.close();         
		} catch (IOException e) {
			e.printStackTrace();
		}                
	}

	public void transMsgSent(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		
		if (message instanceof GIAQueryMessage) {
			GIAQueryMessage queryMessage = (GIAQueryMessage) message.getPayload();
			BigInteger descriptor = queryMessage.getDescriptor();
			long time = Simulator.getInstance().getCurrentTime();
			int timeStep = (int) (time / 10000000);
			if(!queries.contains(descriptor)){
				queries.add(descriptor);
			}
			if(!queriesWithTime.containsKey(descriptor)){
				queriesWithTime.put(descriptor, time);
			}
			if(!queriesFailedWithTime.containsKey(descriptor)){
				queriesFailedWithTime.put(descriptor, time);
			}
			if(!queriesFailed.contains(descriptor)){
				queriesFailed.add(descriptor);
			}
			int hops = queryMessage.getHops();

			if(!queryHopCount.containsKey(descriptor)){
				queryHopCount.put(descriptor, hops);
			} else {
				int oldHops = queryHopCount.get(descriptor);
				if(oldHops < hops){
					queryHopCount.put(descriptor, hops);
				}
			}
			
			if(!aktiveNodes.containsKey(timeStep)){
				aktiveNodes.put(timeStep, new LinkedList<Integer>());
			}
			List<Integer> temporaer = aktiveNodes.get(timeStep);
			temporaer.add(GIABootstrapManager.getInstance().getJoinableSize());
			aktiveNodes.put(timeStep, temporaer);
			if(!timeSteps.contains(timeStep)){
				timeSteps.add(timeStep);
			}
			
		} else if (message instanceof GIAQueryHitMessage) {
			int queryHops = 0;
			GIAQueryHitMessage queryHitMessage = (GIAQueryHitMessage) message.getPayload();
			BigInteger descriptor = queryHitMessage.getDescriptor();
			long time = Simulator.getInstance().getCurrentTime();
			int timeStep = (int) (time / 10000000);
			if(!queryHits.contains(descriptor)){
				queryHits.add(descriptor);
			}
			if(!queryHitsWithTime.containsKey(descriptor)){
				queryHitsWithTime.put(descriptor, time);
			}
			queryHops = queryHitMessage.getHops();
			if(!queryHitCount.containsKey(descriptor)){
				queryHitCount.put(descriptor, new LinkedList<Integer>());
			}
			queryHitCount.get(descriptor).add(queryHops);
			if(!timeSteps.contains(timeStep)){
				timeSteps.add(timeStep);
			}
			if (!queryHitsWithListOfTimes.containsKey(descriptor)){
				queryHitsWithListOfTimes.put(descriptor, new LinkedList<Long>());
			}
			queryHitsWithListOfTimes.get(descriptor).add(time);
		}
		
	}

	public void transMsgReceived(AbstractTransMessage msg) {
		// TODO Auto-generated method stub
		
	}
	
	

}
