package de.tud.kom.p2psim.impl.overlay.gia;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.Component;
import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.stat.distributions.zipf.ZipfConfigurer;
import de.tud.kom.p2psim.impl.util.stat.distributions.zipf.ZipfDistribution;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class FileFactory implements ComponentFactory{

	private static FileFactory singletonInstance;
	private List<Integer> files;
	private ZipfDistribution distribution = new ZipfDistribution();
	private ZipfConfigurer config = new ZipfConfigurer(100, 1.0);
	
	/**
	 * generate new file factory with new list of files
	 */
	public FileFactory() {
		this.files = new LinkedList<Integer>();
		distribution.config(config);
	}
	
	/**
	 * @return the unique file factory, if none exists make new one
	 */
	public static FileFactory getInstance() {
		if (singletonInstance == null) {
			singletonInstance = new FileFactory();
		}
		return singletonInstance;
	}
	
	
	public Component createComponent(Host host) {
		return null;
	}
	
	/**
	 * @return new file
	 */
	public Integer createNewFileSharingKey(){
		int newFile = distribution.returnRank();
		//FileSharingKey newFile = new FileSharingKey(Simulator.getRandom().nextInt(1000));
		if(!files.contains(newFile)){
			files.add(newFile);
		}
		return newFile;
	}
	
	public List<Integer> createDocuments(int numberOfDocuments){
		List<Integer> documents = new LinkedList<Integer>();
		List<Integer> control = new LinkedList<Integer>();
		while(numberOfDocuments != 0){
			int file = Simulator.getRandom().nextInt(1000);
			if(!documents.contains(file)){
				documents.add(file);
				numberOfDocuments--;
				System.out.println("File " +file);
			}
			if(!files.contains(file)){
				files.add(file);
			}
//			int rank = distribution.returnRank();
//			if(!documents.contains(rank)){
//				control.add(rank);
//				FileSharingKey file = new FileSharingKey(rank);
//				documents.add(rank);
//				numberOfDocuments--;
//				System.out.println("File " +file+ " " +rank);
//			}
			
			//GIAOverlayNodeFactory.log.debug("file" +file);
		}
		return documents;
	}
	
	/**
	 * @return number of different files in the network
	 */
	public int getNumberOfFiles(){
		return files.size();
	}
	
	/**
	 * @param placeOfFile index of the file searched for
	 * @return key of the file
	 */
	public Integer getFileKey(int placeOfFile){
		return files.get(placeOfFile);
	}
	
	public List<Integer> getFileSharingKeys(){
		return files;
	}

	public ZipfDistribution getZipf(){
		return distribution;
	}

}
