package de.tud.kom.p2psim.impl.overlay.gia;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.api.transport.TransInfo;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIABootstrapManager implements BootstrapManager{
	
	private List<OverlayNode> activeNodes;
	private List<OverlayNode> joinableNodes;
	private Map<Integer, LinkedList<OverlayNode>> sortedByCapacities;
	private static GIABootstrapManager singletonInstance;
	
	/**
	 * create new bootstrap manager
	 */
	public GIABootstrapManager(){
		this.activeNodes = new LinkedList<OverlayNode>();
		this.joinableNodes = new LinkedList<OverlayNode>();
	}
	
	public List<TransInfo> getBootstrapInfo(){
		List<TransInfo> list = new LinkedList<TransInfo>();
		
		for(OverlayNode cNode : this.activeNodes)
			list.add(((GIAOverlayNode)cNode).getTransLayer().getLocalTransInfo(cNode.getPort()));
		
		return list;
	}
	
	public List<TransInfo> getJoinableBootstrapInfo(){
		List<TransInfo> list = new LinkedList<TransInfo>();
		
		for(OverlayNode cNode : this.joinableNodes)
			list.add(((GIAOverlayNode)cNode).getTransLayer().getLocalTransInfo(cNode.getPort()));
		
		return list;
	}

	/**
	 * @return unique bootstrap manager
	 */
	public static GIABootstrapManager getInstance() {
		if(singletonInstance == null) {
			singletonInstance = new GIABootstrapManager();
		}
		return singletonInstance;
	}
	
	public void registerNode(OverlayNode node){
		if (!activeNodes.contains(node)){
			this.activeNodes.add(node);
		}
	}
	
	public void unregisterNode(OverlayNode node){
		this.activeNodes.remove(node);
	}
	
	/**
	 * @return number of nodes in the network
	 */
	public int getSize(){
		return activeNodes.size();
	}
	public int getJoinableSize(){
		return joinableNodes.size();
	}

	public List<OverlayNode> getNodes(){
		return activeNodes;
	}
	
	public void registerJoinableNodes(OverlayNode node){
		if (!joinableNodes.contains(node)){
			this.joinableNodes.add(node);
		}
	}
	
	public void unregisterJoinableNodes(OverlayNode node){
		if (joinableNodes.contains(node)){
			this.joinableNodes.remove(node);
		}
	}
	
}
