package de.tud.kom.p2psim.impl.overlay.gia;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.api.transport.TransInfo;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAOverlayContact implements OverlayContact<GIAOverlayID>, Comparable<GIAOverlayContact>{
	
	private GIAOverlayID giaID;
	private TransInfo transInfo;
	private boolean isAlive;
	private int capacity;
	
	private long timeActivated;
	private long lastRefresh;
	
	/**
	 * @param id overlay ID of the contact
	 * @param transInfo transport info of the contact
	 * @param capacity capacity of the contact
	 */
	public GIAOverlayContact(GIAOverlayID id, TransInfo transInfo, int capacity){
		this.giaID = id;
		this.transInfo = transInfo;
		this.capacity = capacity;
	}
	
	/**
	 * @param contact contact data of the contact
	 * @param capacity capacity of the contact
	 */
//	public GIAOverlayContact(OverlayContact<GIAOverlayID> contact, int capacity) {
//		this.giaID = contact.getOverlayID();
//		this.transInfo = contact.getTransInfo();
//		this.capacity = ;
//	}
	
	public GIAOverlayID getOverlayID(){
		return this.giaID;
	}
	
	public TransInfo getTransInfo(){
		return this.transInfo;
	}
	
	/**
	 * @param alive true if contact is in the network, false otherwise
	 */
	public void setAlive(boolean alive){
		this.isAlive = alive;
	}
	
	/**
	 * @return true if contact is in the network, false otherwise
	 */
	public boolean isAlive(){
		return this.isAlive;
	}
	
	public GIAOverlayContact clone(){
		GIAOverlayContact newCon = new GIAOverlayContact(this.giaID, this.transInfo, this.capacity);
		newCon.setAlive(this.isAlive);
		return newCon;
	}
	
	/**
	 * @param con contact info
	 * @return distance between con and this contact
	 */
	public int getDistance(GIAOverlayContact con){
		return this.giaID.getDistance(con.getOverlayID());
	}
	
	public int compareTo(GIAOverlayContact o){
		return this.giaID.compareTo(o.getOverlayID());
	}
	
	/**
	 * @param o overlay ID
	 * @return true if o equals overlay ID of the contact
	 */
	public boolean equals(GIAOverlayContact o){
		return this.giaID.equals(o.getOverlayID());
	}
	
	/**
	 * sets the time this contact was activated to the current time
	 */
	public void reset() {
		this.timeActivated = Simulator.getCurrentTime();
	}
	
	/**
	 * @return rank
	 */
	public double getRank() {
		return Double.MAX_VALUE / (Simulator.getCurrentTime() - this.timeActivated);
	}

	/**
	 * @return the time this contact was refreshed last
	 */
	public long getLastRefresh() {
		return lastRefresh;
	}

	/**
	 *  sets the time this contact was refreshed last
	 */
	public void refresh() {
		this.lastRefresh = Simulator.getCurrentTime();
	}
	
	/**
	 * @return capacity of the contact
	 */
	public int getCapacity(){
		return capacity;
	}
}
