package de.tud.kom.p2psim.impl.overlay.gia;

import java.math.BigInteger;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.math.distribution.ExponentialDistributionImpl;
import org.apache.log4j.*;


import de.tud.kom.p2psim.impl.util.logging.SimLogger;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayRoutingTable;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.*;
import de.tud.kom.p2psim.impl.overlay.gia.Operations.*;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.api.transport.TransMessageListener;
import de.tud.kom.p2psim.api.transport.TransMsgEvent;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.api.common.ConnectivityEvent;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAOverlayNode extends AbstractOverlayNode implements TransMessageListener{
	
	protected final static Logger log = SimLogger.getLogger(GIAOverlayNode.class);
	
	public ExponentialDistributionImpl expDist;

	private int maxContacts = GIAConfiguration.GIA_MAXCONTACTS;
	private int minContacts = GIAConfiguration.GIA_MINCONTACTS;
	private GIABootstrapManager bootstrapManager;
	
	private int capacity;	//number of messages a node can handle
	private int userType;
	private double connectivityFactor;
	private long sessionTime;
	private float satisfaction;
	private GIAOverlayRoutingTable routingTable; 
	private TransLayer transLayer;
	private boolean active;
	private GIAOverlayContact ownContactData;
	private boolean biasedRandomWalk;
	private boolean topologyAdaption;
	private boolean oneHopReplication;
	private boolean flowControl;
	
	private List<Integer> documents = new LinkedList<Integer>();
	private List<BigInteger> pingInitiated = new LinkedList<BigInteger>();
	private List<BigInteger> queryInitiated = new LinkedList<BigInteger>();
	// queries initiated by application layer (Descriptor, FilesharingKey)
	private Map<BigInteger, Integer> queryInitiatedWithKey = new HashMap<BigInteger, Integer>();
	// results, ready to get for application layer (Descriptor, Contacts)
	private Map<BigInteger, List<GIAOverlayContact>> queryResults = new HashMap<BigInteger, List<GIAOverlayContact>>();
	private Map<BigInteger, Long> queryTimeout = new HashMap<BigInteger, Long>();
	// List of Token a peers got, stored with ID of sender
	private LinkedList<TransInfo> bootstrapList;
	private LinkedList<TransInfo> joinableList;
	private Map<GIAOverlayID , List<Integer>> documentsOfNeighbors = new HashMap<GIAOverlayID, List<Integer>>();
	private Map<GIAOverlayID, Long> tokensForNeighbors = new HashMap<GIAOverlayID, Long>();
	private Map<GIAOverlayID, Long> tokensFromNeighbors = new HashMap<GIAOverlayID, Long>();
	private Map<GIAOverlayID, Long> lastQuerySent = new HashMap<GIAOverlayID, Long>();
	private Map<GIAOverlayID, Long> lastQueryReceived = new HashMap<GIAOverlayID, Long>();
	private Map<GIAOverlayID, Long> lastMessageReceived = new HashMap<GIAOverlayID, Long>();
	private List<GIAOverlayID> connectToDenied = new LinkedList<GIAOverlayID>();
	
	
	/**
	 * @param transLayer the transport layer of the node
	 * @param peerID the overlay ID of the node
	 * @param port the port of the node
	 * @param capacity the number of queries this node can handle per second
	 * @param documents list of documents stored on the node
	 * @param biasedRandomWalk to switch biased random walk on or off
	 * @param topologyAdaption to switch topology adaption on or off
	 * @param oneHopReplication to switch one hop replication on or off
	 * @param flowControl to switch flow control on or off
	 */
	public GIAOverlayNode(TransLayer transLayer, OverlayID peerID, short port, int capacity, int userType, List<Integer> documents, GIABootstrapManager bootstrapManager, boolean biasedRandomWalk, boolean topologyAdaption, boolean oneHopReplication, boolean flowControl){
		super(peerID, port);
		this.documents = documents;
		this.capacity = capacity;
		this.transLayer = transLayer;
		routingTable = new GIAOverlayRoutingTable(this.getOverlayID());
		transLayer.addTransMsgListener(this, this.getPort());
		active = true;
		ownContactData = new GIAOverlayContact((GIAOverlayID)this.getOverlayID(), this.transLayer.getLocalTransInfo(this.getPort()), this.capacity);
		this.biasedRandomWalk = biasedRandomWalk;
		this.topologyAdaption = topologyAdaption;
		this.oneHopReplication = oneHopReplication;
		this.flowControl = flowControl;
		this.userType = userType;
		this.bootstrapManager = bootstrapManager;
		
		if(topologyAdaption){	
			if (capacity == 1000){
				this.maxContacts = this.maxContacts/2;
			}
			else if(capacity == 100){
				this.maxContacts = this.maxContacts/4; 
			}
			else if(capacity == 10){
				this.maxContacts = this.maxContacts/8; 
			} 
			else if(capacity == 1){
				this.maxContacts = this.maxContacts/12; 
			}
		}	
		else maxContacts = maxContacts/4;
		
		/*if(userType == 0){
			connectivityFactor= 0.75;
			sessionTime = 100 * GIAConfiguration.GIA_MEANSESSIONTIME;
		}
		else if(userType ==1){
			connectivityFactor= 0.5;
			sessionTime = 10 * GIAConfiguration.GIA_MEANSESSIONTIME;
		}
		else{
			connectivityFactor = (1 - GIAConfiguration.GIA_CHURNFACTOR - 0.275) / 0.5;
			sessionTime = GIAConfiguration.GIA_MEANSESSIONTIME;
		}*/
		//expDist = new ExponentialDistributionImpl(sessionTime);
	}
	
	@Override
	public TransLayer getTransLayer(){
		return transLayer;
	}
	
	public void connectivityChanged(ConnectivityEvent ce){
		//
	}
	
	public void messageArrived(TransMsgEvent receivingEvent) {
		Message message = receivingEvent.getPayload();
//		GIAOverlayNode.log.debug("Message Arrived");
		
		if(active){
			
			if (message instanceof GIAConnectMessage) {
//			GIAOverlayNode.log.debug("prozessConnect");
				processConnect(receivingEvent);
			} 
			else if (message instanceof GIAConnectAcceptMessage) {
//	 			GIAOverlayNode.log.debug("prozessConnectAccept");
				processConnectAccept(receivingEvent);
			}
			else if (message instanceof GIAPingMessage) {
//				GIAOverlayNode.log.debug("prozessPing");
				processPing(receivingEvent);
			} 
			else if (message instanceof GIAPongMessage) {
//				GIAOverlayNode.log.debug("prozessPong");
				processPong(receivingEvent);
			} 
			else if (message instanceof GIAQueryMessage) {
//				GIAOverlayNode.log.debug("processQuery");
				processQuery(receivingEvent);
			} 
			else if (message instanceof GIAQueryHitMessage) {
//				GIAOverlayNode.log.debug("processQueryHit");
				processQueryHit(receivingEvent);
			}
			else if (message instanceof GIAQueryIDMessage) {
//				GIAOverlayNode.log.debug("processQueryID");
				processQueryID(receivingEvent);
			}
			else if (message instanceof GIAQueryIDHitMessage) {
//				GIAOverlayNode.log.debug("processQuery");
				processQueryIDHit(receivingEvent);
			}
			else if (message instanceof GIATokenMessage) {
//				GIAOverlayNode.log.debug("prozessToken");
				processToken(receivingEvent);
			}
			else if (message instanceof GIAByeMessage) {
				processBye(receivingEvent);
			}
			else if (message instanceof GIAConnectionBreakMessage) {
				processConnectionBreak(receivingEvent);
			}
			else if (message instanceof GIAConnectionDeniedMessage) {
				processConnectionDenied(receivingEvent);
			}
			else if (message instanceof GIADocumentsMessage) {
//				GIAOverlayNode.log.debug("processDocuments");
				processDocumentsMessage(receivingEvent);
			}
		}

	}
	
	private void processConnectionDenied(TransMsgEvent receivingEvent) {
		GIAConnectionDeniedMessage connDeniedMessage = (GIAConnectionDeniedMessage) receivingEvent.getPayload();
//		GIAOverlayNode.log.debug(connDeniedMessage.getSender() + " hat Connection denied mit : " + this.getOverlayID());
		if (!connectToDenied.contains(connDeniedMessage.getSender())){
			connectToDenied.add(connDeniedMessage.getSender());
		}
		if(routingTable.inactiveContacts().size() == 0 && routingTable.allContacts().size() == 0){
			bootstrapManager.unregisterNode(this);
			bootstrapManager.unregisterJoinableNodes(this);
			this.join();
		}
		else{
			this.connect();
		}
	}

	private void processConnectionBreak(TransMsgEvent receivingEvent) {
		//put peer from active neighbors to inactive neighbors
		GIAConnectionBreakMessage connBreakMessage = (GIAConnectionBreakMessage) receivingEvent.getPayload();
		routingTable.removeContact(connBreakMessage.getSender());
		documentsOfNeighbors.remove(connBreakMessage.getSender());
		//lastMessageReceived.remove(connBreakMessage.getSender());
		routingTable.addInactiveContact(connBreakMessage.getContact());
		GIAOverlayNode.log.debug(connBreakMessage.getSender() + " hat Connection abgebrochen mit : " + this.getOverlayID());
		this.sendToken();
		if(routingTable.allContacts().size() == 0){
			bootstrapManager.unregisterNode(this);
			bootstrapManager.unregisterJoinableNodes(this);
			this.join();
		}
		else this.connect();
	}

	private void processBye(TransMsgEvent receivingEvent) {
		//take peer from the list of active neighbors 
		GIAByeMessage byeMessage = (GIAByeMessage) receivingEvent.getPayload();
		routingTable.removeContact(byeMessage.getSender());
		documentsOfNeighbors.remove(byeMessage.getSender());
		//lastMessageReceived.remove(byeMessage.getSender());
		routingTable.removeInactiveContact(routingTable.getContact(byeMessage.getSender()));
		routingTable.removeNumberOfNeighbors(byeMessage.getSender());
//		GIAOverlayNode.log.debug(this.getOverlayID() + " hat von: "+ byeMessage.getSender() + " eine bye-Message erhalten");
		this.sendToken();
		if(routingTable.allContacts().size() == 0){
			bootstrapManager.unregisterNode(this);
			bootstrapManager.unregisterJoinableNodes(this);
			this.join();
		}
		else this.connect();
	}

	private void processToken(TransMsgEvent receivingEvent) {
		//put rate at witch we can send to neighbor into token list
		GIATokenMessage tokenMessage = (GIATokenMessage) receivingEvent.getPayload();
		
//		System.out.println(this.getOverlayID() + " hat Token erhalten von: " + tokenMessage.getSender());
		if(tokensFromNeighbors.containsKey(tokenMessage.getSender())){
			tokensFromNeighbors.remove(tokenMessage.getSender());
		}	
		tokensFromNeighbors.put(tokenMessage.getSender(), tokenMessage.getTokenRate());
		lastMessageReceived.put(tokenMessage.getSender(), Simulator.getCurrentTime());
			
	}

	private void processConnect(TransMsgEvent receivingEvent) {
		
		GIAConnectMessage connectMessage = (GIAConnectMessage) receivingEvent.getPayload();
		
		int size = routingTable.numberOfActiveContacts();
//		GIAOverlayNode.log.debug("Aktive Neighbors " +size);
		//if the peer has not enough neighbors => always accept connection
		if(size < maxContacts) {
			routingTable.addContact(connectMessage.getContact());
			routingTable.addContactWithCapacity(connectMessage.getContact().getCapacity(), connectMessage.getContact());
			routingTable.setNumberOfNeighbors(connectMessage.getSender(), connectMessage.getNumberOfNeighbors()+1);
			lastMessageReceived.put(connectMessage.getSender(), Simulator.getCurrentTime());
			if(routingTable.inactiveContacts().contains(connectMessage.getContact())){
				routingTable.removeInactiveContact(connectMessage.getContact());
			}
			GIAConnectAcceptMessage connectAcceptMessage = new GIAConnectAcceptMessage((GIAOverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), ownContactData, routingTable.numberOfActiveContacts());
//			GIAOverlayNode.log.debug(connectMessage.getContact().getOverlayID() + " hat ConnectTo aufgerufen mit: " + this.getOverlayID() );
			this.getTransLayer().send(connectAcceptMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			//send the new neighbor the list of documents one has
			if(oneHopReplication){
				GIADocumentsMessage documentsMessage = new GIADocumentsMessage((GIAOverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), documents);
				this.getTransLayer().send(documentsMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			}
			//recalculate the token rate and send it to all neighbors
			this.sendToken();
		}
		//if the peer already has enough neighbors => deny connection or discard old neighbor?
		else if(topologyAdaption){
			int neighbors = 0;
			GIAOverlayContact peerWithMostNeighbors = null;
			int capacityOfPeer = 0;
			int highestCapacity = 0;
			
			//of all neighbors with lower capacity then the new contact get the one with the most neighbors
			for(Integer capacity : routingTable.getSortedCapacities()){
				if(capacity > highestCapacity){
					highestCapacity = capacity;
				}
				if(capacity <= connectMessage.getContact().getCapacity()){
					for(GIAOverlayContact giaContact : routingTable.getNodesWithCapacity(capacity)){
						if(routingTable.getNumberOfNeighbors(giaContact.getOverlayID()) > neighbors){
							neighbors = routingTable.getNumberOfNeighbors(giaContact.getOverlayID());
							peerWithMostNeighbors = giaContact;
							capacityOfPeer = capacity;
						}
					}
				}
			}
			//if no neighbor with lower capacity exists => deny connection
			if(peerWithMostNeighbors == null){
				GIAConnectionDeniedMessage connectionDenied = new GIAConnectionDeniedMessage((GIAOverlayID)this.getOverlayID(), connectMessage.getContact().getOverlayID(), "not good enough");
				this.getTransLayer().send(connectionDenied, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			
			}
			//if the new contact has a higher capacity then any other neighbor or if the neighbor with most neighbors, has more neighbors then the new contact, accept connection and drop neighbor with most neighbors
			else if((connectMessage.getContact().getCapacity() > highestCapacity) || (neighbors > connectMessage.getNumberOfNeighbors() + GIAConfiguration.GIA_HYSTERESISFACTOR)){
				
				GIAConnectionBreakMessage connectionBreak = new GIAConnectionBreakMessage((GIAOverlayID)this.getOverlayID(), peerWithMostNeighbors.getOverlayID(), ownContactData, "Better Peer found");
				this.getTransLayer().send(connectionBreak, peerWithMostNeighbors.getTransInfo(), this.getPort(), TransProtocol.UDP);
			
				routingTable.removeContact(peerWithMostNeighbors.getOverlayID());
				routingTable.addInactiveContact(peerWithMostNeighbors);
				routingTable.removeContactWithCapacity(capacityOfPeer, peerWithMostNeighbors);
				documentsOfNeighbors.remove(peerWithMostNeighbors.getOverlayID());
				lastMessageReceived.remove(peerWithMostNeighbors.getOverlayID());
				
				routingTable.addContact(connectMessage.getContact());
				routingTable.addContactWithCapacity(connectMessage.getContact().getCapacity(), connectMessage.getContact());
				routingTable.setNumberOfNeighbors(connectMessage.getSender(), connectMessage.getNumberOfNeighbors()+1);
				lastMessageReceived.put(connectMessage.getSender(), Simulator.getCurrentTime());
				if(routingTable.inactiveContacts().contains(connectMessage.getContact())){
					routingTable.inactiveContacts().remove(connectMessage.getContact());
				}
				
				GIAConnectAcceptMessage connectAcceptMessage = new GIAConnectAcceptMessage((GIAOverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), ownContactData, routingTable.numberOfActiveContacts());
				this.getTransLayer().send(connectAcceptMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
				if(oneHopReplication){	
					GIADocumentsMessage documentsMessage = new GIADocumentsMessage((GIAOverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), documents);
					this.getTransLayer().send(documentsMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
				}
				this.sendToken();
			}
			// if none of these apply => deny connection 
			else {
				GIAConnectionDeniedMessage connectionDenied = new GIAConnectionDeniedMessage((GIAOverlayID)this.getOverlayID(), connectMessage.getContact().getOverlayID(), "not good enough");
				this.getTransLayer().send(connectionDenied, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			
			}
		}
	}
	
	private void processConnectAccept(TransMsgEvent receivingEvent) {
		GIAConnectAcceptMessage message = (GIAConnectAcceptMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
//		GIAOverlayNode.log.debug(message.getSender() + " hat ConnectAcceptiert von " + this.getOverlayID());
		routingTable.addContact(message.getContact());
		routingTable.addContactWithCapacity(message.getContact().getCapacity(), message.getContact());
		routingTable.setNumberOfNeighbors(message.getSender(), message.getNumberOfNeighbors()+1);
		if(routingTable.inactiveContacts().contains(message.getContact())){
			routingTable.inactiveContacts().remove(message.getContact());
		}
		
		bootstrapManager.registerJoinableNodes(this);
		if(oneHopReplication){
			GIADocumentsMessage documentsMessage = new GIADocumentsMessage((GIAOverlayID) this.getOverlayID(), message.getContact().getOverlayID(), documents);
			this.getTransLayer().send(documentsMessage, message.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
		}	
		this.sendToken();
	}
	
	private void processPing(TransMsgEvent receivingEvent) {
		GIAPingMessage message = (GIAPingMessage) receivingEvent.getPayload();
		//GIAOverlayNode.log.debug(this.getOverlayID() + " hat Ping erhalten von: " + message.getSender());
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
		if (routingTable.incomingPing(message.getSender(), message.getDescriptor())
				&& !this.pingInitiated.contains(message.getDescriptor())) {
			// route ping
			if (message.getTTL() - 1 > 0) {
				for(GIAOverlayContact receiver : routingTable.allContacts()){
					if(receiver.getOverlayID() != message.getSender()){
						GIAPingOperation pingOperation = new GIAPingOperation(this, receiver, message.getTTL() - 1, message.getHops() + 1, message.getDescriptor(), new OperationCallback<Object>() {
							public void calledOperationFailed(Operation<Object> op) {
								//
							}
							public void calledOperationSucceeded(Operation<Object> op) {
								// 
							}
						});
						pingOperation.scheduleImmediately();
					}
				}
			}
			// reply with pong
			GIAPongOperation pongOperation = new GIAPongOperation(this, message.getDescriptor(), this.getNodesRoutingTable().numberOfActiveContacts(), new OperationCallback<Object>() {
				public void calledOperationFailed(Operation<Object> op) {
					// 
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					// 
				}
			});
			pongOperation.scheduleImmediately();
		}	
	}

	private void processPong(TransMsgEvent receivingEvent) {
		GIAPongMessage message = (GIAPongMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
		//GIAOverlayNode.log.debug(this.getOverlayID() + " hat Pong erhalten von: " + message.getSender());
		//GIAOverlayNode.log.debug("Pong senden");

		boolean acceptPong = routingTable.incomingPong(message.getSender(), message.getDescriptor());
		// route pong
		if(acceptPong) {
			if (message.getTTL() -1 > 0) {
				GIAPongOperation pongOperation = new GIAPongOperation(this, message.getTTL() - 1, message.getHops() + 1, message.getDescriptor(), message.getContact(), message.getNumberOfNeighbors(), new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						// 
					}
				});
				pongOperation.scheduleImmediately();
			}
		}
		// add contact
		if(!routingTable.allContacts().contains(message.getContact()) && !routingTable.inactiveContacts().contains(message.getContact())){
			routingTable.addInactiveContact((GIAOverlayContact) message.getContact());
		}
		routingTable.setNumberOfNeighbors(message.getContact().getOverlayID(), message.getNumberOfNeighbors());
	}

	private void processQuery(TransMsgEvent receivingEvent) {
		GIAQueryMessage message = (GIAQueryMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
//		GIAOverlayNode.log.debug(this.getOverlayID() + " hat Query erhalten von: " + message.getSender());
		int maxReplies = message.getMaxReplies();
		List<GIAOverlayID> queryHitSent = message.getQueryHitSent();
		
		//check if neighbor has a token
		if(this.canNeighborSend(message.getSender())){
			lastQueryReceived.put(message.getSender(), Simulator.getCurrentTime());
			// do not check documents twice for the same query or for queries sent from here
			if (routingTable.incomingQuery(message.getSender(), message.getDescriptor())
					&& !this.queryInitiatedWithKey.containsKey(message.getDescriptor())) {
			
				//do not check documents if a query hit for this OverlayID was already sent
				if(!queryHitSent.contains(this.getOverlayID())){
					for (Integer document : documents) {
						if(message.getKey() == document) {
//							System.out.println("QueryHit : 	" + message.getKey());
							GIAQueryHitOperation queryHitOperation = new GIAQueryHitOperation(this, message.getDescriptor(), ownContactData, routingTable.numberOfActiveContacts(), message.getInitiator(), message.getHops()+1, new OperationCallback<Object>() {
								public void calledOperationFailed(Operation<Object> op) {
									//
								}
								public void calledOperationSucceeded(Operation<Object> op) {
									//
								}
							});

//							System.out.println(this.getOverlayID() +" sends QueryHit");
//							GIAOverlayNode.log.debug(this.getOverlayID() +" sends QueryHit");

							//System.out.println("from " +message.getReceiver()+ " to " +message.getSender()+ " query hit for " +message.getKey()+ " after " +(message.getHops() + 1));
//							GIAOverlayNode.log.debug(this.getOverlayID() +" sends QueryHit");

							queryHitSent.add((GIAOverlayID)this.getOverlayID());
							maxReplies--;
							queryHitOperation.scheduleImmediately();
						}	
					}
				}
				//check documents of neighbors for a query hit
				if(oneHopReplication){	
					for (GIAOverlayContact overlayContact : routingTable.allContacts()){
						//if there already was a query hit for this neighbor => do not check his documents
						if(maxReplies > 0 && !queryHitSent.contains(overlayContact.getOverlayID())){
							GIAOverlayID id = overlayContact.getOverlayID();
							if(documentsOfNeighbors.containsKey(id)){
								List<Integer> docs = documentsOfNeighbors.get(id);
								for (Integer document : docs){
									if(message.getKey() ==document) {
										int hops = message.getHops() + 1;
//										System.out.println("QueryHit : 	" + message.getKey() + " hops: " + hops);
										GIAQueryHitOperation queryHitOperation = new GIAQueryHitOperation(this, message.getDescriptor(), overlayContact, routingTable.getNumberOfNeighbors(overlayContact.getOverlayID()), message.getInitiator(), hops, new OperationCallback<Object>() {
											public void calledOperationFailed(Operation<Object> op) {
												//
											}
											public void calledOperationSucceeded(Operation<Object> op) {
												//
											}
										});
										//System.out.println("from " +overlayContact.getOverlayID()+ " to " +message.getSender()+ " query hit for " +message.getKey()+ " after " +(message.getHops() + 1));
										queryHitSent.add(overlayContact.getOverlayID());
										maxReplies--;
										queryHitOperation.scheduleImmediately();
									}
								}
							}
						}
					}
				}	
			}
			// route query
			int maxTTL;
			if(biasedRandomWalk){
				maxTTL = GIAConfiguration.GIA_QUERY_MAX_TTL;
				if (message.getTTL() - 1 > 0 && maxReplies > 0) {
					
					GIAQueryOperation queryOperation = new GIAQueryOperation(this, message.getTTL() - 1, message.getHops() + 1, message.getDescriptor(), message.getSender(), message.getKey(), maxReplies, queryHitSent, message.getInitiator(), maxTTL, biasedRandomWalk, new OperationCallback<Object>() {
						public void calledOperationFailed(Operation<Object> op) {
							//
						}
						public void calledOperationSucceeded(Operation<Object> op) {
							// 
						}
					});
					queryOperation.scheduleImmediately();
//					System.out.println(biasedRandomWalk+ " Query with hops :" +(message.getHops() +1));
				}
			}
			else{
				maxTTL = GIAConfiguration.GIA_QUERYFLOOD_MAX_TTL;
				if(routingTable.getQuerySentTo(message.getDescriptor()).isEmpty()){
					if (message.getTTL() - 1 > 0) {
						GIAQueryOperation queryOperation = new GIAQueryOperation(this, message.getTTL() - 1, message.getHops() + 1, message.getDescriptor(), message.getSender(), message.getKey(), maxReplies, queryHitSent, message.getInitiator(), maxTTL, biasedRandomWalk, new OperationCallback<Object>() {
							public void calledOperationFailed(Operation<Object> op) {
								//
							}
							public void calledOperationSucceeded(Operation<Object> op) {
								// 
							}
						});
						queryOperation.scheduleImmediately();
//						System.out.println(biasedRandomWalk+ " Query with hops :" +(message.getHops() +1));
					}
				}
			}		
		}		
	}

	private void processQueryHit(TransMsgEvent receivingEvent) {
		GIAQueryHitMessage message = (GIAQueryHitMessage) receivingEvent.getPayload();
		boolean acceptQuery = queryInitiated.contains(message.getDescriptor());
//		GIAOverlayNode.log.debug(this.getOverlayID()+ " QueryHit received from " +message.getSender());
		// addContact
		if(!routingTable.inactiveContacts().contains(message.getContact()) && !routingTable.allContacts().contains(message.getContact())){
			routingTable.addInactiveContact(message.getContact());
			routingTable.setNumberOfNeighbors(message.getContact().getOverlayID(), message.getNumberOfNeighbors());
		}
		// add contact to result list
		if(acceptQuery){
			if(queryResults.get(message.getDescriptor()) == null) {
				queryResults.put(message.getDescriptor(), new LinkedList<GIAOverlayContact>());
			}
			queryResults.get(message.getDescriptor()).add(message.getContact());
			if(queryResults.get(message.getDescriptor()).size() >= GIAConfiguration.GIA_MAXREPLIES){
				documents.add(queryInitiatedWithKey.get(message.getDescriptor()));
				if(oneHopReplication){	
					for(GIAOverlayContact contact : routingTable.allContacts()){
						GIADocumentsMessage documentsMessage = new GIADocumentsMessage((GIAOverlayID) this.getOverlayID(), contact.getOverlayID(), documents);
						this.getTransLayer().send(documentsMessage, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
					}
				}
				
				queryTimeout.remove(message.getDescriptor());
				queryInitiated.remove(message.getDescriptor());
				queryResults.remove(queryInitiatedWithKey.get(message.getDescriptor()));
				queryInitiatedWithKey.remove(message.getDescriptor());
			}
		}
	}
	
	private void processQueryID(TransMsgEvent receivingEvent){
		GIAQueryIDMessage message = (GIAQueryIDMessage) receivingEvent.getPayload();
		GIAOverlayID id = new GIAOverlayID(message.getID());
		
		if(this.getOverlayID() == id){
			GIAQueryIDHitMessage queryIDHit = new GIAQueryIDHitMessage((GIAOverlayID) this.getOverlayID(), message.getInitiator().getOverlayID(), 1, 0, message.getDescriptor(), this.ownContactData, this.getNumberOfNeighbors());
			this.getTransLayer().send(queryIDHit, message.getInitiator().getTransInfo(), this.getPort(), TransProtocol.UDP);
		}
		// route query
		else if (message.getTTL() - 1 > 0) {
			GIAQueryIDOperation queryIDOperation = new GIAQueryIDOperation(this, message.getTTL() - 1, message.getHops() + 1, message.getDescriptor(), message.getSender(), message.getID(), message.getInitiator(), new OperationCallback<Object>() {
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					// 
				}
			});
			queryIDOperation.scheduleImmediately();
		}
	}
	
	private void processQueryIDHit(TransMsgEvent receivingEvent){
		GIAQueryIDHitMessage message = (GIAQueryIDHitMessage) receivingEvent.getPayload();
		boolean acceptQuery = queryInitiated.contains(message.getDescriptor());
//		GIAOverlayNode.log.debug(this.getOverlayID()+ " QueryHit received from " +message.getSender());
		// addContact
		if(!routingTable.inactiveContacts().contains(message.getContact()) && !routingTable.allContacts().contains(message.getContact())){
			routingTable.addInactiveContact(message.getContact());
			routingTable.setNumberOfNeighbors(message.getContact().getOverlayID(), message.getNumberOfNeighbors());
		}
		// add contact to result list
		if(acceptQuery){		
				queryTimeout.remove(message.getDescriptor());
				queryInitiated.remove(message.getDescriptor());
				queryResults.remove(queryInitiatedWithKey.get(message.getDescriptor()));
				queryInitiatedWithKey.remove(message.getDescriptor());
		}
	}
	
	private void processDocumentsMessage(TransMsgEvent receivingEvent) {
		//put the list of documents of neighbor in own map
		GIADocumentsMessage documentsMessage = (GIADocumentsMessage) receivingEvent.getPayload();
		lastMessageReceived.put(documentsMessage.getSender(), Simulator.getCurrentTime());
		documentsOfNeighbors.put(documentsMessage.getSender(), documentsMessage.getDocuments());
		
	}

	/**
	 * @param descriptor message ID of the query
	 * @param key the document searched for
	 */
	public void registerQuery(BigInteger descriptor, Integer key) {
		this.queryResults.put(descriptor, new LinkedList<GIAOverlayContact>());
		this.queryInitiatedWithKey.put(descriptor, key);
		this.queryTimeout.put(descriptor, Simulator.getCurrentTime());
		this.queryInitiated.add(descriptor);
	}
	
	/**
	 * node joins the network by bootstrapping
	 */
	public void join(){
		//Join the p2p network
		active = true;
		bootstrapList = (LinkedList<TransInfo>) bootstrapManager.getBootstrapInfo();
		joinableList = (LinkedList<TransInfo>) bootstrapManager.getJoinableBootstrapInfo();
//		if(!bootstrapList.isEmpty()){
//			int numberOfThePeerToConnect = Simulator.getRandom().nextInt(bootstrapList.size());
//			TransInfo connectInfo = bootstrapList.get(numberOfThePeerToConnect);
//			GIAConnectOperation connect = new GIAConnectOperation(this, connectInfo, routingTable.numberOfActiveContacts(), new OperationCallback<Object>(){
//				public void calledOperationFailed(Operation<Object> op) {
//					//
//				}
//				public void calledOperationSucceeded(Operation<Object> op) {
//					//
//				}});
//			//GIAOverlayNode.log.debug("Bootstrap " +this.getOverlayID());
//			connect.scheduleImmediately();
//		}
		
		if(!joinableList.isEmpty()){
			int numberOfThePeerToConnect = Simulator.getRandom().nextInt(joinableList.size());
			TransInfo connectInfo = joinableList.get(numberOfThePeerToConnect);
			GIAConnectOperation connect = new GIAConnectOperation(this, connectInfo, routingTable.numberOfActiveContacts(), new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			//GIAOverlayNode.log.debug("Bootstrap " +this.getOverlayID());
			connect.scheduleImmediately();
		}
		
		if(joinableList.isEmpty()){
			bootstrapManager.registerJoinableNodes(this);
		}
		bootstrapManager.registerNode(this);

		this.sendPing();
		GIACheckNeighborsOperation operation = new GIACheckNeighborsOperation(this, GIAConfiguration.GIA_SCHEDULEDCHECKNEIGHBORS, biasedRandomWalk, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		operation.scheduleWithDelay(5000000);

		
		GIAScheduleConnectOperation connect = new GIAScheduleConnectOperation(this, topologyAdaption, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		connect.scheduleWithDelay(5000000);
		// Peer leaves the Network with a certain probability
		if(GIAConfiguration.GIA_CHURN){
			GIAScheduleLeaveOperation leaveOperation = new GIAScheduleLeaveOperation(this, userType, new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			int random = 30 - Simulator.getRandom().nextInt(60);
			if(userType == 0){
				leaveOperation.scheduleWithDelay((GIAConfiguration.GIA_SESSIONTIMEBENEFACTOR + random)*Simulator.SECOND_UNIT);
			}
			else if(userType == 1){
				leaveOperation.scheduleWithDelay((GIAConfiguration.GIA_SESSIONTIMEPEER + random)*Simulator.SECOND_UNIT);
			}
		}
		// Peer schedules Query automatically 
		GIAScheduleQueryOperation scheduleQuery = new GIAScheduleQueryOperation(this, biasedRandomWalk, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		scheduleQuery.scheduleWithDelay(40000000);
	
	}
	
	/**
	 * node leaves the network by sending bye messages to all neighbors
	 */
	public void leave(boolean rejoin){
		//Peer leaves the network
		List<GIAOverlayContact> activeContacts = routingTable.allContacts();
		for (GIAOverlayContact contact : activeContacts){
			GIAByeMessage bye = new GIAByeMessage((GIAOverlayID)this.getOverlayID(), contact.getOverlayID(), "Disconnect");			
			this.getTransLayer().send(bye, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
			
		}
		routingTable.clearContacts();
		bootstrapManager.unregisterNode(this);
		bootstrapManager.unregisterJoinableNodes(this);
		active = false;
		
		// Peer rejoins after x seconds
		if(rejoin){
			GIAScheduleJoinOperation scheduleJoin = new GIAScheduleJoinOperation(this, userType, new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			scheduleJoin.scheduleWithDelay(GIAConfiguration.GIA_SCHEDULEDCONNECT);
		}
	}
	
	/**
	 * node leaves the network without letting its neighbors know
	 */
	public void fail(boolean rejoin){
		active = false;
		routingTable.clearContacts();
		bootstrapManager.unregisterNode(this);
		bootstrapManager.unregisterJoinableNodes(this);
		GIAOverlayNode.log.debug(Simulator.getCurrentTime() + " - "+ this.getOverlayID() + " has failed !");
	
		// Peer rejoins after x seconds
		if(rejoin){
			GIAScheduleJoinOperation scheduleJoin = new GIAScheduleJoinOperation(this, userType, new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			
			scheduleJoin.scheduleWithDelay((long)(sessionTime/ this.connectivityFactor) - sessionTime);
		}
	}


	/**
	 * node starts sending pings to all neighbors periodically
	 */
	public void sendPing(){
		//send Ping to neighbors
		GIASchedulePingOperation schedulePing = new GIASchedulePingOperation(this, GIAConfiguration.GIA_SCHEDULEPING, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		schedulePing.scheduleImmediately();
	}
	
	/**
	 * method to ping each neighbor once
	 */
	public void sendPingOnce(){
		for(GIAOverlayContact receiver : this.getNodesRoutingTable().allContacts()){
			//GIASchedulePingOperation.log.debug(Simulator.getCurrentTime()+ " - " +this.getComponent().getOverlayID() + " sends keep alive " +receiver.getOverlayID());
			GIAPingOperation ping = new GIAPingOperation(this, receiver, 1, 0, null, new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						//
					}});
			ping.scheduleImmediately();
		}
	}

	/**
	 * node randomly selects a document and searches for it
	 */
	public void sendQuery(){
		//  send Query
		// Peer schedules Query automatically 
		GIAScheduleQueryOperation scheduleQuery = new GIAScheduleQueryOperation(this, biasedRandomWalk, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		scheduleQuery.scheduleWithDelay(4000000);
	}
	
	/**
	 * method to send query once
	 */
	public void sendQueryOnce(){
		// einzelnen Query senden
		int file = 0;
		for(int i = 0; i < 5; i++){
			file = FileFactory.getInstance().getZipf().returnRank();
			if(FileFactory.getInstance().getFileSharingKeys().contains(file)){
				i = 5;
			}
		}
//		FileSharingKey file = FileFactory.getInstance().getFileKey(Simulator.getRandom().nextInt(FileFactory.getInstance().getNumberOfFiles()));
		if(!this.getDocuments().contains(file)){
			int maxTTL;
			if(biasedRandomWalk){
				maxTTL = GIAConfiguration.GIA_QUERY_MAX_TTL;
			}
			else{
				maxTTL = GIAConfiguration.GIA_QUERYFLOOD_MAX_TTL;
			}
			GIAQueryOperation query = new GIAQueryOperation(this, file, maxTTL, biasedRandomWalk, new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						//
					}});			
			query.scheduleImmediately();
		}
	}
	
	/**
	 * send query with a given key
	 * 
	 * @param key document searched for
	 */
	public void sendQueryWithKey(int key){
		//FileSharingKey file = new FileSharingKey(key);
		int maxTTL;
		if(biasedRandomWalk){
			maxTTL = GIAConfiguration.GIA_QUERY_MAX_TTL;
		}
		else{
			maxTTL = GIAConfiguration.GIA_QUERYFLOOD_MAX_TTL;
		}
		GIAQueryOperation query = new GIAQueryOperation(this, key, maxTTL, biasedRandomWalk, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		query.scheduleImmediately();
	}
	
	public void sendQueryForOverlayID(int id){
		GIAQueryIDOperation queryID = new GIAQueryIDOperation(this, id, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		queryID.scheduleImmediately();
	}

	/**
	 * node calculates the token rate and sends it to all neighbors
	 */
	public void sendToken(){
		//send token to neighbors
		if(flowControl){
			List<GIAOverlayContact> activeContacts = routingTable.allContacts();
			for (GIAOverlayContact contact : activeContacts){
				long tokenRate = this.calculateTokenRate(contact.getCapacity());
				GIATokenMessage token = new GIATokenMessage((GIAOverlayID)this.getOverlayID(), contact.getOverlayID(), tokenRate);			
				this.getTransLayer().send(token, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
				tokensForNeighbors.put(contact.getOverlayID(), tokenRate);
	
				//GIAOverlayNode.log.debug(this.getOverlayID() + " sendet Token an: " + contact.getOverlayID());
				//GIAOverlayNode.log.debug(" mit tokenRate: " + tokenRate + " und Kapazit�t: " + this.getCapacity());
			}
		}	
	}

	/**
	 * if node is not satisfied with its current neighbors it selects a better neighbor to connet to
	 */
	public void connect(){
		// connects with other peers to get a higher satisfaction-rate
//		GIAOverlayNode.log.debug(this.getOverlayID() + " hat Connect aufgerufen! Neighbors: " + this.getNumberOfNeighbors());
//		for(GIAOverlayContact contact : routingTable.allContacts()){
//			GIAOverlayNode.log.debug(this.getOverlayID() + " neighbors: " +contact+ " - "+ contact.getOverlayID()+ " - " + contact.getCapacity());
//		}
		if(topologyAdaption){
			if(this.getSatisfaction() < 1){
	//			GIAOverlayNode.log.debug(this.getOverlayID()+ " Satisfaction: " +this.getSatisfaction());
				int size = routingTable.inactiveContacts().size();
	//			GIAOverlayNode.log.debug("Anzahl der inaktiven Nachbarn: " + size);
				if (size != 0){
					List<GIAOverlayContact> contacts = new LinkedList<GIAOverlayContact>();
					for(int i=0; i < 10; i++){
						int random = Simulator.getRandom().nextInt(size);
						if (connectToDenied.size() > this.getNodesRoutingTable().inactiveContacts().size() + 10){
							connectToDenied.clear();
						}
						GIAOverlayContact contact = routingTable.inactiveContacts().get(random);
						if(!contacts.contains(contact)){
							contacts.add(contact);
	//						GIAOverlayNode.log.debug(this.getOverlayID() + " hat " + contact.getOverlayID() + "als neuen Kontakt dazugef�gt");
						} 
						else if(contacts.size() == size){
							i=10;
						}
						else {
							i--;
						}
						
					}
					int maxCapacity = 0;
					GIAOverlayContact connectTo = null;
					for(GIAOverlayContact con : contacts){
						if(con.getCapacity() > maxCapacity){
							connectTo = con;
							maxCapacity = con.getCapacity();
	//						GIAOverlayNode.log.debug(con.getOverlayID()+ " - " + maxCapacity);
						}
					}
					if(maxCapacity <= capacity){
						connectTo = contacts.get(Simulator.getRandom().nextInt(contacts.size()));
					}
					if(this.getNumberOfNeighbors() < this.maxContacts){
						if(!routingTable.allContacts().contains(connectTo) && !connectToDenied.contains(connectTo.getOverlayID())){
							GIAConnectMessage connect = new GIAConnectMessage((GIAOverlayID)this.getOverlayID(), connectTo.getOverlayID(), ownContactData, routingTable.numberOfActiveContacts());
							this.getTransLayer().send(connect, connectTo.getTransInfo(), this.getPort(), TransProtocol.UDP);
						}
					} 
				}
	//			else {System.out.println("keine Inativen Kontakte vorhanden");}
			}
		}
//		GIAOverlayNode.log.debug(this.getOverlayID()+  " Satisfaction: " + this.getSatisfaction());
		else{
			if(routingTable.allContacts().size() < maxContacts && !routingTable.inactiveContacts().isEmpty()){
				GIAOverlayContact contact = routingTable.inactiveContacts().get(Simulator.getRandom().nextInt(routingTable.inactiveContacts().size()));
				GIAConnectMessage connect = new GIAConnectMessage((GIAOverlayID)this.getOverlayID(), contact.getOverlayID(), ownContactData, routingTable.numberOfActiveContacts());
				this.getTransLayer().send(connect, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
			}
		}
	}
	
	/**
	 * @return the level of satisfaction a node has with its neighbors form 0.0 to 1.0
	 */
	public float getSatisfaction(){
		// calculates the satisfactionlevel of a node
		satisfaction = 0;
		if(routingTable.numberOfActiveContacts() < minContacts){
			return satisfaction;
		}
		for(Integer capacity : routingTable.getSortedCapacities()){
			for(OverlayContact<GIAOverlayID> contact : routingTable.getNodesWithCapacity(capacity)){
				int numberOfNeighbors = (routingTable.getNumberOfNeighbors(contact.getOverlayID()));
//				GIAOverlayNode.log.debug("NumberOfNeighbors: " + numberOfNeighbors + " : von : " + contact.getOverlayID() + " : bei : " + this.getOverlayID());
				if (numberOfNeighbors != 0){
					satisfaction += capacity/ numberOfNeighbors;
				}
			}
		}
		satisfaction = satisfaction/capacity;
//		GIAOverlayNode.log.debug("Satisfaction berechnet f�r : " + this.getOverlayID() + " - Ergebnis: " + satisfaction);
		if(satisfaction > 1 || routingTable.numberOfActiveContacts() >= maxContacts){
			satisfaction = 1;
		}
		return satisfaction;
	}
	
	/**
	 * @return the capacity of the node
	 */
	public int getCapacity(){
		return capacity;
	}
	
	/**
	 * @return the number of neighbors of the node
	 */
	public int getNumberOfNeighbors(){
		return routingTable.numberOfActiveContacts();
	}
	
	/**
	 * @param capacity the capacity of the neighbor the node wants to calculate the token rate to
	 * @return token rate
	 */
	public long calculateTokenRate(int capacity){
		long tokenRate = 0;
		long totalCapacity = 0;
		LinkedList<GIAOverlayContact> contacts = (LinkedList<GIAOverlayContact>) routingTable.allContacts();
		for (GIAOverlayContact contact : contacts){
			totalCapacity += contact.getCapacity();
		}
		
		tokenRate = (1000 * totalCapacity)/(capacity * this.getCapacity()) ;
		//GIAOverlayNode.log.debug(tokenRate + " = " + totalCapacity + " / " + capacity + " * " + this.getCapacity());
		return tokenRate;
	}
	
	/**
	 * @param id overlay ID of the receiver
	 * @return true if node has token, false if not
	 */
	public boolean canISend(GIAOverlayID id){
//		System.out.println(id);
		if(flowControl){
			if (tokensFromNeighbors.containsKey(id)){
				float tokenRate = tokensFromNeighbors.get(id);
//				System.out.println("Tokenrate: " + tokenRate);
				if (lastQuerySent.containsKey(id)){
					if (Simulator.getCurrentTime() - tokenRate >= lastQuerySent.get(id)){
						return true;
					} else return false;
				} else return true;
			} else return false;
		}
		else{
			return true;
		}
	}
	
	/**
	 * @param id overlay ID of sender
	 * @return true if sender has token, false if not
	 */
	public boolean canNeighborSend(GIAOverlayID id){
		if(flowControl){
			float tokenRate = tokensForNeighbors.get(id);
			if (lastQueryReceived.containsKey(id)){
				if (Simulator.getCurrentTime() - tokenRate >= lastQueryReceived.get(id)){
					return true;
				} else return false;
			} else return true;
		}
		else{
			return true;
		}
	}
	
	/**
	 * @param id overlay ID of the receiver of this query
	 */
	public void lastQuerySent(GIAOverlayID id) {
		lastQuerySent.put(id, Simulator.getCurrentTime());
	}

	
	/**
	 * @return routing table of this node
	 */
	public GIAOverlayRoutingTable getNodesRoutingTable(){
		return routingTable;
	}
	
	/**
	 * @return true if node is still in the network
	 */
	public boolean isActive(){
		if(active){
			return true;
		}
		else return false;
	}
	
	/**
	 *  sets the peer to active
	 */
	public void setActive(){
		active = true;
	}
	
	/**
	 * @param id overlay ID of the neighbor
	 * @return time the last message from this neighbor was received
	 */
	public long getLastMessageReceived(GIAOverlayID id){
		if(lastMessageReceived.get(id) != null){
			return lastMessageReceived.get(id);
		}
		return 0;
	}
	
	/**
	 * @return contact data of the peer
	 */
	public GIAOverlayContact getOwnContactData(){
		return ownContactData;
	}
	
	/**
	 * @return list of documents
	 */
	public List<Integer> getDocuments(){
		return documents;
	}
	
	public String toString() {
		return this.getOverlayID().toString();
	}
	
	/**
	 * @return map of timeouts of query-messages
	 */
	public Map<BigInteger, Long> getQueryTimeout(){
		return queryTimeout;
	}
	
	/**
	 * @param descriptor messageId of the query-message
	 */
	public void setQueryTimeout(BigInteger descriptor){
		queryTimeout.put(descriptor, Simulator.getCurrentTime());
	}
	
	/**
	 * @return list of queries initiated at this peer
	 */
	public List<BigInteger> getQueryInitiated(){
		return queryInitiated;
	}
	
	/**
	 * @return map of message IDs to fileSharingkeys
	 */
	public Map<BigInteger, Integer> getQueryInitiatedWithKey(){
		return queryInitiatedWithKey;
	}
	
	/**
	 * @return maximum number of contacts
	 */
	public int getMaxContacts(){
		return this.maxContacts;
	}
	
}
