package de.tud.kom.p2psim.impl.overlay.gia;

import java.math.BigInteger;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.*;


import de.tud.kom.p2psim.impl.util.logging.SimLogger;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayID;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.Component;
import de.tud.kom.p2psim.api.common.Host;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAOverlayNodeFactory implements ComponentFactory{
	
	protected final static Logger log = SimLogger.getLogger(GIAOverlayNodeFactory.class);
	private short port = 123;
	private static long id = 0;
	private List<Integer> documents;
	private int capacity;
	private GIABootstrapManager bootstrapManager = new GIABootstrapManager();
	
	private boolean biasedRandomWalk;
	private boolean topologyAdaption;
	private boolean oneHopReplication;
	private boolean flowControl;
	
	public Component createComponent(Host host){
		return new GIAOverlayNode(host.getTransLayer(), newOverlayID(), port, calculateCapacity(), calculateUserType(), FileFactory.getInstance().createDocuments(Simulator.getRandom().nextInt(20)), bootstrapManager, biasedRandomWalk, topologyAdaption, oneHopReplication, flowControl);
	}
	
	/**
	 * @param host 
	 * @return new node with up to 20 documents
	 */
	public GIAOverlayNode createNode(Host host){
		capacity = calculateCapacity();
		int numberOfDocuments = Simulator.getRandom().nextInt(20);
		documents = new LinkedList<Integer>();
		while(numberOfDocuments != 0){
			int file = FileFactory.getInstance().createNewFileSharingKey();
			//documents.add(file);
			numberOfDocuments--;
			//GIAOverlayNodeFactory.log.debug("file" +file);
		}
		GIAOverlayNode newOverlayNode = new GIAOverlayNode(host.getTransLayer(), newOverlayID(), port, capacity, calculateUserType(), documents, bootstrapManager, biasedRandomWalk, topologyAdaption, oneHopReplication, flowControl);
//		GIAOverlayNodeFactory.log.debug(Simulator.getCurrentTime() + " - " + newOverlayNode.getOverlayID() + " - capacity: " +capacity);
//		GIAOverlayNodeFactory.log.debug(FileFactory.getInstance().getFileSharingKeys());
		return newOverlayNode;
	}
	
	/**
	 * @return new overlay ID
	 */
	public GIAOverlayID newOverlayID(){
		this.id += 1;
//		System.out.println(new GIAOverlayID(BigInteger.valueOf(this.id)));
		return new GIAOverlayID(BigInteger.valueOf(this.id));
	}
	
	/**
	 * @return capacity of the peer in operations per minute
	 */
	public int calculateCapacity(){
		int calcCapacity;
		calcCapacity = Simulator.getRandom().nextInt(100);
		if (calcCapacity <=1){
			return 10000;
		}
		else if (calcCapacity >= 2 && calcCapacity < 6){
			return 1000;
		}
		else if (calcCapacity >= 6 && calcCapacity < 35){
			return 100;
		}
		else if (calcCapacity >= 35 && calcCapacity < 80){
			return 10;
		}
		else if (calcCapacity >= 80){
			return 1;
		
		}
		return 10;
	}
	
	public int calculateUserType(){
		int calcUserType = Simulator.getRandom().nextInt(100);
		
		if(calcUserType <= 10){
			//User is benefactor
			return 0;
		}
		else if(calcUserType <= 100){
			//User is normal peer
			return 1;
		}
		else{
			//User is peeper
			return 2;
		}
	}

	public void setBiasedRandomWalk(boolean biasedRandomWalk){
		this.biasedRandomWalk = biasedRandomWalk;
	}
	
	public void setTopologyAdaption(boolean topologyAdaption){
		this.topologyAdaption = topologyAdaption;
	}
	
	public void setOneHopReplication(boolean oneHopReplication){
		this.oneHopReplication = oneHopReplication;
	}
	
	public void setFlowControl(boolean flowControl){
		this.flowControl = flowControl;
	}
	
}
