package de.tud.kom.p2psim.impl.overlay.gia;

import java.math.BigInteger;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayRoutingTable;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAOverlayRoutingTable implements OverlayRoutingTable<GIAOverlayID, GIAOverlayContact> {

	private OverlayID overlayID;
	private Map<GIAOverlayID, GIAOverlayContact> overlayIDs; // = new HashMap<OverlayID, GIAOverlayContact>();
	private List<GIAOverlayContact> activeContacts = new LinkedList<GIAOverlayContact>();
	private List<GIAOverlayContact> inactiveContacts = new LinkedList<GIAOverlayContact>();
	
	private Map<BigInteger, List<GIAOverlayContact>> acceptPong = new HashMap<BigInteger, List<GIAOverlayContact>>();
	private Map<BigInteger, GIAOverlayContact> routePong = new HashMap<BigInteger, GIAOverlayContact>();
	private Map<BigInteger, List<GIAOverlayContact>> acceptQueryHit = new HashMap<BigInteger, List<GIAOverlayContact>>();
	private Map<BigInteger, GIAOverlayContact> routeQueryHit = new HashMap<BigInteger, GIAOverlayContact>();
	private Map<Integer, List<GIAOverlayContact>> sortedByCapacity = new HashMap<Integer, List<GIAOverlayContact>>();
	private List<Integer> sortedCapacities = new LinkedList<Integer>();
	private Map<BigInteger, List<GIAOverlayContact>> querySent;
	private Map<GIAOverlayID, Integer> numberOfNeighborsOf = new HashMap<GIAOverlayID, Integer>();
	
	/**
	 * @param overlayID overlay ID of the node
	 */
	public GIAOverlayRoutingTable(OverlayID overlayID) {
		this.overlayID = overlayID;
		this.querySent = new HashMap<BigInteger, List<GIAOverlayContact>>();
		overlayIDs = new HashMap<GIAOverlayID, GIAOverlayContact>();
	}

	/**
	 * @param overlayID2 overlay ID of initiator
	 * @param descriptor message ID of the ping
	 */
	public void initiatedPing(GIAOverlayID overlayID2, BigInteger descriptor) {
		routePong.put(descriptor, null);
	}

	/**
	 * @param id overlay ID of sender
	 * @param descriptor message ID of ping
	 * @return true if ping was not received before
	 */
	public boolean incomingPing(GIAOverlayID id, BigInteger descriptor) {
		GIAOverlayContact contact = getContact(id);
		if(routePong.containsKey(descriptor) || contact == null) {
			return false;
		}
		routePong.put(descriptor, contact);
		return true;
	}
	
	/**
	 * @param contact receiver of the message
	 * @param descriptor message ID
	 */
	public void outgoingPing(GIAOverlayContact contact, BigInteger descriptor) {
		if(!acceptPong.containsKey(descriptor)) {
			acceptPong.put(descriptor, new LinkedList<GIAOverlayContact>());
		}
		acceptPong.get(descriptor).add(contact);
	}
	
	/**
	 * @param idSender sender
	 * @param descriptor message ID
	 * @return true if pong can be routed
	 */
	public boolean incomingPong(GIAOverlayID idSender, BigInteger descriptor) {
		GIAOverlayContact contactSender = getContact(idSender);
		// return contact to forward message to
		if(!acceptPong.containsKey(descriptor) || !routePong.containsKey(descriptor)) {
			// do not route messages with wrong descriptor
			return false;
		}
		if(!acceptPong.get(descriptor).contains(contactSender)) {
			// do not reply messages without having sent a request 
			return false;
		}
		return true;
	}

	/**
	 * @param descriptor message ID
	 * @return contact to forward message to
	 */
	public GIAOverlayContact outgoingPong(BigInteger descriptor) {
		// return contact to forward message to
		return routePong.get(descriptor);
	}

	/**
	 * @param overlayID2 overlay ID of this node
	 * @param descriptor message ID
	 */
	public void initiatedQuery(GIAOverlayID overlayID2, BigInteger descriptor) {
		routeQueryHit.put(descriptor, null);
	}

	/**
	 * @param id
	 * @param descriptor
	 * @return true if query was 
	 */
	public boolean incomingQuery(GIAOverlayID id, BigInteger descriptor) {
		GIAOverlayContact contact = getContact(id);
		if(routeQueryHit.containsKey(descriptor) || contact == null) {
			return false;
		}
		routeQueryHit.put(descriptor, contact);
		return true;
	}
	
	public void outgoingQuery(GIAOverlayContact contact, BigInteger descriptor) {
		if(!acceptQueryHit.containsKey(descriptor)) {
			acceptQueryHit.put(descriptor, new LinkedList<GIAOverlayContact>());
		}
		acceptQueryHit.get(descriptor).add(contact);
	}
	
	public GIAOverlayContact outgoingQueryHit(BigInteger descriptor) {
		// return contact to forward message to
		return routeQueryHit.get(descriptor);
	}
	
	/**
	 * @param contact contact info
	 */
	public void addInactiveContact(GIAOverlayContact contact) {
		if(inactiveContacts.size() < 35){
			if(contact != null){
				if(!inactiveContacts.contains(contact)) {
					inactiveContacts.add(contact);
				}
				if(!overlayIDs.containsKey(contact.getOverlayID())){
					overlayIDs.put(contact.getOverlayID(), contact);
				}
			}
		}
	}
	
	public void addContact(GIAOverlayContact contact){
		
		if(!this.activeContacts.contains(contact)) {
			inactiveContacts.remove(contact);

			this.overlayIDs.put(contact.getOverlayID(), contact);
			this.activeContacts.add(contact);
		}
	}

	public List<GIAOverlayContact> allContacts() {
		//System.out.println("AllContacts aufgerufen " +activeContacts.size() );
		return new LinkedList<GIAOverlayContact>(activeContacts);
	}
	
	/**
	 * @return the number of active contacts
	 */
	public int numberOfActiveContacts() {
		return activeContacts.size();
	}
	
	/**
	 * @return the number of inactive contacts
	 */
	public List<GIAOverlayContact> inactiveContacts() {
		return new LinkedList<GIAOverlayContact>(inactiveContacts);
	}
	
	public void removeInactiveContact(GIAOverlayContact contact) {
		inactiveContacts.remove(contact);
	}
	
	public GIAOverlayContact getContact(GIAOverlayID oid) {
		return this.overlayIDs.get(oid);
	}

	public void removeContact(GIAOverlayID oid) {
		GIAOverlayContact contact = this.getContact(oid);
		activeContacts.remove(contact);
	}

	public void clearContacts() {
		this.overlayIDs.clear();
		this.activeContacts.clear();
		this.inactiveContacts.clear();
	}
	
	/**
	 * @param capacity capacity of the contact
	 * @param overlayContact contact info
	 */
	public void addContactWithCapacity(int capacity, GIAOverlayContact overlayContact){
		if(sortedByCapacity.containsKey(capacity)){
			List<GIAOverlayContact> temp = sortedByCapacity.get(capacity);
			temp.add(overlayContact);
			sortedByCapacity.remove(capacity);
			sortedByCapacity.put(capacity, temp);
		}
		else {
			List<GIAOverlayContact> newList = new LinkedList<GIAOverlayContact>();
			newList.add(overlayContact);
			sortedByCapacity.put(capacity, newList);
		}
		
		if(!sortedCapacities.contains(capacity)){
			sortedCapacities.add(capacity);
			Collections.sort(sortedCapacities);
		}
	}
	
	/**
	 * @param capacity capacity of contact
	 * @param overlayContact contact info
	 */
	public void removeContactWithCapacity(int capacity, GIAOverlayContact overlayContact){
		if(sortedByCapacity.containsKey(capacity)){
			List<GIAOverlayContact> temp = sortedByCapacity.get(capacity);
			temp.remove(overlayContact);
			sortedByCapacity.remove(capacity);
			if(!temp.isEmpty()){
				sortedByCapacity.put(capacity, temp);
			}
		}
		if(!sortedByCapacity.containsKey(capacity) && sortedCapacities.contains(capacity)){
			Integer capacitie = capacity;
			sortedCapacities.remove(capacitie);
		}
	}
	
	/**
	 * @param capacity capacity looked for
	 * @return all neighbors with the capacity
	 */
	public List<GIAOverlayContact> getNodesWithCapacity(int capacity){
		return sortedByCapacity.get(capacity);
	}
	
	/**
	 * @return sorted capacities of all neighbors
	 */
	public List<Integer> getSortedCapacities(){
		return sortedCapacities;
	}
	
	/**
	 * @param descriptor message ID
	 * @return List of all neighbors this query was already sent to
	 */
	public List<GIAOverlayContact> getQuerySentTo(BigInteger descriptor){
		if (querySent.containsKey(descriptor)){
			if(querySent.get(descriptor).isEmpty()){
				List<GIAOverlayContact> querySentTo = new LinkedList<GIAOverlayContact>();
				return querySentTo;
			}
			else return querySent.get(descriptor);
		} else {
			List<GIAOverlayContact> querySentTo = new LinkedList<GIAOverlayContact>();
			querySent.put(descriptor, querySentTo);
			return querySentTo;
		}
	}
	
	/**
	 * @param descriptor message ID
	 * @param contact contact info
	 */
	public void setQuerySentTo(BigInteger descriptor, GIAOverlayContact contact){
		if(querySent.containsKey(descriptor)){
			if(querySent.get(descriptor).isEmpty()){
				List<GIAOverlayContact> list = new LinkedList<GIAOverlayContact>();
				list.add(contact);
				querySent.put(descriptor, list);
			}
			else {
				List<GIAOverlayContact> list = querySent.get(descriptor);
				list.add(contact);
				querySent.put(descriptor, list);
			}
		}
		else{
			List<GIAOverlayContact> list = new LinkedList<GIAOverlayContact>();
			list.add(contact);
			querySent.put(descriptor, list);
		}
	}
	
	public void flushQuerySentTo(BigInteger descriptor){
		if(querySent.containsKey(descriptor)){
			querySent.remove(descriptor);
		}
	}
	
	/**
	 * @param id overlay ID of neighbor
	 * @param numberOfNeighbors number of neighbors of neighbor
	 */
	public void setNumberOfNeighbors(GIAOverlayID id, int numberOfNeighbors){
		if(numberOfNeighborsOf.containsKey(id)){
			numberOfNeighborsOf.remove(id);
		}
		if(numberOfNeighbors > GIAConfiguration.GIA_MAXCONTACTS){
			numberOfNeighbors = GIAConfiguration.GIA_MAXCONTACTS;
		}
		numberOfNeighborsOf.put(id, numberOfNeighbors);
	}
	
	/**
	 * @param id overlay ID of neighbor who left the network
	 */
	public void removeNumberOfNeighbors(GIAOverlayID id){
		if(numberOfNeighborsOf.containsKey(id)){
			numberOfNeighborsOf.remove(id);
		}
	}
	
	/**
	 * @param id overlay ID of neighbor
	 * @return number of Neighbors of neighbor
	 */
	public int getNumberOfNeighbors(GIAOverlayID id){
		if(numberOfNeighborsOf.containsKey(id)){	
			return numberOfNeighborsOf.get(id);
		}
		else return 0;
	}
	
}
