package de.tud.kom.p2psim.impl.overlay.gia.Messages;

import java.math.BigInteger;
import java.util.List;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.FileSharingKey;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAQueryMessage extends BaseMessage {
	
	private static final long GIA_QUERY_MESSAGE_SIZE = 3;
	private long searchStringSize;
	private Integer key;
	private int maxReplies;
	private List<GIAOverlayID> queryHitSent;
	private GIAOverlayContact initiator;
	
	/**
	 * @param sender overlay ID of the sender
	 * @param receiver overlay ID of the receiver
	 * @param ttl time to live of the message
	 * @param hops hop count for the message
	 * @param descriptor message ID
	 * @param key document searched for
	 * @param maxReplies number of replies wanted
	 * @param queryHitSent list of all the nodes that responded with query hit
	 * @param initiator contact info of the initiator of the query
	 */
	public GIAQueryMessage(GIAOverlayID sender, GIAOverlayID receiver, int ttl, int hops, BigInteger descriptor, Integer key, int maxReplies, List<GIAOverlayID> queryHitSent, GIAOverlayContact initiator) {
		super(sender, receiver, ttl, hops, descriptor);
		searchStringSize = (long) (Simulator.getRandom().nextDouble() * 40);
		this.key = key;
		this.maxReplies = maxReplies;
		this.queryHitSent = queryHitSent;
		this.initiator = initiator;
	}

	/**
	 * @return document
	 */
	public Integer getKey() {
		return this.key;
	}
	
	/**
	 * @return max replies
	 */
	public int getMaxReplies() {
		return this.maxReplies;
	}
	
	public long getSize() { 
		return super.getSize() + GIA_QUERY_MESSAGE_SIZE + searchStringSize;
	}
	
	/**
	 * @return list of all nodes a query hit has been sent for
	 */
	public List<GIAOverlayID> getQueryHitSent(){
		return queryHitSent;
	}
	
	/**
	 * @return contact info of initiator
	 */
	public GIAOverlayContact getInitiator(){
		return initiator;
	}
}
