package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;
import java.util.LinkedList;

import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.gia.FileFactory;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 25.06.2008
 *
 */
public class GIACheckNeighborsOperation extends AbstractOperation<GIAOverlayNode, Object> {

	private long delay;
	private boolean biasedRandomWalk;
	
	/**
	 * @param node initiator of this operation
	 * @param delay interval the massage should be send in
	 * @param biasedRandomWalk true if random walk is active
	 * @param callback
	 */
	public GIACheckNeighborsOperation(GIAOverlayNode node, long delay, boolean biasedRandomWalk, OperationCallback<Object> callback){
		super(node, callback);
		this.delay = delay;
		this.biasedRandomWalk = biasedRandomWalk;
	}

	@Override
	protected void execute() {
		if(this.getComponent().isActive()){
			for(GIAOverlayContact contact : this.getComponent().getNodesRoutingTable().allContacts()){
				if(!(this.getComponent().getLastMessageReceived(contact.getOverlayID()) == 0)){
					if(Simulator.getCurrentTime() - this.getComponent().getLastMessageReceived(contact.getOverlayID()) > GIAConfiguration.GIA_TIMEOUT){
						this.getComponent().getNodesRoutingTable().removeContact(contact.getOverlayID());
						this.getComponent().getNodesRoutingTable().removeContactWithCapacity(contact.getCapacity(), contact);
						this.getComponent().getNodesRoutingTable().removeNumberOfNeighbors(contact.getOverlayID());
						this.getComponent().getNodesRoutingTable().removeInactiveContact(contact);
						this.getComponent().connect();
						
					}
				}
			}
			for(BigInteger descriptor : this.getComponent().getQueryInitiated()){
				if(Simulator.getCurrentTime() - this.getComponent().getQueryTimeout().get(descriptor) > GIAConfiguration.GIA_QUERYTIMEOUT){
					System.out.println(Simulator.getCurrentTime()+ " query timeout " +this.getComponent().getQueryInitiatedWithKey().get(descriptor)+ " is in the network " +FileFactory.getInstance().getFileSharingKeys().contains(this.getComponent().getQueryInitiatedWithKey().get(descriptor)));
					int maxTTL;
					if(biasedRandomWalk){
						maxTTL = GIAConfiguration.GIA_QUERY_MAX_TTL;
						GIAQueryOperation query = new GIAQueryOperation(this.getComponent(), maxTTL, 0, descriptor,(GIAOverlayID) this.getComponent().getOverlayID(), this.getComponent().getQueryInitiatedWithKey().get(descriptor), GIAConfiguration.GIA_MAXREPLIES, new LinkedList<GIAOverlayID>() , this.getComponent().getOwnContactData(), maxTTL, biasedRandomWalk, new OperationCallback<Object>(){
							public void calledOperationFailed(Operation<Object> op) {
								//
							}
							public void calledOperationSucceeded(Operation<Object> op) {
								//
							}});
						query.scheduleImmediately();
						this.getComponent().setQueryTimeout(descriptor);
					}
					else{
						maxTTL = GIAConfiguration.GIA_QUERYFLOOD_MAX_TTL;
					}
				}
			}
			this.scheduleWithDelay(delay);
			operationFinished(true);
		}
	}

	@Override
	public Object getResult() {
		return this;
	}
}
