package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;
import org.apache.log4j.*;


import de.tud.kom.p2psim.impl.util.logging.SimLogger;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAPingMessage;


/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAPingOperation extends BaseOperation{

	protected final static Logger log = SimLogger.getLogger(GIAPingOperation.class);
	private GIAOverlayContact receiver;
	
	/**
	 * @param component initiator of this operation
	 * @param receiver contact info of receiver
	 * @param callback
	 */
	public GIAPingOperation(GIAOverlayNode component, GIAOverlayContact receiver, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_PING_MAX_TTL, GIAConfiguration.GIA_PING_MAX_TTL, 0, null, callback);
		this.receiver = receiver;
		component.getNodesRoutingTable().initiatedPing((GIAOverlayID) component.getOverlayID(), this.getDescriptor());
	}
	
	/**
	 * @param component initiator of this operation
	 * @param receiver contact info of receiver
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param callback
	 */
	public GIAPingOperation(GIAOverlayNode component, GIAOverlayContact receiver, int ttl, int hops, BigInteger descriptor, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.receiver = receiver;
	}

	@Override
	protected void execute() {
		if(ttl > 0) {
			GIAPingMessage pingMessage = new GIAPingMessage((GIAOverlayID) this.getComponent().getOverlayID(), receiver.getOverlayID(), this.ttl, this.hops, this.descriptor);
			this.getComponent().getTransLayer().send(pingMessage, receiver.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
			// inform routing table about outgoing ping
			this.getComponent().getNodesRoutingTable().outgoingPing(receiver, this.getDescriptor());
//			GIAPingOperation.log.debug("ping senden: " + this.getComponent().getOverlayID() + " " + receiver.getOverlayID() + " " + "TTL = " + ttl);
			operationFinished(true);
			
		}
		
	}

}