package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAPongMessage;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAPongOperation extends BaseOperation {

	private GIAOverlayContact contact;
	private int numberOfNeighbors;
	
	/**
	 * @param node initiator of this operation
	 * @param descriptor message ID
	 * @param numberOfNeighbors number of neighbors
	 * @param callback
	 */
	public GIAPongOperation(GIAOverlayNode node, BigInteger descriptor, int numberOfNeighbors, OperationCallback<Object> callback) {
		super(node, GIAConfiguration.GIA_PING_MAX_TTL, GIAConfiguration.GIA_PING_MAX_TTL, 0, descriptor, callback);
		this.contact = node.getOwnContactData();
		this.numberOfNeighbors = numberOfNeighbors;
	}

	/**
	 * @param node initiator of this operation
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param contact contact info of sender
	 * @param numberOfNeighbors number of neighbors
	 * @param callback
	 */
	public GIAPongOperation(GIAOverlayNode node, int ttl, int hops, BigInteger descriptor, GIAOverlayContact contact, int numberOfNeighbors, OperationCallback<Object> callback) {
		super(node, GIAConfiguration.GIA_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.contact = contact;
		this.numberOfNeighbors = numberOfNeighbors;
	}

	/**
	 * @return contact info
	 */
	public GIAOverlayContact getContact() {
		return contact;
	}
	
	@Override
	protected void execute() {
		GIAOverlayContact contactReceiver =  this.getComponent().getNodesRoutingTable().outgoingPong(this.descriptor);
		if(contactReceiver != null) {
			GIAPongMessage message = new GIAPongMessage((GIAOverlayID) this.getComponent().getOverlayID(), contactReceiver.getOverlayID(), this.ttl, this.hops, this.descriptor, this.contact, this.numberOfNeighbors);
			this.getComponent().getTransLayer().send(message, contactReceiver.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		}
		operationFinished(true);
	}

}
