package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryHitMessage;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAQueryHitOperation extends BaseOperation {

	private GIAOverlayContact contact;
	private int numberOfNeighbors;
	private GIAOverlayContact initiator;
	private int hopsOfQuery;
	

	/**
	 * @param component initiator of this operation
	 * @param descriptor message ID
	 * @param contact contact info of the owner of the document
	 * @param numberOfNeighbors number of neighbors of the owner of the document
	 * @param initiator contact info of the initiator of the query
	 * @param callback
	 */
	public GIAQueryHitOperation(GIAOverlayNode component, BigInteger descriptor, GIAOverlayContact contact, int numberOfNeighbors, GIAOverlayContact initiator, int hopsOfQuery, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_QUERY_MAX_TTL, GIAConfiguration.GIA_QUERY_MAX_TTL, hopsOfQuery, descriptor, callback);
		this.contact = contact;
		this.initiator = initiator;
		this.hopsOfQuery = hopsOfQuery;
	}

	/**
	 * @param component initiator of this operation
	 * @param ttl
	 * @param hops
	 * @param descriptor message ID
	 * @param contact contact info of the owner of the document
	 * @param initiator contact info of the initiator of the query
	 * @param callback
	 */
	public GIAQueryHitOperation(GIAOverlayNode component, int ttl, int hops, BigInteger descriptor, GIAOverlayContact contact, GIAOverlayContact initiator, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_QUERY_MAX_TTL, ttl, hops, descriptor, callback);
		this.contact = contact;
		this.initiator = initiator;
	}

	@Override
	protected void execute() {
		GIAQueryHitMessage message = new GIAQueryHitMessage((GIAOverlayID) this.getComponent().getOverlayID(), initiator.getOverlayID(), this.ttl, this.hops, this.descriptor, this.contact, this.numberOfNeighbors);
		this.getComponent().getTransLayer().send(message, initiator.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		operationFinished(true);
	}

}