package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gia.FileSharingKey;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryIDMessage;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAQueryIDOperation extends BaseOperation {
	
	private OverlayID exclude;
	private Integer id;
	private GIAOverlayContact initiator;

	/**
	 * @param component initiator of this operation
	 * @param key document searched for
	 * @param callback
	 */
	public GIAQueryIDOperation(GIAOverlayNode component, Integer id, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_QUERY_MAX_TTL, GIAConfiguration.GIA_QUERY_MAX_TTL, 0, null, callback);
		this.exclude = component.getOverlayID();
		// inform node about new Query in order to get results
		component.registerQuery(this.getDescriptor(), id);
		this.id = id;
		initiator = component.getOwnContactData();
	}

	/**
	 * @param component initiator of this operation
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param exclude node the query came from
	 * @param id OverlayID searched for
	 * @param initiator contact info of the initiator
	 * @param callback
	 */
	public GIAQueryIDOperation(GIAOverlayNode component, int ttl, int hops, BigInteger descriptor, GIAOverlayID exclude, Integer id, GIAOverlayContact initiator, OperationCallback<Object> callback) {
		super(component, GIAConfiguration.GIA_QUERY_MAX_TTL, ttl, hops, descriptor, callback);
		this.exclude = exclude;
		this.id = id;
		this.initiator = initiator;
	}

	@Override
	protected void execute() {
		if(ttl > 0) {
			List<GIAOverlayContact> querySentTo = this.getComponent().getNodesRoutingTable().getQuerySentTo(this.getDescriptor());
			List<Integer> sortedCapacities = this.getComponent().getNodesRoutingTable().getSortedCapacities();
			boolean queryMessageSent = false;
			if(querySentTo.size() == this.getComponent().getNodesRoutingTable().numberOfActiveContacts()){
				querySentTo.clear();
				this.getComponent().getNodesRoutingTable().flushQuerySentTo(this.getDescriptor());
			}
			else if((querySentTo.size() + 1 == this.getComponent().getNodesRoutingTable().numberOfActiveContacts()) && !querySentTo.contains(exclude)){
				exclude = null;
			}
			//for(Integer capacity : sortedCapacities){
			for(int i = sortedCapacities.size() - 1; i >= 0; i--){	
//				System.out.println(Simulator.getCurrentTime()+ " ID: " +this.getComponent().getOverlayID()+ " with " +this.getComponent().getNumberOfNeighbors() +" chosen capacity: " +sortedCapacities.get(i));
//				for(int j = 0; j < sortedCapacities.size(); j++){
//					System.out.println("capacity: " +sortedCapacities.get(j));
//				}
				int capacity = sortedCapacities.get(i);
				List<GIAOverlayContact> nodesWithCapacity = this.getComponent().getNodesRoutingTable().getNodesWithCapacity(capacity);
				for(GIAOverlayContact contact : nodesWithCapacity){
					if(!queryMessageSent){
						//GIAOverlayContact contact = nodesWithCapacity.get(i);
						if(!querySentTo.contains(contact) && !contact.getOverlayID().equals(exclude) && this.getComponent().canISend(contact.getOverlayID()) && (Simulator.getCurrentTime() - this.getComponent().getLastMessageReceived(contact.getOverlayID()) < GIAConfiguration.GIA_SCHEDULEPING + 2000000)){
							GIAQueryIDMessage message = new GIAQueryIDMessage((GIAOverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor, this.id, this.initiator);
							this.getComponent().getTransLayer().send(message, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
							this.getComponent().lastQuerySent(contact.getOverlayID());
							// inform routing table about outgoing query
							this.getComponent().getNodesRoutingTable().outgoingQuery(contact, this.getDescriptor());
							this.getComponent().getNodesRoutingTable().setQuerySentTo(this.getDescriptor(), contact);
							queryMessageSent = true;
							//System.out.println(Simulator.getCurrentTime()+ " " + this.getComponent().getOverlayID()+ " to " +contact.getOverlayID()+ " for " +this.key+ " Query hops " + hops);
						}
					}	
				}
			}
		}
		operationFinished(true);
	}
}
