package de.tud.kom.p2psim.impl.overlay.gia.Operations;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gia.FileSharingKey;
import de.tud.kom.p2psim.impl.overlay.gia.GIAConfiguration;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayID;
import de.tud.kom.p2psim.impl.overlay.gia.GIAOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gia.Messages.GIAQueryMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Robert Schadler
 * @author Jonas K�hne
 * @version 3.0, 22.06.2008
 *
 */
public class GIAQueryOperation extends BaseOperation {
	
	private OverlayID exclude;
	private Integer key;
	private int maxReplies;
	private List<GIAOverlayID> queryHitSent;
	private GIAOverlayContact initiator;
	private boolean biasedRandomWalk;

	/**
	 * @param component initiator of this operation
	 * @param key document searched for
	 * @param callback
	 */
	public GIAQueryOperation(GIAOverlayNode component, Integer key, int maxTTL, boolean biasedRandomWalk, OperationCallback<Object> callback) {
		super(component, maxTTL, maxTTL, 0, null, callback);
		this.exclude = component.getOverlayID();
		// inform node about new Query in order to get results
		component.registerQuery(this.getDescriptor(), key);
		this.key = key;
		this.maxReplies = GIAConfiguration.GIA_MAXREPLIES;
		this.queryHitSent = new LinkedList<GIAOverlayID>();
		initiator = component.getOwnContactData();
		this.biasedRandomWalk = biasedRandomWalk;

	}

	/**
	 * @param component initiator of this operation
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param exclude node the query came from
	 * @param key document searched for
	 * @param maxReplies number of max replies
	 * @param queryHitSent list of nodes already sent a query hit
	 * @param initiator contact info of the initiator
	 * @param callback
	 */
	public GIAQueryOperation(GIAOverlayNode component, int ttl, int hops, BigInteger descriptor, GIAOverlayID exclude, Integer key, int maxReplies, List<GIAOverlayID> queryHitSent, GIAOverlayContact initiator, int maxTTL, boolean biasedRandomWalk, OperationCallback<Object> callback) {
		super(component, maxTTL, ttl, hops, descriptor, callback);
		this.exclude = exclude;
		this.key = key;
		this.maxReplies = maxReplies;
		this.queryHitSent = queryHitSent;
		this.initiator = initiator;
		this.biasedRandomWalk = biasedRandomWalk;
	}

	@Override
	protected void execute() {
		if(ttl > 0) {
			if(biasedRandomWalk){
				List<GIAOverlayContact> querySentTo = this.getComponent().getNodesRoutingTable().getQuerySentTo(this.getDescriptor());
				List<Integer> sortedCapacities = this.getComponent().getNodesRoutingTable().getSortedCapacities();
				boolean queryMessageSent = false;
				if(querySentTo.size() == this.getComponent().getNodesRoutingTable().numberOfActiveContacts()){
					querySentTo.clear();
					this.getComponent().getNodesRoutingTable().flushQuerySentTo(this.getDescriptor());
				}
				else if((querySentTo.size() + 1 == this.getComponent().getNodesRoutingTable().numberOfActiveContacts()) && !querySentTo.contains(exclude)){
					exclude = null;
				}
				for(int i = sortedCapacities.size() - 1; i >= 0; i--){	
	//				System.out.println(Simulator.getCurrentTime()+ " ID: " +this.getComponent().getOverlayID()+ " with " +this.getComponent().getNumberOfNeighbors() +" chosen capacity: " +sortedCapacities.get(i));
	//				for(int j = 0; j < sortedCapacities.size(); j++){
	//					System.out.println("capacity: " +sortedCapacities.get(j));
	//				}
					int capacity = sortedCapacities.get(i);
					List<GIAOverlayContact> nodesWithCapacity = this.getComponent().getNodesRoutingTable().getNodesWithCapacity(capacity);
					for(GIAOverlayContact contact : nodesWithCapacity){
						if(!queryMessageSent){
							//GIAOverlayContact contact = nodesWithCapacity.get(i);
							if(!querySentTo.contains(contact) && !contact.getOverlayID().equals(exclude) && this.getComponent().canISend(contact.getOverlayID()) && (Simulator.getCurrentTime() - this.getComponent().getLastMessageReceived(contact.getOverlayID()) < GIAConfiguration.GIA_SCHEDULEPING + 2000000)){
								GIAQueryMessage message = new GIAQueryMessage((GIAOverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor, this.key, this.maxReplies, this.queryHitSent, this.initiator);
								this.getComponent().getTransLayer().send(message, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
								this.getComponent().lastQuerySent(contact.getOverlayID());
								// inform routing table about outgoing query
								this.getComponent().getNodesRoutingTable().outgoingQuery(contact, this.getDescriptor());
								this.getComponent().getNodesRoutingTable().setQuerySentTo(this.getDescriptor(), contact);
								queryMessageSent = true;
								System.out.println(Simulator.getCurrentTime()+ " " + this.getComponent().getOverlayID()+ " to " +contact.getOverlayID()+ " for " +this.key+ " Query hops " + hops);
							}
						}	
					}
				}
			}
			else{
				for(GIAOverlayContact contact : this.getComponent().getNodesRoutingTable().allContacts()){
					if(!(contact.getOverlayID() == exclude)){
						GIAQueryMessage message = new GIAQueryMessage((GIAOverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor, this.key, this.maxReplies, this.queryHitSent, this.initiator);
						this.getComponent().getTransLayer().send(message, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
						this.getComponent().lastQuerySent(contact.getOverlayID()); 
						// inform routing table about outgoing query
						this.getComponent().getNodesRoutingTable().outgoingQuery(contact, this.getDescriptor());
						this.getComponent().getNodesRoutingTable().setQuerySentTo(this.getDescriptor(), contact);
						
					}
				}
			}
		}
		operationFinished(true);	
	}
}
