package de.tud.kom.p2psim.impl.overlay.gnutella.operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaConfiguration;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayNode;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public abstract class BaseOperation extends AbstractOperation<GnutellaOverlayNode, Object> {

	protected int ttl;
	protected int hops;
	protected BigInteger descriptor;

	protected BaseOperation(GnutellaOverlayNode component, int maxTTL, int ttl, int hops, BigInteger descriptor, OperationCallback<Object> callback) {
		super(component, callback);
		this.ttl = ttl;
		this.hops = hops;
		this.descriptor = descriptor;
		
		if(this.descriptor == null) {
			this.descriptor = GnutellaConfiguration.generateDescriptor((BigInteger) component.getOverlayID().getUniqueValue());
		}

		// don't process if negative hop count
		if(this.hops < 0) {
			this.hops = 0;
			this.ttl = 0;
		}
		
		// decrease ttl, if higher than maximal ttl
		if(this.hops + this.ttl > maxTTL) {
			this.ttl = maxTTL - this.hops;
		}
	}
	
	public BigInteger getDescriptor() {
		return this.descriptor;
	}

	@Override
	public Object getResult() {
		return this;
	}
}
