package de.tud.kom.p2psim.impl.overlay.gnutella.operations;

import java.math.BigInteger;
import java.util.List;

import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.common.AbstractOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaConfiguration;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayRoutingTable;
import de.tud.kom.p2psim.impl.overlay.gnutella.messages.PingMessage;

public class PingOperation extends BaseOperation {
	
	private OverlayID exclude;
	private OverlayContact<GnutellaOverlayID> singleContact = null;

	public PingOperation(GnutellaOverlayNode component, OperationCallback<Object> callback) {
		super(component, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, 0, null, callback);
		this.exclude = component.getOverlayID();
		((GnutellaOverlayRoutingTable) component.getRoutingTable()).initiatedPing((GnutellaOverlayID) component.getOverlayID(), this.getDescriptor());
	}
	
	public PingOperation(GnutellaOverlayNode component, OverlayContact<GnutellaOverlayID> singleContact, OperationCallback<Object> callback) {
		super(component, 1, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, 0, null, callback);
		this.singleContact = singleContact;
		((GnutellaOverlayRoutingTable) component.getRoutingTable()).initiatedPing((GnutellaOverlayID) component.getOverlayID(), this.getDescriptor());
	}
	
	public PingOperation(GnutellaOverlayNode component, int ttl, int hops, BigInteger descriptor, GnutellaOverlayID exclude, OperationCallback<Object> callback) {
		super(component, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.exclude = exclude;
	}
	
	@Override
	protected void execute() {
//		if (this.getComponent().isActive()){
			if(ttl > 0) {
				if(singleContact == null) {
					for (OverlayContact<GnutellaOverlayID> contact : ((List<OverlayContact<GnutellaOverlayID>>) this.getComponent().getRoutingTable().allContacts())) {
						// prevent ping from being sent back
						if(!this.exclude.equals(contact.getOverlayID())) {
							pingContact(contact);
						}
					}
				} else {
					pingContact(singleContact);
				}
//			}
		}
	}
	
	private void pingContact(OverlayContact<GnutellaOverlayID> contact) {
		PingMessage pingMessage = new PingMessage((GnutellaOverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor);
		this.getComponent().getTransLayer().send(pingMessage, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		// inform routing table about outgoing ping
		((GnutellaOverlayRoutingTable) getComponent().getRoutingTable()).outgoingPing(contact, this.getDescriptor());
	}

}
