package de.tud.kom.p2psim.impl.overlay.gnutella.operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaConfiguration;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayRoutingTable;
import de.tud.kom.p2psim.impl.overlay.gnutella.messages.PongMessage;

public class PongOperation extends BaseOperation {

	private OverlayContact<GnutellaOverlayID> contact;

	public PongOperation(GnutellaOverlayNode node, BigInteger descriptor, OperationCallback<Object> callback) {
		super(node, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, 0, descriptor, callback);
		this.contact = new GnutellaOverlayContact((GnutellaOverlayID) node.getOverlayID(), node.getTransLayer().getLocalTransInfo(node.getPort()));
	}

	public PongOperation(GnutellaOverlayNode node, int ttl, int hops, BigInteger descriptor, OverlayContact<GnutellaOverlayID> contact, OperationCallback<Object> callback) {
		super(node, GnutellaConfiguration.GNUTELLA_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.contact = contact;
	}

	public OverlayContact<GnutellaOverlayID> getContact() {
		return contact;
	}
	
	@Override
	protected void execute() {
		OverlayContact<GnutellaOverlayID> contactReceiver = ((GnutellaOverlayRoutingTable) getComponent().getRoutingTable()).outgoingPong(this.descriptor);
		if(contactReceiver != null) {
			PongMessage message = new PongMessage((GnutellaOverlayID) this.getComponent().getOverlayID(), contactReceiver.getOverlayID(), this.ttl, this.hops, this.descriptor, this.contact);
			this.getComponent().getTransLayer().send(message, contactReceiver.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		}
	}

}
