package de.tud.kom.p2psim.impl.overlay.gnutella.operations;

import java.math.BigInteger;
import java.util.List;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaConfiguration;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayRoutingTable;
import de.tud.kom.p2psim.impl.overlay.gnutella.filesharing.FilesharingKey;
import de.tud.kom.p2psim.impl.overlay.gnutella.messages.QueryMessage;

public class QueryOperation extends BaseOperation {
	
	private OverlayID exclude;
	private FilesharingKey key;

	public QueryOperation(GnutellaOverlayNode component, FilesharingKey key, OperationCallback<Object> callback) {
		super(component, GnutellaConfiguration.GNUTELLA_QUERY_MAX_TTL, GnutellaConfiguration.GNUTELLA_QUERY_MAX_TTL, 0, null, callback);
		this.exclude = component.getOverlayID();
		// inform node about new Query in order to get results
		component.registerQuery(this.getDescriptor(), key);
		this.key = key;
	}

	public QueryOperation(GnutellaOverlayNode component, int ttl, int hops, BigInteger descriptor, GnutellaOverlayID exclude, FilesharingKey key, OperationCallback<Object> callback) {
		super(component, GnutellaConfiguration.GNUTELLA_QUERY_MAX_TTL, ttl, hops, descriptor, callback);
		this.exclude = component.getOverlayID();
		this.key = key;
	}

	@Override
	protected void execute() {
//		if(this.getComponent().isActive()){
			if(ttl > 0) {
				for (OverlayContact<GnutellaOverlayID> contact : (List<OverlayContact<GnutellaOverlayID>>) this.getComponent().getRoutingTable().allContacts()) {
					// prevent ping from being sent back
					if(!this.exclude.equals(contact.getOverlayID())) {
						QueryMessage message = new QueryMessage((GnutellaOverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor, this.key);
						this.getComponent().getTransLayer().send(message, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
						// inform routing table about outgoing query
						((GnutellaOverlayRoutingTable) getComponent().getRoutingTable()).outgoingQuery(contact, this.getDescriptor());
					}
//				}
			}
		}
	}
}
