package de.tud.kom.p2psim.impl.overlay.gnutella06.Analyzer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigInteger;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tud.kom.p2psim.api.analyzer.Analyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.OperationAnalyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.TransAnalyzer;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06BootstrapManager;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06FileFactory;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PingMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryHitMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.BaseOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.Gnutella06ConnectOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.Gnutella06PingOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.Gnutella06QueryHitOperation;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.Gnutella06QueryOperation;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.AbstractTransMessage;

public class Gnutella06Analyzer implements Analyzer, TransAnalyzer, OperationAnalyzer{
	
	private Map<BigInteger,Double> lookupHopCount;
	private Map<BigInteger,Long> lookupStarted;
	private Map<BigInteger,Long> lookupDelay;
	private Map<BigInteger,Boolean> dataOnline;
	private Map<BigInteger,Long> lookupReplyDelay;
	private Map<BigInteger,Double> relativDelayPenalty;
	private Map<Gnutella06OverlayID, Integer> loadBalancePeers;
	private Map<Gnutella06OverlayID, Integer> loadBalanceSuperpeers;
	private List<BigInteger> opIDs;
	private List<Double> loadBalanceSuper;
	private List<Double> loadBalance;
	private double lookups;
	private double succeeded;
	private long totalDelay;
	private double successRate;
	private Map<Gnutella06OverlayID, Gnutella06OverlayNode> nodes;
	private List<Gnutella06OverlayID> superpeers;
	private List<Gnutella06OverlayID> peers;
	private Map<Long, Double> hopsPerSecond;
	private Map<Long, Long> delayPerSecond;
	private Map<Long, Double> rdpPerSecond;
	private Map<Long, Double> successPerSecond;
	private Map<Long, Integer> queriesPerSecond;
	private List<Long> seconds;
	private double hopCount;
	private double numberOfMessages;
	private double numberOfQueryMessages;
	private int maxHops;
	private double rdp;
	private double loadBalanceRatioSuper;
	private double loadBalanceRatio;

	public void start() {
		lookupHopCount = new HashMap<BigInteger, Double>();
		lookupDelay = new HashMap<BigInteger, Long>();
		lookupStarted = new HashMap<BigInteger, Long>();
		dataOnline = new HashMap<BigInteger, Boolean>();
		loadBalancePeers = new HashMap<Gnutella06OverlayID, Integer>();
		loadBalanceSuperpeers = new HashMap<Gnutella06OverlayID, Integer>();
		lookupReplyDelay = new HashMap<BigInteger, Long>();
		relativDelayPenalty = new HashMap<BigInteger, Double>();
		hopsPerSecond = new HashMap<Long, Double>();
		delayPerSecond = new HashMap<Long, Long>();
		rdpPerSecond = new HashMap<Long, Double>();
		successPerSecond = new HashMap<Long, Double>();
		queriesPerSecond = new HashMap<Long, Integer>();
		seconds = new LinkedList<Long>();
		opIDs = new LinkedList<BigInteger>();
		loadBalanceSuper = new LinkedList<Double>();
		loadBalance = new LinkedList<Double>();
		lookups = 0;
		succeeded = 0;
		nodes = new HashMap<Gnutella06OverlayID, Gnutella06OverlayNode>();
		superpeers = new LinkedList<Gnutella06OverlayID>();
		peers = new LinkedList<Gnutella06OverlayID>();
		numberOfMessages = 0;
		numberOfQueryMessages = 0;
		maxHops = 0;
		loadBalanceRatioSuper = 0;
		loadBalanceRatio = 0;
	}

	public void stop(Writer output) {
		/*for(Gnutella06OverlayNode node : nodes){
			System.out.println(node.getOverlayID().toString()+ " is SuperPeer " +node.isSuperpeer() + " is connected to:");
			for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().allContacts()){
				System.out.println("Superpeer: " + contact.getOverlayID().toString());
			}
			for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().inactiveContacts()){
				System.out.println("inactive Superpeer: " + contact.getOverlayID().toString());
			}
			if(node.isSuperpeer()){
				for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().activeLeafContacts()){
					System.out.println("Peer: " + contact.getOverlayID().toString());
				}
				for(Gnutella06OverlayContact contact : node.getNodesRoutingTable().inactiveLeafContacts()){
					System.out.println("inactive Peer: " + contact.getOverlayID().toString());
				}
			}
		}*/
		
		
		int counter = 1;
		for(BigInteger opID : opIDs){
			if(lookupDelay.get(opID) != null){
				System.out.println(counter + " Hops: " +lookupHopCount.get(opID)+ " Delay: " + lookupDelay.get(opID));
				totalDelay += lookupDelay.get(opID);
				counter++;
			}
			if(lookupHopCount.get(opID) !=null){
				hopCount += lookupHopCount.get(opID);
			}
			if(lookupReplyDelay.get(opID) != null && lookupDelay.get(opID) != null){
				double delayPenalty = (double)lookupDelay.get(opID)/(double)lookupReplyDelay.get(opID);
				relativDelayPenalty.put(opID, delayPenalty);
				rdp += delayPenalty;
			}
		}
		if(succeeded != 0){
			successRate = (succeeded/lookups)*100;
			System.out.println("Lookups: " +lookups+ " Succeeded; " +succeeded+ " Rate: " +successRate);
			System.out.println("Average Delay: " + totalDelay/lookupDelay.size());
			System.out.println("Average Hops: " + hopCount/lookupHopCount.size());
		}
			
		double superpeerMessages = 0;
		for(Gnutella06OverlayID peer : superpeers){
			if(loadBalanceSuperpeers.containsKey(peer)){
				loadBalanceSuper.add((double)loadBalanceSuperpeers.get(peer));
				superpeerMessages += loadBalanceSuperpeers.get(peer);
			}
		}
		Collections.sort(loadBalanceSuper);
		
		double peerMessages = 0;
		for(Gnutella06OverlayID peer : peers){
			if(loadBalancePeers.containsKey(peer)){
				loadBalance.add((double)loadBalancePeers.get(peer));
				peerMessages += loadBalancePeers.get(peer);
			}
		}
		Collections.sort(loadBalance);
		
		for(BigInteger opID: opIDs){
			long second = lookupStarted.get(opID)/10000000;
			if(second >= 5){
				if(!seconds.contains(second)){
					seconds.add(second);
				}
				if(queriesPerSecond.containsKey(second)){
					queriesPerSecond.put(second, queriesPerSecond.get(second) + 1);
				}
				else{
					queriesPerSecond.put(second, 1);
				}
				if(lookupHopCount.containsKey(opID)){
					if(hopsPerSecond.containsKey(second)){
						double temp = hopsPerSecond.get(second) + (double)lookupHopCount.get(opID);
						hopsPerSecond.put(second, temp);
					}
					else{
						hopsPerSecond.put(second, (double)lookupHopCount.get(opID));
					}
				}
				if(lookupDelay.containsKey(opID)){
					if(delayPerSecond.containsKey(second)){
						delayPerSecond.put(second, delayPerSecond.get(second) + lookupDelay.get(opID));
					}
					else{
						delayPerSecond.put(second, lookupDelay.get(opID));
					}
				}
				if(relativDelayPenalty.containsKey(opID)){
					if(rdpPerSecond.containsKey(second)){
						rdpPerSecond.put(second, rdpPerSecond.get(second) + relativDelayPenalty.get(opID));
					}
					else{
						rdpPerSecond.put(second, relativDelayPenalty.get(opID));
					}
				}
				if(lookupHopCount.containsKey(opID)){
					if(successPerSecond.containsKey(second)){
						successPerSecond.put(second, successPerSecond.get(second) + 1);
					}
					else{
						successPerSecond.put(second, 1.0);
					}
				}
			}
		}
		
		successRate = 0;
		for(long second : seconds){
			if(second >= 5){
				if(successPerSecond.get(second) != null){
					hopsPerSecond.put(second, hopsPerSecond.get(second)/successPerSecond.get(second));
					delayPerSecond.put(second, delayPerSecond.get(second)/(int)(1000*successPerSecond.get(second)));
					rdpPerSecond.put(second, rdpPerSecond.get(second)/successPerSecond.get(second));
					successPerSecond.put(second, (100*successPerSecond.get(second))/queriesPerSecond.get(second));
					successRate += successPerSecond.get(second);
				}
			}
		}
		successRate /= seconds.size();
		
		loadBalanceRatioSuper = loadBalanceSuper.get(loadBalanceSuper.size() - 1)/loadBalanceSuper.get(loadBalanceSuper.size()/2);
		loadBalanceRatio = loadBalance.get(loadBalance.size() - 1)/loadBalance.get(loadBalance.size()/2);
		double percentageOfQueries = numberOfQueryMessages/numberOfMessages;
		
		String f = "Gnutella06-Analyzerausgabe1.dat";
	    FileWriter fstream = null;
		try {
			fstream = new FileWriter(f);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write("	Gnutella06-Analyzer	\n");
			out.write("	-----------------------	\n");
			out.write("	\n");
			out.write("	\n");
			out.write("number of nodes "+nodes.size()+" \n");
			out.write("	\n");
			out.write("Lookups: " +lookups+ " Succeeded; " +succeeded+ " Rate: " +successRate + " \n");
			out.write("	\n");
			out.write("Average Delay: " + totalDelay/lookupDelay.size() + " \n");
			out.write("	\n");
			out.write("Average Hops: " + hopCount/lookupHopCount.size() + " \n");
			out.write("	\n");
			out.write("Percentage of Queries: " +percentageOfQueries+"\n");
			out.write("	\n");
			out.write("QueryMessages/Query: " +numberOfQueryMessages/lookups+" \n");
			out.write("	\n");
			out.write("Average RDP: " + rdp/relativDelayPenalty.size()+" \n");
			out.write("	\n");
			out.write("Load Balance Ratio Super: " + loadBalanceRatioSuper + " \n");
			out.write(" \n");
			out.write("Load Balance Ratio: " + loadBalanceRatio + " \n");
			out.write(" \n");
			for(BigInteger opID : opIDs){
				if(lookupDelay.get(opID) != null){
					out.write(lookupStarted.get(opID) + " Hops: " +lookupHopCount.get(opID)+ " Delay: " + lookupDelay.get(opID) + " RDP: " +relativDelayPenalty.get(opID) +" \n");
					totalDelay += lookupDelay.get(opID);
					counter++;
				}
				if(lookupHopCount.get(opID) !=null){
					hopCount += lookupHopCount.get(opID);
				}
			}
			out.write("	\n");
			out.write("Failed Queries: \n");
			out.write("	\n");
			for(BigInteger opID : opIDs){
				if(lookupStarted.containsKey(opID) && !lookupDelay.containsKey(opID)){
					out.write(lookupStarted.get(opID) + " data available: " +dataOnline.get(opID)+" \n");
				}
			}
			out.write("second   success \n");
			for(long second : seconds){
				out.write(second*10+ "  " +successPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   hops \n");
			for(long second : seconds){
				out.write(second*10+ "  " +hopsPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   delay \n");
			for(long second : seconds){
				out.write(second *10+ "  " +delayPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   rdp \n");
			for(long second : seconds){
				out.write(second*10+ "  " +rdpPerSecond.get(second)+ " \n");
			}
			out.write("	\n");
			out.write("second   queries \n");
			for(long second : seconds){
				out.write(second*10+ " " +queriesPerSecond.get(second) + " \n");
			}
			out.write("	\n");
			out.write("Loadbalance Superpeers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalanceSuper.size(); i++){
				double percent = (loadBalanceSuper.get(loadBalanceSuper.size()- 1 - i)/peerMessages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average Superpeer load: "+superpeerMessages/loadBalanceSuperpeers.size()+ " \n");
			out.write("	\n");
			out.write("Loadbalance Peers \n");
			out.write("Peer   Messages \n");
			for(int i=0; i<loadBalance.size(); i++){
				double percent = (loadBalance.get(loadBalance.size()- 1 - i)/peerMessages) * 100;
				out.write((i + 1)+"       " +percent+" \n");
		
			}
			out.write("	\n");
			out.write("Average Peer load: "+peerMessages/loadBalancePeers.size()+ " \n");
			out.write(" \n");
			for(Gnutella06OverlayID id : superpeers){
				if(nodes.get(id).isSuperpeerCapable() && !nodes.get(id).isSuperpeer()){
					out.write(id+ "is not Superpeer \n");
				}
			}
			out.write("Diameter: " + maxHops);
			out.close();         
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void transMsgReceived(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		AbstractOverlayMessage<Gnutella06OverlayID> abMessage = (AbstractOverlayMessage<Gnutella06OverlayID>) message;
		if(superpeers.contains(abMessage.getReceiver())){
			if(loadBalanceSuperpeers.containsKey(abMessage.getReceiver())){
				loadBalanceSuperpeers.put(abMessage.getReceiver(), loadBalanceSuperpeers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalanceSuperpeers.put(abMessage.getReceiver(), 1);
		}
		else if(peers.contains(abMessage.getReceiver())){
			if(loadBalancePeers.containsKey(abMessage.getReceiver())){
				loadBalancePeers.put(abMessage.getReceiver(), loadBalancePeers.get(abMessage.getReceiver()) + 1);
			}
			else
				loadBalancePeers.put(abMessage.getReceiver(), 1);
		}
		
		if (message instanceof Gnutella06QueryHitMessage){
			Gnutella06QueryHitMessage reply = (Gnutella06QueryHitMessage)message.getPayload();
			if(opIDs.contains(reply.getDescriptor()) && lookupStarted.containsKey(reply.getDescriptor()) && !lookupDelay.containsKey(reply.getDescriptor())){
				lookupHopCount.put(reply.getDescriptor(), (double)reply.getHops());
				lookupDelay.put(reply.getDescriptor(), Simulator.getCurrentTime() - lookupStarted.get(reply.getDescriptor()));
				succeeded++;
				long temp = lookupReplyDelay.get(reply.getDescriptor());
				lookupReplyDelay.put(reply.getDescriptor(), Simulator.getCurrentTime() - temp);
			}
		}
		else if(message instanceof Gnutella06PingMessage){
			Gnutella06PingMessage ping = (Gnutella06PingMessage)message.getPayload();
			
			if(ping.getHops() >= maxHops){
				maxHops = ping.getHops();
			}
		}
	}

	public void transMsgSent(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		numberOfMessages++;
		
		if (message instanceof Gnutella06QueryMessage){
			numberOfQueryMessages++;
			Gnutella06QueryMessage query = (Gnutella06QueryMessage)message.getPayload();
			BigInteger opID = query.getDescriptor();
			if(!opIDs.contains(opID)){
				lookupStarted.put(opID, Simulator.getCurrentTime());
				opIDs.add(opID);
				lookups++;
				if(Gnutella06FileFactory.getInstance().getFileSharingKeys().contains(query.getKey())){
					dataOnline.put(opID, true);
				}
				else {
					dataOnline.put(opID, false);
				}	
			}
		}
		else if (message instanceof Gnutella06QueryHitMessage){
			Gnutella06QueryHitMessage reply = (Gnutella06QueryHitMessage)message.getPayload();
			lookupReplyDelay.put(reply.getDescriptor(), Simulator.getCurrentTime());
		}
		
	}

	public void operationFinished(Operation<?> op) {
		// TODO Auto-generated method stub
		
	}

	public void operationInitiated(Operation<?> op) {
		if(op instanceof Gnutella06PingOperation){
			BaseOperation operation = (BaseOperation) op;
			if(!nodes.containsKey((Gnutella06OverlayID)operation.getComponent().getOverlayID())){
				nodes.put((Gnutella06OverlayID)operation.getComponent().getOverlayID(), operation.getComponent());
				if(operation.getComponent().isSuperpeer()){
					superpeers.add((Gnutella06OverlayID)operation.getComponent().getOverlayID());
				}
				else {
					peers.add((Gnutella06OverlayID)operation.getComponent().getOverlayID());
				}
			}
		}
		else if(op instanceof Gnutella06ConnectOperation){
			Gnutella06ConnectOperation operation = (Gnutella06ConnectOperation) op;
			if(!nodes.containsKey((Gnutella06OverlayID)operation.getComponent().getOverlayID())){
				nodes.put((Gnutella06OverlayID)operation.getComponent().getOverlayID(), operation.getComponent());
				if(operation.getComponent().isSuperpeer()){
					superpeers.add((Gnutella06OverlayID)operation.getComponent().getOverlayID());
				}
				else {
					peers.add((Gnutella06OverlayID)operation.getComponent().getOverlayID());
				}
			}
		}
		
		
		/*if(op instanceof Gnutella06QueryOperation){
			int opID = op.getOperationID();
			if(!lookupStarted.containsKey(opID)){
				lookupStarted.put(opID, Simulator.getCurrentTime());
				opIDs.add(opID);
				lookups++;
			}
		}
		else if (op instanceof Gnutella06QueryHitOperation){
			Gnutella06QueryHitOperation queryHit = (Gnutella06QueryHitOperation) op;
			if(lookupStarted.containsKey(queryHit.getOperationID())){
				lookupHopCount.put(queryHit.getOperationID(), queryHit.getHops());
				lookupDelay.put(queryHit.getOperationID(), Simulator.getCurrentTime() - lookupStarted.get(queryHit.getOperationID()));
				lookupStarted.remove(queryHit.getOperationID());
				succeeded++;
			}
		}*/
	}

}
