package de.tud.kom.p2psim.impl.overlay.gnutella06.Analyzer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigInteger;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tud.kom.p2psim.api.analyzer.Analyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.TransAnalyzer;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06ByeMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06ConnectAcceptMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06ConnectMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06ConnectionDeniedMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06DocumentsMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PingMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PongMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryHitMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.AbstractTransMessage;

public class Gnutella06MessageAnalyzer implements Analyzer, TransAnalyzer {

	private Map<BigInteger, Integer> queryHopCount;
	private Map<BigInteger, List<Integer>> queryHitCount;
//	private Map<Long, BigInteger> queryHopsWithTime;
//	private Map<Long, BigInteger> queryHitsWithTime;
	private List<BigInteger> queryIds;
	private List<BigInteger> queryHitIds;
	private List<BigInteger> querySucceed;
	private List<BigInteger> queryFailed;
	private static double queryAnteil;
	private static double queryHitAnteil;
	private static double pingAnteil;
	private static double pongAnteil;
	private static double tokenAnteil;
	private static double documentsAnteil;
	private static double connectAnteil;
	private static double connectAcceptAnteil;
	private static double connectionDeniedAnteil;
	private static double connectionBreakAnteil;
	private static double byeAnteil;
	private static double anzahlMessages;
	private static double anzahlGesendeteMails;
	private static double queryGesendetAnteil;
	private static double queryHitGesendetAnteil;
	private static double pingGesendetAnteil;
	private static double pongGesendetAnteil;
	private static double tokenGesendetAnteil;
	private static double documentsGesendetAnteil;
	private static double connectGesendetAnteil;
	private static double connectAcceptGesendetAnteil;
	private static double connectionDeniedGesendetAnteil;
	private static double connectionBreakGesendetAnteil;
	private static double byeGesendetAnteil;
	private double queryHopsAveraged;
	private double hopsWhenQueryHitAveraged;
	private int counter = 0;
	private double secCounter = 0;
	private int queryHops = 0;
	private double hopsWhenQueryHit = 0;
	private int tester;
	private BigInteger exclude;
	
	public void start() {
		queryHopCount = new HashMap<BigInteger, Integer>();
		queryHitCount = new HashMap<BigInteger, List<Integer>>();
//		queryHopsWithTime = new HashMap<Long, BigInteger>();
//		queryHitsWithTime = new HashMap<Long, BigInteger>();
		queryIds = new LinkedList<BigInteger>();
		queryHitIds = new LinkedList<BigInteger>();
		queryFailed = new LinkedList<BigInteger>();
		querySucceed = new LinkedList<BigInteger>();
	}

	public void stop(Writer output) {
		NumberFormat n = NumberFormat.getInstance();
		n.setMaximumFractionDigits(2);
		for (int i=0; i< queryIds.size(); i++){
			BigInteger query;
			query = queryIds.get(i);
			if(queryHopCount.containsKey(query)){
				queryHops = queryHops + queryHopCount.get(query);
				counter++;
				System.out.println("-	" + query.toString() + " - "+ queryHopCount.get(query).toString() + " - " + queryHops + " - " + counter);
			}
		}
		for (int i=0; i< queryHitIds.size(); i++){
			BigInteger queryHits;
			queryHits = queryHitIds.get(i);
			if(queryHitCount.containsKey(queryHits)){
				List<Integer> queryHopsTemp = queryHitCount.get(queryHits);
				for (int j=0; j< queryHopsTemp.size(); j++){
					queryHops = queryHops + queryHopsTemp.get(j);
					hopsWhenQueryHit = hopsWhenQueryHit + queryHopsTemp.get(j);
					counter++;	
					secCounter++;
//					System.out.println("-	QueryHits	-	" + queryHits.toString() + " - "+ queryHopsTemp.get(j) + " - " + hopsWhenQueryHit + " - " + secCounter);
				}
								
			}
		}
		if(counter != 0){
			queryHopsAveraged = queryHops / counter;
		}
		if(secCounter !=0){
			hopsWhenQueryHitAveraged = hopsWhenQueryHit / secCounter;
		}
		
		
		double queryPerCent = (100.0*queryAnteil)/anzahlMessages;
		double queryHitPerCent = (double)((100*queryHitAnteil)/anzahlMessages);
		double pingPerCent = (double)((100*pingAnteil)/anzahlMessages);
		double pongPerCent = (double)((100*pongAnteil)/anzahlMessages);
		double connectPerCent = (double)((100*connectAnteil)/anzahlMessages);
		double connectAcceptPerCent = (double)((100*connectAcceptAnteil)/anzahlMessages);
		double connectDeniedPerCent =(double)((100*connectionDeniedAnteil)/anzahlMessages);
		double connectBreakPerCent =(double)((100*connectionBreakAnteil)/anzahlMessages);
		double byePerCent =(double)((100*byeAnteil)/anzahlMessages);
		double tokenPerCent =(double)((100*tokenAnteil)/anzahlMessages);
		double documentsPerCent =(double)((100*documentsAnteil)/anzahlMessages);
		double queryNotSucceded = (double)((100*(double)queryFailed.size())/((double)queryIds.size()));
		double querySucceded = (double)((100*(double)queryHitIds.size())/(double)queryIds.size());

		String f = "Gnutella06-Analyzerausgabe2.dat";
	    FileWriter fstream = null;
		try {
			fstream = new FileWriter(f);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write("	Gnutella06-Analyzer	\n");
			out.write("	-----------------------	\n");
			out.write("	\n");
			out.write("	\n");
			out.write("Messages gesendet: \n");
			out.write("Gesamt		query		queryhit		ping		pong		connect		connectAccept		connectDenied		connectBreak		bye			documents		token	\n");
			out.write(anzahlGesendeteMails + "	" + queryGesendetAnteil + "		" + queryHitGesendetAnteil + "			" + pingGesendetAnteil + "		" + pongGesendetAnteil + "		" + connectGesendetAnteil + "		" + connectAcceptGesendetAnteil + "				" + connectionDeniedGesendetAnteil + "				" + connectionBreakGesendetAnteil + "				" + byeGesendetAnteil + "		" + documentsGesendetAnteil + "			" + tokenGesendetAnteil + " \n");
			out.write("	\n");
			out.write("Messages empfangen: \n");
			out.write("Gesamt		query		queryhit		ping		pong		connect		connectAccept		connectDenied		connectBreak		bye			documents		token	\n");
			out.write(anzahlMessages + "	" + queryAnteil + "		" + queryHitAnteil + "			" + pingAnteil + "		" + pongAnteil + "		" + connectAnteil + "		" + connectAcceptAnteil + "				" + connectionDeniedAnteil + "				" + connectionBreakAnteil + "				" + byeAnteil + "		" + documentsAnteil + "			" + tokenAnteil + " \n");
			out.write("			" + n.format(queryPerCent) + "		" + n.format(queryHitPerCent) + "			" + n.format(pingPerCent) + "		" + n.format(pongPerCent) + "		" + n.format(connectPerCent) + "		" + n.format(connectAcceptPerCent) + "				" + n.format(connectDeniedPerCent) + "				" + n.format(connectBreakPerCent) + "				" + n.format(byePerCent) + "		" + n.format(documentsPerCent) + "			" + n.format(tokenPerCent) + " \n");
			out.write("	\n");
			out.write("Durchschnittliche Hops			\n");
			out.write("Gesamt	QueryHit	queries		queryHits(Queries)	GesamtQueryHits		query Failed	\n");
			out.write(n.format(queryHopsAveraged) + "		" + n.format(hopsWhenQueryHitAveraged) + "				" + queryIds.size() + "				" + queryHitIds.size() +  "			" + secCounter +"					" + queryFailed.size() + "	\n");			out.write("	\n");
			out.write("	\n");
			out.write("Prozent query Erfolgreich		Prozent query gescheitert	\n");
			out.write(n.format(querySucceded ) + "							" + n.format(queryNotSucceded ) + "	\n");
			out.close();         
		} catch (IOException e) {
			e.printStackTrace();
		}                
		
		
	}

	public void transMsgReceived(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		anzahlMessages++;
					
		if (message instanceof Gnutella06ConnectMessage) {
			connectAnteil++;
			}
		else if (message instanceof Gnutella06ConnectionDeniedMessage) {
			connectionDeniedAnteil++;
			}
		else if (message instanceof Gnutella06ByeMessage) {
			byeAnteil++;
			}
		else if (message instanceof Gnutella06DocumentsMessage) {
			documentsAnteil++;
		}
		else if (message instanceof Gnutella06ConnectAcceptMessage) {
			connectAcceptAnteil++;
		}
		else if (message instanceof Gnutella06PingMessage) {
			pingAnteil++;
		} 
		else if (message instanceof Gnutella06PongMessage) {
			pongAnteil++;
		} 
		else if (message instanceof Gnutella06QueryMessage) {
			queryAnteil++;
			Gnutella06QueryMessage queryMessage = (Gnutella06QueryMessage) message.getPayload();
//			long simulatorTime = Simulator.getCurrentTime();
			BigInteger descriptor = queryMessage.getDescriptor();
			if(!queryIds.contains(descriptor)){
				queryIds.add(descriptor);
			}
			if(!queryFailed.contains(descriptor) && !querySucceed.contains(descriptor)){
				queryFailed.add(descriptor);
			}
			int hops = queryMessage.getHops();
//			int ttl = queryMessage.getTTL();

//				System.out.println("Hops = " + hops);
//				System.out.println("TTL = " + ttl);

			
			if(!queryHopCount.containsKey(descriptor)){
				queryHopCount.put(descriptor, hops);
			} else {
				int oldHops = queryHopCount.get(descriptor);
				if(oldHops < hops){
					queryHopCount.put(descriptor, hops);
				}
			 } 
			
		} 
		else if (message instanceof Gnutella06QueryHitMessage) {
			queryHitAnteil++;
			int queryHops = 0;
			Gnutella06QueryHitMessage queryHitMessage = (Gnutella06QueryHitMessage) message.getPayload();
			//ToDo wenn descriptor der QueryHitMessage bereinstimmt, dann Hops der Query Ausgeben
			long simulatorTime = Simulator.getCurrentTime();
			BigInteger descriptor = queryHitMessage.getDescriptor();
			if(queryHopCount.containsKey(descriptor)){
//				queryHops = queryHopCount.get(descriptor);
				queryHopCount.remove(descriptor);
			}
			queryHops = queryHitMessage.getHops();
//			System.out.println(queryHops);
	// TODO Map<descriptor, List<queryHops> fr mehr als ein QueryHit auf eine query
			if (!queryHitCount.containsKey(descriptor)){
				queryHitCount.put(descriptor, new LinkedList<Integer>());
			}
			queryHitCount.get(descriptor).add(queryHops);
//			System.out.println("Hops in Liste einegefgt: " + queryHops);
//			queryHitsWithTime.put(simulatorTime, descriptor);
			if (!queryHitIds.contains(descriptor)){
				queryHitIds.add(descriptor);
			}
			if(queryFailed.contains(descriptor)){
				queryFailed.remove(descriptor);
			}
			if (!querySucceed.contains(descriptor)){
				querySucceed.add(descriptor);
			}
		}
	}

	public void transMsgSent(AbstractTransMessage msg) {
		Message message = msg.getPayload();
		anzahlGesendeteMails++;
		if (message instanceof Gnutella06QueryMessage) {
			queryGesendetAnteil++;
		} else 		if (message instanceof Gnutella06ConnectMessage) {
			connectGesendetAnteil++;
		}
		else if (message instanceof Gnutella06ConnectionDeniedMessage) {
			connectionDeniedGesendetAnteil++;
		}
		else if (message instanceof Gnutella06ByeMessage) {
			byeGesendetAnteil++;
		}
		else if (message instanceof Gnutella06DocumentsMessage) {
			documentsGesendetAnteil++;
		}
		else if (message instanceof Gnutella06ConnectAcceptMessage) {
			connectAcceptGesendetAnteil++;
		}
		else if (message instanceof Gnutella06PingMessage) {
			pingGesendetAnteil++;
		} 
		else if (message instanceof Gnutella06PongMessage) {
			pongGesendetAnteil++;
		}else if (message instanceof Gnutella06QueryHitMessage) {
			queryHitGesendetAnteil++;
		}
	}
	
}
