package de.tud.kom.p2psim.impl.overlay.gnutella06;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;

/**
 * @author Jonas Khne
 * @version 3.0, 06.07.2008
 *
 */
public class Gnutella06BootstrapManager implements BootstrapManager{
	
	private List<OverlayNode> activeNodes;
	private List<OverlayNode> joinableNodes;
	private static Gnutella06BootstrapManager singletonInstance;
	
	/**
	 * create new bootstrap manager
	 */
	public Gnutella06BootstrapManager(){
		this.activeNodes = new LinkedList<OverlayNode>();
		this.joinableNodes = new LinkedList<OverlayNode>();
	}
	
	public List<TransInfo> getBootstrapInfo(){
		List<TransInfo> list = new LinkedList<TransInfo>();
		
		for(OverlayNode cNode : this.activeNodes)
			list.add(((Gnutella06OverlayNode)cNode).getTransLayer().getLocalTransInfo(cNode.getPort()));
		
		return list;
	}
	
	public List<TransInfo> getJoinableBootstrapInfo(){
		List<TransInfo> list = new LinkedList<TransInfo>();
		
		for(OverlayNode cNode : this.joinableNodes)
			list.add(((Gnutella06OverlayNode)cNode).getTransLayer().getLocalTransInfo(cNode.getPort()));
		
		return list;
	}

	/**
	 * @return unique bootstrap manager
	 */
	public static Gnutella06BootstrapManager getInstance() {
		if(singletonInstance == null) {
			singletonInstance = new Gnutella06BootstrapManager();
		}
		return singletonInstance;
	}
	
	public void registerNode(OverlayNode node){
//		if (!activeNodes.contains(node)){
			this.activeNodes.add(node);
//		}
	}
	
	public void unregisterNode(OverlayNode node){
		this.activeNodes.remove(node);
	}
	
	public void registerJoinableNodes(OverlayNode node){
		if (!joinableNodes.contains(node)){
			this.joinableNodes.add(node);
		}
	}
	
	public void unregisterJoinableNodes(OverlayNode node){
		if (joinableNodes.contains(node)){
			this.joinableNodes.remove(node);
		}
	}
	
	/**
	 * @return number of nodes in the network
	 */
	public int getSize(){
		return activeNodes.size();
	}
	
}
