package de.tud.kom.p2psim.impl.overlay.gnutella06;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.Component;
import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.stat.distributions.zipf.ZipfConfigurer;
import de.tud.kom.p2psim.impl.util.stat.distributions.zipf.ZipfDistribution;

/**
 * @author Jonas Khne
 * @version 3.0, 06.07.2008
 *
 */
public class Gnutella06FileFactory implements ComponentFactory{

	private static Gnutella06FileFactory singletonInstance;
	private List<Integer> files;
	private ZipfDistribution distribution = new ZipfDistribution();
	private ZipfConfigurer config = new ZipfConfigurer(750, 1.0);
	
	/**
	 * generate new file factory with new list of files
	 */
	public Gnutella06FileFactory() {
		this.files = new LinkedList<Integer>();
		distribution.config(config);
	}
	
	/**
	 * @return the unique file factory, if none exists make new one
	 */
	public static Gnutella06FileFactory getInstance() {
		if (singletonInstance == null) {
			singletonInstance = new Gnutella06FileFactory();
		}
		return singletonInstance;
	}
	
	
	public Component createComponent(Host host) {
		return null;
	}
	
	/**
	 * @return new file
	 */
	public Integer createNewFileSharingKey(){
		int newFile = Simulator.getRandom().nextInt(500);
		if(!files.contains(newFile)){
			files.add(newFile);
		}
		return newFile;
	}
	
	/**
	 * @return number of different files in the network
	 */
	public int getNumberOfFiles(){
		return files.size();
	}
	
	/**
	 * @param placeOfFile index of the file searched for
	 * @return key of the file
	 */
	public Integer getFileKey(int placeOfFile){
		return files.get(placeOfFile);
	}
	
	/**
	 * @return list of files in the network
	 */
	public List<Integer> getFileSharingKeys(){
		return files;
	}
	
	public ZipfDistribution getZipf(){
		return distribution;
	}

}
