package de.tud.kom.p2psim.impl.overlay.gnutella06;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * @author Jonas Khne
 * @version 3.0, 06.07.2008
 *
 */
public class Gnutella06OverlayContact implements OverlayContact<Gnutella06OverlayID>, Comparable<Gnutella06OverlayContact>{
	
	private Gnutella06OverlayID gnutella06ID;
	private TransInfo transInfo;
	private boolean isAlive;
	private boolean superpeerCapable;
	
	private long timeActivated;
	private long lastRefresh;
	
	/**
	 * @param id overlay ID of the contact
	 * @param transInfo transport info of the contact
	 */
	public Gnutella06OverlayContact(Gnutella06OverlayID id, TransInfo transInfo, boolean superpeerCapable){
		this.gnutella06ID = id;
		this.transInfo = transInfo;
		this.superpeerCapable = superpeerCapable;
	}
	
	public Gnutella06OverlayID getOverlayID(){
		return this.gnutella06ID;
	}
	
	public TransInfo getTransInfo(){
		return this.transInfo;
	}
	
	public boolean getSuperpeerCapable(){
		return this.superpeerCapable;
	}
	
	/**
	 * @param alive true if contact is in the network, false otherwise
	 */
	public void setAlive(boolean alive){
		this.isAlive = alive;
	}
	
	/**
	 * @return true if contact is in the network, false otherwise
	 */
	public boolean isAlive(){
		return this.isAlive;
	}
	
	public Gnutella06OverlayContact clone(){
		Gnutella06OverlayContact newCon = new Gnutella06OverlayContact(this.gnutella06ID, this.transInfo, this.superpeerCapable);
		newCon.setAlive(this.isAlive);
		return newCon;
	}
	
	/**
	 * @param con contact info
	 * @return distance between con and this contact
	 */
	public int getDistance(Gnutella06OverlayContact con){
		return this.gnutella06ID.getDistance(con.getOverlayID());
	}
	
	public int compareTo(Gnutella06OverlayContact contact){
		return this.gnutella06ID.compareTo(contact.getOverlayID());
	}
	
	/**
	 * @param contact overlay ID
	 * @return true if o equals overlay ID of the contact
	 */
	public boolean equals(Gnutella06OverlayContact contact){
		return this.gnutella06ID.equals(contact.getOverlayID());
	}
	
	/**
	 * sets the time this contact was activated to the current time
	 */
	public void reset() {
		this.timeActivated = Simulator.getCurrentTime();
	}
	
	/**
	 * @return rank
	 */
	public double getRank() {
		return Double.MAX_VALUE / (Simulator.getCurrentTime() - this.timeActivated);
	}

	/**
	 * @return the time this contact was refreshed last
	 */
	public long getLastRefresh() {
		return lastRefresh;
	}

	/**
	 *  sets the time this contact was refreshed last
	 */
	public void refresh() {
		this.lastRefresh = Simulator.getCurrentTime();
	}
}
