package de.tud.kom.p2psim.impl.overlay.gnutella06;

import java.math.BigInteger;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.ConnectivityEvent;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.api.transport.TransMessageListener;
import de.tud.kom.p2psim.api.transport.TransMsgEvent;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.*;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Operations.*;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * @author Jonas Khne
 * @version 3.0, 01.08.2008
 *
 */
public class Gnutella06OverlayNode extends AbstractOverlayNode implements TransMessageListener{
	
	protected final static Logger log = SimLogger.getLogger(Gnutella06OverlayNode.class);

	private int maxSuperpeerContacts = Gnutella06Configuration.GNUTELLA06_MAXCONTACTSFORLEAF;
	
	private Gnutella06OverlayRoutingTable routingTable; 
	private TransLayer transLayer;
	private boolean active;
	private boolean superpeerCapable;
	private boolean isSuperpeer = false;
	private Gnutella06OverlayContact ownContactData;
	private int userType;
	
	private List<Integer> documents = new LinkedList<Integer>();
	private List<BigInteger> pingInitiated = new LinkedList<BigInteger>();
	private List<BigInteger> queryInitiated = new LinkedList<BigInteger>();
	// queries initiated by application layer (Descriptor, FilesharingKey)
	private Map<BigInteger, Integer> queryInitiatedWithKey = new HashMap<BigInteger, Integer>();
	// results, ready to get for application layer (Descriptor, Contacts)
	private Map<BigInteger, List<Gnutella06OverlayContact>> queryResults = new HashMap<BigInteger, List<Gnutella06OverlayContact>>();
	// List of Token a peers got, stored with ID of sender
	private LinkedList<TransInfo> bootstrapList;
	private LinkedList<TransInfo> joinableList;
	private Map<Gnutella06OverlayID , List<Integer>> documentsOfNeighbors = new HashMap<Gnutella06OverlayID, List<Integer>>();
	private Map<Gnutella06OverlayID, Long> lastMessageReceived = new HashMap<Gnutella06OverlayID, Long>();
	
	/**
	 * @param transLayer the transport layer of the node
	 * @param peerID the overlay ID of the node
	 * @param port the port of the node
	 * @param documents list of documents stored on the node
	 */
	public Gnutella06OverlayNode(TransLayer transLayer, OverlayID peerID, short port, boolean superpeerCapable, int userType, List<Integer> documents){
		super(peerID, port);
		this.documents = documents;
		this.transLayer = transLayer;
		routingTable = new Gnutella06OverlayRoutingTable(this.getOverlayID());
		transLayer.addTransMsgListener(this, this.getPort());
		active = true;
		ownContactData = new Gnutella06OverlayContact((Gnutella06OverlayID)this.getOverlayID(), this.transLayer.getLocalTransInfo(this.getPort()), superpeerCapable);
		this.superpeerCapable = superpeerCapable;
		this.userType = userType;
		
	}
	
	@Override
	public TransLayer getTransLayer(){
		return transLayer;
	}
	
	public void connectivityChanged(ConnectivityEvent ce){
		//
	}
	
	public void messageArrived(TransMsgEvent receivingEvent) {
		Message message = receivingEvent.getPayload();
		
		if(active){
			
			if (message instanceof Gnutella06ConnectMessage) {
				processConnect(receivingEvent);
			} 
			else if (message instanceof Gnutella06ConnectAcceptMessage) {
				processConnectAccept(receivingEvent);
			}
			else if (message instanceof Gnutella06PingMessage) {
				processPing(receivingEvent);
			} 
			else if (message instanceof Gnutella06PongMessage) {
				processPong(receivingEvent);
			} 
			else if (message instanceof Gnutella06QueryMessage) {
				processQuery(receivingEvent);
			} 
			else if (message instanceof Gnutella06QueryHitMessage) {
				processQueryHit(receivingEvent);
			}
			else if (message instanceof Gnutella06ByeMessage) {
				processBye(receivingEvent);
			}
			else if (message instanceof Gnutella06ConnectionDeniedMessage) {
				processConnectionDenied(receivingEvent);
			}
			else if (message instanceof Gnutella06DocumentsMessage) {
				processDocumentsMessage(receivingEvent);
			}
		}

	}
	
	private void processConnectionDenied(TransMsgEvent receivingEvent) {
		Gnutella06ConnectionDeniedMessage connDeniedMessage = (Gnutella06ConnectionDeniedMessage) receivingEvent.getPayload();
		System.out.println(connDeniedMessage.getSender() + " hat Connection denied mit : " + this.getOverlayID());
		/*if(connDeniedMessage.getSwitchToLeaf()){
			if(routingTable.activeLeafContacts().size() == 0){
				for(Gnutella06OverlayContact contact : routingTable.allContacts()){
					Gnutella06ByeMessage bye = new Gnutella06ByeMessage((Gnutella06OverlayID)this.getOverlayID(), contact.getOverlayID(), "Disconnect");			
					this.getTransLayer().send(bye, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
				}
				isSuperpeer = false;
				maxSuperpeerContacts = Gnutella06Configuration.GNUTELLA06_MAXCONTACTSFORLEAF;
				
				Gnutella06ConnectMessage connect = new Gnutella06ConnectMessage((Gnutella06OverlayID)this.getOverlayID(), connDeniedMessage.getSender(), ownContactData, isSuperpeer);
				this.getTransLayer().send(connect, routingTable.getContact(connDeniedMessage.getSender()).getTransInfo(), this.getPort(), TransProtocol.UDP);
			}
		}*/
		Gnutella06OverlayContact con = this.getNodesRoutingTable().getContact(connDeniedMessage.getSender());
		if(this.getNodesRoutingTable().inactiveContacts().contains(con)){
			this.getNodesRoutingTable().inactiveContacts().remove(con);
		}
		else if(this.getNodesRoutingTable().inactiveLeafContacts().contains(con)){
			this.getNodesRoutingTable().inactiveLeafContacts().remove(con);
		}
		if (!connDeniedMessage.getSuperpeers().isEmpty()){
			for(Gnutella06OverlayContact contact : connDeniedMessage.getSuperpeers()){
				if(contact.getOverlayID()!= this.getOverlayID()){
					routingTable.addInactiveContact(contact);
				}
			}
//			Gnutella06OverlayContact contact = connDeniedMessage.getSuperpeers().get(Simulator.getRandom().nextInt(connDeniedMessage.getSuperpeers().size()));
//			Gnutella06ConnectMessage connect = new Gnutella06ConnectMessage((Gnutella06OverlayID)this.getOverlayID(), connDeniedMessage.getSender(), ownContactData, isSuperpeer);
//			this.getTransLayer().send(connect, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
		}
		else if(routingTable.allContacts().size() == 0){
			if(routingTable.inactiveContacts().size() == 0){
				Gnutella06BootstrapManager.getInstance().unregisterNode(this);
				Gnutella06BootstrapManager.getInstance().unregisterJoinableNodes(this);
				this.join();
			}
			else{
				this.connect();
			}
		}
	}

	private void processBye(TransMsgEvent receivingEvent) {
		//take peer from the list of active neighbors 
		Gnutella06ByeMessage byeMessage = (Gnutella06ByeMessage) receivingEvent.getPayload();
		routingTable.removeContact(byeMessage.getSender());
		documentsOfNeighbors.remove(byeMessage.getSender());
		//lastMessageReceived.remove(byeMessage.getSender());
		routingTable.removeInactiveContact(routingTable.getContact(byeMessage.getSender()));
		if(routingTable.inactiveContacts().size() == 0 && routingTable.allContacts().size() == 0){
			Gnutella06BootstrapManager.getInstance().unregisterNode(this);
			Gnutella06BootstrapManager.getInstance().unregisterJoinableNodes(this);
			this.join();
		}
		else this.connect();
	}

	private void processConnect(TransMsgEvent receivingEvent) {
		
		Gnutella06ConnectMessage connectMessage = (Gnutella06ConnectMessage) receivingEvent.getPayload();
		
		boolean spaceForNeighbor = false;
		if(connectMessage.getSuperPeer()){
			if(routingTable.allContacts().size() < this.maxSuperpeerContacts){
				spaceForNeighbor = true;
			}
		}
		else{
			if(this.isSuperpeer && routingTable.activeLeafContacts().size() < Gnutella06Configuration.GNUTELLA06_MAXLEAFCONTACTS){
				spaceForNeighbor = true;
			}
		}

		//if the peer has not enough neighbors => always accept connection
		if(spaceForNeighbor) {
			if(connectMessage.getSuperPeer()){
				routingTable.addContact(connectMessage.getContact());
			}
			else{
				routingTable.addLeafContact(connectMessage.getContact());
			}
			lastMessageReceived.put(connectMessage.getSender(), Simulator.getCurrentTime());
			if(routingTable.inactiveContacts().contains(connectMessage.getContact())){
				routingTable.removeInactiveContact(connectMessage.getContact());
			}
			Gnutella06ConnectAcceptMessage connectAcceptMessage = new Gnutella06ConnectAcceptMessage((Gnutella06OverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), ownContactData, isSuperpeer);
			this.getTransLayer().send(connectAcceptMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			
			//send the new neighbor the list of documents one has if he is a super peer and one self is a leaf
			if(connectMessage.getSuperPeer() && !this.isSuperpeer){
				Gnutella06DocumentsMessage documentsMessage = new Gnutella06DocumentsMessage((Gnutella06OverlayID) this.getOverlayID(), connectMessage.getContact().getOverlayID(), documents);
				this.getTransLayer().send(documentsMessage, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			}
		}
		//if the peer already has enough neighbors => deny connection
		else {
			Gnutella06ConnectionDeniedMessage connectionDenied;
			if(this.isSuperpeer && connectMessage.getSuperPeer() && routingTable.activeLeafContacts().size() < Gnutella06Configuration.GNUTELLA06_MAXLEAFCONTACTS/2){
				List<Gnutella06OverlayContact> superpeers = routingTable.allContacts();
				superpeers.add(ownContactData);
				connectionDenied = new Gnutella06ConnectionDeniedMessage((Gnutella06OverlayID)this.getOverlayID(), connectMessage.getContact().getOverlayID(), "switch to Leaf mode", true, superpeers);	
			}
			else{
				connectionDenied = new Gnutella06ConnectionDeniedMessage((Gnutella06OverlayID)this.getOverlayID(), connectMessage.getContact().getOverlayID(), "no space available", false, routingTable.allContacts());	
			}
			this.getTransLayer().send(connectionDenied, connectMessage.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
		}
		
	}
	
	private void processConnectAccept(TransMsgEvent receivingEvent) {
		Gnutella06ConnectAcceptMessage message = (Gnutella06ConnectAcceptMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
		System.out.println(Simulator.getCurrentTime()+" - "+message.getSender() + " hat ConnectAcceptiert von " + this.getOverlayID());
		if(message.getSuperpeer()){
			routingTable.addContact(message.getContact());
			if(routingTable.inactiveContacts().contains(message.getContact())){
				routingTable.inactiveContacts().remove(message.getContact());
			}
			if(!isSuperpeer){
				Gnutella06DocumentsMessage documentsMessage = new Gnutella06DocumentsMessage((Gnutella06OverlayID) this.getOverlayID(), message.getContact().getOverlayID(), documents);
				this.getTransLayer().send(documentsMessage, message.getContact().getTransInfo(), this.getPort(), TransProtocol.UDP);
			}
			
		}
		else{
			routingTable.addLeafContact(message.getContact());
			if(routingTable.inactiveLeafContacts().contains(message.getContact())){
				routingTable.inactiveLeafContacts().remove(message.getContact());
			}
		}
		Gnutella06BootstrapManager.getInstance().registerJoinableNodes(this);
	}
	
	private void processPing(TransMsgEvent receivingEvent) {
		Gnutella06PingMessage message = (Gnutella06PingMessage) receivingEvent.getPayload();
	
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
		if (routingTable.incomingPing(message.getSender(), message.getDescriptor())
				&& !this.pingInitiated.contains(message.getDescriptor())) {
			// route ping if TTL is greater 1
			if (message.getTTL() - 1 > 0) {
				for(Gnutella06OverlayContact receiver : routingTable.allContacts()){
					if(receiver.getOverlayID() != message.getSender()){
						Gnutella06PingOperation pingOperation = new Gnutella06PingOperation(this, receiver, message.getTTL(), message.getHops(), message.getDescriptor(), new OperationCallback<Object>() {
							public void calledOperationFailed(Operation<Object> op) {
								//
							}
							public void calledOperationSucceeded(Operation<Object> op) {
								// 
							}
						});
						pingOperation.scheduleImmediately();
					}
				}
				if(isSuperpeer){
					for(Gnutella06OverlayContact receiver : routingTable.activeLeafContacts()){
						if(receiver.getOverlayID() != message.getSender()){
							Gnutella06PingOperation pingOperation = new Gnutella06PingOperation(this, receiver, message.getTTL(), message.getHops(), message.getDescriptor(), new OperationCallback<Object>() {
								public void calledOperationFailed(Operation<Object> op) {
									//
								}
								public void calledOperationSucceeded(Operation<Object> op) {
									// 
								}
							});
							pingOperation.scheduleImmediately();
						}
					}
				}	
			}
			// reply with pong
			Gnutella06PongOperation pongOperation = new Gnutella06PongOperation(this, message.getDescriptor(), isSuperpeer, new OperationCallback<Object>() {
				public void calledOperationFailed(Operation<Object> op) {
					// 
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					// 
				}
			});
			pongOperation.scheduleImmediately();
		}	
	}

	private void processPong(TransMsgEvent receivingEvent) {
		Gnutella06PongMessage message = (Gnutella06PongMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());

		boolean acceptPong = routingTable.incomingPong(message.getSender(), message.getDescriptor());
		// route pong
		if(acceptPong) {
			if (message.getTTL() -1 > 0) {
				Gnutella06PongOperation pongOperation = new Gnutella06PongOperation(this, message.getTTL(), message.getHops(), message.getDescriptor(), message.getContact(), message.getSuperpeer(), new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						// 
					}
				});
				pongOperation.scheduleImmediately();
			}
		}
		// add contact
		if(message.getSuperpeer()){
			if(!routingTable.allContacts().contains(message.getContact()) && !routingTable.inactiveContacts().contains(message.getContact())){
				routingTable.addInactiveContact(message.getContact());
			}
		}
		
		else{
			if(!routingTable.activeLeafContacts().contains(message.getContact()) && !routingTable.inactiveLeafContacts().contains(message.getContact())){
				routingTable.addInactiveLeafContact(message.getContact());
			}
		}
	}

	private void processQuery(TransMsgEvent receivingEvent) {
		//this.sendPingOnce();
		Gnutella06QueryMessage message = (Gnutella06QueryMessage) receivingEvent.getPayload();
		lastMessageReceived.put(message.getSender(), Simulator.getCurrentTime());
//		Gnutella06OverlayNode.log.debug(this.getOverlayID() + " hat Query erhalten von: " + message.getSender());

		boolean dontForward = false;
		// do not check documents twice for the same query or for queries sent from here
		if (routingTable.incomingQuery(message.getSender(), message.getDescriptor())
				&& !this.queryInitiatedWithKey.containsKey(message.getDescriptor())) {
		
			for (Integer document : documents) {
				if(message.getKey() == document) {
					Gnutella06QueryHitOperation queryHit = new Gnutella06QueryHitOperation(this, message.getDescriptor(),  ownContactData, message.getInitiator(), message.getHops(), new OperationCallback<Object>() {
						public void calledOperationFailed(Operation<Object> op) {
							//
						}
						public void calledOperationSucceeded(Operation<Object> op) {
								// 
							}
						});
					queryHit.scheduleImmediately();
					System.out.println(Simulator.getCurrentTime() +" "+ this.getOverlayID().toString() +" sends QueryHit");
					Gnutella06OverlayNode.log.debug(this.getOverlayID() +" sends QueryHit");
					dontForward = true;
					//System.out.println("from " +message.getReceiver()+ " to " +message.getSender()+ " query hit for " +message.getKey()+ " after " +(message.getHops() + 1));
				}	
			}
			//check documents of leafs for a query hit if so forward the query
			for (Gnutella06OverlayContact overlayContact : routingTable.activeLeafContacts()){
				Gnutella06OverlayID id = overlayContact.getOverlayID();
				if(documentsOfNeighbors.containsKey(id)){
					List<Integer> docs = documentsOfNeighbors.get(id);
					for (Integer document : docs){
						if(message.getKey() ==document) {
							Gnutella06QueryMessage queryMessage = new Gnutella06QueryMessage((Gnutella06OverlayID)this.getOverlayID(), overlayContact.getOverlayID(), 1, message.getHops() + 1, message.getDescriptor(), message.getKey(), message.getInitiator());
							this.getTransLayer().send(queryMessage, overlayContact.getTransInfo(), this.getPort(), TransProtocol.UDP);
							//System.out.println("from " +overlayContact.getOverlayID()+ " to " +message.getSender()+ " query hit for " +message.getKey()+ " after " +(message.getHops() + 1));
							dontForward = true;
						}
					}
				}
			}
		}
		// route query
		if (message.getTTL() - 1 > 0 && !dontForward) {
			Gnutella06QueryOperation queryOperation = new Gnutella06QueryOperation(this, message.getTTL(), message.getHops(), message.getDescriptor(), message.getSender(), message.getKey(), message.getInitiator(), new OperationCallback<Object>() {
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
						// 
					}
				});
			queryOperation.scheduleImmediately();
		}	
	}		


	private void processQueryHit(TransMsgEvent receivingEvent) {
		Gnutella06QueryHitMessage message = (Gnutella06QueryHitMessage) receivingEvent.getPayload();
		boolean acceptQuery = queryInitiated.contains(message.getDescriptor());
		
		// add contact to result list
		if(acceptQuery){
			if(queryResults.get(message.getDescriptor()) == null) {
				queryResults.put(message.getDescriptor(), new LinkedList<Gnutella06OverlayContact>());
			}
			queryResults.get(message.getDescriptor()).add(message.getContact());
			
			documents.add(queryInitiatedWithKey.get(message.getDescriptor()));
			if(!isSuperpeer){
				for(Gnutella06OverlayContact contact : routingTable.allContacts()){
					Gnutella06DocumentsMessage documentsMessage = new Gnutella06DocumentsMessage((Gnutella06OverlayID) this.getOverlayID(), contact.getOverlayID(), documents);
					this.getTransLayer().send(documentsMessage, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
				}
			}
				
			queryInitiated.remove(message.getDescriptor());
			queryResults.remove(queryInitiatedWithKey.get(message.getDescriptor()));
			queryInitiatedWithKey.remove(message.getDescriptor());
			
		}
	}
	
	private void processDocumentsMessage(TransMsgEvent receivingEvent) {
		//put the list of documents of neighbor in own map
		Gnutella06DocumentsMessage documentsMessage = (Gnutella06DocumentsMessage) receivingEvent.getPayload();
		lastMessageReceived.put(documentsMessage.getSender(), Simulator.getCurrentTime());
		documentsOfNeighbors.put(documentsMessage.getSender(), documentsMessage.getDocuments());
		
	}

	/**
	 * @param descriptor message ID of the query
	 * @param key the document searched for
	 */
	public void registerQuery(BigInteger descriptor, Integer key) {
		this.queryResults.put(descriptor, new LinkedList<Gnutella06OverlayContact>());
		this.queryInitiatedWithKey.put(descriptor, key);
		this.queryInitiated.add(descriptor);
	}
	
	/**
	 * node joins the network by bootstrapping
	 */
	public void join(){
		//Join the p2p network
		System.out.println(this.getOverlayID().toString()+ " is in the network and Superpeer " + this.superpeerCapable);
		active = true;
		bootstrapList = (LinkedList<TransInfo>) Gnutella06BootstrapManager.getInstance().getBootstrapInfo();
		joinableList = (LinkedList<TransInfo>) Gnutella06BootstrapManager.getInstance().getJoinableBootstrapInfo();
		isSuperpeer = superpeerCapable;
		if(isSuperpeer){
			this.maxSuperpeerContacts = Gnutella06Configuration.GNUTELLA06_MAXSUPERPEERCONTACTS; 
		}
		
		if(!joinableList.isEmpty()){
			int numberOfThePeerToConnect = Simulator.getRandom().nextInt(joinableList.size());
			TransInfo connectInfo = joinableList.get(numberOfThePeerToConnect);
			Gnutella06ConnectOperation connect = new Gnutella06ConnectOperation(this, connectInfo, isSuperpeer, new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			connect.scheduleImmediately();
		}
		
		if(joinableList.isEmpty()){
			Gnutella06BootstrapManager.getInstance().registerJoinableNodes(this);
		}
		Gnutella06BootstrapManager.getInstance().registerNode(this);

		this.sendPing();
		Gnutella06CheckNeighborsOperation operation = new Gnutella06CheckNeighborsOperation(this, Gnutella06Configuration.GNUTELLA06_SCHEDULEDCHECKNEIGHBORS, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		operation.scheduleWithDelay(5000000);

		
		Gnutella06ScheduleConnectOperation connect = new Gnutella06ScheduleConnectOperation(this, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		connect.scheduleWithDelay(5000000);
		
		// Peer leaves the Network with a certain probability
		if(Gnutella06Configuration.GNUTELLA06_CHURN){
			Gnutella06ScheduleLeaveOperation leaveOperation = new Gnutella06ScheduleLeaveOperation(this, userType, new OperationCallback<Object>(){
				public void calledOperationFailed(Operation<Object> op) {
					//
				}
				public void calledOperationSucceeded(Operation<Object> op) {
					//
				}});
			int random = 30 - Simulator.getRandom().nextInt(60);
			if(userType == 0){
				leaveOperation.scheduleWithDelay((Gnutella06Configuration.GNUTELLA06_SESSIONTIMEBENEFACTOR + random)*Simulator.SECOND_UNIT);
			}
			else if(userType == 1){
				leaveOperation.scheduleWithDelay((Gnutella06Configuration.GNUTELLA06_SESSIONTIMEPEER + random)*Simulator.SECOND_UNIT);
			}
		}
		// Peer schedules Query automatically 
		Gnutella06ScheduleQueryOperation scheduleQuery = new Gnutella06ScheduleQueryOperation(this, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		int delay = Gnutella06Configuration.GNUTELLA06_QUERYRATE - Simulator.getRandom().nextInt(20) * 1000000;
		scheduleQuery.scheduleWithDelay(delay);
		
	}
	
	/**
	 * node leaves the network by sending bye messages to all neighbors
	 */
	public void leave(){
		//Peer leaves the network
		List<Gnutella06OverlayContact> activeContacts = routingTable.allContacts();
		for (Gnutella06OverlayContact contact : activeContacts){
			Gnutella06ByeMessage bye = new Gnutella06ByeMessage((Gnutella06OverlayID)this.getOverlayID(), contact.getOverlayID(), "Disconnect");			
			this.getTransLayer().send(bye, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
			
		}
		routingTable.clearContacts();
		Gnutella06BootstrapManager.getInstance().unregisterNode(this);
		Gnutella06BootstrapManager.getInstance().unregisterJoinableNodes(this);
		active = false;
		
	}
	
	/**
	 * node leaves the network without letting its neighbors know
	 */
	public void fail(){
		active = false;
		routingTable.clearContacts();
		Gnutella06BootstrapManager.getInstance().unregisterNode(this);
		Gnutella06BootstrapManager.getInstance().unregisterJoinableNodes(this);
	
	}


	/**
	 * node starts sending pings to all neighbors periodically
	 */
	public void sendPing(){
		//send Ping to neighbors
		Gnutella06SchedulePingOperation schedulePing = new Gnutella06SchedulePingOperation(this, Gnutella06Configuration.GNUTELLA06_SCHEDULEPING, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		schedulePing.scheduleImmediately();
	}
	
	/**
	 * sendPing() sends Pings periodically sendPingOnce() sends Pings to all neighbors but only once
	 */
	public void sendPingOnce(){
		for(Gnutella06OverlayContact receiver : this.getNodesRoutingTable().allContacts()){
			Gnutella06PingOperation ping = new Gnutella06PingOperation(this, receiver, 10, 0, null, new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						//
					}});
			ping.scheduleImmediately();
		}
		if(isSuperpeer){
			for(Gnutella06OverlayContact receiver : this.getNodesRoutingTable().activeLeafContacts()){
				Gnutella06PingOperation ping = new Gnutella06PingOperation(this, receiver, 1, 0, null, new OperationCallback<Object>(){
						public void calledOperationFailed(Operation<Object> op) {
							//
						}
						public void calledOperationSucceeded(Operation<Object> op) {
							//
						}});
				ping.scheduleImmediately();
			}
		}
	}

	/**
	 * node randomly selects a document and searches for it
	 */
	public void sendQuery(){
		//  send Query
		// Peer schedules Query automatically 
		Gnutella06ScheduleQueryOperation scheduleQuery = new Gnutella06ScheduleQueryOperation(this, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		scheduleQuery.scheduleWithDelay(4000000);
	}
	
	public void sendQueryOnce(){
		// einzelnen Query senden
		int file = 0;
		for(int i = 0; i < 5; i++){
			file = Gnutella06FileFactory.getInstance().getZipf().returnRank();
			if(Gnutella06FileFactory.getInstance().getFileSharingKeys().contains(file)){
				i = 5;
			}
		}
		if(!this.getDocuments().contains(file)){
			Gnutella06QueryOperation query = new Gnutella06QueryOperation(this, file, new OperationCallback<Object>(){
					public void calledOperationFailed(Operation<Object> op) {
						//
					}
					public void calledOperationSucceeded(Operation<Object> op) {
						//
					}});			
			query.scheduleImmediately();
		}
	}
	
	public void sendQueryWithKey(int key){
		Gnutella06QueryOperation query = new Gnutella06QueryOperation(this, key, new OperationCallback<Object>(){
			public void calledOperationFailed(Operation<Object> op) {
				//
			}
			public void calledOperationSucceeded(Operation<Object> op) {
				//
			}});
		query.scheduleImmediately();
	}

	/**
	 * if node is not satisfied with its current neighbors it selects a better neighbor to connet to
	 */
	public void connect(){
		// connects with other peers

		if(routingTable.allContacts().size() < this.maxSuperpeerContacts){
			if (routingTable.inactiveContacts().size() > 0){
				Gnutella06OverlayContact contact = routingTable.inactiveContacts().get(Simulator.getRandom().nextInt(routingTable.inactiveContacts().size()));
	
				if(!routingTable.allContacts().contains(contact)){
					Gnutella06ConnectMessage connect = new Gnutella06ConnectMessage((Gnutella06OverlayID)this.getOverlayID(), contact.getOverlayID(), ownContactData, isSuperpeer);
					this.getTransLayer().send(connect, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
				}
			} 
//			else {System.out.println("keine Inativen Kontakte vorhanden");}
		}
		if(routingTable.activeLeafContacts().size() < Gnutella06Configuration.GNUTELLA06_MAXLEAFCONTACTS && this.isSuperpeer){

			if (routingTable.inactiveLeafContacts().size() > 0){
				Gnutella06OverlayContact contact = routingTable.inactiveLeafContacts().get(Simulator.getRandom().nextInt(routingTable.inactiveLeafContacts().size()));
	
				if(!routingTable.activeLeafContacts().contains(contact)){
					Gnutella06ConnectMessage connect = new Gnutella06ConnectMessage((Gnutella06OverlayID)this.getOverlayID(), contact.getOverlayID(), ownContactData, isSuperpeer);
					this.getTransLayer().send(connect, contact.getTransInfo(), this.getPort(), TransProtocol.UDP);
				}
			} 
//			else {System.out.println("keine Inativen Kontakte vorhanden");}
		}
	}
	
	/**
	 * @return routing table of this node
	 */
	public Gnutella06OverlayRoutingTable getNodesRoutingTable(){
		return routingTable;
	}
	
	/**
	 * @return true if node is still in the network
	 */
	public boolean isActive(){
		if(active){
			return true;
		}
		else return false;
	}
	
	public void setActive(){
		active = true;
	}
	
	/**
	 * @param id overlay ID of the neighbor
	 * @return time the last message from this neighbor was received
	 */
	public long getLastMessageReceived(Gnutella06OverlayID id){
		if(lastMessageReceived.get(id) != null){
			return lastMessageReceived.get(id);
		}
		return 0;
	}
	
	public Gnutella06OverlayContact getOwnContactData(){
		return ownContactData;
	}
	
	public List<Integer> getDocuments(){
		return documents;
	}
	
	public String toString() {
		return this.getOverlayID().toString();
	}
	
	public List<BigInteger> getQueryInitiated(){
		return queryInitiated;
	}
	
	public Map<BigInteger, Integer> getQueryInitiatedWithKey(){
		return queryInitiatedWithKey;
	}

	public int getMaxContacts() {
		return this.maxSuperpeerContacts;
	}
	
	public boolean isSuperpeer(){
		return this.isSuperpeer;
	}
	
	public boolean isSuperpeerCapable(){
		return this.superpeerCapable;
	}
	
}
