package de.tud.kom.p2psim.impl.overlay.gnutella06;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.Component;
import de.tud.kom.p2psim.api.common.ComponentFactory;
import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

/**
 * @author Jonas Khne
 * @version 3.0, 06.07.2008
 *
 */
public class Gnutella06OverlayNodeFactory implements ComponentFactory{
	
	protected final static Logger log = SimLogger.getLogger(Gnutella06OverlayNodeFactory.class);
	private short port = 123;
	private static long id = 0;
	private List<Integer> documents;
	private boolean superpeerCapable;
	private int userType;
	
	/**
	 * constructor for the node factory
	 */
	public Gnutella06OverlayNodeFactory(){
		// constructor for the node factory
	}
	
	public Component createComponent(Host host){
		int numberOfDocuments = Simulator.getRandom().nextInt(10);
		documents = new LinkedList<Integer>();
		while(numberOfDocuments != 0){
			Integer file = Gnutella06FileFactory.getInstance().createNewFileSharingKey();
			documents.add(file);
			numberOfDocuments--;
		}
		Gnutella06OverlayNode newNode = new Gnutella06OverlayNode(host.getTransLayer(), newOverlayID(), port, calculateSuperpeerCapable(), userType, documents);
		return newNode;
	}
	
	/**
	 * @param host 
	 * @return new node with up to 40 documents
	 */
	public Gnutella06OverlayNode createNode(Host host){
		int numberOfDocuments = Simulator.getRandom().nextInt(40);
		documents = new LinkedList<Integer>();
		while(numberOfDocuments != 0){
			Integer file = Gnutella06FileFactory.getInstance().createNewFileSharingKey();
			documents.add(file);
			numberOfDocuments--;
		}
		Gnutella06OverlayNode newOverlayNode = new Gnutella06OverlayNode(host.getTransLayer(), newOverlayID(), port, calculateSuperpeerCapable(), userType, documents);
		Gnutella06OverlayNodeFactory.log.debug(Gnutella06FileFactory.getInstance().getFileSharingKeys());
		return newOverlayNode;
	}
	
	/**
	 * @return new overlay ID
	 */
	private Gnutella06OverlayID newOverlayID(){
		this.id += 1;
		Gnutella06OverlayID newID = new Gnutella06OverlayID(BigInteger.valueOf(this.id));
		return newID;
	}
	
	private boolean calculateSuperpeerCapable(){
		if(this.id == 1){
			superpeerCapable = true;
			userType = 0;
			return true;
		}
		else if(Simulator.getRandom().nextInt(100) <= Gnutella06Configuration.GNUTELLA06_SUPERPEERPERCENTAGE){
			superpeerCapable = true;
			if(Simulator.getRandom().nextInt(100)<= 40){
				userType = 0;
			}
			else{
				userType = 1;
			}
			return true;
		}
		else{
			superpeerCapable = false;
			userType = 1;
			return false;
		}
	}
}