package de.tud.kom.p2psim.impl.overlay.gnutella06;

import java.math.BigInteger;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.tud.kom.p2psim.api.overlay.OverlayContact;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.overlay.OverlayRoutingTable;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class Gnutella06OverlayRoutingTable implements OverlayRoutingTable<Gnutella06OverlayID, Gnutella06OverlayContact> {

	private OverlayID overlayID;
	private Map<Gnutella06OverlayID, Gnutella06OverlayContact> overlayIDs;
	private List<Gnutella06OverlayContact> activeSuperpeerContacts = new LinkedList<Gnutella06OverlayContact>();
	private List<Gnutella06OverlayContact> activeLeafContacts = new LinkedList<Gnutella06OverlayContact>();
	private List<Gnutella06OverlayContact> inactiveContacts = new LinkedList<Gnutella06OverlayContact>();
	private List<Gnutella06OverlayContact> inactiveLeafContacts = new LinkedList<Gnutella06OverlayContact>();
	
	private Map<BigInteger, List<Gnutella06OverlayContact>> acceptPong = new HashMap<BigInteger, List<Gnutella06OverlayContact>>();
	private Map<BigInteger, Gnutella06OverlayContact> routePong = new HashMap<BigInteger, Gnutella06OverlayContact>();
	private Map<BigInteger, List<Gnutella06OverlayContact>> acceptQueryHit = new HashMap<BigInteger, List<Gnutella06OverlayContact>>();
	private Map<BigInteger, Gnutella06OverlayContact> routeQueryHit = new HashMap<BigInteger, Gnutella06OverlayContact>();
	
	/**
	 * @param overlayID overlay ID of the node
	 */
	public Gnutella06OverlayRoutingTable(OverlayID overlayID) {
		this.overlayID = overlayID;
		overlayIDs = new HashMap<Gnutella06OverlayID, Gnutella06OverlayContact>();
	}

	/**
	 * @param overlayID2 overlay ID of initiator
	 * @param descriptor message ID of the ping
	 */
	public void initiatedPing(Gnutella06OverlayID overlayID2, BigInteger descriptor) {
		routePong.put(descriptor, null);
	}
	
	/**
	 * @param id overlay ID of sender
	 * @param descriptor message ID of ping
	 * @return true if ping was not received before
	 */
	public boolean incomingPing(Gnutella06OverlayID id, BigInteger descriptor) {
		Gnutella06OverlayContact contact = getContact(id);
		if(routePong.containsKey(descriptor) || contact == null) {
			return false;
		}
		routePong.put(descriptor, contact);
		return true;
	}
	
	/**
	 * @param contact receiver of the message
	 * @param descriptor message ID
	 */
	public void outgoingPing(Gnutella06OverlayContact contact, BigInteger descriptor) {
		if(!acceptPong.containsKey(descriptor)) {
			acceptPong.put(descriptor, new LinkedList<Gnutella06OverlayContact>());
		}
		acceptPong.get(descriptor).add(contact);
	}
	
	/**
	 * @param idSender sender
	 * @param descriptor message ID
	 * @return true if pong can be routed
	 */
	public boolean incomingPong(Gnutella06OverlayID idSender, BigInteger descriptor) {
		Gnutella06OverlayContact contactSender = getContact(idSender);
		// return contact to forward message to
		if(!acceptPong.containsKey(descriptor) || !routePong.containsKey(descriptor)) {
			// do not route messages with wrong descriptor
			return false;
		}
		if(!acceptPong.get(descriptor).contains(contactSender)) {
			// do not reply messages without having sent a request 
			return false;
		}
		return true;
	}

	/**
	 * @param descriptor message ID
	 * @return contact to forward message to
	 */
	public Gnutella06OverlayContact outgoingPong(BigInteger descriptor) {
		// return contact to forward message to
		return routePong.get(descriptor);
	}

	/**
	 * @param overlayID2 overlay ID of this node
	 * @param descriptor message ID
	 */
	public void initiatedQuery(Gnutella06OverlayID overlayID2, BigInteger descriptor) {
		routeQueryHit.put(descriptor, null);
	}

	/**
	 * @param id
	 * @param descriptor
	 * @return true if query was 
	 */
	public boolean incomingQuery(Gnutella06OverlayID id, BigInteger descriptor) {
		Gnutella06OverlayContact contact = getContact(id);
		if(routeQueryHit.containsKey(descriptor) || contact == null) {
			return false;
		}
		routeQueryHit.put(descriptor, contact);
		return true;
	}
	
	public void outgoingQuery(Gnutella06OverlayContact contact, BigInteger descriptor) {
		if(!acceptQueryHit.containsKey(descriptor)) {
			acceptQueryHit.put(descriptor, new LinkedList<Gnutella06OverlayContact>());
		}
		acceptQueryHit.get(descriptor).add(contact);
	}
	
	public Gnutella06OverlayContact outgoingQueryHit(BigInteger descriptor) {
		// return contact to forward message to
		return routeQueryHit.get(descriptor);
	}
	
	/**
	 * @param contact contact info
	 */
	public void addInactiveContact(Gnutella06OverlayContact contact) {
		if(inactiveContacts.size() < 35){
			if(contact != null){
				if(!inactiveContacts.contains(contact)) {
					inactiveContacts.add(contact);
				}
				if(!overlayIDs.containsKey(contact.getOverlayID())){
					overlayIDs.put(contact.getOverlayID(), contact);
				}
			}
		}
		else{
			if(contact != null){
				inactiveContacts.remove(Simulator.getRandom().nextInt(inactiveContacts.size()));
				if(!inactiveContacts.contains(contact)) {
					inactiveContacts.add(contact);
				}
				if(!overlayIDs.containsKey(contact.getOverlayID())){
					overlayIDs.put(contact.getOverlayID(), contact);
				}
			}
		}
	}
	

	/**
	 * @return the number of inactive contacts
	 */
	public List<Gnutella06OverlayContact> inactiveContacts() {
		return new LinkedList<Gnutella06OverlayContact>(inactiveContacts);
	}
	
	public void removeInactiveContact(Gnutella06OverlayContact contact) {
		inactiveContacts.remove(contact);
	}
	
	public void addInactiveLeafContact(Gnutella06OverlayContact contact) {
		if(inactiveLeafContacts.size() < 35){
			if(contact != null){
				if(!inactiveLeafContacts.contains(contact)) {
					inactiveLeafContacts.add(contact);
				}
				if(!overlayIDs.containsKey(contact.getOverlayID())){
					overlayIDs.put(contact.getOverlayID(), contact);
				}
			}
		}
		else{
			if(contact != null){
				inactiveLeafContacts.remove(Simulator.getRandom().nextInt(inactiveLeafContacts.size()));
				if(!inactiveLeafContacts.contains(contact)) {
					inactiveLeafContacts.add(contact);
				}
				if(!overlayIDs.containsKey(contact.getOverlayID())){
					overlayIDs.put(contact.getOverlayID(), contact);
				}
			}
		}
	}
	

	/**
	 * @return the number of inactive contacts
	 */
	public List<Gnutella06OverlayContact> inactiveLeafContacts() {
		return new LinkedList<Gnutella06OverlayContact>(inactiveLeafContacts);
	}
	
	public void removeInactiveLeafContact(Gnutella06OverlayContact contact) {
		inactiveLeafContacts.remove(contact);
	}
	
	public void addContact(Gnutella06OverlayContact contact){
		
		if(!this.activeSuperpeerContacts.contains(contact)) {
			inactiveContacts.remove(contact);

			this.overlayIDs.put(contact.getOverlayID(), contact);
			this.activeSuperpeerContacts.add(contact);
		}
	}

	public List<Gnutella06OverlayContact> allContacts() {
		//System.out.println("AllContacts aufgerufen " +activeContacts.size() );
		return new LinkedList<Gnutella06OverlayContact>(activeSuperpeerContacts);
	}
	
	public void removeContact(Gnutella06OverlayID oid) {
		Gnutella06OverlayContact contact = this.getContact(oid);
		activeSuperpeerContacts.remove(contact);
	}
	
	/**
	 * @return the number of active contacts
	 */
	public int numberOfActiveContacts() {
		return activeSuperpeerContacts.size();
	}
	
	public void addLeafContact(Gnutella06OverlayContact contact){
		if(!this.activeLeafContacts.contains(contact)) {
			inactiveContacts.remove(contact);

			this.overlayIDs.put(contact.getOverlayID(), contact);
			this.activeLeafContacts.add(contact);
		}
	}
	
	public List<Gnutella06OverlayContact> activeLeafContacts() {
		return new LinkedList<Gnutella06OverlayContact>(activeLeafContacts);
	}
	
	public void removeLeafContact(Gnutella06OverlayID oid) {
		Gnutella06OverlayContact contact = this.getContact(oid);
		activeLeafContacts.remove(contact);
	}
	
	public Gnutella06OverlayContact getContact(Gnutella06OverlayID oid) {
		return this.overlayIDs.get(oid);
	}

	public void clearContacts() {
		this.overlayIDs.clear();
		this.activeSuperpeerContacts.clear();
		this.activeLeafContacts.clear();
		this.inactiveContacts.clear();
	}
}
