package de.tud.kom.p2psim.impl.overlay.gnutella06.Messages;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;

public abstract class BaseMessage extends AbstractOverlayMessage<Gnutella06OverlayID> {

	private int ttl;
	private int hops;
	private BigInteger descriptor;
	private final static long GNUTELLA06_BASE_MESSAGE_SIZE = 23;
	
	/**
	 * @param sender overlay ID of sender
	 * @param receiver overlay ID of receiver
	 * @param ttl time to live for message
	 * @param hops number of hops already taken
	 * @param descriptor message ID
	 */
	public BaseMessage(Gnutella06OverlayID sender, Gnutella06OverlayID receiver, int ttl, int hops, BigInteger descriptor) {
		super(sender, receiver);
		this.ttl = ttl;
		this.hops = hops;
		this.descriptor = descriptor;
	}
	
	public Message getPayload() {
		return this;
	}
	
	/**
	 * @return time to live
	 */
	public Integer getTTL() {
		return ttl;
	}
	
	/**
	 * @return return hop count
	 */
	public Integer getHops() {
		return hops;
	}
	
	/**
	 * @return message ID
	 */
	public BigInteger getDescriptor() {
		return descriptor;
	}

	public long getSize() {
		return GNUTELLA06_BASE_MESSAGE_SIZE;
	}
}