package de.tud.kom.p2psim.impl.overlay.gnutella06.Messages;

import java.math.BigInteger;
import java.util.List;

import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class Gnutella06QueryMessage extends BaseMessage {
	
	private static final long GNUTELLA06_QUERY_MESSAGE_SIZE = 3;
	private long searchStringSize;
	private Integer key;
	private Gnutella06OverlayContact initiator;
	
	/**
	 * @param sender overlay ID of the sender
	 * @param receiver overlay ID of the receiver
	 * @param ttl time to live of the message
	 * @param hops hop count for the message
	 * @param descriptor message ID
	 * @param key document searched for
	 * @param maxReplies number of replies wanted
	 * @param queryHitSent list of all the nodes that responded with query hit
	 * @param initiator contact info of the initiator of the query
	 */
	public Gnutella06QueryMessage(Gnutella06OverlayID sender, Gnutella06OverlayID receiver, int ttl, int hops, BigInteger descriptor, Integer key, Gnutella06OverlayContact initiator) {
		super(sender, receiver, ttl, hops, descriptor);
		searchStringSize = (long) (Simulator.getRandom().nextDouble() * 40);
		this.key = key;
		this.initiator = initiator;
	}

	/**
	 * @return document
	 */
	public Integer getKey() {
		return this.key;
	}
	
	public long getSize() { 
		return super.getSize() + GNUTELLA06_QUERY_MESSAGE_SIZE + searchStringSize;
	}

	/**
	 * @return contact info of initiator
	 */
	public Gnutella06OverlayContact getInitiator(){
		return initiator;
	}
}

