package de.tud.kom.p2psim.impl.overlay.gnutella06.Operations;

import java.math.BigInteger;

import org.apache.log4j.Logger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06Configuration;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PingMessage;
import de.tud.kom.p2psim.impl.util.logging.SimLogger;

public class Gnutella06PingOperation extends BaseOperation{

	protected final static Logger log = SimLogger.getLogger(Gnutella06PingOperation.class);
	private Gnutella06OverlayContact receiver;
	
	/**
	 * @param component initiator of this operation
	 * @param receiver contact info of receiver
	 * @param callback
	 */
	public Gnutella06PingOperation(Gnutella06OverlayNode component, Gnutella06OverlayContact receiver, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_PING_MAX_TTL, 3, 0, null, callback);
		this.receiver = receiver;
		component.getNodesRoutingTable().initiatedPing((Gnutella06OverlayID) component.getOverlayID(), this.getDescriptor());
	}
	
	/**
	 * @param component initiator of this operation
	 * @param receiver contact info of receiver
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param callback
	 */
	public Gnutella06PingOperation(Gnutella06OverlayNode component, Gnutella06OverlayContact receiver, int ttl, int hops, BigInteger descriptor, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.receiver = receiver;
	}

	@Override
	protected void execute() {
		if(ttl > 0) {
			Gnutella06PingMessage pingMessage = new Gnutella06PingMessage((Gnutella06OverlayID) this.getComponent().getOverlayID(), receiver.getOverlayID(), this.ttl, this.hops, this.descriptor);
			this.getComponent().getTransLayer().send(pingMessage, receiver.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
			// inform routing table about outgoing ping
			this.getComponent().getNodesRoutingTable().outgoingPing(receiver, this.getDescriptor());
			//System.out.println("Ping from " +this.getComponent().getOverlayID()+ " to "+ receiver.getOverlayID());
			operationFinished(true);
		}
		
	}

}
