package de.tud.kom.p2psim.impl.overlay.gnutella06.Operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06Configuration;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PongMessage;

public class Gnutella06PongOperation extends BaseOperation {

	private Gnutella06OverlayContact contact;
	private boolean isSuperpeer;
	
	/**
	 * @param node initiator of this operation
	 * @param descriptor message ID
	 * @param isSuperpeer true if sender is super peer
	 * @param callback
	 */
	public Gnutella06PongOperation(Gnutella06OverlayNode node, BigInteger descriptor, boolean isSuperpeer, OperationCallback<Object> callback) {
		super(node, Gnutella06Configuration.GNUTELLA06_PING_MAX_TTL, Gnutella06Configuration.GNUTELLA06_PING_MAX_TTL, 0, descriptor, callback);
		this.contact = node.getOwnContactData();
	}

	/**
	 * @param node initiator of this operation
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param contact contact info of sender
	 * @param isSuperpeer true if sender is super peer
	 * @param callback
	 */
	public Gnutella06PongOperation(Gnutella06OverlayNode node, int ttl, int hops, BigInteger descriptor, Gnutella06OverlayContact contact, boolean isSuperpeer, OperationCallback<Object> callback) {
		super(node, Gnutella06Configuration.GNUTELLA06_PING_MAX_TTL, ttl, hops, descriptor, callback);
		this.contact = contact;
	}

	/**
	 * @return contact info
	 */
	public Gnutella06OverlayContact getContact() {
		return contact;
	}
	
	@Override
	protected void execute() {
		Gnutella06OverlayContact contactReceiver =  this.getComponent().getNodesRoutingTable().outgoingPong(this.descriptor);
		if(contactReceiver != null) {	
			Gnutella06PongMessage message = new Gnutella06PongMessage((Gnutella06OverlayID) this.getComponent().getOverlayID(), contactReceiver.getOverlayID(), this.ttl, this.hops, this.descriptor, this.contact, isSuperpeer);
			this.getComponent().getTransLayer().send(message, contactReceiver.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		}
		operationFinished(true);
	}

}
