package de.tud.kom.p2psim.impl.overlay.gnutella06.Operations;

import java.math.BigInteger;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06Configuration;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryHitMessage;

public class Gnutella06QueryHitOperation extends BaseOperation {

	private Gnutella06OverlayContact contact;
	private Gnutella06OverlayContact initiator;
	private int hopsOfQuery;
	

	/**
	 * @param component initiator of this operation
	 * @param descriptor message ID
	 * @param contact contact info of the owner of the document
	 * @param initiator contact info of the initiator of the query
	 * @param callback
	 */
	public Gnutella06QueryHitOperation(Gnutella06OverlayNode component, BigInteger descriptor, Gnutella06OverlayContact contact, Gnutella06OverlayContact initiator, int hopsOfQuery, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, hopsOfQuery, descriptor, callback);
		this.contact = contact;
		this.initiator = initiator;
		this.hopsOfQuery = hopsOfQuery;
	}

	/**
	 * @param component initiator of this operation
	 * @param ttl
	 * @param hops
	 * @param descriptor message ID
	 * @param contact contact info of the owner of the document
	 * @param initiator contact info of the initiator of the query
	 * @param callback
	 */
	public Gnutella06QueryHitOperation(Gnutella06OverlayNode component, int ttl, int hops, BigInteger descriptor, Gnutella06OverlayContact contact, Gnutella06OverlayContact initiator, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, ttl, hops, descriptor, callback);
		this.contact = contact;
		this.initiator = initiator;
	}

	@Override
	protected void execute() {
		Gnutella06QueryHitMessage message = new Gnutella06QueryHitMessage((Gnutella06OverlayID) this.getComponent().getOverlayID(), initiator.getOverlayID(), this.ttl, this.hops, this.descriptor, this.contact);
		this.getComponent().getTransLayer().send(message, initiator.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
		operationFinished(true);
	}

	public int getHops(){
		return hopsOfQuery;
	}
}
