package de.tud.kom.p2psim.impl.overlay.gnutella06.Operations;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.common.OperationCallback;
import de.tud.kom.p2psim.api.overlay.OverlayID;
import de.tud.kom.p2psim.api.transport.TransProtocol;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06Configuration;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayContact;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayID;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Gnutella06OverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06PingMessage;
import de.tud.kom.p2psim.impl.overlay.gnutella06.Messages.Gnutella06QueryMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class Gnutella06QueryOperation extends BaseOperation {
	
	private OverlayID exclude;
	private Integer key;
	private Gnutella06OverlayContact initiator;

	/**
	 * @param component initiator of this operation
	 * @param key document searched for
	 * @param callback
	 */
	public Gnutella06QueryOperation(Gnutella06OverlayNode component, Integer key, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, 0, null, callback);
		this.exclude = component.getOverlayID();
		// inform node about new Query in order to get results
		component.registerQuery(this.getDescriptor(), key);
		this.key = key;
		initiator = component.getOwnContactData();
	}

	/**
	 * @param component initiator of this operation
	 * @param ttl time to live
	 * @param hops hop count
	 * @param descriptor message ID
	 * @param exclude node the query came from
	 * @param key document searched for
	 * @param initiator contact info of the initiator
	 * @param callback
	 */
	public Gnutella06QueryOperation(Gnutella06OverlayNode component, int ttl, int hops, BigInteger descriptor, Gnutella06OverlayID exclude, Integer key, Gnutella06OverlayContact initiator, OperationCallback<Object> callback) {
		super(component, Gnutella06Configuration.GNUTELLA06_QUERY_MAX_TTL, ttl, hops, descriptor, callback);
		this.exclude = exclude;
		this.key = key;
		this.initiator = initiator;
	}

	@Override
	protected void execute() {
		if(ttl > 0) {
			for(Gnutella06OverlayContact contact : this.getComponent().getNodesRoutingTable().allContacts()) {
				if(contact.getOverlayID() != exclude){
					Gnutella06QueryMessage queryMessage = new Gnutella06QueryMessage((Gnutella06OverlayID) this.getComponent().getOverlayID(), contact.getOverlayID(), this.ttl, this.hops, this.descriptor, key, initiator);
					this.getComponent().getTransLayer().send(queryMessage, contact.getTransInfo(), this.getComponent().getPort(), TransProtocol.UDP);
					// inform routing table about outgoing ping
					this.getComponent().getNodesRoutingTable().outgoingQuery(contact, descriptor);
	//				Gnutella06QueryOperation.log.debug("query senden: " + this.getComponent().getOverlayID() + " " + contact.getOverlayID() + " " + "TTL = " + ttl);
				}
			}
		}
		operationFinished(true);
	}
}

