package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.MouseInfo;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.tud.kom.p2psim.impl.network.gnp.topology.Host;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PDragEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PImage;

public class ComponentWorldMap extends PCanvas implements ChangeListener, MouseListener, ActionListener {

	private HostMap mapRef;

	private File worldMapFile = new File("src/de/tud/kom/p2psim/impl/util/gui/gnp/map.gif");

	private PLayer rootLayer;

	private PImage worldLayer = new PImage();

	private PImage hostLayerAll = new PImage();

	private PImage hostLayerSelected = new PImage();

	private PNode gridSelectionLayer = new PNode();

	private int resolution = 8;

	private boolean editMode = false;

	JPopupMenu menu = new JPopupMenu();

	JMenuItem itemAddSelectionToGroup = new JMenuItem("Add Selection to current Group");

	JMenuItem itemSwitchToSingleMode = new JMenuItem("Switch to Single-Selection Mode");

	JMenuItem itemSwitchToGroupMode = new JMenuItem("Switch to Grid-Selection Mode");

	JSlider itemZoom = new JSlider();

	private int noOfGridLon = 96;

	private int noOfGridLat = 48;

	private boolean group[][];

	private PNode selectionGrid[][];

	private String currentGroup = null;

	private Set<Host> currentHosts = new HashSet<Host>();

	HashMap<JCheckBox, Host> content = new HashMap<JCheckBox, Host>();

	private InterfacePanel parent;

	public void setEditMode(boolean on) {
		editMode = on;
		updateMenue();
	}

	public void setCurrentGroups(String group) {
		currentGroup = group;
		currentHosts.clear();
		currentHosts.addAll(mapRef.getGroup(group));
		switchToSingleMode();
		clearGroupGrid();
		updateMenue();
		updateHostLayerSelected();
	}

	public ComponentWorldMap(HostMap mapRef, InterfacePanel parent) {
		super();
		this.mapRef = mapRef;
		this.parent = parent;

		rootLayer = getLayer();

		initialize();

		this.removeInputEventListener(this.getZoomEventHandler());

		this.addMouseListener(this);

		itemZoom.setMinimum(1000);
		itemZoom.setMaximum(10000);
		itemZoom.setValue(1000);
		itemZoom.addChangeListener(this);
		updateMenue();

	}

	private void initialize() {
		initWorldLayer();
		initGridSelectionLayer();
		updateHostLayerAll();
		rootLayer.addChild(hostLayerSelected);
		rootLayer.addChild(gridSelectionLayer);
		rootLayer.addChild(worldLayer);
		rootLayer.addChild(hostLayerAll);
		hostLayerAll.translate(-180, -90);
		hostLayerAll.setScale(1 / (double) resolution);
		hostLayerSelected.translate(-180, -90);
		hostLayerSelected.setScale(1 / (double) resolution);
		this.getCamera().scaleViewAboutPoint(2, -360, -180);
		hostLayerSelected.addInputEventListener(new SingleSelectionEventHandler());
		gridSelectionLayer.addInputEventListener(new GridSelectionEventHandler());
		itemAddSelectionToGroup.addActionListener(this);
		itemSwitchToSingleMode.addActionListener(this);
		itemSwitchToGroupMode.addActionListener(this);
	}

	private void switchToGridMode() {
		gridSelectionLayer.moveToFront();

	}

	private void switchToSingleMode() {
		hostLayerSelected.moveToFront();
	}

	private void initWorldLayer() {
		try {
			worldLayer.setImage(ImageIO.read(worldMapFile));
			worldLayer.setHeight(180);
			worldLayer.setWidth(360);
			worldLayer.translate(-180, -90);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void updateHostLayerAll() {
		BufferedImage image = new BufferedImage(360 * resolution, 180 * resolution, BufferedImage.TYPE_INT_ARGB);
		Graphics2D graphic = image.createGraphics();
		graphic.setColor(Color.BLACK);
		ArrayList<Host> monitors = new ArrayList<Host>();
		for (Host host : mapRef.getHostIndex().values()) {
			if (host.isLocatable()) {
				if (host.getHostType() == Host.MONITOR)
					monitors.add(host);
				int y1 = (int) Math.floor(resolution * (((-1) * host.getLatitude()) + 90));
				int x1 = (int) Math.floor(resolution * (host.getLongitude() + 180));
				graphic.drawRect(x1, y1, 1, 1);
			}
		}
		graphic.setColor(Color.BLUE);
		for (Host monitor : monitors) {
			int y1 = (int) Math.floor(resolution * (((-1) * monitor.getLatitude()) + 90));
			int x1 = (int) Math.floor(resolution * (monitor.getLongitude() + 180));
			graphic.fill(new Ellipse2D.Float(x1 - 6, y1 - 6, 13, 13));
		}
		hostLayerAll.setImage(image);
	}

	public void updateHostLayerSelected() {
		BufferedImage image = new BufferedImage(360 * resolution, 180 * resolution, BufferedImage.TYPE_INT_ARGB);
		Graphics2D graphic = image.createGraphics();
		graphic.setColor(Color.RED);
		for (Host host : currentHosts) {
			if (host.isLocatable()) {
				int y1 = (int) Math.floor(resolution * (((-1) * host.getLatitude()) + 90));
				int x1 = (int) Math.floor(resolution * (host.getLongitude() + 180));
				graphic.fill(new Ellipse2D.Double(x1 - 3, y1 - 3, 7, 7));
			}
		}
		hostLayerSelected.setImage(image);
	}

	private void initGridSelectionLayer() {
		double stepLon = 360 / (double) noOfGridLon;
		double stepLat = 180 / (double) noOfGridLat;
		selectionGrid = new PNode[noOfGridLon][noOfGridLat];
		group = new boolean[noOfGridLon][noOfGridLat];
		for (int x = 0; x < noOfGridLon; x++) {
			for (int y = 0; y < noOfGridLat; y++) {
				PNode aNode = new PNode();
				aNode.setBounds(0, 0, stepLon, stepLat);
				aNode.addAttribute("selectionGrid", true);
				aNode.setPaint(Color.BLUE);
				aNode.setTransparency(0.0F);
				double posX = (-180 + stepLon * x);
				double posY = (-90 + stepLat * y + stepLat);
				aNode.setX(posX);
				aNode.setY(-posY);
				selectionGrid[x][y] = aNode;
				group[x][y] = false;
				gridSelectionLayer.addChild(aNode);
			}
		}
	}

	private void clearGroupGrid() {
		for (int x = 0; x < noOfGridLon; x++) {
			for (int y = 0; y < noOfGridLat; y++) {
				group[x][y] = false;
				selectionGrid[x][y].setTransparency(0.0F);
			}
		}
	}

	private PNode getGroupNode(double x, double y) {
		double longitude = x;
		double latitude = y / -1.0;
		double stepLon = 360 / (double) noOfGridLon;
		double stepLat = 180 / (double) noOfGridLat;

		int posLon = (int) Math.floor((180 + longitude) / stepLon);
		int posLat = (int) Math.floor((90 + latitude) / stepLat);
		return selectionGrid[posLon][posLat];

	}

	public void setMapRef(HostMap mapRef) {
		this.mapRef = mapRef;
		this.updateHostLayerAll();
	}

	class SingleSelectionEventHandler extends PBasicInputEventHandler {

		@Override
		public void mouseClicked(PInputEvent e) {
			super.mouseClicked(e);

			double posX = e.getPosition().getX();
			double posY = e.getPosition().getY();
			Set<Host> hosts = mapRef.getNearestHosts(posX, -posY);
			if (!hosts.isEmpty() && e.getButton() == MouseEvent.BUTTON1) {
				JPopupMenu menu = new JPopupMenu();
				JPanel panel = new JPanel();
				panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
				content.clear();

				for (Host h : hosts) {
					JCheckBox cb = null;
					if (currentHosts.contains(h))
						cb = new JCheckBox(h.toString(), true);
					else
						cb = new JCheckBox(h.toString(), false);
					cb.addActionListener(new ActionListener() {

						public void actionPerformed(ActionEvent e) {
							JCheckBox current = (JCheckBox) e.getSource();
							if (current.isSelected())
								currentHosts.add(content.get(current));
							else
								currentHosts.remove(content.get(current));
						}
					});

					panel.add(cb);
					content.put(cb, h);
				}

				panel.setMaximumSize(new Dimension(300, 300));
				JScrollPane scroll = new JScrollPane(panel);
				scroll.setMaximumSize(new Dimension(300, 300));

				JMenuItem itemSelectAll = new JMenuItem("Select All");
				itemSelectAll.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						for (JCheckBox cb : content.keySet()) {
							cb.setSelected(true);
							currentHosts.add(content.get(cb));
						}
					}
				});
				JMenuItem itemRemoveAll = new JMenuItem("Remove Selected");
				itemRemoveAll.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						for (JCheckBox cb : content.keySet()) {
							cb.setSelected(false);
							currentHosts.remove(content.get(cb));
						}
					}
				});
				menu.add(itemSelectAll);
				menu.add(itemRemoveAll);
				menu.add(scroll);

				int x = (int) MouseInfo.getPointerInfo().getLocation().getX();
				int y = (int) MouseInfo.getPointerInfo().getLocation().getY();
				menu.show(null, x, y);
			}

			updateHostLayerSelected();
		}

	}

	class GridSelectionEventHandler extends PDragEventHandler {

		private boolean delete = false;

		@Override
		protected void startDrag(PInputEvent e) {
			if (e.getPickedNode().getAttribute("selectionGrid") != null) {
				super.startDrag(e);
				e.setHandled(true);
				e.getPickedNode().moveToFront();
				double posX = e.getPositionRelativeTo(e.getPickedNode()).getX();
				double posY = e.getPositionRelativeTo(e.getPickedNode()).getY();
				int x = getGridPosLongitude(posX);
				int y = getGridPosLatitude(-posY);
				delete = (group[x][y]) ? true : false;
			}
		}

		@Override
		protected void drag(PInputEvent e) {
			if (e.getPickedNode().getAttribute("selectionGrid") != null) {
				double posX = e.getPositionRelativeTo(e.getPickedNode()).getX();
				double posY = e.getPositionRelativeTo(e.getPickedNode()).getY();
				PNode groupGrid = getGroupNode(posX, posY);

				int x = getGridPosLongitude(posX);
				int y = getGridPosLatitude(-posY);
				if (delete) {
					groupGrid.setTransparency(0.0f);
					group[x][y] = false;
				} else {
					groupGrid.setTransparency(0.7f);
					group[x][y] = true;
				}
			}
		}
	}

	public int getGridPosLongitude(double longitude) {
		double step = 360 / (double) noOfGridLon;
		return (int) Math.floor((180 + longitude) / step);
	}

	public int getGridPosLatitude(double latitude) {
		double step = 180 / (double) noOfGridLat;
		return (int) Math.floor((90 + latitude) / step);
	}

	public void stateChanged(ChangeEvent e) {
		JSlider slider = (JSlider) e.getSource();
		this.getCamera().setScale(slider.getValue() / 1000.0);

	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON3)
			menu.show((Component) e.getSource(), e.getX(), e.getY());
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == itemAddSelectionToGroup) {
			actionAddSelectionToGroup();
		} else if (e.getSource() == itemSwitchToSingleMode) {
			switchToSingleMode();
		} else if (e.getSource() == itemSwitchToGroupMode) {
			switchToGridMode();
		}

	}

	private void updateMenue() {
		menu.removeAll();
		if (currentGroup != null) {
			itemAddSelectionToGroup.setText("Add Selected Hosts to Group \"" + currentGroup + "\"");
			menu.add(itemAddSelectionToGroup);
		}
		if (editMode) {
			menu.add(itemSwitchToSingleMode);
			menu.add(itemSwitchToGroupMode);
		}
		menu.add(itemZoom);
	}

	private void actionAddSelectionToGroup() {
		mapRef.addHostToGroup(currentGroup, currentHosts);
		mapRef.addHostToGroup(currentGroup, group);
		if (parent instanceof PanelHostsAndGroups)
			((PanelHostsAndGroups) parent).updateLeftPanel();
	}
}
