package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.statistics.DefaultBoxAndWhiskerCategoryDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import de.tud.kom.p2psim.impl.network.gnp.topology.GnpSpace;
import de.tud.kom.p2psim.impl.network.gnp.topology.Host;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;

public class PanelGNP extends JPanel implements ActionListener, MouseListener, InterfacePanel {

	private HostMap mapRef;

	private Frontend frontend;

	private JPanel topLeft = new JPanel();

	private JPanel topRight = new JPanel();

	private JPanel bottomLeft = new JPanel();

	private JPanel bottomRight = new JPanel();

	private JTextField noOfDimensions = new JTextField();

	private JTextField resheduleMonitors = new JTextField();

	private JTextField resheduleHosts = new JTextField();

	private JButton runCalculation = new JButton("Start");

	private JButton cancelCalculation = new JButton("Cancel");

	private JScrollPane scrollHostConnectivity;

	private JTable adjacencyHosts;

	private JProgressBar progressBar = new JProgressBar();

	private JLabel progressLabel = new JLabel();

	GnpThread thread = new GnpThread();

	JPopupMenu menuHosts;

	JMenuItem itemRemoveSelectedHosts = new JMenuItem("Remove Selected Hosts ...");

	public PanelGNP(HostMap map, Frontend frontend) {

		this.frontend = frontend;

		runCalculation.addActionListener(this);
		cancelCalculation.addActionListener(this);

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		topLeft.setLayout(new BoxLayout(topLeft, BoxLayout.Y_AXIS));
		topRight.setLayout(new BoxLayout(topRight, BoxLayout.Y_AXIS));
		bottomLeft.setLayout(new BoxLayout(bottomLeft, BoxLayout.Y_AXIS));
		bottomRight.setLayout(new BoxLayout(bottomRight, BoxLayout.Y_AXIS));

		JSplitPane splitTop = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, topLeft, topRight);
		splitTop.setDividerSize(15);
		splitTop.setBorder(BorderFactory.createEmptyBorder());

		JSplitPane splitBottom = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, bottomLeft, bottomRight);
		splitBottom.setDividerSize(15);
		splitBottom.setBorder(BorderFactory.createEmptyBorder());

		JSplitPane splitHorizontal = new JSplitPane(JSplitPane.VERTICAL_SPLIT, splitTop, splitBottom);
		splitHorizontal.setDividerSize(15);
		splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
		add(splitHorizontal);

		setMap(map);

		splitTop.setDividerLocation(400);
		splitBottom.setDividerLocation(600);
		splitHorizontal.setDividerLocation(200);

		builtMenues();

	}

	public void builtMenues() {

		menuHosts = new JPopupMenu();
		itemRemoveSelectedHosts.addActionListener(this);
		menuHosts.add(itemRemoveSelectedHosts);

	}

	public void setMap(HostMap map) {
		mapRef = map;
		updatePanel();

	}

	public void updatePanel() {
		updateTopLeftPanel();
		updateTopRightPanel();
		updateBottomLeftPanel();
		updateBottomRightPanel();
	}

	private void updateTopRightPanel() {
		topRight.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Hosts and Relative Error:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		HostsTabelModel tm = new HostsTabelModel(mapRef);
		adjacencyHosts = new JTable(tm);
		scrollHostConnectivity = new JScrollPane(adjacencyHosts);
		scrollHostConnectivity.addMouseListener(this);
		adjacencyHosts.addMouseListener(this);
		topRight.add(header);
		topRight.add(Box.createRigidArea(new Dimension(10, 10)));
		topRight.add(scrollHostConnectivity);
		this.revalidate();
	}

	private void updateTopLeftPanel() {
		topLeft.removeAll();
		topLeft.setAlignmentX(0.0f);

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Calculation of GNP-Space (Downhill-Simplex)");
		header.add(titel);
		header.add(Box.createHorizontalGlue());

		JPanel line1 = new JPanel();
		JPanel line2 = new JPanel();
		JPanel line3 = new JPanel();
		JPanel line4 = new JPanel();
		line1.setLayout(new BoxLayout(line1, BoxLayout.X_AXIS));
		line2.setLayout(new BoxLayout(line2, BoxLayout.X_AXIS));
		line3.setLayout(new BoxLayout(line3, BoxLayout.X_AXIS));
		line4.setLayout(new BoxLayout(line4, BoxLayout.X_AXIS));

		noOfDimensions.setPreferredSize(new Dimension(100, (int) noOfDimensions.getMinimumSize().getHeight()));
		resheduleMonitors.setPreferredSize(new Dimension(100, (int) resheduleMonitors.getMinimumSize().getHeight()));
		resheduleHosts.setPreferredSize(new Dimension(100, (int) resheduleHosts.getMinimumSize().getHeight()));

		noOfDimensions.setMaximumSize(new Dimension(100, (int) noOfDimensions.getMinimumSize().getHeight()));
		resheduleMonitors.setMaximumSize(new Dimension(100, (int) resheduleMonitors.getMinimumSize().getHeight()));
		resheduleHosts.setMaximumSize(new Dimension(100, (int) resheduleHosts.getMinimumSize().getHeight()));

		line1.add(Box.createHorizontalGlue());
		line2.add(Box.createHorizontalGlue());
		line3.add(Box.createHorizontalGlue());
		line1.add(new JLabel("Dimensions:"));
		noOfDimensions.setText(Integer.toString(mapRef.getNoOfMonitors() - 1));
		line1.add(noOfDimensions);
		line2.add(new JLabel("Rescheduling Monitor-Positioning:"));
		resheduleMonitors.setText("100");
		line2.add(resheduleMonitors);
		line3.add(new JLabel("Rescheduling Host-Positioning:"));
		resheduleHosts.setText("10");
		line3.add(resheduleHosts);
		topLeft.add(header);
		topLeft.add(Box.createVerticalGlue());
		topLeft.add(line1);
		topLeft.add(line2);
		topLeft.add(line3);
		topLeft.add(Box.createVerticalGlue());
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(runCalculation);
		buttonPanel.add(cancelCalculation);
		runCalculation.setAlignmentX(0.5f);
		cancelCalculation.setAlignmentX(0.5f);
		cancelCalculation.setEnabled(false);
		topLeft.add(buttonPanel);
		topLeft.add(Box.createVerticalGlue());
		topLeft.add(progressBar);
		line4.add(Box.createHorizontalGlue());
		progressLabel.setText("");
		line4.add(progressLabel);
		topLeft.add(line4);
		topLeft.add(Box.createVerticalGlue());
		this.revalidate();
	}

	private void updateBottomLeftPanel() {
		bottomLeft.removeAll();

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Cumulative Distribution of Realtiv Error");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		bottomLeft.add(header);
		bottomLeft.add(Box.createRigidArea(new Dimension(10, 10)));

		XYSeries xySeries = new XYSeries("Cumulative Distribution of Realtiv Error");
		XYSeriesCollection xyDataset = new XYSeriesCollection();

		if (mapRef.getGnpRef() != null) {
			double cd[] = mapRef.getGnpRef().getRelativeErrorDistribution();
			for (int y = 0; y < cd.length; y++) {
				xySeries.add(cd[y], y);
			}
			xyDataset = new XYSeriesCollection(xySeries);
		}

		JFreeChart chartXY = ChartFactory.createXYLineChart(null, "Relative Error", "Cumulative Probability", xyDataset, PlotOrientation.VERTICAL, false, // Legend
				false, // Tooltip
				false); // urls;
		// XYDotRenderer dotRenderer = new XYDotRenderer();
		// dotRenderer.setDotHeight(1);
		// dotRenderer.setDotWidth(1);
		// chartXY.getXYPlot().setRenderer(dotRenderer);
		ChartPanel chartPanel = new ChartPanel(chartXY);

		bottomLeft.add(chartPanel);
		this.revalidate();
	}

	private void updateBottomRightPanel() {
		bottomRight.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Directional Relativ Error");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		bottomRight.add(header);
		bottomRight.add(Box.createRigidArea(new Dimension(10, 10)));

		DefaultBoxAndWhiskerCategoryDataset boxPlotDataset = new DefaultBoxAndWhiskerCategoryDataset();
		// DefaultBoxAndWhiskerXYDataset boxPlotDataset = new
		// DefaultBoxAndWhiskerXYDataset("Test");

		// BoxPlot boxPlotDataset = new BoxPlot();

		if (mapRef.getGnpRef() != null) {
			boxPlotDataset = getErrorBoxPlot();
		}

		JFreeChart chartXY = ChartFactory.createBoxAndWhiskerChart(null, "Measured Distance in ms (50ms Per Box-Plot)", "Directional Relativ Error", boxPlotDataset, false);

		ChartPanel chartPanel = new ChartPanel(chartXY);

		bottomRight.add(chartPanel);

		this.revalidate();
	}

	public DefaultBoxAndWhiskerCategoryDataset getErrorBoxPlot() {
		DefaultBoxAndWhiskerCategoryDataset boxAndWhiskerCategoryDataset = new DefaultBoxAndWhiskerCategoryDataset();

		LinkedList<Double>[] errors = mapRef.getGnpRef().getDirectionalRelativeErrorsGroupedByRtt();

		for (int c = 0; c < errors.length; c++) {
			if (errors[c].size() > 0) {
				if (c < errors.length - 1)
					boxAndWhiskerCategoryDataset.add(errors[c], "X", (c * 50 + 50));
				else
					boxAndWhiskerCategoryDataset.add(errors[c], "X", "> 1000");
			}
		}
		return boxAndWhiskerCategoryDataset;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == runCalculation) {
			actionCalculateGnp();
		} else if (e.getSource() == cancelCalculation) {
			actionCancelCalculateGnp();
		} else if (e.getSource() == itemRemoveSelectedHosts)
			actionRemoveSelectedHosts();
	}

	private void actionRemoveSelectedHosts() {
		int[] rows = adjacencyHosts.getSelectedRows();
		GnpSpace gnp = mapRef.getGnpRef();
		for (int c : rows) {
			mapRef.removeHosts(gnp.getHostsGroupedByMaxRelativeError()[c]);
		}
		gnp.builtStatisticalData();
		this.updatePanel();
		frontend.panelLocateSkitter.updatePanel();
		frontend.panelImportSkitter.updatePanel();
		frontend.panelGroups.updatePanel();
	}

	private void actionCancelCalculateGnp() {
		GnpSpace.calculationInProgress = false;
		runCalculation.setEnabled(true);
		cancelCalculation.setEnabled(false);
		runCalculation.revalidate();
		progressBar.setValue(0);

	}

	private void actionCalculateGnp() {
		thread = new GnpThread();
		runCalculation.setEnabled(false);
		cancelCalculation.setEnabled(true);
		thread.start();
	}

	private class HostsTabelModel extends AbstractTableModel {

		private String[] columnName = { "Relativ Error", "# of related Hosts" };

		HashSet<Host>[] stat2;

		double maxError;

		private HostsTabelModel(HostMap mapRef) {
			if (mapRef.getGnpRef() != null) {
				stat2 = mapRef.getGnpRef().getHostsGroupedByMaxRelativeError();
				maxError = mapRef.getGnpRef().getMaximumRelativeError();
			}
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			if (stat2 == null)
				return 0;
			else
				return stat2.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				if (rowIndex < stat2.length - 1)
					return "[ " + (rowIndex / 10.0) + " - " + ((rowIndex + 1) / 10.0) + " )";
				else
					return "[ " + (rowIndex / 10.0) + " - " + maxError + " ]";
			} else {
				return stat2[rowIndex].size();
			}
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

	class GnpThread extends Thread {
		@Override
		public void run() {
			Timer timer = new Timer();
			timer.schedule(new ProgressUpdate(Integer.parseInt(resheduleMonitors.getText()), mapRef.getNoOfHosts()), 1000, 1000);
			GnpSpace gnp = GnpSpace.getGnp(Integer.parseInt(noOfDimensions.getText()), Integer.parseInt(resheduleMonitors.getText()), Integer.parseInt(resheduleHosts.getText()), mapRef);
			gnp.builtStatisticalData();
			mapRef.setGnpRef(gnp);
			updateBottomLeftPanel();
			updateBottomRightPanel();
			updateTopRightPanel();
			timer.cancel();
			runCalculation.setEnabled(true);
			cancelCalculation.setEnabled(false);
			progressBar.setValue(0);
		}
	}

	class ProgressUpdate extends TimerTask {

		int arg1, arg2;

		public ProgressUpdate(int arg1, int arg2) {
			this.arg1 = arg1;
			this.arg2 = arg2;
			progressBar.setMinimum(0);
		}

		@Override
		public void run() {
			if (GnpSpace.calculationStepStatus == 1) {
				progressBar.setMaximum(arg1);
				progressLabel.setText(GnpSpace.calculationProgressStatus + " / " + arg1 + " Rescheduled Monitor-Positioning");
			} else if (GnpSpace.calculationStepStatus == 2) {
				progressBar.setMaximum(arg2);
				progressLabel.setText(GnpSpace.calculationProgressStatus + " / " + (mapRef.getNoOfHosts() - mapRef.getNoOfMonitors()) + " Hosts");
			} else if (GnpSpace.calculationStepStatus == 3) {
				progressBar.setMaximum(arg2);
				progressLabel.setText(GnpSpace.calculationProgressStatus + " / " + (mapRef.getNoOfHosts() - mapRef.getNoOfMonitors()) + " Hosts");
			}
			progressBar.setValue(GnpSpace.calculationProgressStatus);

		}

	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		if (e.getSource() == adjacencyHosts)
			menuHosts.show((Component) e.getSource(), e.getX(), e.getY());

	}

}
