package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;

import de.tud.kom.p2psim.impl.network.gnp.topology.Host;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;

public class PanelGeolocation extends JPanel implements ActionListener, MouseListener, InterfacePanel {

	private HostMap mapRef;

	ComponentWorldMap wm;

	private Frontend frontend;

	private JPanel left = new JPanel();

	private JPanel right = new JPanel();

	private JScrollPane scrollLocationTable;

	private JTable locationTable;

	/*
	 * JPopupMenu menuSkitterData; JMenuItem itemImportSkitter = new
	 * JMenuItem("Import Skitter-Data ...");
	 */

	JScrollBar sb = new JScrollBar();

	JPopupMenu menuHosts;

	JMenuItem itemRemoveSelectedHosts = new JMenuItem("Remove Selected Hosts ...");

	JPopupMenu menuGeoIpDB;

	JMenuItem itemOpenDatabase = new JMenuItem("Select Database ...");

	JTextField database;

	public PanelGeolocation(HostMap map, Frontend frontend) {

		this.frontend = frontend;

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		right.setLayout(new BoxLayout(right, BoxLayout.X_AXIS));

		JSplitPane splitHorizontal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left, right);
		splitHorizontal.setDividerSize(15);
		splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
		add(splitHorizontal);

		menuGeoIpDB = new JPopupMenu();
		itemOpenDatabase.addActionListener(this);
		menuGeoIpDB.add(itemOpenDatabase);

		menuHosts = new JPopupMenu();
		itemRemoveSelectedHosts.addActionListener(this);
		menuHosts.add(itemRemoveSelectedHosts);

		setMap(map);

		splitHorizontal.setDividerLocation(300);

		// builtMenues();

	}

	public void setMap(HostMap map) {
		mapRef = map;
		updateLeftPanel();
		updateRightPanel();
	}

	public void updatePanel() {
		updateLeftPanel();
		updateRightPanel();
	}

	private void updateRightPanel() {
		right.removeAll();
		wm = new ComponentWorldMap(mapRef, this);
		wm.updateHostLayerAll();
		/*
		 * sb.setValues(1, 10, 1, 100); sb.addAdjustmentListener(this);
		 * wm.setZoom(sb.getValue()/10.0); right.add(sb);
		 */right.add(wm);
	}

	private void updateLeftPanel() {

		mapRef.setLocationOfHosts();

		left.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("GeoIP-Database: ");
		header.add(titel);
		// header.add(Box.createHorizontalGlue());
		database = new JTextField(mapRef.getGeolocationDatabase());
		database.setEditable(false);
		database.addMouseListener(this);
		database.setMaximumSize(new Dimension((int) database.getMaximumSize().getWidth(), (int) database.getPreferredSize().getHeight()));
		header.add(database);
		left.add(header);
		// left.add(database);

		TableSorter sorter = new TableSorter(new LocationTabelModel(mapRef));
		locationTable = new JTable(sorter);
		boolean[] numeric = { false, true };
		sorter.setTableHeader(locationTable.getTableHeader());
		sorter.setSortingStatus(0, TableSorter.ASCENDING);
		sorter.setNumeric(numeric);

		scrollLocationTable = new JScrollPane(locationTable);
		locationTable.addMouseListener(this);
		scrollLocationTable.addMouseListener(this);

		locationTable.addMouseListener(this);

		left.add(scrollLocationTable);

		// bottomLeft.add(Box.createVerticalGlue());

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == itemOpenDatabase)
			actionLoadDatabase();
		else if (e.getSource() == itemRemoveSelectedHosts)
			actionRemoveHosts();
	}

	private void actionRemoveHosts() {
		int[] rows = locationTable.getSelectedRows();
		for (int i : rows) {
			HashMap<String, HashSet<Host>> locations = mapRef.getLocations();
			String key = (String) locationTable.getValueAt(i, 0);
			HashSet<Host> hosts = locations.get(key);
			mapRef.removeHosts(hosts);
		}
		frontend.updatePanelHostsChanged();
	}

	private void actionLoadDatabase() {
		JFileChooser fileChooser = new JFileChooser();
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			String db = fileChooser.getSelectedFile().getAbsolutePath();
			mapRef.setGeolocationDatabase(db);
			frontend.updatePanelHostsChanged();
		}
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		if (e.getSource() == database)
			menuGeoIpDB.show((Component) e.getSource(), e.getX(), e.getY());
		if (e.getSource() == locationTable)
			menuHosts.show((Component) e.getSource(), e.getX(), e.getY());

	}

	private class LocationTabelModel extends AbstractTableModel {

		HashMap<String, HashSet<Host>> locations;

		HostMap mapRef;

		private String[] columnName = { "Location", "# Hosts" };

		private LocationTabelModel(HostMap mapRef) {
			locations = mapRef.getLocations();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return locations.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			String key = (String) locations.keySet().toArray()[rowIndex];
			if (columnIndex == 0)
				return key;
			else if (columnIndex == 1)
				return locations.get(key).size();
			else
				return locations.get(key);
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

}
