package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import de.tud.kom.p2psim.impl.network.gnp.topology.Host;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;

public class PanelHostsAndGroups extends JPanel implements ActionListener,
		MouseListener, InterfacePanel, ListSelectionListener {

	private HostMap mapRef;

	ComponentWorldMap wm;
	private Frontend frontend;

	private JPanel left = new JPanel();
	private JPanel right = new JPanel();

	private GroupTabelModel tableModelGroups;
	private JTable tableGroups;
	private JScrollPane scrollTableGroup;

	private JPopupMenu menuHostsAndGroups;
	private JMenuItem itemAddNewGroup = new JMenuItem("Add New Group");
	private JMenuItem itemBuiltCountryGroups = new JMenuItem(
			"Add Country Groups");
	private JMenuItem itemBuiltRegionGroups = new JMenuItem("Add Region Groups");
	private JMenuItem itemBuiltWorldGroups = new JMenuItem("Add World Group");
	private JMenuItem itemRemoveGroup = new JMenuItem("Remove Selected Group");

	JTextField database;

	public PanelHostsAndGroups(HostMap map, Frontend frontend) {

		this.frontend = frontend;

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		right.setLayout(new BoxLayout(right, BoxLayout.X_AXIS));

		JSplitPane splitHorizontal = new JSplitPane(
				JSplitPane.HORIZONTAL_SPLIT, left, right);
		splitHorizontal.setDividerSize(15);
		splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
		add(splitHorizontal);

		menuHostsAndGroups = new JPopupMenu();
		itemAddNewGroup.addActionListener(this);
		itemRemoveGroup.addActionListener(this);
		itemBuiltCountryGroups.addActionListener(this);
		itemBuiltRegionGroups.addActionListener(this);
		itemBuiltWorldGroups.addActionListener(this);
		menuHostsAndGroups.add(itemAddNewGroup);
		menuHostsAndGroups.add(itemBuiltCountryGroups);
		menuHostsAndGroups.add(itemBuiltRegionGroups);
		menuHostsAndGroups.add(itemBuiltWorldGroups);
		menuHostsAndGroups.add(itemRemoveGroup);

		splitHorizontal.setDividerLocation(300);

		setMap(map);

	}

	public void setMap(HostMap map) {
		mapRef = map;
		initLeftPanel();
		initRightPanel();
	}

	public void updatePanel() {
		updateLeftPanel();
		updateRightPanel();
	}

	private void initLeftPanel() {

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		header.add(new JLabel("Hosts and Groups: "));
		header.add(Box.createHorizontalGlue());

		tableModelGroups = new GroupTabelModel(mapRef);
		TableSorter sorter = new TableSorter(tableModelGroups);
		tableGroups = new JTable(sorter);
		sorter.setTableHeader(tableGroups.getTableHeader());
		sorter.setSortingStatus(0, TableSorter.ASCENDING);
		boolean[] numeric = { false, true };
		sorter.setNumeric(numeric);
		scrollTableGroup = new JScrollPane(tableGroups);

		tableGroups.addMouseListener(this);
		tableGroups.getSelectionModel().addListSelectionListener(this);
		scrollTableGroup.addMouseListener(this);

		left.removeAll();
		left.add(header);
		left.add(Box.createRigidArea(new Dimension(10, 10)));
		left.add(scrollTableGroup);

		updateLeftPanel();
	}

	public void updateLeftPanel() {
		tableModelGroups.fireTableDataChanged();
		((TableSorter) tableGroups.getModel()).fireTableDataChanged();
	}

	private void initRightPanel() {
		wm = new ComponentWorldMap(mapRef, this);
		wm.setEditMode(true);

		right.removeAll();
		right.add(wm);
	}

	private void updateRightPanel() {
		wm.updateHostLayerAll();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == itemAddNewGroup)
			actionAddNewGroup();
		else if (e.getSource() == itemRemoveGroup)
			actionRemoveGroup();
		else if (e.getSource() == itemBuiltCountryGroups)
			actionBuiltCountryGroups();
		else if (e.getSource() == itemBuiltRegionGroups)
			actionBuiltRegionGroups();
		else if (e.getSource() == itemBuiltWorldGroups)
			actionBuiltWorldGroups();
		updateLeftPanel();
//		frontend.panelSimulation.updatePanel();
	}

	private void actionRemoveGroup() {
		int[] rows = tableGroups.getSelectedRows();
		HashSet<String> groupNames = new HashSet<String>();
		for (int i : rows) {
			groupNames.add((String) tableGroups.getValueAt(i, 0));
		}
		mapRef.removeGroup(groupNames);
	}

	private void actionAddNewGroup() {
		this.mapRef.addHostToGroup("(newGroup)", new HashSet<Host>());
	}

	public void actionBuiltCountryGroups() {
		this.mapRef.builtCountryGroups();
	}

	public void actionBuiltRegionGroups() {
		this.mapRef.builtRegionGroups();
	}

	private void actionBuiltWorldGroups() {
		mapRef.builtWorldGroups();
	}

	public void mouseReleased(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON3) {
			if (e.getSource() == tableGroups
					|| e.getSource() == scrollTableGroup)
				menuHostsAndGroups.show((Component) e.getSource(), e.getX(), e
						.getY());
		}
	}

	private class GroupTabelModel extends AbstractTableModel {

		HashMap<String, Set<Host>> groups;
		HostMap mapRef;

		private String[] columnName = { "Name", "# Hosts" };

		private GroupTabelModel(HostMap mapRef) {
			this.mapRef = mapRef;
			this.groups = mapRef.getGroups();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return groups.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			String key = (String) groups.keySet().toArray()[rowIndex];
			if (columnIndex == 0)
				return key;
			else
				return groups.get(key).size();
		}

		@Override
		public boolean isCellEditable(int row, int column) {
			return true;
		}

		@Override
		public void setValueAt(Object obj, int row, int column) {
			String key = (String) getValueAt(row, 0);
			if (column == 0) {
				if (!key.equals(obj)) {
					mapRef.addHostToGroup(obj.toString(), mapRef.getGroup(key));
					mapRef.removeGroup(key);
				}
			} else if (column == 1) {
				try {
					int newSize = Integer.parseInt((String) obj);
					mapRef.scaleGroup(key, newSize);
					updateRightPanel();
				} catch (NumberFormatException e) {
				}
			}
			this.fireTableDataChanged();
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == tableGroups.getSelectionModel()) {
			if (tableGroups.getSelectedRow() != -1) {
				String groupName = (String) tableGroups.getValueAt(tableGroups
						.getSelectedRow(), 0);
				this.wm.setCurrentGroups(groupName);
			}
		}
	}

	public void mouseClicked(MouseEvent e) {
	};

	public void mouseEntered(MouseEvent e) {
	};

	public void mouseExited(MouseEvent e) {
	};

	public void mousePressed(MouseEvent e) {
	};

}
