package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.AbstractTableModel;

import de.tud.kom.p2psim.impl.network.gnp.topology.CountryLookup;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;
import de.tud.kom.p2psim.impl.network.gnp.topology.PingErLookup;

public class PanelPingER extends JPanel implements InterfacePanel, MouseListener, ActionListener {

	private HostMap mapRef;

	private Frontend frontend;

	private JPanel left1 = new JPanel();

	private JPanel right1 = new JPanel();

	private JPanel right2 = new JPanel();

	private JScrollPane scrollPingErFiles;

	private JTable pingErFiles;

	private JTable pingErData;

	private JTable pingErNameMapping;

	JPopupMenu menuPingErFiles = new JPopupMenu();

	JMenuItem itemImportMinRtt = new JMenuItem("Import Minimum RTT - Skitter-Data ...");

	JMenuItem itemImportAveRtt = new JMenuItem("Import Average RTT - Skitter-Data ...");

	JMenuItem itemImportDelayVariation = new JMenuItem("Import Delay Variation - Skitter-Data ...");

	JMenuItem itemImportPacketLoss = new JMenuItem("Import Packet Loss - Skitter-Data ...");

	JPopupMenu menuPingErCountry = new JPopupMenu();

	JMenuItem itemImportCountry = new JMenuItem("Import \"region_country.txt\" ...");

	JTextField database;

	public PanelPingER(HostMap map, Frontend frontend) {

		this.frontend = frontend;

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		left1.setLayout(new BoxLayout(left1, BoxLayout.Y_AXIS));
		right1.setLayout(new BoxLayout(right1, BoxLayout.Y_AXIS));
		right2.setLayout(new BoxLayout(right2, BoxLayout.Y_AXIS));

		JSplitPane splitRight = new JSplitPane(JSplitPane.VERTICAL_SPLIT, right1, right2);
		splitRight.setDividerLocation(300);
		splitRight.setBorder(BorderFactory.createEmptyBorder());

		JSplitPane splitHorizontal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left1, splitRight);
		splitHorizontal.setDividerLocation(300);
		splitHorizontal.setBorder(BorderFactory.createEmptyBorder());

		add(splitHorizontal);

		menuPingErFiles.add(itemImportMinRtt);
		menuPingErFiles.add(itemImportAveRtt);
		menuPingErFiles.add(itemImportDelayVariation);
		menuPingErFiles.add(itemImportPacketLoss);
		itemImportMinRtt.addActionListener(this);
		itemImportAveRtt.addActionListener(this);
		itemImportDelayVariation.addActionListener(this);
		itemImportPacketLoss.addActionListener(this);

		menuPingErCountry.add(itemImportCountry);
		itemImportCountry.addActionListener(this);

		setMap(map);

	}

	public void setMap(HostMap map) {
		mapRef = map;
		updatePanel();
	}

	public void updatePanel() {
		updateLeftPanel();
		updateTopRightPanel();
		updateBottomRightPanel();
		this.revalidate();

	}

	public void updateLeftPanel() {
		left1.removeAll();

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("PingER Summary Reports:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());

		TableSorter sorter = new TableSorter(new PingErFilesTabelModel(mapRef));
		pingErFiles = new JTable(sorter);
		scrollPingErFiles = new JScrollPane(pingErFiles);
		pingErFiles.addMouseListener(this);
		scrollPingErFiles.addMouseListener(this);
		boolean[] numeric = { false, false };
		sorter.setTableHeader(pingErFiles.getTableHeader());
		sorter.setSortingStatus(1, TableSorter.ASCENDING);
		sorter.setNumeric(numeric);

		left1.add(header);
		left1.add(Box.createRigidArea(new Dimension(10, 10)));
		left1.add(scrollPingErFiles);
	}

	public void updateTopRightPanel() {
		right1.removeAll();

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Region to Region Data:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());

		// TableSorter sorter = new TableSorter(new
		// PingErAdjacencyTabelModel(mapRef));
		pingErData = new JTable(new PingErAdjacencyTabelModel(mapRef));
		pingErData.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		JScrollPane scrollPingErData = new JScrollPane(pingErData, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		// boolean[] numeric = {false, false};
		// sorter.setTableHeader(pingErFiles.getTableHeader());
		// sorter.setSortingStatus(0, TableSorter.ASCENDING);
		// sorter.setNumeric(numeric);

		right1.add(header);
		right1.add(Box.createRigidArea(new Dimension(10, 10)));
		right1.add(scrollPingErData);
	}

	public void updateBottomRightPanel() {
		right2.removeAll();

		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Skitter-PingEr Mapping:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());

		JPanel header2 = new JPanel();
		header2.setLayout(new BoxLayout(header2, BoxLayout.X_AXIS));
		JLabel titel2 = new JLabel("PingER \"region_country.txt\": ");
		header2.add(titel2);
		database = new JTextField(mapRef.getCountryLookup().getPingErCountryRegionFilename());
		database.setEditable(false);
		database.addMouseListener(this);
		database.setMaximumSize(new Dimension((int) database.getMaximumSize().getWidth(), (int) database.getPreferredSize().getHeight()));
		header2.add(database);

		TableSorter sorter = new TableSorter(new CountryLockupTableModel(mapRef));
		pingErNameMapping = new JTable(sorter);
		boolean[] numeric = { false, false, false, false };
		sorter.setTableHeader(pingErNameMapping.getTableHeader());
		sorter.setSortingStatus(0, TableSorter.DESCENDING);
		sorter.setNumeric(numeric);
		pingErNameMapping.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		ArrayList<String> pingErCountry = mapRef.getCountryLookup().getPingErCountrys();
		JComboBox comboBox1 = new JComboBox(pingErCountry.toArray());
		pingErNameMapping.getColumn("PingER Country").setCellEditor(new DefaultCellEditor(comboBox1));

		ArrayList<String> pingErRegion = mapRef.getCountryLookup().getPingErRegions();
		JComboBox comboBox2 = new JComboBox(pingErRegion.toArray());
		pingErNameMapping.getColumn("PingER Region").setCellEditor(new DefaultCellEditor(comboBox2));

		right2.add(header);
		right2.add(Box.createRigidArea(new Dimension(10, 10)));
		right2.add(header2);
		// right2.add(Box.createRigidArea(new Dimension(10,10)));
		right2.add(new JScrollPane(pingErNameMapping));

	}

	private class PingErFilesTabelModel extends AbstractTableModel {

		private PingErLookup pingEr;

		private String[] columnName = { "File", "# Type" };

		private PingErFilesTabelModel(HostMap mapRef) {
			this.pingEr = mapRef.getPingErLookup();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return pingEr.getFiles().size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			String file = (String) pingEr.getFiles().keySet().toArray()[rowIndex];
			if (columnIndex == 0)
				return file;
			if (columnIndex == 1)
				return pingEr.getFiles().get(file);
			else
				return null;
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

	private class PingErAdjacencyTabelModel extends AbstractTableModel {

		private PingErLookup pingEr;

		private Set<String> regions = new HashSet<String>();

		private String[] columnName = { "File", "# Type" };

		private PingErAdjacencyTabelModel(HostMap mapRef) {
			this.pingEr = mapRef.getPingErLookup();
			for (String name : pingEr.getData().keySet()) {
				if (mapRef.getCountryLookup().containsPingErCountryOrRegion(name))
					regions.add(name);
			}
		}

		public int getColumnCount() {
			if (regions.size() == 0)
				return 0;
			else
				return regions.size() + 1;
		}

		public int getRowCount() {
			return regions.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				return regions.toArray()[rowIndex];
			} else {
				String from = (String) regions.toArray()[rowIndex];
				String to = (String) regions.toArray()[columnIndex - 1];
				Object value = pingEr.getData().get(from).get(to);
				if (value == null) {
					value = pingEr.getData().get(to).get(from);
					if (value != null)
						value = "( " + value + " )";
				}
				return value;
			}
		}

		@Override
		public String getColumnName(int num) {
			if (num == 0)
				return "";
			else
				return (String) regions.toArray()[num - 1];
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == itemImportMinRtt)
			actionAddPingErMinimumRtt();
		else if (e.getSource() == itemImportAveRtt)
			actionAddPingErAverageRtt();
		else if (e.getSource() == itemImportDelayVariation)
			actionAddPingErDelayVariation();
		else if (e.getSource() == itemImportPacketLoss)
			actionAddPingErPacketLoss();
		else if (e.getSource() == itemImportCountry)
			actionImportPingErCountryRegionFile();
		updatePanel();
	}

	private void actionImportPingErCountryRegionFile() {
		JFileChooser fileChooser = new JFileChooser();
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			mapRef.getCountryLookup().importPingErCountryRegionFile(fileChooser.getSelectedFile());
		}
	}

	public void actionAddPingErMinimumRtt() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(true);
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			for (File file : fileChooser.getSelectedFiles()) {
				mapRef.importPingErMinimumRtt(file);
			}
		}
	}

	public void actionAddPingErAverageRtt() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(true);
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			for (File file : fileChooser.getSelectedFiles()) {
				mapRef.importPingErAverageRtt(file);
			}
		}
	}

	public void actionAddPingErDelayVariation() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(true);
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			for (File file : fileChooser.getSelectedFiles()) {
				mapRef.importPingErDelayVariation(file);
			}
		}
	}

	public void actionAddPingErPacketLoss() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(true);
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			for (File file : fileChooser.getSelectedFiles()) {
				mapRef.importPingErPacketLoss(file);
			}
		}
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		if (e.getSource() == pingErFiles || e.getSource() == scrollPingErFiles) {
			menuPingErFiles.show((Component) e.getSource(), e.getX(), e.getY());
		} else if (e.getSource() == database)
			menuPingErCountry.show((Component) e.getSource(), e.getX(), e.getY());

	}

	private class CountryLockupTableModel extends AbstractTableModel {

		CountryLookup countryLockup = null;

		String[] columnName = { "Code", "GeoIP Country", "PingER Country", "PingER Region" };

		public CountryLockupTableModel(HostMap mapRef) {
			this.countryLockup = mapRef.getCountryLookup();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return countryLockup.getCountryCodes().size();
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

		@Override
		public boolean isCellEditable(int row, int column) {
			if (column == 2 || column == 3)
				return true;
			else
				return false;
		}

		public Object getValueAt(int row, int column) {
			String cc = (String) countryLockup.getCountryCodes().toArray()[row];
			if (column == 0)
				return cc;
			if (column == 1)
				return countryLockup.getGeoIpCountryName(cc);
			if (column == 2)
				return countryLockup.getPingErCountryName(cc);
			if (column == 3)
				return countryLockup.getPingErRegionName(cc);
			return null;
		}

		@Override
		public void setValueAt(Object obj, int row, int column) {
			if (column == 2)
				countryLockup.assignCountryCodeToPingErData((String) getValueAt(row, 0), (String) obj, countryLockup.getPingErRegionName((String) obj));
			if (column == 3)
				countryLockup.assignCountryCodeToPingErData((String) getValueAt(row, 0), "", (String) obj);
			this.fireTableDataChanged();
		}
	}

}
