package de.tud.kom.p2psim.impl.util.gui.gnp;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;
import de.tud.kom.p2psim.impl.network.gnp.topology.Host;
import de.tud.kom.p2psim.impl.network.gnp.topology.HostMap;

public class PanelSkitter extends JPanel implements ActionListener,
		MouseListener, InterfacePanel {

	private HostMap mapRef;
	private Frontend frontend;

	private JPanel topLeft = new JPanel();
	private JPanel topRight = new JPanel();
	private JPanel bottomLeft = new JPanel();
	private JPanel bottomRight = new JPanel();

	private JScrollPane scrollSkitterFiles;
	private JTable skitterFiles;
	private JScrollPane scrollAdjacencyMonitors;
	private JTable adjacencyMonitors;
	private JScrollPane scrollHostConnectivity;
	private JTable adjacencyHosts;

	JPopupMenu menuSkitterData;
	JMenuItem itemImportSkitter = new JMenuItem("Import Skitter-Data ...");

	JPopupMenu menuMonitors;
	JMenuItem itemRemoveSelectedMonitor = new JMenuItem(
			"Remove Selected Monitors ...");
	JMenuItem itemMakeSymmetrical = new JMenuItem("Make Symmetrical");
	JMenuItem itemRemoveMaximumSeparated = new JMenuItem(
			"Keep Maximum Separated Monitor ...");

	JPopupMenu menuHosts;
	JMenuItem itemRemoveSelectedHosts = new JMenuItem(
			"Remove Selected Hosts ...");

	public PanelSkitter(HostMap map, Frontend frontend) {

		this.frontend = frontend;

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		topLeft.setLayout(new BoxLayout(topLeft, BoxLayout.Y_AXIS));
		topRight.setLayout(new BoxLayout(topRight, BoxLayout.Y_AXIS));
		bottomLeft.setLayout(new BoxLayout(bottomLeft, BoxLayout.Y_AXIS));
		bottomRight.setLayout(new BoxLayout(bottomRight, BoxLayout.Y_AXIS));

		JSplitPane splitTop = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				topLeft, topRight);
		splitTop.setDividerSize(15);
		splitTop.setBorder(BorderFactory.createEmptyBorder());

		JSplitPane splitBottom = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				bottomLeft, bottomRight);
		splitBottom.setDividerSize(15);
		splitBottom.setBorder(BorderFactory.createEmptyBorder());

		JSplitPane splitHorizontal = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				splitTop, splitBottom);
		splitHorizontal.setDividerSize(15);
		splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
		add(splitHorizontal);

		setMap(map);

		splitTop.setDividerLocation(400);
		splitBottom.setDividerLocation(600);
		splitHorizontal.setDividerLocation(200);

		builtMenues();

	}

	public void builtMenues() {
		menuSkitterData = new JPopupMenu();
		itemImportSkitter.addActionListener(this);
		menuSkitterData.add(itemImportSkitter);

		menuMonitors = new JPopupMenu();
		itemRemoveSelectedMonitor.addActionListener(this);
		itemMakeSymmetrical.addActionListener(this);
		itemRemoveMaximumSeparated.addActionListener(this);
		menuMonitors.add(itemRemoveSelectedMonitor);
		menuMonitors.add(itemMakeSymmetrical);
		menuMonitors.add(itemRemoveMaximumSeparated);

		menuHosts = new JPopupMenu();
		itemRemoveSelectedHosts.addActionListener(this);
		menuHosts.add(itemRemoveSelectedHosts);

	}

	public void setMap(HostMap map) {
		mapRef = map;
		updatePanel();
	}

	public void updatePanel() {
		updateTopLeftPanel();
		updateTopRightPanel();
		updateBottomLeftPanel();
		updateBottomRightPanel();
	}

	private void updateTopRightPanel() {
		topRight.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Skitter-Data:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		SkitterFilesTabelModel tm = new SkitterFilesTabelModel(mapRef);

		skitterFiles = new JTable(tm);
		scrollSkitterFiles = new JScrollPane(skitterFiles);
		skitterFiles.addMouseListener(this);
		scrollSkitterFiles.addMouseListener(this);
		topRight.add(header);
		topRight.add(scrollSkitterFiles);
		this.revalidate();
	}

	private void updateTopLeftPanel() {
		topLeft.removeAll();
		this.revalidate();
	}

	private void updateBottomLeftPanel() {
		bottomLeft.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Monitor to Monitors RTTs:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		MonitorsTabelModel tm = new MonitorsTabelModel(mapRef);
		adjacencyMonitors = new JTable(tm);
		scrollAdjacencyMonitors = new JScrollPane(adjacencyMonitors);
		scrollAdjacencyMonitors.addMouseListener(this);
		adjacencyMonitors.addMouseListener(this);
		bottomLeft.add(header);
		bottomLeft.add(scrollAdjacencyMonitors);
		this.revalidate();
	}

	private void updateBottomRightPanel() {
		bottomRight.removeAll();
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
		JLabel titel = new JLabel("Monitor to Hosts RTTs:");
		header.add(titel);
		header.add(Box.createHorizontalGlue());
		HostsTabelModel tm = new HostsTabelModel(mapRef);
		adjacencyHosts = new JTable(tm);
		scrollHostConnectivity = new JScrollPane(adjacencyHosts);
		scrollHostConnectivity.addMouseListener(this);
		adjacencyHosts.addMouseListener(this);
		bottomRight.add(header);
		bottomRight.add(scrollHostConnectivity);
		this.revalidate();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == itemImportSkitter)
			actionImportSkitterFiles();
		else if (e.getSource() == itemRemoveSelectedMonitor)
			actionRemoveSelectedMonitor();
		else if (e.getSource() == itemMakeSymmetrical)
			actionMakeSymmetrical();
		else if (e.getSource() == itemRemoveMaximumSeparated)
			actionRemoveMaximumSeparated();
		else if (e.getSource() == itemRemoveSelectedHosts)
			actionRemoveSelectedHosts();
	}

	private void actionRemoveSelectedHosts() {
		int[] rows = adjacencyHosts.getSelectedRows();
		Set<Integer> delete = new HashSet<Integer>();
		for (int i : rows) {
			delete.add((Integer) adjacencyHosts.getValueAt(i, 1));
		}
		for (Integer connections : delete) {
			mapRef.removeHosts(connections);
		}
		frontend.updatePanelHostsChanged();
	}

	private void actionRemoveMaximumSeparated() {
		JTextField tfKeep = new JTextField();
		String keepString = new String();
		keepString = (String) JOptionPane.showInputDialog(tfKeep,
				"Keep # maximum separated Monitors:", "Keep Monitors",
				JOptionPane.PLAIN_MESSAGE, null, null, keepString);
		int keepInt = Integer.parseInt(keepString);
		mapRef.removeMonitorsKeepMaximumSparation(keepInt);
		frontend.updatePanelHostsChanged();
	}

	private void actionMakeSymmetrical() {
		mapRef.makeSymmetrical();
		updateBottomLeftPanel();
	}

	private void actionRemoveSelectedMonitor() {
		int[] rows = adjacencyMonitors.getSelectedRows();
		Set<Host> delete = new HashSet<Host>();
		for (int i : rows) {
			delete.add((Host) adjacencyMonitors.getValueAt(i, 0));
		}
		for (Host host : delete) {
			mapRef.removeMonitor(host);
		}
		frontend.updatePanelHostsChanged();
	}

	private void actionImportSkitterFiles() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(true);
		int status = fileChooser.showOpenDialog(this);
		if (status == JFileChooser.APPROVE_OPTION) {
			for (File skitterFile : fileChooser.getSelectedFiles()) {
				mapRef.importSkitterFile(skitterFile);
				updateBottomLeftPanel();
				updateBottomRightPanel();
				updateTopRightPanel();
			}
			frontend.updatePanelHostsChanged();
		}
	}

	private class SkitterFilesTabelModel extends AbstractTableModel {

		private HashMap<String, HashSet<Host>> skitterFiles;

		private String[] columnName = { "File", "Hosts" };

		private SkitterFilesTabelModel(HostMap mapRef) {
			skitterFiles = mapRef.getImportedSkitterFiles();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return skitterFiles.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			String file = (String) skitterFiles.keySet().toArray()[rowIndex];
			if (columnIndex == 0)
				return file;
			if (columnIndex == 1)
				return skitterFiles.get(file).size();
			else
				return null;
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

	private class MonitorsTabelModel extends AbstractTableModel {

		private Collection<Host> monitors;

		private MonitorsTabelModel(HostMap mapRef) {
			monitors = mapRef.getMonitorIndex().values();
		}

		public int getColumnCount() {
			if (monitors.isEmpty())
				return 0;
			else
				return monitors.size() + 1;
		}

		public int getRowCount() {
			return monitors.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				return monitors.toArray()[rowIndex];
			} else {
				Host sender = (Host) monitors.toArray()[rowIndex];
				Host receiver = (Host) monitors.toArray()[columnIndex - 1];
				return (sender.getRtt(receiver) >= 0) ? sender.getRtt(receiver)
						: null;
			}
		}

		@Override
		public String getColumnName(int num) {
			if (monitors.isEmpty())
				return "";
			else if (num == 0)
				return "";
			else
				return monitors.toArray()[num - 1].toString();
		}

	}

	private class HostsTabelModel extends AbstractTableModel {

		private String[] columnName = { "# Hosts ", "RTTs to # Monitors" };

		private int[] connectivity;

		private HostsTabelModel(HostMap mapRef) {
			connectivity = mapRef.getConnectivityOfHosts();
		}

		public int getColumnCount() {
			return columnName.length;
		}

		public int getRowCount() {
			return connectivity.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				return connectivity[rowIndex];
			} else {
				return rowIndex;
			}
		}

		@Override
		public String getColumnName(int num) {
			return columnName[num];
		}

	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {

	}

	public void mousePressed(MouseEvent e) {

	}

	public void mouseReleased(MouseEvent e) {
		if (e.getSource() == scrollSkitterFiles
				|| e.getSource() == skitterFiles)
			menuSkitterData.show((Component) e.getSource(), e.getX(), e.getY());
		else if (e.getSource() == scrollAdjacencyMonitors
				|| e.getSource() == adjacencyMonitors)
			menuMonitors.show((Component) e.getSource(), e.getX(), e.getY());
		if (e.getSource() == scrollHostConnectivity
				|| e.getSource() == adjacencyHosts)
			menuHosts.show((Component) e.getSource(), e.getX(), e.getY());

	}

}
