package de.tud.kom.p2psim.impl.util.guirunner;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import de.tud.kom.p2psim.impl.util.guirunner.impl.DirView;
import de.tud.kom.p2psim.impl.util.guirunner.impl.LastOpened;
import de.tud.kom.p2psim.impl.util.guirunner.impl.RunnerController;
import de.tud.kom.p2psim.impl.util.vis.util.Config;
import de.tud.kom.p2psim.impl.util.vis.util.GlobalKeyEventDispatcher;

/**
 * A window to select a configuration file to run PFS from.
 * Useful for developers switching to different configuration files
 * many times, as well as for presentations etc.
 * 
 * @author Leo Nobach
 * @version 3.0, 25.11.2008
 *
 */
public class GUIRunner extends JFrame implements WindowListener, KeyListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6126914669745711438L;
	
	/*
	 * Konfigurationspfade
	 */
	static final String CONF_PATH = "GUIRunner/MainWindow";
	static final String CONF_PATH_WIDTH = CONF_PATH + "Width";
	static final String CONF_PATH_HEIGHT = CONF_PATH + "Height";
	static final String CONF_PATH_POSX = CONF_PATH + "PosX";
	static final String CONF_PATH_POSY = CONF_PATH + "PosY";
	
	public static final String DEFAULT_CONFIG_DIR = "./config";
	public static final Image frameIcon = new ImageIcon("images/icons/frame_icon.png").getImage();
	
	public static LastOpened lastOpened = new LastOpened();
	public static RunnerController ctrl = new RunnerController();
	
	public GUIRunner() {
		ctrl.setLastOpened(lastOpened);
		ctrl.setMainWindow(this);
		
		this.setTitle("PeerfactSim.KOM - Select Launch Configuration");
		this.setIconImage(frameIcon);
		
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);
		
		this.setSize(Config.getValue(CONF_PATH_WIDTH, 600), Config.getValue(CONF_PATH_HEIGHT, 600));
		this.setLocation(new Point(Config.getValue(CONF_PATH_POSX, 0), Config.getValue(CONF_PATH_POSY, 0)));
		
		this.setLayout(new BorderLayout());
		
		JScrollPane sp = new JScrollPane(new DirView(ctrl, lastOpened));
		
		this.add(sp, BorderLayout.CENTER);
		this.add(new ButtonBar(), BorderLayout.SOUTH);
		GlobalKeyEventDispatcher disp = new GlobalKeyEventDispatcher(this);
		disp.addKeyListener(this);
		
		this.setVisible(true);
	}
	
	public class ButtonBar extends JPanel implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = -7482502225920138689L;
		
		private JButton cancelBtn;

		public ButtonBar() {
			this.setLayout(new FlowLayout());
			
			JButton launchBtn = new JButton("Start Simulation");
			ctrl.setLaunchButton(launchBtn);
			this.add(launchBtn);
			
			JButton startVisBtn = new JButton("Visualization Stand-Alone");
			ctrl.setStartVisBtn(startVisBtn);
			this.add(startVisBtn);
			
			cancelBtn = new JButton("Cancel");
			this.add(cancelBtn);
			
			cancelBtn.addActionListener(this);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == cancelBtn) closeRunner();
		}

	}

	public void closeRunner() {
		disposeRunner();
		System.exit(0);
	}
	
	public void disposeRunner() {
		saveSettings();
		this.setVisible(false);
		this.dispose();
		Config.writeXMLFile();
	}
	
	/**
	 * Speichert Einstellungen wie Fenstergröße o.ä.
	 */
	public void saveSettings() {
		
		Config.setValue(CONF_PATH_WIDTH, this.getWidth());
		Config.setValue(CONF_PATH_HEIGHT, this.getHeight());
		Config.setValue(CONF_PATH_POSX, this.getX());
		Config.setValue(CONF_PATH_POSY, this.getY());
	}

	@Override
	public void windowActivated(WindowEvent arg0) {
		//Nothing to do
		
	}

	@Override
	public void windowClosed(WindowEvent arg0) {
		//Nothing to do
		
	}

	@Override
	public void windowClosing(WindowEvent arg0) {
		closeRunner();
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) {
		//Nothing to do
		
	}

	@Override
	public void windowDeiconified(WindowEvent arg0) {
		//Nothing to do
		
	}

	@Override
	public void windowIconified(WindowEvent arg0) {
		//Nothing to do
		
	}

	@Override
	public void windowOpened(WindowEvent arg0) {
		//Nothing to do
		
	}
	
	public static void main(String[] args) {
		new GUIRunner();
	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER) ctrl.invokeRunSimulator();
		else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) this.closeRunner();
	}

	@Override
	public void keyReleased(KeyEvent e) {
		//Nothing to do
		
	}

	@Override
	public void keyTyped(KeyEvent e) {
		//Nothing to do
	}
	
}
