package de.tud.kom.p2psim.impl.util.guirunner.impl;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

/**
 * A tree view that shows available and recently opened configuration files.
 * 
 * @author Leo Nobach
 * @version 3.0, 25.11.2008
 *
 */
public class DirView extends JTree implements TreeSelectionListener, MouseListener {

	private RunnerController ctrl;
	
	private ConfigTreeNode selectedConfigNode = null;

	public DirView(RunnerController ctrl, LastOpened lastOpened) {
		super();
		
		this.ctrl = ctrl;
		
		NodeModel mdl = new NodeModel(lastOpened);
		
		this.setModel(new DefaultTreeModel(mdl.getRoot()));
		this.getSelectionModel().setSelectionMode
        (TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.addTreeSelectionListener(this);
		this.expandAll(this);
		
		this.addMouseListener(this);
		
		this.setCellRenderer(new NodeRenderer());
		
		initialSelect();


	}

	/**
	 * Selects the row that should be selected on startup.
	 */
	private void initialSelect() {
		this.setSelectionRow(2);
	}

	/**
	 * Expands all entries in the tree view.
	 * @param tree
	 */
	public void expandAll(JTree tree) {
	    int row = 0;
		while (row < tree.getRowCount()) {
			tree.expandRow(row);
			row++;
	    }
	}

	
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		Object selectedNode = this.getLastSelectedPathComponent();
		
		if (selectedNode instanceof ConfigTreeNode) {
			selectedConfigNode = (ConfigTreeNode)selectedNode;
		} else {
			selectedConfigNode = null;
		}
		
		ctrl.selectFile(selectedConfigNode==null?null:selectedConfigNode.getConfigFile());
		
		//System.out.println("Ausgewählte ConfigNode: " + selectedConfigNode);
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
		if (arg0.getClickCount() == 2 && selectedConfigNode != null)
			ctrl.invokeRunSimulator();
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
}
