package de.tud.kom.p2psim.impl.util.guirunner.impl;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * The manager for recently opened files.
 * 
 * @author Leo Nobach
 * @version 3.0, 25.11.2008
 *
 */
public class LastOpened {

	public static final String LAST_OPENED_CONF = "./guiCfg/runnerLastOpened.files";
	public static final int LAST_OPENED_LENGTH = 5;
	
	List<File> lastOpened = new ArrayList<File>();
	
	public LastOpened() {
		this.loadFromFile();
	}
	
	/**
	 * Appends a file to the recently opened files.
	 * @param f
	 */
	public void append(File f) {
		
		removeOccurrences(f);
		
		lastOpened.add(0, f);
		
		correctSize();
	}
	
	/**
	 * Removes the occurrence from the last opened files.
	 * 
	 * Nötig, da equals und damit remove entsprechend
	 * bei Dateien nicht funktioniert!
	 * @param f
	 */
	private void removeOccurrences(File f) {
		for (File f2 : lastOpened) {
			if (f2.getAbsolutePath().equals(f.getAbsolutePath())) {
				lastOpened.remove(f2);
				break;
			}
		}
	}
	
	/**
	 * Returns all last opened files that are saved.
	 * @return
	 */
	public List<File> getLastOpened() {
		return lastOpened;
	}

	/**
	 * Cuts the size to the maximum entries allowed.
	 */
	private void correctSize() {
		if (lastOpened.size() > LAST_OPENED_LENGTH)
			lastOpened = lastOpened.subList(0, LAST_OPENED_LENGTH -1);
	}
	
	/**
	 * Loads the entries from the configuration file.
	 */
	public void loadFromFile() {
		
		lastOpened = new ArrayList<File>();
		
		BufferedReader cfgFileStream;
		try {
			cfgFileStream = new BufferedReader(new FileReader(LAST_OPENED_CONF));
			
			String line;
			while((line = cfgFileStream.readLine()) != null) {
				
				File openedFile = new File(line);
				if (openedFile.exists()) {
					lastOpened.add(openedFile);
				} else {
					System.err.println("Datei \"" + line + "\" existiert nicht mehr. Wird herausgenommen.");
				}
			}
			
		} catch (FileNotFoundException e) {
			//einfach leere Liste übergeben.
		} catch (IOException e) {
			System.err.println("Fehler beim Einlesen der zuletzt geöffneten Dateien: ");
			e.printStackTrace();
		}
		
		correctSize();
	}
	
	/**
	 * Saves the entries to the configuration file
	 */
	public void saveToFile() {
		try {
			
			BufferedWriter w = new BufferedWriter(new FileWriter(LAST_OPENED_CONF));
			
			for (File f : lastOpened) {
				w.write(f.getAbsolutePath() + "\r\n");
			}
			
			w.close();
			
		} catch (IOException e) {
			System.err.println("Fehler beim Schreiben der zuletzt geöffneten Dateien: ");
			e.printStackTrace();
		}
	}
	
}
