package de.tud.kom.p2psim.impl.util.guirunner.impl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;

import de.tud.kom.p2psim.impl.util.guirunner.GUIRunner;
import de.tud.kom.p2psim.impl.util.vis.controller.Controller;

/**
 * 
 * Runs the simulator or invokes operations on the view.
 * 
 * @author Leo Nobach
 * @version 3.0, 25.11.2008
 *
 */
public class RunnerController implements ActionListener {

	LastOpened lastOpened = null;
	File selectedFile = null;
	JButton launchButton = null;
	private GUIRunner runner;
	private JButton startVisBtn = null;

	/**
	 * Runs the simulator with the specified config file string.
	 * @param configFile
	 */
	private void runSimulator(String configFile) {
		new SimulationThread(configFile).start();
	}
	
	public void setLastOpened(LastOpened lastOpened) {
		this.lastOpened = lastOpened;
	}
	
	/**
	 * Called when the user wants to start the simulation.
	 */
	public void invokeRunSimulator() {
		
		if (selectedFile == null) return;	//No file is selected!
		
		if (lastOpened != null) lastOpened.append(selectedFile);
		lastOpened.saveToFile();
		
		System.out.println("GUIRunner: Starte Simulator mit " + selectedFile.getAbsolutePath());
		runner.disposeRunner();
		
		runSimulator(selectedFile.getAbsolutePath());
	}

	/**
	 * Called when the user has selected a file.
	 * @param file
	 */
	public void selectFile(File file) {
		selectedFile = file;
		
		if (launchButton != null) {
			launchButton.setEnabled(selectedFile != null);
		}
	}
	
	/**
	 * Sets the launch button that invokes the launch of the
	 * simulation
	 * @param b
	 */
	public void setLaunchButton(JButton b) {
		b.addActionListener(this);
		this.launchButton = b;
	}
	
	/**
	 * Sets the button that invokes the standalone execution of
	 * the visualization.
	 * @param b
	 */
	public void setStartVisBtn(JButton b) {
		b.addActionListener(this);
		this.startVisBtn  = b;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() == launchButton) {
			invokeRunSimulator();
		} else if (arg0.getSource() == startVisBtn) {
			invokeStartVis();
		}
	}

	/**
	 * Starts the visualization stand-alone.
	 */
	private void invokeStartVis() {
		System.out.println("GUIRunner: Starte Visualisierung");
		runner.disposeRunner();
		Controller.init();
	}

	/**
	 * Sets the main window of the GUIRunner.
	 * @param runner
	 */
	public void setMainWindow(GUIRunner runner) {
		this.runner=runner;
	}
	

}
