package de.tud.kom.p2psim.impl.util.guirunner.progress;

import java.io.Writer;

import de.tud.kom.p2psim.api.analyzer.Analyzer.NetAnalyzer;
import de.tud.kom.p2psim.api.analyzer.Analyzer.OperationAnalyzer;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.network.NetMessage;
import de.tud.kom.p2psim.impl.simengine.Simulator;

public class ProgressAnalyzer implements NetAnalyzer, OperationAnalyzer {

	SimulationProgressView view;
	
	int messagesIn = 0;
	int messagesOut = 0;
	int operationsInitiated = 0;
	int operationsFinished = 0;
	int netMsgsDropped = 0;
	int netMsgsReceived = 0;
	int netMsgsSent = 0;
	
	public ProgressAnalyzer() {
		view = SimulationProgressView.getInstance();
	
		view.addProgressValue(new SimulationTime());
		view.addProgressValue(new MessagesIn());
		view.addProgressValue(new MessagesOut());
		view.addProgressValue(new OperationsInitiated());
		view.addProgressValue(new OperationsFinished());
		view.addProgressValue(new NetMsgsDropped());
		view.addProgressValue(new NetMsgsReceived());
		view.addProgressValue(new NetMsgsSent());
		view.rebuildProgressValues();
		
		Thread.setDefaultUncaughtExceptionHandler(view);
		Thread.currentThread().setUncaughtExceptionHandler(view);
		
	}
	
	@Override
	public void start() {
		view.notifySimulationRunning();
		view.update();
	}

	@Override
	public void stop(Writer output) {
		view.notifySimulationFinished();
		view.update();
	}

	@Override
	public void operationFinished(Operation<?> op) {
		operationsFinished++;
		view.updateIfNecessary();
	}

	@Override
	public void operationInitiated(Operation<?> op) {
		operationsInitiated++;
		view.updateIfNecessary();
	}
	
	@Override
	public void netMsgDrop(NetMessage msg, NetID id) {
		netMsgsDropped++;
		view.updateIfNecessary();
	}

	@Override
	public void netMsgReceive(NetMessage msg, NetID id) {
		netMsgsReceived++;
		view.updateIfNecessary();
	}

	@Override
	public void netMsgSend(NetMessage msg, NetID id) {
		netMsgsSent++;
		view.updateIfNecessary();
	}
	
	public class SimulationTime implements ProgressValue {

		@Override
		public String getName() {
			return "Simulationszeit";
		}

		@Override
		public String getValue() {
			long time = Simulator.getCurrentTime();
			return String.valueOf(Simulator.getFormattedTime(time));
		}
		
	}
	
	public class MessagesIn implements ProgressValue {

		@Override
		public String getName() {
			return "Empfangene Nachrichten";
		}

		@Override
		public String getValue() {
			return String.valueOf(messagesIn);
		}
		
	}
	
	public class MessagesOut implements ProgressValue {

		@Override
		public String getName() {
			return "Gesendete Nachrichten";
		}

		@Override
		public String getValue() {
			return String.valueOf(messagesOut);
		}
		
	}
	
	public class OperationsInitiated implements ProgressValue {

		@Override
		public String getName() {
			return "Gestartete Operationen";
		}

		@Override
		public String getValue() {
			return String.valueOf(operationsInitiated);
		}
		
	}
	
	public class OperationsFinished implements ProgressValue {

		@Override
		public String getName() {
			return "Beendete Operationen";
		}

		@Override
		public String getValue() {
			return String.valueOf(operationsFinished);
		}
		
	}
	
	public class NetMsgsDropped implements ProgressValue {

		@Override
		public String getName() {
			return "Verlorengenangene Pakete";
		}

		@Override
		public String getValue() {
			return String.valueOf(netMsgsDropped);
		}
		
	}
	
	public class NetMsgsReceived implements ProgressValue {

		@Override
		public String getName() {
			return "Empfangene Pakete";
		}

		@Override
		public String getValue() {
			return String.valueOf(netMsgsReceived);
		}
		
	}
	
	public class NetMsgsSent implements ProgressValue {

		@Override
		public String getName() {
			return "Gesendete Pakete";
		}

		@Override
		public String getValue() {
			return String.valueOf(netMsgsSent);
		}
		
	}

}











