package de.tud.kom.p2psim.impl.util.guirunner.progress;

import java.util.HashMap;
import java.util.Map;

import de.tud.kom.p2psim.impl.simengine.Simulator;

/**
 * Modification that adds the Progress analyzer to each execution.
 * 
 * This class is used to run simulations in <a href="http://www.peerfact.org/">PeerfactSim.KOM</a>.
 * @author Konstantin Pussep
 * @author Sebastian Kaune
 * @version 3.0, 04.12.2007
 *
 */
public class ProgressRunner {

//		private final static Logger log = SimLogger.getLogger(SimulatorRunner.class);

		
	/**
	 * Set private to prevent instantiation.
	 * 
	 */
	private ProgressRunner() {
		// set private
	}

	
	/**
	 * This method can be used to run a simulation.
	 * The expected arguments are: <code>config file</code> and an optional list of 
	 * zero or many variable assignments<code>(variable=value)*")</code>
	 * or from the command line <code> java Scenario {config file} {variable=value}*</code>.
	 * 
	 * @param args
	 *            expect an array with the name of the
	 *            configuration file and optional variable assignments
	 */
	public static void main(String[] args) {
		// TODO if more arguments required - use apache cli
		if (args.length >= 1) {
			Simulator sim =  Simulator.getInstance();
			Map<String, String> variables = parseVariables(args, 1);
			sim.configure(args[0], variables);
			addAnalyzer(sim);
			sim.start();
		} else
			System.err.println("usage: Scenario <config file> (<variable=value>)*");
	}


	private static void addAnalyzer(Simulator sim) {
		Simulator.getMonitor().setAnalyzer(new ProgressAnalyzer());
	}

	private static Map<String, String> parseVariables(String[] args, int startAt) {
		Map<String, String> variables = new HashMap<String, String>();
		// assert (args.length-1)%2==0;
		for (int j = startAt; j < args.length; j++) {
			String[] tokens = args[j].split("=");
			assert tokens.length == 2 : "Bad format " + args[j];
			variables.put(tokens[0], tokens[1]);
		}
		return variables;
	}
	
	
}
