package de.tud.kom.p2psim.impl.util.guirunner.progress;

import java.awt.Image;
import java.awt.Point;
import java.lang.Thread.UncaughtExceptionHandler;

import javax.swing.ImageIcon;

import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.util.vis.util.Config;

public class SimulationProgressView extends RichProgressView implements UncaughtExceptionHandler {

	public static SimulationProgressView inst = null;
	
	public static SimulationProgressView getInstance() {
		if (inst == null) inst = new SimulationProgressView();
		return inst;
	}
	
	/*
	 * Configuration paths
	 */
	static final String CONF_PATH = "GUIRunner/ProgressWindow";
	static final String CONF_PATH_POSX = CONF_PATH + "PosX";
	static final String CONF_PATH_POSY = CONF_PATH + "PosY";
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1701832807987594850L;

	public static final String JOB_PREPARING = "Vorbereitung...";

	public static final String JOB_SIMULATION = "Simulation...";
	
	public static final String JOB_EXCEPTION_PREFIX = "Exception: ";

	public static final String JOB_FINISHED = "Erfolgreich beendet";

	public static final Image frameIcon = new ImageIcon("images/icons/frame_icon.png").getImage();
	
	/**
	 * Angezeigter Fortschritt nach Vorbereitung
	 */
	public static final int PREPARING_PROGR = 100;

	static final long UPDATE_STEP_TIME = 100;
	
	String actualJobName = JOB_PREPARING;
	
	boolean simulationRunning = false;
	boolean simulationFinished = false;
	
	long lastUpdateTime = -1;

	int progress = 0;

	protected SimulationProgressView() {
		super();
		this.setLocation(new Point(Config.getValue(CONF_PATH_POSX, 0), Config.getValue(CONF_PATH_POSY, 0)));
		this.setIconImage(frameIcon);
		this.setTitle("Simulationsfortschritt");
		this.setMaximum(1000 + PREPARING_PROGR);
		update();
		
	}

	@Override
	public String getActualJobName() {
		return actualJobName;
	}

	@Override
	public int getProgress() {
		if (!simulationRunning)
			return 0;
		else if (simulationFinished)
			return 1000 + PREPARING_PROGR;
		else
			return PREPARING_PROGR + (int) ((double) getSimCurrentTime() / (double) getSimEndTime() * 1000);
	}

	@Override
	public void onCancel(boolean cancelled) {
		this.saveSettings();
		Config.writeXMLFile();
		if (cancelled) System.exit(0);
	}

	public void notifySimulationRunning() {
		actualJobName = JOB_SIMULATION;
		simulationRunning = true;
	}
	
	public void notifySimulationFinished() {
		actualJobName = JOB_FINISHED;
		simulationFinished = true;
		this.setFinished();
	}
	
	public void updateIfNecessary() {
		
		long now = System.currentTimeMillis();
		
		if (lastUpdateTime <= now - UPDATE_STEP_TIME) {
			update();
			lastUpdateTime = now;
		}
	}

	@Override
	public long getEstimatedTime() {
		if (!simulationRunning)
			return -1;
		
		double timePerVirt = (double)getElapsedTime() / (double)getSimCurrentTime();
		long result = (long)((getSimEndTime() - getSimCurrentTime()) * timePerVirt);
		//System.out.println("Estimated Time: " + result);
		return result;

	}
	
	protected long getSimCurrentTime() {
		return Simulator.getCurrentTime();
	}
	
	protected long getSimEndTime() {
		return Simulator.getEndTime();
	}
	
	/**
	 * Saves settings, like window size etc.
	 */
	public void saveSettings() {
		
		Config.setValue(CONF_PATH_POSX, this.getX());
		Config.setValue(CONF_PATH_POSY, this.getY());
	}

	@Override
	public void uncaughtException(Thread t, Throwable e) {
		e.printStackTrace();
		actualJobName = JOB_EXCEPTION_PREFIX + e.getMessage();
		progress = 1000 + PREPARING_PROGR;
		this.notifySimulationFinished();
		this.update();
	}

}













