package de.tud.kom.p2psim.impl.util.helpers;

/**
 * Interface for predicate checks on objects. The check itself has to be
 * implemented in the method {@link Predicate#isTrue(Object) isTrue}.
 * 
 * For example, that method could check whether passed objects are greater than
 * some value given in a constructor.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 * 
 * @param <T>
 *            the type of the objects to be checked.
 */
public interface Predicate<T> {

	/**
	 * Decides whether a certain predicate, defined by the implementing class of
	 * this interface, holds for <code>object</code>.
	 * 
	 * @param object
	 *            the Object to be checked for the predicate.
	 * @return <code>true</code> if and only if the predicate holds for
	 *         <code>object</code>.
	 */
	public boolean isTrue(T object);
}
