/**
 * 
 */
package de.tud.kom.p2psim.impl.util.stat.distributions.zipf;

import de.tud.kom.p2psim.impl.util.stat.distributions.Distribution;
import de.tud.kom.p2psim.impl.util.stat.distributions.DistributionConfigurer;
import de.tud.kom.p2psim.impl.util.stat.distributions.normal.NormalDistribution;

/**
 * @author Kalman Graffi
 * @version 0.1, 26.09.2007
 * see http://en.wikipedia.org/wiki/Zipf's_law
 */
public class ZipfDistribution implements Distribution {

	private int maximum_Number_Of_Ranks;
	private double zipfExponent;
	private double harmonicNormFactor;
	
	
	public void config(DistributionConfigurer dc) {
		maximum_Number_Of_Ranks = ((ZipfConfigurer) dc).getMaxNrOfRanks();
		zipfExponent = ((ZipfConfigurer) dc).getZipfExponent();
		harmonicNormFactor = 0;
		for (int i=1; i<=maximum_Number_Of_Ranks;i++){
			harmonicNormFactor += Math.pow(1/i, zipfExponent);
		}
	}

	/**
	 * Returns a RANK, not a probability 
	 */
	public double returnValue() {
		// prob = 1 / (rank^zipfExp * harmonicNormFactor)
		// => rank^zipfExp = 1 / (prob * harmonicNormFactor)
		// => rank = zipfExp-sqrt (1 / (prob*harmonicNormFactor))
		// rank = 1 ... maximum_Number_Of_Ranks => 1/rank = 0..1
				
		return 1/(
				Math.pow(
						1 / (Math.random() * harmonicNormFactor) 
						, 
						1/zipfExponent
				)
		);
	}
	
	public int returnRank(){
		return (int) Math.floor(1 / returnValue());
	}

}
