package de.tud.kom.p2psim.impl.util.vis.analyzer.ol.gnutella;

import java.awt.Color;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;
import de.tud.kom.p2psim.impl.overlay.gnutella.GnutellaOverlayNode;
import de.tud.kom.p2psim.impl.overlay.gnutella.messages.*;
import de.tud.kom.p2psim.impl.util.vis.analyzer.OverlayAdapter;

/**
 * Adapter für Gnutella
 * @author Leo Nobach
 * @version 3.0, 03.11.2008
 *
 */
public class GnutellaAdapter extends OverlayAdapter{

	public GnutellaAdapter() {
		addOverlayImpl(GnutellaOverlayNode.class);
		
		addOverlayImpl(PingMessage.class);
		addOverlayImpl(PongMessage.class);
		addOverlayImpl(QueryMessage.class);
		addOverlayImpl(PushMessage.class);
		addOverlayImpl(OkMessage.class);
		addOverlayImpl(ConnectMessage.class);
		
		addOverlayNodeMetric(DocCountM.class);
	}
	
	Map<GnutellaOverlayNode, Integer> docCounts = new HashMap<GnutellaOverlayNode, Integer>();
	
	@Override
	public String getOverlayName() {
		return "Gnutella";
	}

	@Override
	public void handleLeavingHost(Host host) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void handleNewHost(Map<String, Serializable> attributes, Host host, OverlayNode overlayNode) {

		if (overlayNode instanceof GnutellaOverlayNode) {
			GnutellaOverlayNode gn_node = ((GnutellaOverlayNode)overlayNode);
			
			attributes.putAll(analyzeAttributes(gn_node));
			
		}
		
	}

	@Override
	public void handleNewHostAfter(Host host, OverlayNode overlayNode) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void handleOverlayMsg(Message omsg, Host from, NetID fromID, Host to, NetID toID) {
		// Hier wird festgelegt wie bei dem Auftreten von Nachrichten visualisiert wird
		
		if(omsg instanceof PingMessage) {
//			getTranslator().overlayEdgeFlash(from, to, Color.gray);
		} else if(omsg instanceof PongMessage) {
//			getTranslator().overlayEdgeFlash(from, to, Color.darkGray);
		} else if(omsg instanceof QueryMessage) {
			flashEdge(fromID, toID, Color.blue, "Query", omsg.getClass());
		} else if(omsg instanceof QueryHitMessage) {
			flashEdge(fromID, toID, Color.green, "QueryHit", omsg.getClass());
		} else if(omsg instanceof PushMessage) {
			flashEdge(fromID, toID, Color.red, "Push", omsg.getClass());
		} else if(omsg instanceof OkMessage){
			flashEdge(fromID, toID, Color.magenta, "OK", omsg.getClass());
		} else if(omsg instanceof ConnectMessage){
			flashEdge(fromID, toID, Color.yellow, "Connect", omsg.getClass());
		}
		
		refreshAttributesFor(from);
		refreshAttributesFor(to);
	}
	
	/**
	 * Überprüft, ob für den Host alle Attribute noch aktuell sind und ändert
	 * sie ggf.
	 * @param host
	 */
	protected void refreshAttributesFor(Host host) {
		if (host == null) {
			return;
		}
		OverlayNode ov_node = host.getOverlay(GnutellaOverlayNode.class);
		
		if (ov_node instanceof GnutellaOverlayNode) {
			Map<String, Serializable> attrsToRefresh = 
				analyzeAttributes((GnutellaOverlayNode)ov_node);
			
			if (!attrsToRefresh.isEmpty()) {
				getTranslator().nodeAttributesChanged(host.getNetLayer().getNetID(), 
						attrsToRefresh);
			}
			
		}
	}
	
	/**
	 * Gibt alle Attribute zurück, die sich geändert haben.
	 * Bei neu erzeugten Knoten alle.
	 * @param gn_node
	 * @return
	 */
	protected Map<String, Serializable> analyzeAttributes(GnutellaOverlayNode gn_node) {
		Map<String, Serializable> attrs = new HashMap<String, Serializable>();
		
		refreshDocCount(gn_node, attrs);
		
		return attrs;
	}

	/**
	 * Refresht die Dokumenten-Zahl, falls 
	 * @param gn_node
	 * @param attrs
	 */
	private void refreshDocCount(GnutellaOverlayNode gn_node, Map<String, Serializable> attrs) {
		int docCount = gn_node.getDocuments().size();
		if (docCounts.get(gn_node) == null || docCount != docCounts.get(gn_node)) {
			attrs.put("doc_count", docCount);
			docCounts.put(gn_node, docCount);
		}
	}

	@Override
	public BootstrapManager getBootstrapManagerFor(OverlayNode nd) {
		return null;
		
		//Nicht implementiert für Gnutella
	}

	@Override
	public void handleOperation(Host host, Operation<?> op, boolean finished) {
		// TODO Auto-generated method stub
		
	}

}
