package de.tud.kom.p2psim.impl.util.vis.analyzer.ol.napster;

import java.awt.Color;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.api.common.Message;
import de.tud.kom.p2psim.api.common.Operation;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.overlay.OverlayKey;
import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.impl.overlay.BootstrapManager;
import de.tud.kom.p2psim.impl.overlay.AbstractOverlayNode.PeerStatus;
import de.tud.kom.p2psim.impl.overlay.cd.DistributionStrategyImpl;
import de.tud.kom.p2psim.impl.overlay.cd.messages.DownloadRequestMsg;
import de.tud.kom.p2psim.impl.overlay.cd.messages.DownloadResultMsg;
import de.tud.kom.p2psim.impl.overlay.dht.centralized.ClientNode;
import de.tud.kom.p2psim.impl.overlay.dht.centralized.ServerNode;
import de.tud.kom.p2psim.impl.util.vis.analyzer.OverlayAdapter;
import de.tud.kom.p2psim.impl.util.vis.analyzer.Translator.EdgeHandle;
import de.tud.kom.p2psim.impl.util.vis.analyzer.positioners.SchematicPositioner;


/**
 * Dieser Overlay-Adapter visualisiert spezielle Eigenschaften des Napster-Protokolls.
 * 
 * @author Julius Rueckert
 * @author Leo Nobach
 *
 */
public class NapsterAdapter extends OverlayAdapter {	
	
	/**
	 * Bisher gezeichneter Peer Status eines jeden Hosts.
	 */
	public Map<NetID, PeerStatus> peerStatuses = new HashMap<NetID, PeerStatus>();
	
	public Map<DownloadTransaction, EdgeHandle> dl_edges = new HashMap<DownloadTransaction, EdgeHandle>();
	
	public NapsterAdapter() {
	
		addOverlayImpl(ClientNode.class);
		addOverlayImpl(ServerNode.class);
		addOverlayImpl(DownloadRequestMsg.class);
		addOverlayImpl(DownloadResultMsg.class);
		
		addOverlayNodeMetric(PeerStatusM.class);

	}

	@Override
	public String getOverlayName() {
		return "Napster";
	}

	@Override
	public void handleNewHost(Map<String, Serializable> attributes, Host host, OverlayNode overlayNode) {
		if(overlayNode instanceof ClientNode) {
			ClientNode clientNode = (ClientNode) overlayNode;
			attributes.put("peer_status", clientNode.getPeerStatus());
		} else if (overlayNode instanceof ServerNode) {
			ServerNode serverNode = (ServerNode) overlayNode;
			attributes.put("peer_status", serverNode.getPeerStatus());
		}
	}

	@Override
	public void handleNewHostAfter(Host host, OverlayNode overlayNode) {
		
		if(overlayNode instanceof ClientNode) {
			
			ClientNode clientNode = (ClientNode) overlayNode;
			
			NetID ServerID = clientNode.getServerAddress().getNetId();
			
			addEdge(host.getNetLayer().getNetID(), ServerID, Color.GREEN, "server");	
			
			
		} else if(overlayNode instanceof DistributionStrategyImpl){
			//nothing 2 do
			
		} else if(overlayNode instanceof ServerNode){
			//TODO: Vielleicht Kante zu allen existierenden Clients mit diesem Server herstellen?
		}			
	}
	
	@Override
	public void handleLeavingHost(Host host) {
		//Nichts zu tun.
	}
			

	@Override
	public void handleOverlayMsg(Message omsg, Host from, NetID fromID, Host to, NetID toID) {
		
		if(omsg instanceof DownloadRequestMsg){ // Anfrage bei einem Clienten um Download
			
			// Neue Kante zwischen den zwei Clienten herstellen (Dies ist Beginn eines Downloads)
			//Attribute setzen
			EdgeHandle edge = addEdge(fromID, toID, Color.RED, "download");
			OverlayKey key = ((DownloadRequestMsg)omsg).getKey();
			dl_edges.put(new DownloadTransaction(fromID, toID, key), edge);
					
		} else if (omsg instanceof DownloadResultMsg) { // Gewuenschtes Dokument wurde empfangen
			
			OverlayKey key = ((DownloadResultMsg)omsg).getDoc().getKey();
			EdgeHandle edge = dl_edges.remove(new DownloadTransaction(toID, fromID, key));
			// Kante zwischen den zwei Clients entfernen (Download ist abgeschlossen)
			if (edge != null) edge.remove();
		}
		
		checkPeerStatus(from);
		checkPeerStatus(to);
	}

	private void checkPeerStatus(Host from) {
		OverlayNode onode = from.getOverlay(ClientNode.class);
		
		if (onode instanceof ClientNode) {
			PeerStatus pst = ((ClientNode)onode).getPeerStatus();
			if (peerStatuses.get(from.getNetLayer().getNetID()) != pst) {
				getTranslator().nodeAttributeChanged(from.getNetLayer().getNetID(), "peer_status", pst);
				peerStatuses.put(from.getNetLayer().getNetID(), pst);
			}
		}
	}

	@Override
	public BootstrapManager getBootstrapManagerFor(OverlayNode nd) {
		return null;
		
		//Sowas ist für Napster nicht implementiert.
	}
	
	public SchematicPositioner getNewPositioner() {
		return new NapsterPositioner();
	}

	@Override
	public void handleOperation(Host host, Operation<?> op, boolean finished) {
		// TODO Auto-generated method stub
		
	}
	
	public class DownloadTransaction {
	
		private NetID requestor;
		private NetID responder;
		private OverlayKey key;

		public DownloadTransaction(NetID requestor, NetID responder, OverlayKey key) {
			this.requestor = requestor;
			this.responder = responder;
			this.key = key;
		}
		
		public boolean equals(Object o) {
			if (!(o instanceof DownloadTransaction)) return false;
			DownloadTransaction other = (DownloadTransaction)o;
			return other.requestor.equals(this.requestor) &&
				other.responder.equals(this.responder) &&
				other.key.equals(this.key);
		}
		
		public int hashCode() {
			return requestor.hashCode()*23+responder.hashCode()*13+key.hashCode();
		}
	}

	
}