package de.tud.kom.p2psim.impl.util.vis.analyzer.positioners.multi;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import de.tud.kom.p2psim.api.common.Host;
import de.tud.kom.p2psim.api.overlay.OverlayNode;
import de.tud.kom.p2psim.impl.util.vis.analyzer.OverlayAdapter;
import de.tud.kom.p2psim.impl.util.vis.analyzer.positioners.MultiPositioner;
import de.tud.kom.p2psim.impl.util.vis.analyzer.positioners.SchematicPositioner;
import de.tud.kom.p2psim.impl.util.vis.util.visualgraph.Coords;

/**
 * Ein MultiPositioner, der die OverlayAdapter der Reihe nach abgeht und
 * den ersten, der eine Position für host hat, verwendet um die Position
 * zu bestimmen.
 * 
 * @author Leo Nobach
 * @version 3.0, 20.11.2008
 *
 */
public class TakeFirstPositioner extends MultiPositioner {

	Map<OverlayAdapter, SchematicPositioner> loadedPositioners
	= new HashMap<OverlayAdapter, SchematicPositioner>();
	
	@Override
	public Coords getSchematicHostPosition(Host host) {
		
		Iterator<OverlayNode> it = host.getOverlays();
		
		while(it.hasNext()){
			OverlayNode node = it.next();
			for (OverlayAdapter adapter : this.getAllAdapters()) {
				if (adapter.isDedicatedOverlayImplFor(node.getClass())) {
					
					SchematicPositioner positioner = loadedPositioners.get(adapter);
					if (positioner == null) {
						positioner = adapter.getNewPositioner();
						loadedPositioners.put(adapter, positioner);
					}
					
					Coords pos = positioner.getSchematicHostPosition(host, node);
					if (pos != null) return pos;
				}
			}
		}
		return null;	//Es gibt keinen Adapter, der eine Position für den Knoten hat.
	}

}
