package de.tud.kom.p2psim.impl.util.vis.controller.player;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.model.EventTimeline;

/**
 * Thread, von der Player-Klasse verwendet, um den Abspielvorgang zu steuern.
 * 
 * Hinweis: Dies ist nicht der einzige Thread, der zeichnet. Die AWT-EventQueue
 * zeichnet die meiste Zeit!.
 * @author Leo Nobach
 *
 */
public class PlayThread extends Thread{
	
	boolean open = false;
	
	Player invokingPlayer;
	
	long lastFrameBegun = 0;
	
	long lastFrameDone = 0;
	
	int framesNotDone;
	
	/**
	 * Standard-Konstruktor
	 * 
	 * @param p
	 */
	public PlayThread(Player p) {
		this.invokingPlayer = p;
	}
	
	@Override
	public void run() {
		EventTimeline tl = invokingPlayer.getTimeline();
		
		while(true) {
			invokingPlayer.setPlaying(true);
			
			while(tl.getActualTime() < tl.getMaxTime() && open) {
				this.doFrame(tl);
				try {
					
					
					long timeToSleep = lastFrameBegun - lastFrameDone + 
					(long)(invokingPlayer.getSpeed()*invokingPlayer.getQuantization());
					
					dbg("Paint-Dauer: " + (lastFrameDone - lastFrameBegun) +
							". Warte jetzt " + timeToSleep + "Sekunden");
					
					
					if (timeToSleep > 0) Thread.sleep(timeToSleep);
					
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			invokingPlayer.setPlaying(false);
			
			if (!invokingPlayer.isLooping() || !open) break;
			Controller.resetView();
		}
		
		if (tl.getActualTime() >= tl.getMaxTime()) invokingPlayer.notifyStopping();
		
	}
	
	/**
	 * Zeichnet einen Frame, bzw. stellt ihn in die AWT-EventQueue.
	 * @param tl
	 */
	public void doFrame(EventTimeline tl) {
		
		this.lastFrameBegun = System.currentTimeMillis();
		
		try {
			this.setName("PlayerThread");
			tl.jumpToTime(tl.getActualTime() + (long)(invokingPlayer.getQuantization()*Player.TIME_UNIT_MULTIPLICATOR));

			
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}
		
		this.lastFrameDone = System.currentTimeMillis();
	}
	
	/**
	 * Schließt den Thread, wenn open=false.
	 * 
	 * @param open
	 */
	public void setOpen(boolean open) {
		this.open = open;
	}
	
	/**
	 * Debug-Ausgabe
	 * @param o
	 */
	public void dbg (Object o) {
		//System.err.println(this.getClass().getSimpleName() + ": " + (o==null?"null":o.toString()));
	}

}
