package de.tud.kom.p2psim.impl.util.vis.controller.player;

import java.util.ArrayList;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.model.EventTimeline;
import de.tud.kom.p2psim.impl.util.vis.util.Config;

/**
 * Spielt die Aufzeichnung ab und führt verwandte Operationen aus.
 * 
 * @author Leo Nobach
 *
 */
public class Player {
	
	/**
	 * Maximale Framerate in ms (optimiert die Rechenleistung)
	 */
	public static final int FPS = 25;
	
	/**
	 * So viele Eingabe-Zeiteinheiten sind eine Sekunde.
	 */
	public static final int TIME_UNIT_MULTIPLICATOR = 1000000;
	
	/**
	 * Schritt beim Vor- und Zurückspulen.
	 */
	public static final int FWD_STEP_SIZE = 50;
	
	
	ArrayList<PlayerEventListener> listeners = new ArrayList<PlayerEventListener>();
	EventTimeline timeline;
	PlayThread playthread = new PlayThread(this);
	boolean isPlaying;
	boolean loop = false;
	
	/**
	 * Geschwindigkeit in Millisekunden pro Frame
	 */
	public int speed = Config.getValue("Player/Speed", 400);
	
	/**
	 * Schritt beim Abspielen in Sekunden.
	 */
	protected double quantization = Config.getValue("Player/Quantization", 2500)/10000d;

	public EventTimeline getTimeline() {
		return timeline;
	}

	public void setTimeline(EventTimeline timeline) {
		this.timeline = timeline;
	}
	
	
	public void setSpeed(float speed) {
		System.out.println("Speed gesetzt auf:" + speed);
	}
	
	public void play() {
		if (timeline.getMaxTime() > 0) {
			if (!isPlaying) {
				if (timeline.getActualTime() >= timeline.getMaxTime()) Controller.resetView();
				playthread = new PlayThread(this);
				playthread.setOpen(true);
				playthread.start();
			}
			this.notifyPlaying();
		}
	}
	
	public void pause() {
		playthread.setOpen(false);
		this.notifyPause();
	}
	
	public void stop() {
		playthread.setOpen(false);
		Controller.resetView();
		this.notifyStopping();
	}
	
	public void rev() {
		long jumptime = timeline.getActualTime() - FWD_STEP_SIZE * TIME_UNIT_MULTIPLICATOR;
		
		if (jumptime <= 0)	timeline.jumpToTime(0);
		else timeline.jumpToTime(jumptime);
		
		this.notifyReverse();
	}
	
	public void fwd() {
		
		long jumptime = timeline.getActualTime() + FWD_STEP_SIZE * TIME_UNIT_MULTIPLICATOR;
		
		if (jumptime >= timeline.getMaxTime())	timeline.jumpToTime(timeline.getMaxTime());
		else timeline.jumpToTime(jumptime);
	
		this.notifyForward();
	}
	
	public void reset() {
		
	}
	
	public int getSpeed() {
		return speed;
	}
	
	/**
	 * Setzt die Geschwindigkeit des Players in Millisekunden
	 * @param speed
	 */
	public void setSpeed(int speed) {
		this.speed = speed;
		this.notifySpeedChange(speed);
	}
	
	public double getQuantization() {
		return quantization;
	}
	
	/**
	 * Setzt die Geschwindigkeit des Players in Millisekunden
	 * @param speed
	 */
	public void setQuantization(double quantization) {
		this.quantization = quantization;
		this.notifyQuantizationChange(quantization);
	}

	public void showTime() {
		System.out.println(timeline.getActualTime());
	}

	/**
	 * Gibt zurück, ob der Player gerade abspielt.
	 * @return
	 */
	public boolean isPlaying() {
		return isPlaying;
	}
	
	protected void setPlaying(boolean playing) {
		isPlaying = playing;
	}
	
	public void addEventListener(PlayerEventListener l) {
		this.listeners.add(l);
	}
	
	protected void notifySpeedChange(int speed) {
		for (PlayerEventListener l : listeners) {
			l.speedChange(speed);
		}
	}
	
	protected void notifyPlaying() {
		for (PlayerEventListener l : listeners) {
			l.play();
		}
	}
	
	protected void notifyStopping() {
		for (PlayerEventListener l : listeners) {
			l.stop();
		}
	}
	
	protected void notifyPause() {
		for (PlayerEventListener l : listeners) {
			l.pause();
		}
	}
	
	protected void notifyForward() {
		for (PlayerEventListener l : listeners) {
			l.forward();
		}
	}
	
	protected void notifyReverse() {
		for (PlayerEventListener l : listeners) {
			l.reverse();
		}
	}
	
	private void notifyQuantizationChange(double q) {
		for (PlayerEventListener l : listeners) {
			l.quantizationChange(q);
		}
	}
	
	public void saveSettings() {
		Config.setValue("Player/Speed", speed);
		Config.setValue("Player/Quantization", (int)(quantization*10000d));
		
	}

	public boolean isLooping() {
		return loop;
	}

	public void setLooping(boolean loop) {
		this.loop = loop;
	}
	
}
