package de.tud.kom.p2psim.impl.util.vis.metrics;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import de.tud.kom.p2psim.impl.util.vis.api.metrics.Metric;
import de.tud.kom.p2psim.impl.util.vis.api.metrics.overlay.OverlayEdgeMetric;
import de.tud.kom.p2psim.impl.util.vis.api.metrics.overlay.OverlayNodeMetric;
import de.tud.kom.p2psim.impl.util.vis.api.metrics.overlay.OverlayUniverseMetric;

public class MetricsPack implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6782000462960510486L;
	
	protected MetricsPackPart<OverlayNodeMetric> nodes = new MetricsPackPart<OverlayNodeMetric>();
	protected MetricsPackPart<OverlayEdgeMetric> edges = new MetricsPackPart<OverlayEdgeMetric>();
	protected MetricsPackPart<OverlayUniverseMetric> universe = new MetricsPackPart<OverlayUniverseMetric>();
	

	/**
	 * Entfernt alle Instanzen der enthaltenen Metriken. Werden beim nächsten Aufruf
	 * neu instanziiert.
	 */
	public void clearInstances() {
		nodes.clearInstances();
		edges.clearInstances();
		universe.clearInstances();
	}
	
	public MetricsPackPart<OverlayNodeMetric> getNodes() {
		return nodes;
	}


	public MetricsPackPart<OverlayEdgeMetric> getEdges() {
		return edges;
	}


	public MetricsPackPart<OverlayUniverseMetric> getUniverse() {
		return universe;
	}
	
	public class MetricsPackPart<M extends Metric> implements Serializable {
	
		/**
		 * 
		 */
		private static final long serialVersionUID = 6094347201051011597L;

		Set<Class<? extends M>> metrics = new HashSet<Class<? extends M>>();
	
		Set<M> metricsInst = null;
	
		
		public void addMetric(Class<? extends M> m) {
			metrics.add(m);
			if (metricsInst != null) try {
				metricsInst.add(m.newInstance());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		public void clearInstances() {
			this.metricsInst = null;
			System.gc();
		}

		public Set<M> getMetrics() {
			if (metricsInst == null) {
				metricsInst = new HashSet<M>();
				try {
					for (Class<? extends M> m : metrics) metricsInst.add(m.newInstance());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			
			
			return metricsInst;
			
		}
		
	}


	
}
