package de.tud.kom.p2psim.impl.util.vis.model.overlay;

import java.awt.Color;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import javax.swing.ImageIcon;

import de.tud.kom.p2psim.impl.util.vis.api.metrics.BoundMetric;
import de.tud.kom.p2psim.impl.util.vis.api.metrics.overlay.OverlayNodeMetric;
import de.tud.kom.p2psim.impl.util.vis.model.MetricObject;
import de.tud.kom.p2psim.impl.util.vis.model.ModelIterator;
import de.tud.kom.p2psim.impl.util.vis.util.visualgraph.PositionInfo;

/**
 * Knoten (Peer) im Overlay-Netzwerk.
 * 
 * @author Leo Nobach
 */
public class VisOverlayNode extends de.tud.kom.p2psim.impl.util.vis.util.visualgraph.Node<VisOverlayNode, VisOverlayEdge> implements MetricObject, AttributeObject {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1932483666858398117L;

	Hashtable<String, Serializable> attributes = new Hashtable<String, Serializable>();
	
	protected static final ImageIcon REPR_ICON = new ImageIcon("images/icons/model/OverlayNode16_16.png");
	
	/**
	 * Name des Knotens
	 */
	String name;
	
	public VisOverlayNode(PositionInfo pos, String name) {
		super(pos);
		this.name = name;
	}
	
	/**
	 * Gibt den Namen des Knotens zurück, der in der Darstellung verwendet wird.
	 * @return
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * F�gt ein Attribut ein.
	 * <b> Achtung! Alle Attribute m�ssen serializable sein! </b>
	 * @param name
	 * @param value
	 */
	public void insertAttribute(String name, Serializable value) {
		this.attributes.put(name, value);
	}
	
	public Object getAttribute(String name) {
		return attributes.get(name);
	}
	
	/**
	 * Gibt eine Map aller Attribute zurück.
	 * @return
	 */
	public Map<String, Serializable> getAttributes() {
		return attributes;
	}
	
	/**
	 * Setzt einen Namen f�r diesen Knoten, der in der Darstellung verwendet werden soll.
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gibt einen Satz aller Metriken für diesen Knoten in der an ihn gebundenen Form zurück.
	 * @return
	 */
	public Vector<BoundMetric> getBoundMetrics() {
		Vector<BoundMetric> res = new Vector<BoundMetric>();
		for (OverlayNodeMetric m : de.tud.kom.p2psim.impl.util.vis.metrics.MetricsBase.forOverlayNodes().getListOfAllMetrics()) {
			res.add(m.getBoundTo(this));
		}
		return res;
	}

	@Override
	public Color getColor() {
		return Color.DARK_GRAY;
	}

	@Override
	public void iterate(ModelIterator it) {
		it.overlayNodeVisited(this);
	}
	
	public String toString() {
		return this.getName() + "(" + this.getAttribute("NetID") + ")";
	}
	
	@Override
	public ImageIcon getRepresentingIcon() {
		return REPR_ICON;
	}
	
}
