package de.tud.kom.p2psim.impl.util.vis.ui.common.DetailsPane;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import de.tud.kom.p2psim.impl.util.vis.api.visualization.VisActionListener;
import de.tud.kom.p2psim.impl.util.vis.model.MetricObject;
import de.tud.kom.p2psim.impl.util.vis.model.overlay.VisOverlayEdge;
import de.tud.kom.p2psim.impl.util.vis.model.overlay.VisOverlayGraph;
import de.tud.kom.p2psim.impl.util.vis.model.overlay.VisOverlayNode;
import de.tud.kom.p2psim.impl.util.vis.ui.common.CaptionContainer;
import de.tud.kom.p2psim.impl.util.vis.ui.common.DetailsPane.EdgeFilter.EdgeFilterPane;
import de.tud.kom.p2psim.impl.util.vis.util.Config;
import de.tud.kom.p2psim.impl.util.vis.util.SplitPaneConfigSaver;



/**
 * Fensterabschnitt zum Anzeigen von Peer- oder Verbindungsdetails, seiner Metriken o.Ä.
 * 
 * @author Leo Nobach
 *
 */
public class DetailsPane extends JPanel implements VisActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -5520015429136616404L;
	
	protected static final String SPLITTER_CONF_PATH = "UI/DetailsPane/Splitter1Pos";
	protected static final String SPLITTER2_CONF_PATH = "UI/DetailsPane/Splitter2Pos";
	
	ObjectMetricsTable object_metrics_table = new ObjectMetricsTable();
	CaptionContainer object_caption_container;
	MetricsTable metrics_table = new MetricsTable();
	EdgeFilterPane edgeFilterPane = new EdgeFilterPane();
	
	Component top_pane;
	
	public DetailsPane() {

		this.setLayout(new BorderLayout());
		this.setPreferredSize(new Dimension(300, 100));
		
		object_caption_container = new CaptionContainer("", new JScrollPane(object_metrics_table));
		
		//clickedOn(Controller.getVisApi().getSelectedObject());
		
		JSplitPane sp2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
				new CaptionContainer("Alle Metriken", new JScrollPane(metrics_table)),
				new CaptionContainer("Filter", new JScrollPane(edgeFilterPane)));
		
		sp2.setDividerLocation(Config.getValue(SPLITTER2_CONF_PATH, 250));
		
		sp2.addPropertyChangeListener(new SplitPaneConfigSaver(SPLITTER2_CONF_PATH));
		
		JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, object_caption_container, 
				sp2);
		
		sp.setDividerLocation(Config.getValue(SPLITTER_CONF_PATH, 350));
		
		sp.addPropertyChangeListener(new SplitPaneConfigSaver(SPLITTER_CONF_PATH));
		
		this.add(sp);
		
	}
	
	/**
	 * Setzt c als TopPane. Ist c null, wird kein TopPane gesetzt.
	 * @param c
	 */
	private void makeNewTopPane(Component c) {
		if (top_pane != null) this.remove(top_pane);
		
		top_pane = c;
		
		if (c != null) this.add(top_pane, BorderLayout.NORTH);
		
		this.setVisible(false);
		this.setVisible(true);
	}


	@Override
	public void clickedOn(MetricObject o) {
		
		System.out.println("Showing details for " + o);
		object_metrics_table.setMetrics(o.getBoundMetrics());
		object_caption_container.setCaption("Metriken von " + o);
		
		if (o instanceof VisOverlayNode) {
			makeNewTopPane(new OverlayNodeDescriptionPane((VisOverlayNode)o));
		} else if (o instanceof VisOverlayEdge) {
			makeNewTopPane(new OverlayEdgeDescriptionPane((VisOverlayEdge)o));
		} else if (o instanceof VisOverlayGraph) {
			makeNewTopPane(new OverlayGraphDescriptionPane((VisOverlayGraph)o));
		} else {
			makeNewTopPane(null);
		}
		
	}

	public void reset() {
		this.revalidate();
		this.repaint();
	}

	
}
