package de.tud.kom.p2psim.impl.util.vis.ui.common;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JSlider;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.model.EventTimeline;
import de.tud.kom.p2psim.impl.util.vis.model.ModelRefreshListener;
import de.tud.kom.p2psim.impl.util.vis.model.TimelineEventListener;
import de.tud.kom.p2psim.impl.util.vis.model.VisDataModel;
import de.tud.kom.p2psim.impl.util.vis.ui.common.densityPane.DensityView;

/**
 * Slider, der es ermöglicht, die Position der Timeline zu ändern.
 * @author Leo Nobach
 *
 */
public class TimelineSlider extends JPanel implements TimelineEventListener, MouseListener, ModelRefreshListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 8975112526022783477L;
	
	private static final int SLIDER_SIZE = 1000;
	
	JSlider slider = new JSlider(0, SLIDER_SIZE);
	DensityView density = new DensityView();
	
	public TimelineSlider() {
		
		slider.addMouseListener(this);
		EventTimeline.addEventListener(this);
		//slider.setPreferredSize(new Dimension(800, 20));
		
		this.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		//c.weighty = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		this.add(slider, c);
		
		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 1;
		//1c.weighty = 1/3;
		c.fill = GridBagConstraints.HORIZONTAL;
		this.add(density, c);
		
		density.setEventTimeline(Controller.getTimeline());
		density.setPadding(6, 6);
		
		VisDataModel.addRefreshListener(this);
		enableProperly();

	}

	private void sliderChanged(int sliderValue) {
		
		long timeline_time = (long)(Controller.getTimeline().getMaxTime()*((double)slider.getValue()/(double)SLIDER_SIZE));
		this.changeTimeline(timeline_time);
	}
	
	private void changeTimeline(long time) {
		Controller.getTimeline().jumpToTime(time);
	}

	private void refreshSlider() {
		
		EventTimeline tl = Controller.getTimeline();
		int value = (int)Math.rint(SLIDER_SIZE*((double)tl.getActualTime()/(double)tl.getMaxTime()));
		this.slider.setValue(value);
	}
	
	@Override
	public void actualTimeChanged(EventTimeline invoker) {
		if (invoker == Controller.getTimeline()) {
			this.refreshSlider();
		}
	}

	@Override
	public void maxTimeChanged(EventTimeline invoker) {
		if (invoker == Controller.getTimeline()) {
			this.refreshSlider();
		}
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		if (slider.isEnabled()) this.sliderChanged(slider.getValue());
	}
	
	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void modelNeedsRefresh(VisDataModel model) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void newModelLoaded(VisDataModel model) {
		enableProperly();
		if (model != null) density.setEventTimeline(model.getTimeline());
	}

	private void enableProperly() {
		slider.setEnabled(Controller.getModel() != null);
	}

	@Override
	public void simulationFinished(VisDataModel model) {
		// TODO Auto-generated method stub
		
	}
	
	


}
