package de.tud.kom.p2psim.impl.util.vis.ui.common.dialogs;

import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.model.VisDataModel;
import de.tud.kom.p2psim.impl.util.vis.util.Config;

/**
 * Dateiauswahldialog, mit dem neue Aufzeichnungen geladen und gespeichert
 * werden können.
 * 
 * @author Leo Nobach
 *
 */
public class RecordFileChooser extends JFileChooser{

	/**
	 * 
	 */
	private static final long serialVersionUID = -1308955789418633424L;
	
	static final String CFG_LAST_PATH = "UI/Dialogs/LoadSave/lastPath";
	
	static final String FILE_EXTENSION = "peerfact";
	
	public RecordFileChooser() {
		FileNameExtensionFilter filter = new FileNameExtensionFilter("Visualisierungsaufzeichnungen", FILE_EXTENSION);
		 this.setFileFilter(filter);
		 
		 this.setCurrentDirectory(new File (Config.getValue(CFG_LAST_PATH, "")));
	}
	
	/**
	 * Fragt nach einer zu öffnenden Datei. Wird eine Datei ausgewählt, wird sie geöffnet
	 * und bei Erfolg true zurückgegeben, sonst false.
	 * @return
	 */
	public boolean askForOpen() {
		if (this.showOpenDialog(Controller.getUIMainWindow()) == JFileChooser.APPROVE_OPTION) {
			this.openFile(this.getSelectedFile());
			setLastDirectory();
			return true;
		}
		return false;
	}

	/**
	 * Fragt, wo die Datei gespeichert werden soll. Wird eine Datei 
	 * ausgewählt, wird sie geöffnet und bei Erfolg true zurückgegeben,
	 * sonst false.
	 * @return
	 */
	public boolean askForSave() {
		if (this.showSaveDialog(Controller.getUIMainWindow()) == JFileChooser.APPROVE_OPTION) {
			this.saveFile(this.validateFileName(this.getSelectedFile()));
			setLastDirectory();
			return true;
		}
		return false;
	}
	

	private void setLastDirectory() {
		Config.setValue(CFG_LAST_PATH, this.getCurrentDirectory().getAbsolutePath());
	}
	
	private void saveFile(File selectedFile) {
		try {
			Controller.getModel().saveTo(selectedFile);
			Controller.getUIMainWindow().setTitleFileName(selectedFile.getName());
		} catch (IOException e) {
			System.out.println("Exception" + e.getMessage() + e.getLocalizedMessage());
			e.printStackTrace();
		}
	}

	private void openFile(File selectedFile) {
		try {
			Controller.loadModelFrontend(VisDataModel.fromFile(selectedFile));
			Controller.getModel().setName(selectedFile.getName());
		} catch (IOException e) {
			System.out.println("Exception");
			e.printStackTrace();
		}
	}

	private File validateFileName(final File file) {
		
		FileFilter filter = this.getFileFilter();
		
	    if (filter.accept(file)) {
	        return file;
	    }
	    String fileName = file.getName();
	    final int index = fileName.lastIndexOf(".");
	    if (index > 0) {
	        fileName = fileName.substring(0, index);
	    }
	 
	    final String newFileName = fileName + "." + FILE_EXTENSION;
	 
	    return new File(file.getParent(), newFileName);
	} 

}
