package de.tud.kom.p2psim.impl.util.vis.ui.common.menubar;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;

import javax.swing.JPanel;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.model.ModelRefreshListener;
import de.tud.kom.p2psim.impl.util.vis.model.VisDataModel;
import de.tud.kom.p2psim.impl.util.vis.ui.common.toolbar.MainToolBar;
import de.tud.kom.p2psim.impl.util.vis.ui.common.toolbar.PlayerToolBar;

/**
 * Menüleiste des Hauptfensters. Enthält außerdem Toolbars.
 * 
 * @author Leo Nobach
 */
public class VisMenuBar extends JPanel implements ModelRefreshListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1335636287732847967L;
	
	PlayerToolBar playerToolBar = new PlayerToolBar();
	FlowLayout fl;
	
	/**
	 * Menüleiste des Hauptfensters. Enthält außerdem Toolbars.
	 */
	public VisMenuBar() {
		fl = new FlowLayout();
		fl.setAlignment(FlowLayout.LEADING);
		this.setLayout(fl);
		
		/*
		 * Toolbars
		 */
		this.add(new MainToolBar());
		this.add(playerToolBar);
		this.add(Controller.getVisApi().getVisualizationSpecificToolbar());
		VisDataModel.addRefreshListener(this);
		setPlayerToolBar();
	}

	@Override
	public void modelNeedsRefresh(VisDataModel model) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void newModelLoaded(VisDataModel model) {
		setPlayerToolBar();
	}

	private void setPlayerToolBar() {
		playerToolBar.setVisible(Controller.getModel() != null);
	}
	
	@Override
	public void paint(Graphics g) {
		setCorrectDimensions(this, fl);
		super.paint(g);
	}
	
	/**
	 * Workaround, damit das FlowLayout Zeilen umbricht.
	 * 
	 * Gegebenfalls durch was besseres ersetzen!
	 * 
	 * Sucht die rechte untere
	 * Ecke des umschließenden Quadrates aller Komponenten in
	 * c und setzt die Breite und Höhe von c darauf.
	 */
	private void setCorrectDimensions(Container c, FlowLayout fl) {
		
		int newWidth = 0;
		int newHeight = 0;
		for (int i = 0; i < this.getComponentCount(); i++) {
			
			Component cmp = this.getComponent(i);
			int lastX = cmp.getWidth() + cmp.getX();
			int lastY = cmp.getHeight() + cmp.getY();
			
			if (lastX > newWidth) newWidth = lastX;
			if (lastY > newHeight) newHeight = lastY; 
			
		}
		
		newHeight += fl.getVgap();
		newWidth += fl.getHgap();
		
		Dimension newDimension = new Dimension(newWidth, newHeight);
		
		this.setPreferredSize(newDimension);
		
	}

	@Override
	public void simulationFinished(VisDataModel model) {
		// TODO Auto-generated method stub
		
	}


}
