package de.tud.kom.p2psim.impl.util.vis.ui.common.toolbar.elements;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.SystemColor;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.controller.player.PlayerEventListener;


/**
 * Ermöglicht es, die Geschwindigkeit zu setzen.
 * 
 * Geschwindigkeit = Länge einer virtuellen Zeiteinheit in ms.
 * 
 * @author leo
 *
 */
public class FPSChanger extends JPanel implements ChangeListener, PlayerEventListener {
	
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 2980517499160605264L;

	static final double MIN_Q = 0.25d;
	
	static final double MAX_Q = 100d;
	
	static final double EXPONENT = 0.025d;
	
	protected JLabel label = new JLabel();
	protected JSlider slider = new JSlider();
	
	public FPSChanger() {
		label.setPreferredSize(new Dimension(50, 20));
		label.setBackground(SystemColor.text);
		label.setOpaque(true);
		
		slider.setMinimum(0);
		slider.setMaximum(100);
		slider.addChangeListener(this);
		
		this.setLayout(new FlowLayout());
		this.add(label);
		this.add(slider);
		this.setToolTipText("Quantisierung einstellen");
		
		double initialQuantization = Controller.getPlayer().getQuantization();
		
		Controller.getPlayer().addEventListener(this);
		
		setSliderValueFromQuantization(initialQuantization);
		this.setQuantizationText(initialQuantization);
		
	}
	
	private double getQuantizationFromSliderValue() {
		
		return MIN_Q + (MAX_Q - MIN_Q)*(1f - Math.pow(1f - ((double)slider.getValue()/100), EXPONENT));
	}
	
	private void  setSliderValueFromQuantization(double q) {
		slider.setValue((int)Math.rint((1f - Math.pow(1f -((double) q - MIN_Q)/(MAX_Q - MIN_Q), 1f/EXPONENT))*100));
	}

	private void setQuantizationText(double q) {
		label.setText(q + "s");
	}

	@Override
	public void stateChanged(ChangeEvent arg0) {
		Controller.getPlayer().setQuantization(getQuantizationFromSliderValue());
	}
	
	@Override
	public void forward() {
	}

	@Override
	public void pause() {
	}

	@Override
	public void play() {
	}

	@Override
	public void reverse() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void quantizationChange(double quantization) {
		setSliderValueFromQuantization(quantization);
		this.setQuantizationText(quantization);
	}

	@Override
	public void speedChange(int speed) {
	}


}
