package de.tud.kom.p2psim.impl.util.vis.ui.common.toolbar.elements;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.SystemColor;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.tud.kom.p2psim.impl.util.vis.controller.Controller;
import de.tud.kom.p2psim.impl.util.vis.controller.player.PlayerEventListener;


/**
 * Ermöglicht es, die Geschwindigkeit zu setzen.
 * 
 * Geschwindigkeit = Länge einer virtuellen Zeiteinheit in ms.
 * 
 * @author leo
 *
 */
public class SpeedChanger extends JPanel implements ChangeListener, PlayerEventListener {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 3966024292754025010L;
	
	
	static final float MIN_SPEED = 1000f;
	
	static final float MAX_SPEED = 1f;
	
	static final double EXPONENT = 2.5f;
	
	protected JLabel label = new JLabel();
	protected JSlider slider = new JSlider();
	
	public SpeedChanger() {
		label.setPreferredSize(new Dimension(50, 20));
		label.setBackground(SystemColor.text);
		label.setOpaque(true);
		
		slider.setMinimum(0);
		slider.setMaximum(100);
		slider.addChangeListener(this);
		
		this.setLayout(new FlowLayout());
		this.add(label);
		this.add(slider);
		this.setToolTipText("Geschwindigkeit einstellen");
		
		int initialSpeed = Controller.getPlayer().getSpeed();
		
		Controller.getPlayer().addEventListener(this);
		
		setSliderValueFromSpeed(initialSpeed);
		this.setSpeedText(initialSpeed);
		
	}
	
	private int getSpeedFromSliderValue() {
		
		return (int)Math.rint(MIN_SPEED + (MAX_SPEED - MIN_SPEED)*(1f - Math.pow(1f - ((double)slider.getValue()/100), EXPONENT)));
	}
	
	private void  setSliderValueFromSpeed(int speed) {
		slider.setValue((int)Math.rint((1f - Math.pow(1f -((double) speed - MIN_SPEED)/(MAX_SPEED - MIN_SPEED), 1f/EXPONENT))*100));
	}

	private void setSpeedText(int speed) {
		label.setText(speed + "ms");
	}

	@Override
	public void stateChanged(ChangeEvent arg0) {
		Controller.getPlayer().setSpeed(getSpeedFromSliderValue());
	}

	@Override
	public void speedChange(int speed) {
		setSliderValueFromSpeed(speed);
		this.setSpeedText(speed);
	}
	
	@Override
	public void forward() {
	}

	@Override
	public void pause() {
	}

	@Override
	public void play() {
	}

	@Override
	public void reverse() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void quantizationChange(double quantization) {
		// TODO Auto-generated method stub
		
	}


}
