package de.tud.kom.p2psim.impl.util.vis.util;

import java.awt.Color;

/**
 * Werkzeuge zur Farbmanipulation
 * 
 * @author Leo Nobach
 * @version 3.0, 24.11.2008
 *
 */
public class ColorToolkit {

	/**
	 * Gibt eine hellere Version der übergebenen Farbe zurück, die z.B. als
	 * Hintergrund verwendet werden kann.
	 * @param color
	 * @return
	 */
	public static Color getLightColorFor(Color color) {
		float weaknessFactor = 0.1f;
		
		float[] compArray = color.getRGBColorComponents(null);
		
		//Farben invertieren
		for (int i=0; i<3; i++) {
			compArray[i] = 1-compArray[i];
		}
		
		//Macht die Farbe um den Faktor weißer.
		for (int i=0; i<3; i++) {
			compArray[i] = compArray[i]*weaknessFactor;
		}
		
		//Zieht den Graustich aus der Farbe.
		float grayPart = Math.min(compArray[0], Math.min(compArray[1], compArray[2]));
		for (int i=0; i<3; i++) {
			compArray[i] -= grayPart;
		}
		
		//Farben zurück invertieren
		for (int i=0; i<3; i++) {
			compArray[i] = 1-compArray[i];
		}
		
		return new Color(compArray[0], compArray[1], compArray[2]);
	}
	
	public static Color weighColor(Color cl1, Color cl2, double weight) {
		if (weight <= 0) return cl1;
		if (weight >= 1) return cl2;
		
		double invWeight = 1-weight;
		
		Color result = new Color((int) (cl1.getRed()*weight+cl2.getRed()*invWeight),
							(int) (cl1.getGreen()*weight+cl2.getGreen()*invWeight),
							(int) (cl1.getBlue()*weight+cl2.getBlue()*invWeight));
		
		return result;
	}
	
}
