package de.tud.kom.p2psim.impl.util.vis.util;

import java.awt.AWTEvent;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.AWTEventListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Fängt alle AWT-Key-Events ab und gibt sie an seine
 * KeyListener weiter.
 * @author
 * @version 3.0, 13.11.2008
 *
 */
public class GlobalKeyEventDispatcher {

	List<KeyListener> listeners = new ArrayList<KeyListener>();
	Window frame2BActive = null;
	
	/**
	 * Standard-Konstruktor
	 */
	public GlobalKeyEventDispatcher() {
		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {
		      public void eventDispatched(AWTEvent event) {
		            if (event instanceof KeyEvent)
		                  dispatch((KeyEvent) event);
		      }

		}, AWTEvent.KEY_EVENT_MASK );
	}
	
	/**
	 * GlobalKeyEventDispatcher, der Events nur an die EventListener
	 * meldet, wenn das Fenster frame2BActive im Vordergrund ist
	 * (isFocused)
	 * @param frame2BActive
	 */
	public GlobalKeyEventDispatcher(Window frame2BActive) {
		this();
		this.frame2BActive = frame2BActive;
	}
	
	/**
	 * Fügt einen KeyListener hinzu
	 * @param l
	 */
	public void addKeyListener(KeyListener l) {
		listeners.add(l);
	}
	
	void dispatch(KeyEvent event) {
		if (shallDispatch()) {
			if (event.getID() == KeyEvent.KEY_PRESSED)
			for (KeyListener listener : listeners) {
				listener.keyPressed(event);
			} else if (event.getID() == KeyEvent.KEY_RELEASED)
			for (KeyListener listener : listeners) {
				listener.keyReleased(event);
			} else if (event.getID() == KeyEvent.KEY_TYPED)
			for (KeyListener listener : listeners) {
				listener.keyTyped(event);
			}
		}
	}

	private boolean shallDispatch() {
		return (frame2BActive == null || frame2BActive.isFocused());
	}
	
	
}
