package de.tud.kom.p2psim.impl.util.vis.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Vergleicht zwei Collections und gibt die Differenz als Set zurück.
 * @author Leo Nobach
 * @version 3.0, 07.11.2008
 *
 * @param <T>
 */
public class SetDiff<T> {

	Collection<T> a;
	Collection<T> b;
	
	/**
	 * Standard-Konstruktor
	 * @param setA
	 * @param setB
	 */
	public SetDiff(Collection<T> setA, Collection<T> setB) {
		this.a = setA;
		this.b = setB;
	}
	
	/**
	 * Gibt alle Items aus, die sich in B befinden, aber nicht in A
	 * Ist b null, wird eine leere Liste zurückgegeben, ist a null, wird
	 * b zurückgegeben
	 * 
	 * @return
	 */
	public Collection<T> getItemsNotInA() {
		Set<T> diff = new HashSet<T>();
		if (b == null) return diff;
		if (a == null) return b;
		
		for (T item : b) {
			if (!a.contains(item)) diff.add(item);
		}
		return diff;
	}
	
	/**
	 * Gibt alle Items aus, die sich in A befinden, aber nicht in B.
	 * Ist a null, wird eine leere Liste zurückgegeben, ist b null, wird
	 * b zurückgegeben
	 * @return
	 */
	public Collection<T> getItemsNotInB() {
		Set<T> diff = new HashSet<T>();
		if (a == null) return diff;
		if (b == null) return a;
		for (T item : a) {
			if (!b.contains(item)) diff.add(item);
		}
		return diff;
	}
	
}
