package de.tud.kom.p2psim.impl.util.vis.util.gui;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.table.AbstractTableModel;

/**
 * Ein Tabellenmodell, in dem verschiedene Objekte ausgewählt werden können, und
 * eine Liste dieser Objekte zurückgegeben werden kann.
 * 
 * @author Leo Nobach
 * 
 * @param <TObj>
 */
public class IconObjectTableModel<TObj extends IconObject> extends AbstractTableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7789895021882856840L;

	List<TObj> objects;

	Set<TObj> selected = new HashSet<TObj>();

	String caption;

	public IconObjectTableModel(List<TObj> objects, String caption) {
		this.objects = objects;
		this.caption = caption;
	}

	@Override
	public String getColumnName(int col) {
		if (col == 0)
			return "";
		else if (col == 1)
			return caption;
		else
			return "";
	}

	@Override
	public int getColumnCount() {
		return 3;
	}

	@Override
	public int getRowCount() {
		return objects.size();
	}

	@Override
	public Object getValueAt(int row, int col) {
		if (col == 0)
			return objects.get(row).getRepresentingIcon();
		else if (col == 1)
			return objects.get(row).toString();
		else {
			return selected.contains(objects.get(row));
		}
	}

	@Override
	public void setValueAt(Object value, int row, int col) {
		if (col == 2) {
			if ((Boolean) value)
				selected.add(objects.get(row));
			else
				selected.remove(objects.get(row));
		}

	}

	@SuppressWarnings("unchecked")
	@Override
	public Class getColumnClass(int c) {
		if (c == 0)
			return ImageIcon.class;
		if (c == 1)
			return String.class;
		else
			return Boolean.class;
	}

	@Override
	public boolean isCellEditable(int row, int col) {
		return (col == 2);
	}

	public List<TObj> getListOfSelectedObjects() {
		return new ArrayList<TObj>(selected);

	}

	/**
	 * Wählt entweder alle Einträge der Liste, oder wählt alle ab.
	 * 
	 * @param value
	 */
	public void setAllSelected(boolean value) {
		if (!value) {
			selected.clear();
		} else {
			selected.addAll(objects);
		}
	}

}
