package de.tud.kom.p2psim.impl.util.vis.util.visualgraph;

import java.util.HashSet;
import java.awt.Color;
import java.io.Serializable;

import de.tud.kom.p2psim.impl.util.vis.model.ModelIterator;

/**
 * Abstrakter Knoten eines Graphen (bisher ungerichtet)
 * @author leo
 *
 * @param <TNode>
 * @param <TEdge>
 */
public abstract class Node <TNode extends Node<TNode, TEdge>,TEdge extends Edge<TNode,TEdge>> implements Serializable{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 2354593609095590995L;

	protected PositionInfo position;
	
	protected VisualGraph<TNode, TEdge> graph;
	
	public HashSet<Edge<TNode, TEdge>> edges = new HashSet<Edge<TNode, TEdge>>();

	public Node(PositionInfo pos) {
		this.position = pos;
	}
	
	public void setGraph(VisualGraph<TNode, TEdge> graph) {
		this.graph = graph;
	}
	
	public void unsetGraph(){
		this.graph = null;
	}
	
	public VisualGraph<TNode, TEdge> getGraph() {
		return this.graph;
	}
	
	public void setPos(PositionInfo pos) {
		this.position = pos;
	}
	
	/**
	 * Topologische Position
	 * @return
	 */
	public Coords getTopologicalPosition() {
		//System.out.println(position);
		return this.position.getTopoCoords();
	}
	
	/**
	 * Schematische Position
	 * @return
	 */
	public Coords getSchematicPosition() {
		return this.position.getSchemCoords();
	}
	
	/**
	 * Positionsinformation
	 * @return
	 */
	public PositionInfo getPositionInfo() {
		return this.position;
	}
	
	public abstract void iterate(ModelIterator it);

	public abstract Color getColor();
	
}
