package de.tud.kom.p2psim.impl.util.vis.visualization2d;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

/**
 * Behandelt Mausereignisse und führt gegebenfalls Kommandos in Simple2DVisualization aus.
 * @author Konstantin Pussep
 * @author Sebastian Kaune
 * @version 3.0, 13.11.2008
 *
 */

public class DefaultMouseHandler implements MouseListener, MouseMotionListener, MouseWheelListener {

	Simple2DVisualization vis;
	
	/**
	 * Position of last click. Needed for dragging of the scenario.
	 */
	private Point lastClickPosition = new Point(0,0);
	
	/**
	 * Standard-Konstruktor
	 * @param vis
	 */
	public DefaultMouseHandler(Simple2DVisualization vis) {
		super();
		this.vis = vis;
	}

	/**
	 * Wird aufgerufen, wenn die Maus ins Visualisierungsfenster geklickt wurde.
	 */
	@Override
	public void mouseClicked(MouseEvent event) {
		//Nothing to do
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		//Nothing to do
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		//Nothing to do
	}

	@Override
	public void mousePressed(MouseEvent event) {
		lastClickPosition = event.getPoint();
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		if (event.getButton() == MouseEvent.BUTTON1) {
			vis.handleClick(event.getPoint());
		}
	}
	
	@Override
	public void mouseMoved(MouseEvent e) {
		if (vis.clickHndlr.clickedOnABox(e.getPoint()))
			vis.setCursor(new Cursor(Cursor.HAND_CURSOR));
		else
			vis.setCursor(Cursor.getDefaultCursor());
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		
		vis.zoom(e.getWheelRotation()<0, e.getPoint());
		
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		Point pNew =e.getPoint();
		Point pDiff = new Point(pNew.x - lastClickPosition.x, pNew.y - lastClickPosition.y);
		
		vis.shiftView(pDiff);
		
		this.lastClickPosition = pNew;
	}
	
	
}
