package de.tud.kom.p2psim.impl.network.gnp;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class GeoLocationOracleTest extends Assert {

	GnpNetLayer CH, IT, US1, US2, CN, ISP_City_1, ISP_City_2, Region_1, Region_2, Hessen_DA1, Hessen_DA2, Hessen_KA, Berlin_1, Berlin_2, Johannesburg, California, Oregon, North_Carolina_1,
			North_Carolina_2;

	GnpNetLayerFactory netLayerFactoryGnp;

	GnpLatencyModel latencyModelGnp;

	@Before
	public void setUp() {
		try {
			latencyModelGnp = new GnpLatencyModel();
			latencyModelGnp.setUsePingErJitter(true);

			netLayerFactoryGnp = new GnpNetLayerFactory();
			netLayerFactoryGnp.setGnpFile("test/de/tud/kom/p2psim/impl/network/gnp/testISP.xml");
			netLayerFactoryGnp.setLatencyModel(latencyModelGnp);

			// fake entries (not realistic)
			CH = netLayerFactoryGnp.newNetLayer("Switzerland");
			IT = netLayerFactoryGnp.newNetLayer("Italy");
			US1 = netLayerFactoryGnp.newNetLayer("Massachusetts");
			US2 = netLayerFactoryGnp.newNetLayer("NewYork");
			CN = netLayerFactoryGnp.newNetLayer("EastAsia");
			ISP_City_1 = netLayerFactoryGnp.newNetLayer("ISP_City_Test1");
			ISP_City_2 = netLayerFactoryGnp.newNetLayer("ISP_City_Test2");
			Region_1 = netLayerFactoryGnp.newNetLayer("Region_Test1");
			Region_2 = netLayerFactoryGnp.newNetLayer("Region_Test2");

			// real measured entries (derived from measured data)
			Hessen_DA1 = netLayerFactoryGnp.newNetLayer("Hessen_DA1");
			Hessen_DA2 = netLayerFactoryGnp.newNetLayer("Hessen_DA2");
			Hessen_KA = netLayerFactoryGnp.newNetLayer("Hessen_KA");
			Berlin_1 = netLayerFactoryGnp.newNetLayer("Berlin_1");
			Berlin_2 = netLayerFactoryGnp.newNetLayer("Berlin_2");
			Johannesburg = netLayerFactoryGnp.newNetLayer("Johannesburg");
			California = netLayerFactoryGnp.newNetLayer("California");
			Oregon = netLayerFactoryGnp.newNetLayer("Oregon");
			North_Carolina_1 = netLayerFactoryGnp.newNetLayer("North_Carolina_1");
			North_Carolina_2 = netLayerFactoryGnp.newNetLayer("North_Carolina_2");

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testGeoPriority() {
		assertTrue(GeoLocationOracle.getGeoPriority(CN.getNetID(), US1.getNetID()) == 0);
		assertTrue(GeoLocationOracle.getGeoPriority(CH.getNetID(), IT.getNetID()) == 1);
		assertTrue(GeoLocationOracle.getGeoPriority(US1.getNetID(), US2.getNetID()) == 2);
		assertTrue(GeoLocationOracle.getGeoPriority(Region_1.getNetID(), Region_2.getNetID()) == 3);
		assertTrue(GeoLocationOracle.getGeoPriority(ISP_City_1.getNetID(), ISP_City_2.getNetID()) == 4);
	}

//	@Test
//	public void testGetPropagationDelay() {
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Hessen_DA1.getNetID(), Hessen_DA2.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Hessen_DA1.getNetID(), Hessen_KA.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Hessen_DA1.getNetID(), Berlin_1.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Hessen_DA1.getNetID(), Berlin_2.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Berlin_2.getNetID(), Berlin_1.getNetID()));
//
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(Oregon.getNetID(), California.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(North_Carolina_1.getNetID(), California.getNetID()));
//		System.out.println(GeoLocationOracle.getInternetPropagationDelay(California.getNetID(), North_Carolina_1.getNetID()));
//	}

//	@Test
//	public void testGeoDistance() {
//		System.out.println(GeoLocationOracle.getGeographicalDistance(Hessen_DA1.getNetID(), Hessen_KA.getNetID()));
//		System.out.println(GeoLocationOracle.getGeographicalDistance(Hessen_DA1.getNetID(), Berlin_2.getNetID()));
//		System.out.println(GeoLocationOracle.getGeographicalDistance(Hessen_DA1.getNetID(), North_Carolina_1.getNetID()));
//		System.out.println(GeoLocationOracle.getGeographicalDistance(Hessen_DA1.getNetID(), Johannesburg.getNetID()));
//	}

	@After
	public void tearDown() {
	}

}
