package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import static org.junit.Assert.*;


import java.math.BigInteger;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.AbstractNodeVisitor;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.BranchNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.LeafNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.Node;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.NodeVisitor;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.PseudoRootNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup.ConfigStub;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;

/**
 * Tests for AbstractNodeVisitor.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 * 
 */
public class AbstractNodeVisitorTest {

    private ConfigStub config;

    private static NodeVisitor<KademliaOverlayID> visitor;

    private NodeStub node;

    /**
     * Setup while loading class.
     * 
     * @throws java.lang.Exception
     */
    @BeforeClass
    public static void setUpBeforeClass() {
	visitor = new AbstractNodeVisitorStub();
    }

    /**
     * Setup before each test case.
     * 
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() {
	config = new ConfigStub();
	node = new NodeStub();
    }

    /**
     * Test method for
     * {@link hkademlia.components.routingtable.AbstractNodeVisitor#visit(hkademlia.components.routingtable.Node)}.
     */
    @Test
    public void testVisitNode() {
	try {
	    visitor.visit(node);
	    fail("Expected exception for unknown node type");
	} catch (UnsupportedOperationException ex) {
	    // expected
	}
	assertFalse("Node should not have been visited", node.called);
    }

    /**
     * Test method for
     * {@link hkademlia.components.routingtable.AbstractNodeVisitor#visit(hkademlia.components.routingtable.PseudoRootNode)}.
     */
    @Test
    public void testVisitPseudoRootNode() {
	PseudoRootNode<KademliaOverlayID> pseudoRoot = new PseudoRootNode<KademliaOverlayID>(
		null, config);
	pseudoRoot.registerNewChild(node);
	visitor.visit(pseudoRoot);
	assertTrue("Child node of pseudo root should have been visited",
		node.called);
    }

    /**
     * Test node stub - used to accept visitors.
     * 
     * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
     */
    private static class NodeStub implements Node<KademliaOverlayID> {

	public boolean called = false;

	@Override
	public void accept(NodeVisitor<KademliaOverlayID> visitor) {
	    called = true;
	}

	@Override
	public int getLevel() {
	    return 0; // not used
	}

	@Override
	public KademliaOverlayID getOwnID() {
	    return null; // not used
	}

	@Override
	public BigInteger getPrefix() {
	    return null; // not used
	}
    }

    /**
     * Stub of AbstractNodeVisitor to test the implemented methods.
     * 
     * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
     */
    private static class AbstractNodeVisitorStub extends
	    AbstractNodeVisitor<KademliaOverlayID> {

	@Override
	public void visit(BranchNode<KademliaOverlayID> node) {
	    // not used
	}

	@Override
	public void visit(LeafNode<KademliaOverlayID> node) {
	    // not used
	}
    }
}
