package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Collection;

import org.junit.Test;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.HKademliaRoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;
import de.tud.kom.p2psim.util.helpers.TestHelper;


/**
 * Tests for HKademliaRoutingTable.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class HKademliaRoutingTableTest extends
	AbstractClusterPreferenceRoutingTableTest {

    /**
     * {@inheritDoc}
     */
    @Override
    protected HKademliaRoutingTable<HKademliaOverlayID> getEmptyRoutingTable(
	    KademliaOverlayContact<HKademliaOverlayID> owner) {
	return new HKademliaRoutingTable<HKademliaOverlayID>(owner, config);
    }

    /**
     * Lookup with cluster restriction and reference works as expected.
     */
    @Test
    public void testLookupWithClusterRestriction() {
	HKademliaRoutingTable<HKademliaOverlayID> hRoutingTable;
	hRoutingTable = getEmptyRoutingTable(ownContact);

	KademliaOverlayContact<HKademliaOverlayID> c11001010, c11001000;
	KademliaOverlayContact<HKademliaOverlayID> c10001010, c10001000;
	c11001010 = TestHelper.createContact("11001010", config);
	c11001000 = TestHelper.createContact("11001000", config);
	c10001010 = TestHelper.createContact("10001010", config);
	c10001000 = TestHelper.createContact("10001000", config);

	hRoutingTable.addContact(c11001010);
	hRoutingTable.addContact(c11001000);
	hRoutingTable.addContact(c10001010);
	hRoutingTable.addContact(c10001000);

	KademliaOverlayKey k11000000 = new KademliaOverlayKey("11000000",
		config);
	HKademliaOverlayID i00001010 = new HKademliaOverlayID("1010", config);
	HKademliaOverlayID i00000000 = new HKademliaOverlayID("0", config);
	HKademliaOverlayID i00001000 = new HKademliaOverlayID("1000", config);

	Collection<KademliaOverlayContact<HKademliaOverlayID>> result;

	// look up 11000000 with reference cluster 1010, depth >= 2
	result = hRoutingTable.localLookup(k11000000, 2, 2, i00001010);
	assertTrue("Result contains 11001010", result.contains(c11001010));
	assertTrue("Result contains 10001010", result.contains(c10001010));

	// look up 11000000 with reference cluster 1010, depth >= 1
	result = hRoutingTable.localLookup(k11000000, 2, 1, i00001010);
	assertTrue("Result contains 11001010", result.contains(c11001010));
	assertTrue("Result contains 11001000", result.contains(c11001000));

	// look up 11000000 with reference cluster 0000, depth >= 2
	result = hRoutingTable.localLookup(k11000000, 2, 2, i00000000);
	assertEquals("Result is empty", 0, result.size());

	// look up 11000000 with reference cluster 0000, depth >= 1
	result = hRoutingTable.localLookup(k11000000, 2, 1, i00000000);
	assertEquals("Result is empty", 0, result.size());

	// look up 11000000 with reference cluster 0000, depth >= 0
	result = hRoutingTable.localLookup(k11000000, 2, 0, i00000000);
	assertTrue("Result contains 11001010", result.contains(c11001010));
	assertTrue("Result contains 11001000", result.contains(c11001000));

	// look up 11000000 with reference cluster 1000, depth >= 2
	result = hRoutingTable.localLookup(k11000000, 2, 2, i00001000);
	assertTrue("Result contains 11001000", result.contains(c11001000));
	assertTrue("Result contains 10001000", result.contains(c10001000));

	// look up 11000000 with reference cluster 1000, depth >= 1
	result = hRoutingTable.localLookup(k11000000, 2, 1, i00001000);
	assertTrue("Result contains 11001000", result.contains(c11001000));
	assertTrue("Result contains 11001010", result.contains(c11001010));
    }

}
