package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import static org.junit.Assert.*;

import java.util.Collection;


import org.junit.Test;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.KademliaRoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTable;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayKey;
import de.tud.kom.p2psim.util.helpers.TestHelper;


/**
 * Tests for KademliaRoutingTable.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class KademliaRoutingTableTest extends AbstractRoutingTableTest {

    /**
     * {@inheritDoc}
     */
    @Override
    protected RoutingTable<HKademliaOverlayID> getEmptyRoutingTable(
	    KademliaOverlayContact<HKademliaOverlayID> owner) {
	return new KademliaRoutingTable<HKademliaOverlayID>(owner, config);
    }

    /**
     * Make sure that contacts do not replace each other (that is, clusters do
     * not matter here) and that all contacts from the buckets are visible.
     * 
     * @see KandyRoutingTableTest#testClustersDoMatterVisibilityRestriction()
     * @see HKademliaRoutingTableTest#testClustersDoMatterNoVisibilityRestriction()
     */
    @Test
    public void testClustersDontMatterAllVisible() {
	// own contact is 00001001
	KademliaOverlayContact<HKademliaOverlayID> c01010000 = TestHelper
		.createContact("01010000", config);
	KademliaOverlayContact<HKademliaOverlayID> c01111000 = TestHelper
		.createContact("01111000", config);
	KademliaOverlayContact<HKademliaOverlayID> c01011001 = TestHelper
		.createContact("01011001", config);
	// add 3 contacts, expect to see first 2 (all visible, no replacement)
	routingTable.addContact(c01010000);
	routingTable.addContact(c01111000);
	routingTable.addContact(c01011001);
	Collection<KademliaOverlayContact<HKademliaOverlayID>> result;
	result = routingTable.localLookup(new KademliaOverlayKey("01000000",
		config), 2);
	assertEquals("RT returns 2 results", 2, result.size());
	assertTrue("Result contains 01010000", result.contains(c01010000));
	assertTrue("Result contains 01111000", result.contains(c01111000));
	// 01011001 in cache -> not available.
    }

}
