package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import static org.junit.Assert.*;


import java.math.BigInteger;

import org.junit.Before;
import org.junit.Test;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.AbstractNodeVisitor;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.BranchNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.LeafNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.ParentNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup.ConfigStub;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;

/**
 * Test LeafNode.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class LeafNodeTest {

    private ConfigStub config;

    private ParentNode<KademliaOverlayID> pseudoRoot;

    private LeafNode<KademliaOverlayID> root;

    private LeafNode<KademliaOverlayID> leaf;

    /**
     * Setup.
     */
    @Before
    public void setUp() {
	config = new ConfigStub();
	pseudoRoot = new ParentNodeStub<KademliaOverlayID>(-1, null, null,
		BigInteger.ZERO);
	root = new LeafNode<KademliaOverlayID>(pseudoRoot, config);
	leaf = new LeafNode<KademliaOverlayID>(BigInteger.ONE, pseudoRoot,
		config);
    }

    /**
     * Test root-bucket constructor.
     */
    @Test
    public void testLeafNodeParentNode() {
	assertEquals("Root bucket should have prefix zero", BigInteger.ZERO,
		root.getPrefix());
	assertEquals("Parent should be pseudoRoot", pseudoRoot, root
		.getParent());
	assertNotNull("Bucket map should have been initialised", root.kBucket);
	assertNotNull("Cache map should have been initialised",
		root.replacementCache);
    }

    /**
     * Test more general constructor.
     */
    @Test
    public void testLeafNodeBigIntegerParentNode() {
	assertEquals("Leaf bucket should have prefix one", BigInteger.ONE, leaf
		.getPrefix());
	assertEquals("Parent should be pseudoRoot", pseudoRoot, leaf
		.getParent());
	assertNotNull("Bucket map should have been initialised", leaf.kBucket);
	assertNotNull("Cache map should have been initialised",
		leaf.replacementCache);
    }

    /**
     * Test accept.
     */
    @Test
    public void testAccept() {
	VisitorStub visitor = new VisitorStub();
	leaf.accept(visitor);
	assertTrue("LeafNode-visit-method should have been called in visitor",
		visitor.called);
    }

    /**
     * Visitor stub that permits to check whether the right visit method is
     * called.
     * 
     * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
     */
    private static class VisitorStub extends
	    AbstractNodeVisitor<KademliaOverlayID> {

	public boolean called = false;

	@Override
	public void visit(BranchNode<KademliaOverlayID> node) {
	    fail("This method should not be called as there is no BranchNode involved here.");

	}

	@Override
	public void visit(LeafNode<KademliaOverlayID> node) {
	    this.called = true;
	}
    }

}
