package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;


import java.math.BigInteger;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.Node;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.NodeVisitor;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.ParentNode;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;


/**
 * Test stub for ParentNode; exposes all instance variables as public.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
class ParentNodeStub<T extends KademliaOverlayID> implements ParentNode<T> {

	public int level;
	public T id;
	public Node<T> child = null;
	public BigInteger prefix;

	/**
	 * Creates a new ParentNode stub. Instance variables are public accessible.
	 * All parameters to this method (at least the non-primitive ones) may be
	 * set to null if desired.
	 * 
	 * @param level
	 *            the level of this node.
	 * @param id
	 *            the ID of this node.
	 * @param child
	 *            a preset child of this node.
	 * @param prefix
	 *            the prefix of this node.
	 */
	public ParentNodeStub(int level, T id, Node<T> child,
			BigInteger prefix) {
		this.level = level;
		this.id = id;
		this.child = child;
		this.prefix = prefix;
	}

	@Override
	public int getLevel() {
		return level;
	}

	@Override
	public T getOwnID() {
		return id;
	}

	@Override
	public void registerNewChild(Node<T> child) {
		this.child = child;
	}

	@Override
	public void accept(NodeVisitor<T> visitor) {
		visitor.visit(this);
	}

	@Override
	public BigInteger getPrefix() {
		return prefix;
	}
}