package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTableEntry;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTablePredicates.MinimumClusterDepthRestrictor;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup.ConfigStub;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.util.helpers.TestHelper;

/**
 * Tests for Predicates used in routing tables.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class RoutingTablePredicatesTest {

    static ConfigStub config;

    /**
     * Configure necessary simulator-wide variables
     */
    @BeforeClass
    public static void beforeClass() {
	// Create dummy scheduler
	TestHelper.initSimulator();
	config = new ConfigStub();
	config.idLength = 6;
	config.hierarchyTreeOrder = 1;
	config.hierarchyDepth = 2;
    }

    /**
     * Test HierarchyRestrictor.
     */
    @Test
    public void testHierarchyRestrictor() {
	HKademliaOverlayID i010101 = new HKademliaOverlayID(21, config);
	RoutingTableEntry<HKademliaOverlayID> e010101 = new RoutingTableEntry<HKademliaOverlayID>(
		TestHelper.createContact(i010101, 21));
	HKademliaOverlayID i011101 = new HKademliaOverlayID(29, config);
	RoutingTableEntry<HKademliaOverlayID> e011101 = new RoutingTableEntry<HKademliaOverlayID>(
		TestHelper.createContact(i011101, 29));
	HKademliaOverlayID i110100 = new HKademliaOverlayID(52, config);
	RoutingTableEntry<HKademliaOverlayID> e110100 = new RoutingTableEntry<HKademliaOverlayID>(
		TestHelper.createContact(i110100, 52));
	HKademliaOverlayID i011111 = new HKademliaOverlayID(31, config);
	RoutingTableEntry<HKademliaOverlayID> e011111 = new RoutingTableEntry<HKademliaOverlayID>(
		TestHelper.createContact(i011111, 31));

	MinimumClusterDepthRestrictor<HKademliaOverlayID> r010101_2 = new MinimumClusterDepthRestrictor<HKademliaOverlayID>(
		i010101, 2);
	assertTrue("ref 010101 >= 2: 010101 valid", r010101_2.isTrue(e010101));
	assertTrue("ref 010101 >= 2: 011101 valid", r010101_2.isTrue(e011101));
	assertFalse("ref 010101 >= 2: 110100 invalid", r010101_2
		.isTrue(e110100));
	assertFalse("ref 010101 >= 2: 011111 invalid", r010101_2
		.isTrue(e011111));

	MinimumClusterDepthRestrictor<HKademliaOverlayID> r011101_1 = new MinimumClusterDepthRestrictor<HKademliaOverlayID>(
		i011101, 1);
	assertTrue("ref 011101 >= 1: 010101 valid", r011101_1.isTrue(e010101));
	assertTrue("ref 011101 >= 1: 011101 valid", r011101_1.isTrue(e011101));
	assertTrue("ref 011101 >= 1: 110100 valid", r011101_1.isTrue(e110100));
	assertFalse("ref 011101 >= 1: 011111 invalid", r011101_1
		.isTrue(e011111));

	MinimumClusterDepthRestrictor<HKademliaOverlayID> r011111_0 = new MinimumClusterDepthRestrictor<HKademliaOverlayID>(
		i011111, 0);
	assertTrue("ref 011111 >= 0: 010101 valid", r011111_0.isTrue(e010101));
	assertTrue("ref 011111 >= 0: 011101 valid", r011111_0.isTrue(e011101));
	assertTrue("ref 011111 >= 0: 110100 valid", r011111_0.isTrue(e110100));
	assertTrue("ref 011111 >= 0: 011111 valid", r011111_0.isTrue(e011111));
    }

}
