package de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup;

import static org.junit.Assert.*;

import java.math.BigInteger;

import org.junit.Before;
import org.junit.Test;

import de.tud.kom.p2psim.api.transport.TransLayer;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.Node;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup.HAbstractNodeFactory;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.setup.KademliaSetup;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;

/**
 * Tests for HAbstractNodeFactory.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 */
public class HAbstractNodeFactoryTest {

    private HAbstractNodeFactory factory;

    /**
     * Setup.
     */
    @Before
    public void before() {
	ConfigStub conf = new ConfigStub();
	conf.clusterMappingFilePath = "test/kademlia2/setup/clusterMappingTestFile.properties";
	KademliaSetup.setConfig(conf);
	factory = new HAbstractNodeFactoryStub();
    }

    /**
     * Test getClusterSuffix
     */
    @Test
    public void testGetClusterSuffix() {
	assertEquals("Cluster suffix of 'Europe' is 0100", new BigInteger(
		"0100", 2), factory.getClusterSuffix("Europe"));
	assertEquals("Cluster suffix of 'North+America' is 1000",
		new BigInteger("1000", 2), factory
			.getClusterSuffix("North+America"));
	assertEquals("Cluster suffix of non-existing group identifier is 0",
		BigInteger.ZERO, factory.getClusterSuffix("does-not-exist"));
    }

    /**
     * Test if HKademliaOverlayIDs are correctly constructed using the given
     * data. (This is a general test that is not directly related to
     * HAbstractNodeFactory.)
     */
    @Test
    public void testConstructHKademliaOverlayID() {
	ConfigStub conf = new ConfigStub();
	conf.idLength = 8;
	conf.hierarchyDepth = 1;
	conf.hierarchyTreeOrder = 2;

	HKademliaOverlayID europe = new HKademliaOverlayID("10101010", conf)
		.setCluster(factory.getClusterSuffix("Europe"));
	assertEquals("europe-ID is 10101000", new BigInteger("10101000", 2),
		europe.getBigInt());

	conf.hierarchyDepth = 2;
	HKademliaOverlayID northAmerica = new HKademliaOverlayID("01010101",
		conf).setCluster(factory.getClusterSuffix("North+America"));
	assertEquals("northAmerica-ID is 01011000", new BigInteger("01011000",
		2), northAmerica.getBigInt());
    }

    /**
     * Stub implementation of HAbstractNodeFactory for test purposes.
     * 
     * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
     */
    private static class HAbstractNodeFactoryStub extends HAbstractNodeFactory {

	@Override
	protected Node<HKademliaOverlayID> buildHierarchicalNode(
		HKademliaOverlayID id, short port, TransLayer msgMgr) {
	    // empty
	    return null;
	}

    }
}
