package de.tud.kom.p2psim.util.helpers;

import junitx.util.PrivateAccessor;
import de.tud.kom.p2psim.api.network.NetID;
import de.tud.kom.p2psim.api.transport.TransInfo;
import de.tud.kom.p2psim.impl.network.simple.SimpleNetID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.components.routingtable.RoutingTableConfig;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.HKademliaOverlayID;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayContact;
import de.tud.kom.p2psim.impl.overlay.dht.kademlia2.types.KademliaOverlayID;
import de.tud.kom.p2psim.impl.simengine.Scheduler;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.transport.DefaultTransInfo;

/**
 * This class is used by JUnit-Tests to set up some minimally necessary
 * simulator-wide variables.
 * 
 * @author Tobias Lauinger <tl1003@rbg.informatik.tu-darmstadt.de>
 * 
 */
public class TestHelper {

    /**
     * Makes sure the simulator is in a state in which basic unit tests can be
     * carried out, for instance, a scheduler is registered.
     */
    public static void initSimulator() {
	Simulator.getInstance();
    }

    /**
     * Dirty hack to set the simulation time (only for testing purposes where
     * the simulation engine itself is not running).
     * 
     * @param time
     *                the simulation time to be set.
     * @throws NoSuchFieldException
     *                 if the variable name in the scheduler implementation
     *                 holding the current simulation time has been modified but
     *                 not been adapted in this method.
     */
    public static void setSimulationTime(long time) throws NoSuchFieldException {
	Scheduler sched = (Scheduler) PrivateAccessor.getField(Simulator.class,
		"scheduler");

	PrivateAccessor.setField(sched, "currentTime", time);
    }

    /**
     * Creates a KademliaOverlayContact with the given <code>id</code>, and
     * pseudo TransportAddress.
     * 
     * @param id
     *                the KademliaOverlayID to be used in the new contact.
     * @param randomNumber
     *                some random number used to generate NetIDs and ports
     *                needed in TransportAddresses needed for the returned
     *                KademliaOverlayContact.
     * @return a new contact with the given <code>id</code>.
     */
    public static <T extends KademliaOverlayID> KademliaOverlayContact<T> createContact(
	    T id, int randomNumber) {
	NetID someNetID;
	TransInfo someTransportAddress;
	KademliaOverlayContact<T> result;

	someNetID = new SimpleNetID(randomNumber);
	someTransportAddress = new DefaultTransInfo(someNetID,
		(short) (2 * randomNumber));
	result = new KademliaOverlayContact<T>(id, someTransportAddress);
	return result;
    }

    /**
     * Creates a KademliaOverlayContact with the given <code>id</code> as a
     * HKademliaOverlayID.
     * 
     * @param id
     *                a String that contains the binary representation of the
     *                ID.
     * @param conf
     *                a RoutingTableConfig with constants for ID length etc.
     * @return a new contact with the given <code>id</code>.
     */
    public static KademliaOverlayContact<HKademliaOverlayID> createContact(
	    String id, RoutingTableConfig conf) {
	HKademliaOverlayID oid = new HKademliaOverlayID(id, conf);
	return createContact(oid, oid.hashCode());
    }

    /**
     * Creates a KademliaOverlayContact with the given <code>id</code> as a
     * KademliaOverlayID.
     * 
     * @param id
     *                a String that contains the binary representation of the
     *                ID.
     * @param conf
     *                a RoutingTableConfig with constants for ID length etc.
     * @return a new contact with the given <code>id</code>.
     */
    public static KademliaOverlayContact<KademliaOverlayID> createStdContact(
	    String id, RoutingTableConfig conf) {
	KademliaOverlayID oid = new KademliaOverlayID(id, conf);
	return createContact(oid, oid.hashCode());
    }

}
